package ai; 

import bosses.BelethManager; 
import l2ft.gameserver.ai.DefaultAI; 
import l2ft.gameserver.model.L2Character; 
import l2ft.gameserver.model.L2Skill; 
import l2ft.gameserver.model.L2Skill.SkillType; 
import l2ft.gameserver.model.instances.L2NpcInstance; 
import l2ft.gameserver.tables.SkillTable; 
import l2ft.util.Rnd; 
import l2ft.util.Util; 

public class Beleth extends DefaultAI { 

    private final L2Skill Bleed = SkillTable.getInstance().getInfo(5495, 1); 
    private final L2Skill HornOfRising = SkillTable.getInstance().getInfo(5497, 1); 
    private static final L2Skill Fireball = SkillTable.getInstance().getInfo(5496, 1); 

    public Beleth(L2Character actor) { 
        super(actor); 
        AI_TASK_DELAY = 1000; 
        AI_TASK_ACTIVE_DELAY = 1000; 
    } 

    @Override 
    public boolean isGlobalAI() { 
        return true; 
    } 

    @Override 
    protected boolean createNewTask() { 
        L2NpcInstance actor = getActor(); 
        if (actor == null) { 
            return true; 
        } 

        if (!BelethManager.getZone().checkIfInZone(actor)) { 
            teleportHome(true); 
            return false; 
        } 

        clearTasks(); 

        L2Character target; 
        if ((target = prepareTarget()) == null) { 
            return false; 
        } 

        if (!BelethManager.getZone().checkIfInZone(target)) { 
            target.removeFromHatelist(actor, false); 
            return false; 
        } 
        if(Rnd.chance(40) && !actor.isCastingNow() && Util.checkIfInRange(600, actor, target, false)) 
        { 
            addTaskCast(target, Fireball); 
        } 
        return true; 
    } 

    /** если видит хилл скил 
     * @param skill 
     * @param caster 
     */ 
    @Override 
    protected void onEvtSeeSpell(L2Skill skill, L2Character caster) { 
        L2NpcInstance npc = getActor(); 
        if (npc != null && !npc.isDead() && (npc.getNpcId() == 29118 || npc.getNpcId() == 29119) && !npc.isCastingNow() && skill.getSkillType() == SkillType.HEAL && Rnd.get(100) < 80 && Util.checkIfInRange(600, npc, caster, false)) { 
            clearTasks(); 
            addTaskCast(caster, HornOfRising); 
            //npc.setTarget(caster); 
            //npc.doCast(HornOfRising, caster, true); 
        } 
    } 

    @Override 
    public void onEvtAttacked(L2Character attacker, int damage) { 
        BelethManager.setLastAttackTime(); 
        if (getActor().getNpcId() == 29118 || getActor().getNpcId() == 29119) { 
            BelethManager.onAttack(getActor(), attacker); 
            return; 
        } 
        super.onEvtAttacked(attacker, damage); 
    } 

    @Override 
    public void checkAggression(L2Character target) { 
        if (getActor().getNpcId() == 29118 || getActor().getNpcId() == 29119) { 
            BelethManager.onAggroRangeEnter(getActor(), target); 
        } 
        super.checkAggression(target); 
    } 

    @Override 
    protected void onEvtSpawn() { 
        L2NpcInstance actor = getActor(); 
        if (actor.getNpcId() == 29118 || actor.getNpcId() == 29119) { 
            actor.setRunning(); 
            if (actor.getAroundCharacters(300, 200).size() > 2 && Rnd.get(100) < 60) { 
                clearTasks(); 
                addTaskCast((L2Character) actor.getTarget(), Bleed); 
                //actor.doCast(Bleed, (L2Character) actor.getTarget(), true); 
            } 
        } 
    } 

    @Override 
    protected boolean maybeMoveToHome() { 
        L2NpcInstance actor = getActor(); 
        if (actor != null && !BelethManager.getZone().checkIfInZone(actor)) { 
            teleportHome(true); 
        } 
        return false; 
    } 
}  