package ai;

import l2ft.extensions.scripts.Functions;
import l2ft.gameserver.ai.DefaultAI;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2World;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.serverpackets.ExShowScreenMessage;

public class DestroyedTumor extends DefaultAI
{
	private static final int TUMOR_OF_DEATH = 18708;
	private static final int SOUL_COFFIN = 18711;
	private static final int RADIUS = 700;
	private static final long DEFAULT_REVIVE_TIME = 300000L;
	private static final long COFINS_CHECK_TIME = 30000L;
	private long _reviveTime = 0L;
	private long _cofinsTime = 0L;
	private long _penaltyTime = 0L;

	public DestroyedTumor(L2Character actor)
	{
		super(actor);
		actor.setImobilised(true);
	}

	protected void onEvtSpawn()
	{
		_reviveTime = (System.currentTimeMillis() + DEFAULT_REVIVE_TIME);
		_cofinsTime = (System.currentTimeMillis() + COFINS_CHECK_TIME);

		super.onEvtSpawn();
	}

	protected boolean thinkActive()
	{
		L2NpcInstance actor = getActor();
		if (actor == null) {
			return false;
		}

		if ((_reviveTime > 0L) && (System.currentTimeMillis() > _reviveTime - _penaltyTime))
		{
			_reviveTime = (System.currentTimeMillis() + DEFAULT_REVIVE_TIME);
			L2NpcInstance tumor = Functions.addSpawnToInstance(TUMOR_OF_DEATH, actor.getLoc(), 0, actor.getReflectionId());
			tumor.setCurrentHp(tumor.getMaxHp() / 3, false);
			Reflection r = actor.getReflection();
			if (r.getInstancedZoneId() == 119)
				r.broadcastOnScreenMessage("q.696.TumorRevived", 3000, ExShowScreenMessage.ScreenMessageAlign.MIDDLE_CENTER, false, 1, -1, true, new String[0]);
			else if (r.getInstancedZoneId() == 120)
				r.broadcastOnScreenMessage("q.697.TumorRevived", 3000, ExShowScreenMessage.ScreenMessageAlign.MIDDLE_CENTER, false, 1, -1, true, new String[0]);
			actor.deleteMe();
			return true;
		}

		if ((_cofinsTime > 0L) && (System.currentTimeMillis() > _cofinsTime))
		{
			_cofinsTime = (System.currentTimeMillis() + COFINS_CHECK_TIME);
			for (L2NpcInstance mob : L2World.getAroundNpc(actor, RADIUS, 128))
				if ((mob.getNpcId() == SOUL_COFFIN) && (!mob.isDead()))
					_penaltyTime += 5000L;
			return true;
		}

		return super.thinkActive();
	}
}