package ai;

import l2ft.common.ThreadPoolManager;
import l2ft.gameserver.ai.CtrlIntention;
import l2ft.gameserver.ai.Mystic;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.serverpackets.ExShowScreenMessage;
import l2ft.gameserver.serverpackets.ExShowScreenMessage.ScreenMessageAlign;
import l2ft.gameserver.serverpackets.PlaySound;
import l2ft.gameserver.tables.SkillTable;
import l2ft.util.Location;
import l2ft.util.Rnd;
import bosses.FreyaManager;
import bosses.FreyaManager.World;

public class Freya extends Mystic
{
	public Freya(L2Character actor)
	{
		super(actor);
		AI_TASK_DELAY = 100;
		AI_TASK_ACTIVE_DELAY = 100;
	}

	private int s_freya_eternal_blizzard1 = 6274;
	private int s_freya_eternal_blizzard_power1 = 6275;
	private int s_freya_eternal_blizzard_signal1 = 6276;

	@Override
	protected boolean thinkActive()
	{
		L2NpcInstance actor = getActor();
		if(actor == null)
			return true;
		if(getIntention() == CtrlIntention.AI_INTENTION_ACTIVE)
		{
			for(L2Player player : actor.getReflection().getPlayers())
			{
				if(player != null)
				{
					if(player.isDead())
						continue;
					if(Rnd.getRnd(600000))
						continue;
					player.addDamageHate(actor, 0, 100); // Это нужно, чтобы гвард не перестал атаковать цель после первых ударов
					actor.setRunning(); // Включаем бег...
					actor.setAttackTimeout(Integer.MAX_VALUE + System.currentTimeMillis()); // Это нужно, чтобы не сработал таймаут
					actor.getAI().setAttackTarget(player); // На всякий случай, не обязательно делать
					actor.getAI().changeIntention(CtrlIntention.AI_INTENTION_ATTACK, player, null); // Переводим в состояние атаки
					actor.getAI().addTaskAttack(player); // Добавляем отложенное задание атаки, сработает в самом конце движения
					return true;
				}
			}
		}
		return true;
	}

	@Override
	public void startAITask()
	{
		final L2NpcInstance actor = getActor();
		if(_aiTask == null && actor != null)
		{
			ThreadPoolManager.getInstance().scheduleGeneral(new Runnable()
			{
				@Override
				public void run()
				{
					setAITask(actor);
				}
			}, 6000);
		}
		super.startAITask();
	}

	@Override
	public void onIntentionAttack(L2Character target)
	{
		L2NpcInstance actor = getActor();
		if(actor == null)
			return;
		if(target.getNpcId() == 18855 || target.getNpcId() == 18856 || target.getNpcId() == 18854 || target.getNpcId() == 25699 || target.getNpcId() == 25700 || target.getNpcId() == 18853 || target.getNpcId() == 29179 || target.getNpcId() == 29180 || target.getNpcId() == 29177 || target.getNpcId() == 29178)
		{
			setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
			return;
		}
		super.onIntentionAttack(target);
	}

	public static enum Methods
	{
		AddMoveToDesire,
		AddUseSkillDesire,
		AddTimerEx
	}
	private void runMethod(final Methods method, int time, final Object... arg)
	{
		final L2NpcInstance npc = getActor();
		ThreadPoolManager.getInstance().scheduleGeneral(new Runnable()
		{
			@Override
			public void run()
			{
				switch(method)
				{
					case AddMoveToDesire:
						npc.setRunning();
						addTaskMove(new Location((Integer)arg[0],(Integer)arg[1],(Integer)arg[2],(Integer)arg[3]), true);
						break;
					case AddUseSkillDesire:
						L2Character target = getAttackTarget();
						if(target != null)
							addTaskCast(target, (L2Skill)arg[0]);
						break;
					case AddTimerEx:
						World world = FreyaManager.getWorld(npc.getReflectionId());
						if(((String)arg[0]).equalsIgnoreCase("TIMER_eternal_blizzard") && !world.finish)
						{
							boolean isHardmode = world._ishardMode;
							String voice = "";
							screenMessage(npc.getReflection(), 1801111, ScreenMessageAlign.TOP_CENTER, false); // Я чувствую сильное влияние магии!
							switch(Rnd.get(3))
							{
								case 0:
									voice = "SystemMsg_ru.freya_voice_09";
									break;
								case 1:
									voice = "SystemMsg_ru.freya_voice_10";
									break;
								case 2:
									voice = "SystemMsg_ru.freya_voice_11";
									break;
								default:
									System.out.println("Error > FreyaManeger.runMethod.AddTimerEx.Rnd.get(3)");
									break;
							}
							npc.broadcastPacket(new PlaySound(voice));
							if(isHardmode) // HardMode???
							{
								if((world._Force == 0 && npc.getCurrentHp() < npc.getMaxHp() * 0.80) || (world._Force == 1 && npc.getCurrentHp() < npc.getMaxHp() * 0.60) || (world._Force == 2 && npc.getCurrentHp() < npc.getMaxHp() * 0.40) || (world._Force == 3 && npc.getCurrentHp() < npc.getMaxHp() * 0.20))
								{
									runMethod(Methods.AddUseSkillDesire, 2000, getSkill(6697, 1));
									world._Force++;
								}
								else
									runMethod(Methods.AddUseSkillDesire, 2000, getSkill(s_freya_eternal_blizzard_power1, 1));
								if(world.stages == 1)
									runMethod(Methods.AddTimerEx, ((Rnd.get(5) + 40) * 1000), "TIMER_eternal_blizzard");
								else if(world.stages == 3)
									runMethod(Methods.AddTimerEx, ((Rnd.get(5) + 35) * 1000), "TIMER_eternal_blizzard");
							}
							else
							{
								runMethod(Methods.AddUseSkillDesire, 2000, getSkill(s_freya_eternal_blizzard1, 1));
								runMethod(Methods.AddTimerEx, ((Rnd.get(5) + 55) * 1000), "TIMER_eternal_blizzard");
							}
						}
						else if(((String)arg[0]).equalsIgnoreCase("TIMER_use_freya_buff"))
						{
							addTaskCast(npc, getSkill(6284, 1));
							runMethod(Methods.AddTimerEx, 15 * 1000, "TIMER_use_freya_buff");
						}
						break;
				}
			}
		},
		time);
	}

	private void setAITask(L2NpcInstance npc)
	{
		World world = FreyaManager.getWorld(npc.getReflectionId());

		boolean isHardmode = world._ishardMode;

		switch(world.stages)
		{
			case 1:
				if(isHardmode)
				{
					clearNextAction();
					clearTasks();

					runMethod(Methods.AddUseSkillDesire, 100, getSkill(6285, 1));
					runMethod(Methods.AddMoveToDesire, 3000, 114730,-114805,-11200,50);
					screenMessage(npc.getReflection(), 1801097, ScreenMessageAlign.TOP_CENTER, false); // Фрея начинает двигаться.
					runMethod(Methods.AddTimerEx, 60000, "TIMER_eternal_blizzard", isHardmode);
					//runMethod(Methods.AddTimerEx, 15000, "TIMER_use_freya_buff", isHardmode);
				}
				else
				{
					clearNextAction();
					clearTasks();

					runMethod(Methods.AddMoveToDesire, 3000, 114730,-114805,-11200,50);
					screenMessage(npc.getReflection(), 1801097, ScreenMessageAlign.TOP_CENTER, false); // Фрея начинает двигаться.
					runMethod(Methods.AddTimerEx, 60000, "TIMER_eternal_blizzard", isHardmode);
					//runMethod(Methods.AddTimerEx, 15000, "TIMER_use_freya_buff", isHardmode);
				}
				break;
			case 3:
				if(isHardmode)
				{
					clearNextAction();
					clearTasks();

					runMethod(Methods.AddUseSkillDesire, 100, getSkill(6285, 1),1000000);
					runMethod(Methods.AddMoveToDesire, 10000, 114730,-114805,-11200,50);
					screenMessage(npc.getReflection(), 1801097, ScreenMessageAlign.TOP_CENTER, false); // Фрея начинает двигаться.
					runMethod(Methods.AddTimerEx, 50000, "TIMER_eternal_blizzard", isHardmode);
					//runMethod(Methods.AddTimerEx, 15000, "TIMER_use_freya_buff", isHardmode);
				}
				else
				{
					clearNextAction();
					clearTasks();

					runMethod(Methods.AddMoveToDesire, 5000, 114730,-114805,-11200,50);
					screenMessage(npc.getReflection(), 1801097, ScreenMessageAlign.TOP_CENTER, false); // Фрея начинает двигаться.
					runMethod(Methods.AddTimerEx, 50000, "TIMER_eternal_blizzard", isHardmode);
					//runMethod(Methods.AddTimerEx, 15000, "TIMER_use_freya_buff", isHardmode);
				}
				break;
		}
	}
			/*else if( ( isMove ) == 1 )
			{
				if(Rnd(10000) < 3333 )
				{
					if( gg->Rand(10000) < 5000 )
					{
						if( myself->Skill_GetConsumeMP(Ice_Ball) < ( attacker + 400 ) && myself->Skill_GetConsumeHP(Ice_Ball) < ( attacker + 328 ) && myself->Skill_InReuseDelay(Ice_Ball) == 0 )
						{
							myself->AddUseSkillDesire(attacker,Ice_Ball,0,1,1000000);
						}
					}
					else if( myself->Skill_GetConsumeMP(Ice_Ball) < ( attacker + 400 ) && myself->Skill_GetConsumeHP(Ice_Ball) < ( attacker + 328 ) && myself->Skill_InReuseDelay(Ice_Ball) == 0 )
					{
						myself->AddUseSkillDesire(( myself + 1504 ),Ice_Ball,0,1,1000000);
					}
				}
				if( gg->Rand(10000) < 800 )
				{
					if( gg->Rand(10000) < 5000 )
					{
						if( myself->Skill_GetConsumeMP(Summon_Elemental) < ( attacker + 400 ) && myself->Skill_GetConsumeHP(Summon_Elemental) < ( attacker + 328 ) && myself->Skill_InReuseDelay(Summon_Elemental) == 0 )
						{
							myself->AddUseSkillDesire(attacker,Summon_Elemental,0,1,1000000);
						}
						( myself + 1160 ) = gg->GetIndexFromCreature(attacker);
					}
					else if( myself->Skill_GetConsumeMP(Summon_Elemental) < ( attacker + 400 ) && myself->Skill_GetConsumeHP(Summon_Elemental) < ( attacker + 328 ) && myself->Skill_InReuseDelay(Summon_Elemental) == 0 )
					{
						myself->AddUseSkillDesire(( myself + 1504 ),Summon_Elemental,0,1,1000000);
					}
					( myself + 1160 ) = gg->GetIndexFromCreature(( myself + 1504 ));
				}
				if( gg->Rand(10000) < 1500 )
				{
					if( myself->Skill_GetConsumeMP(Self_Nova) < ( attacker + 400 ) && myself->Skill_GetConsumeHP(Self_Nova) < ( attacker + 328 ) && myself->Skill_InReuseDelay(Self_Nova) == 0 )
					{
						myself->AddUseSkillDesire(attacker,Self_Nova,0,1,1000000);
					}
				}
				if( gg->Rand(10000) < 500 )
				{
							( myself + 1184 ) = 1;
							myself->AddTimerEx(TIMER_enable_death_clack,( 15 * 1000 ));
							h0 = myself->GetMaxHateInfo(0);
							if( myself->IsNullCreature(( h0 + 8 )) == 0 )
							{
								if( debug_mode )
								{
									myself->Say("эLX  а. " + ( ( h0 + 8 ) + 344 ));
								}
								myself->AddUseSkillDesire(( h0 + 8 ),Death_Clack,0,1,10000000);
							}
							if( is_hard_mode != 1 )
							{
								i0 = Death_Clack_Count; // 2
							}
							else
							{
								i0 = Death_Clack_Count_Hard; // 3
							}
							select(i0)
							{
								case 2:
									if( myself->GetHateInfoCount() >= 2 )
									{
										h0 = myself->GetNthHateInfo(0,( gg->Rand(9) + 1 ),0);
										if( myself->IsNullHateInfo(h0) == 0 )
										{
											if( myself->IsNullCreature(( h0 + 8 )) == 0 )
											{
												if( debug_mode )
												{
													myself->Say("эLX  а. " + ( ( h0 + 8 ) + 344 ));
												}
												myself->AddUseSkillDesire(( h0 + 8 ),Death_Clack,0,1,10000000);
											}
										}
									}
									break;
								case 3:
									if( myself->GetHateInfoCount() >= 3 )
									{
										h0 = myself->GetNthHateInfo(0,( gg->Rand(4) + 1 ),0);
										if( myself->IsNullHateInfo(h0) == 0 )
										{
											if( myself->IsNullCreature(( h0 + 8 )) == 0 )
											{
												if( debug_mode )
												{
													myself->Say("эLX  а. " + ( ( h0 + 8 ) + 344 ));
												}
												myself->AddUseSkillDesire(( h0 + 8 ),Death_Clack,0,1,10000000);
											}
										}
										h0 = myself->GetNthHateInfo(0,( gg->Rand(( myself->GetHateInfoCount() - 1 )) + 1 ),0);
										if( myself->IsNullHateInfo(h0) == 0 )
										{
											if( myself->IsNullCreature(( h0 + 8 )) == 0 )
											{
												if( debug_mode )
												{
													myself->Say("эLX  а. " + ( ( h0 + 8 ) + 344 ));
												}
												myself->AddUseSkillDesire(( h0 + 8 ),Death_Clack,0,1,10000000);
											}
										}
									}
									break;
							}
				}
			}*/

	public void screenMessage(Reflection ref, int id, ScreenMessageAlign align, boolean bool)
	{
		for(L2Player members : ref.getPlayers())
			members.sendPacket(new ExShowScreenMessage(id, 6000, align, true, 1, -1, bool));
	}

	private L2Skill getSkill(int id, int level)
	{
		return SkillTable.getInstance().getInfo(id, level);
	}

	@Override
	public boolean isGlobalAI()
	{
		return true;
	}
	
	public boolean isNotReturnHome()
	{
		return true;
	}
}