package ai;

import l2ft.common.ThreadPoolManager;
import l2ft.gameserver.ai.CtrlIntention;
import l2ft.gameserver.ai.Mystic;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.tables.SkillTable;

public class KegorJinia extends Mystic
{
	private static final L2Skill KEGOR_BUFF = SkillTable.getInstance().getInfo(6289, 1); // 18851
	private static final L2Skill JINIA_BUFF = SkillTable.getInstance().getInfo(6288, 1); // 18850

	public KegorJinia(L2Character actor)
	{
		super(actor);
		AI_TASK_DELAY = 100;
		AI_TASK_ACTIVE_DELAY = 100;
		ThreadPoolManager.getInstance().scheduleAi(new BUFF(), 100, false);
	}

	@Override
	protected boolean thinkActive()
	{
		L2NpcInstance actor = getActor();
		if(actor == null)
			return true;
		if(getIntention() == CtrlIntention.AI_INTENTION_ACTIVE)
			return startAtack();
		return true;
	}

	private boolean startAtack()
	{
		L2NpcInstance actor = getActor();
		if(actor == null)
			return true;
		if(getIntention() == CtrlIntention.AI_INTENTION_ACTIVE)
		{
			for(L2NpcInstance mob : actor.getReflection().getMonsters())
			{
				if(mob != null)
				{
					if(mob.getNpcId() != 29179 && mob.getNpcId() != 29180)
						continue;
					if(mob.isDead())
						continue;
					mob.addDamageHate(actor, 0, 100); // Это нужно, чтобы гвард не перестал атаковать цель после первых ударов
					actor.setRunning(); // Включаем бег...
					actor.setAttackTimeout(Integer.MAX_VALUE + System.currentTimeMillis()); // Это нужно, чтобы не сработал таймаут
					actor.getAI().setAttackTarget(mob); // На всякий случай, не обязательно делать
					actor.getAI().changeIntention(CtrlIntention.AI_INTENTION_ATTACK, mob, null); // Переводим в состояние атаки
					actor.getAI().addTaskAttack(mob); // Добавляем отложенное задание атаки, сработает в самом конце движения
					break;
				}
			}
		}
		return true;
	}

	@Override
	public void onIntentionAttack(L2Character target)
	{
		L2NpcInstance actor = getActor();
		if(actor == null)
			return;
		if(target.isPlayer())
		{
			setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
			return;
		}
		super.onIntentionAttack(target);
	}

	private class BUFF implements Runnable
	{
		public void run()
		{
			L2NpcInstance npc = getActor();
			if(npc == null || npc.isDead())
				return;
			try
			{
				if(npc.getNpcId() == 18851)
					npc.doCast(KEGOR_BUFF, npc, true);
				else
					npc.doCast(JINIA_BUFF, npc, true);
				setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
				startAtack();
				ThreadPoolManager.getInstance().scheduleAi(new BUFF(), 29000, false);
			}
			catch (Throwable t)
			{
			}
		}
	}

	@Override
	public boolean isGlobalAI()
	{
		return true;
	}

	public boolean isNotReturnHome()
	{
		return true;
	}
}