package ai;

import l2ft.gameserver.ai.CtrlIntention;
import l2ft.gameserver.ai.Mystic;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Effect;
import l2ft.gameserver.model.L2Playable;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Spawn;
import l2ft.gameserver.model.L2World;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.tables.SkillTable;
import l2ft.util.GArray;
import l2ft.util.Rnd;
import npc.model.SeducedMobInstance;

public class SeducedMage extends Mystic
{
	private static final int SEDUCED_BUFF = 5972;

	public SeducedMage(L2Character actor)
	{
		super(actor);
		this.AI_TASK_DELAY = 500;
		this.AI_TASK_ACTIVE_DELAY = 500;
	}

	protected boolean thinkActive()
	{
		if ((super.thinkActive()) || (!this._task_list.isEmpty())) 
		{
			return true;
		}
		L2NpcInstance actor = getActor();

		if (getIntention() == CtrlIntention.AI_INTENTION_ACTIVE)
		{
			GArray<L2Character> targets = new GArray<L2Character>();
			for (L2NpcInstance obj : L2World.getAroundNpc(actor))
				if ((obj != null) && (!obj.isAlikeDead()) && (!obj.isInvul()) && (obj.isVisible()))
				{
					if (actor.getFactionId().equalsIgnoreCase(obj.getFactionId()))
						continue;
					targets.add(obj);
				}
			checkAggression(targets.get(Rnd.get(targets.size())));

			if ((targets.isEmpty()) && (isGlobalAggro()))
			{
				targets.clear();
				for (L2Player pl : L2World.getAroundPlayers(actor, 100, 64))
					if ((pl != null) && (!pl.isAlikeDead()) && (!pl.isInvul()) && (pl.isVisible()))
					{
						if (alreadyBuffied(pl))
							continue;
						targets.add(pl);
					}
				L2Character target = targets.get(Rnd.get(targets.size()));
				if (target != null) 
				{
					actor.doCast(SkillTable.getInstance().getInfo(5972, 1), target, true);
				}
			}
		}
		return false;
	}

	public boolean canAttackCharacter(L2Character target)
	{
		return (target.getPlayer() == null) && (!(target instanceof SeducedMobInstance));
	}

	private static boolean alreadyBuffied(L2Player pl)
	{
		for (L2Effect e : pl.getEffectList().getAllEffects())
			if ((!e.getSkill().isOffensive()) && (e.getSkill().getName().startsWith("Seduced ")))
				return true;
		return false;
	}
}