package bosses;

import java.util.concurrent.ScheduledFuture;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2ft.common.ThreadPoolManager;
import l2ft.extensions.scripts.Functions;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.idfactory.IdFactory;
import l2ft.gameserver.instancemanager.InstancedZoneManager;
import l2ft.gameserver.instancemanager.TowerOfNaiaManager;
import l2ft.gameserver.model.entity.InstancedZone;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Spawn;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.instances.L2DoorInstance;
import l2ft.gameserver.model.instances.L2MonsterInstance;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.DoorTable;
import l2ft.gameserver.tables.NpcTable;
import l2ft.util.Location;
import l2ft.util.Rnd;

public class TowerOfNaia extends Functions implements ScriptFile
{
	private static Logger _log = Logger.getLogger(TowerOfNaia.class.getName());

	private static final int REFLECTION_ID = 519;

	private static final int BELETHS_SLAVE = 18490;
	private static final int BELETHS_FOLLOWER1 = 22393;
	private static final int BELETHS_DEVOTEE1 = 22394;
	private static final int BELETHS_PROJECTION = 22395;
	private static final int BELETHS_RETAINER = 22411;
	private static final int BELETHS_FIGHTER = 22412;
	private static final int BELETHS_LOW_SERVANT = 22413;
	//private static final int BELETHS_FOLLOWER2 		= 22414;
	//private static final int BELETHS_DEVOTEE2 		= 22415;
	private static final int BELETHS_ASSASSIN1 = 22439;
	private static final int BELETHS_ASSASSIN2 = 22440;
	private static final int BELETHS_MINION1 = 22441;
	private static final int BELETHS_MINION2 = 22442;

	private static final int[][] ROOM_MONSTERS = {
			{ BELETHS_FOLLOWER1, 1, 0, 0, 0, BELETHS_DEVOTEE1, 2, 0, 0, 0, 0, 0, 0, BELETHS_FOLLOWER1, 2, 0, 0, 0, 0, 0, 0,
					BELETHS_DEVOTEE1, 1, 0, 0, 0 },
			{ BELETHS_ASSASSIN1, 8, -48146, 249597, -9124, -48144, 248711, -9124, -48704, 249597, -9104, -49219, 249596,
					-9104, -49715, 249601, -9104, -49714, 248696, -9104, -49225, 248710, -9104, -48705, 248708, -9104 },
			{ BELETHS_MINION1, 2, 0, 0, 0, 0, 0, 0, BELETHS_MINION2, 2, 0, 0, 0, 0, 0, 0 },
			{ BELETHS_ASSASSIN2, 8, -49754, 243866, -9968, -49754, 242940, -9968, -48733, 243858, -9968, -48745, 242936,
					-9968, -49264, 242946, -9968, -49268, 243869, -9968, -48186, 242934, -9968, -48185, 243855, -9968 },
			{ BELETHS_RETAINER, 2, 0, 0, 0, 0, 0, 0, BELETHS_FOLLOWER1, 1, 0, 0, 0, BELETHS_DEVOTEE1, 1, 0, 0, 0,
					BELETHS_FOLLOWER1, 1, 0, 0, 0, BELETHS_DEVOTEE1, 1, 0, 0, 0 },
			{ BELETHS_PROJECTION, 2, 0, 0, 0, 0, 0, 0 },
			{ BELETHS_FOLLOWER1, 1, 0, 0, 0, BELETHS_DEVOTEE1, 2, 0, 0, 0, 0, 0, 0, BELETHS_FOLLOWER1, 2, 0, 0, 0, 0, 0, 0,
					BELETHS_DEVOTEE1, 1, 0, 0, 0, BELETHS_FIGHTER, 1, 0, 0, 0 },
			{ BELETHS_PROJECTION, 2, 0, 0, 0, 0, 0, 0 },
			{ BELETHS_MINION1, 2, 0, 0, 0, 0, 0, 0, BELETHS_MINION2, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
			{ BELETHS_PROJECTION, 2, 0, 0, 0, 0, 0, 0 },
			{ BELETHS_LOW_SERVANT, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
			{ BELETHS_SLAVE, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0 } };

	private static final int BEFORE_SPAWN = 0;
	private static final int AFTER_SPAWN = 1;
	private static final int AFTER_KILL = 2;

	private static final int CLOSE = 0;
	private static final int OPEN = 1;

	private static final int[][] ROOM_DOOR_MECHANICS = {
			{ 18250001, BEFORE_SPAWN, OPEN, 18250001, AFTER_SPAWN, CLOSE, 18250002, AFTER_KILL, OPEN, 18250003, AFTER_KILL,
					OPEN }, { 18250003, AFTER_SPAWN, CLOSE, 18250004, AFTER_KILL, OPEN, 18250005, AFTER_KILL, OPEN },
			{ 18250005, AFTER_SPAWN, CLOSE, 18250006, AFTER_KILL, OPEN, 18250007, AFTER_KILL, OPEN },
			{ 18250007, AFTER_SPAWN, CLOSE, 18250008, AFTER_KILL, OPEN, 18250009, AFTER_KILL, OPEN },
			{ 18250009, AFTER_SPAWN, CLOSE, 18250010, AFTER_KILL, OPEN, 18250011, AFTER_KILL, OPEN },
			{ 18250011, AFTER_SPAWN, CLOSE, 18250101, AFTER_KILL, OPEN, 18250013, AFTER_KILL, OPEN },
			{ 18250013, AFTER_SPAWN, CLOSE, 18250014, AFTER_KILL, OPEN, 18250015, AFTER_KILL, OPEN },
			{ 18250015, AFTER_SPAWN, CLOSE, 18250102, AFTER_KILL, OPEN, 18250017, AFTER_KILL, OPEN },
			{ 18250017, AFTER_SPAWN, CLOSE, 18250018, AFTER_KILL, OPEN, 18250019, AFTER_KILL, OPEN },
			{ 18250019, AFTER_SPAWN, CLOSE, 18250103, AFTER_KILL, OPEN, 18250021, AFTER_KILL, OPEN },
			{ 18250021, AFTER_SPAWN, CLOSE, 18250022, AFTER_KILL, OPEN, 18250023, AFTER_KILL, OPEN },
			{ 18250023, AFTER_SPAWN, CLOSE, 18250024, AFTER_KILL, OPEN } };

	private static final int[][] SPORE_LOCATIONS = { { -46152, 246776, -14207 }, { -46392, 247176, -14208 },
			{ -46392, 247688, -14209 }, { -46152, 248120, -14208 }, { -45736, 248360, -14208 }, { -45208, 248360, -14208 },
			{ -44824, 248136, -14208 }, { -44568, 247688, -14209 }, { -44552, 247208, -14208 }, { -44792, 246776, -14208 },
			{ -45224, 246536, -14208 }, { -45752, 246536, -14208 }, };

	private ScheduledFuture<?> _deadMonsterCheckTask = null;

	public void enterNaia()
	{
		int roofLockHpPercent = 100;
		L2Player player = (L2Player) getSelf();
		L2MonsterInstance roofLock = TowerOfNaiaManager.getRoofLock();
		if(roofLock != null)
		{
			roofLockHpPercent = (int) roofLock.getCurrentHpPercents();
		}

		if(roofLockHpPercent <= 10)
		{
			if(enterInstance(player, REFLECTION_ID, true))
			{
				handleDoors(player.getReflection(), 0, BEFORE_SPAWN);
			}
		}
		else
		{
			getNpc().doDie(null);
		}
	}

	public void startRoomInvasion(String[] args)
	{
		int roomId = -1;
		try
		{
			roomId = Integer.parseInt(args[0]);
		}
		catch(Exception e)
		{}

		if(roomId < 0 || roomId > 11){ return; }

		getNpc().doDie(null);

		Reflection ref = getSelfPlayer().getReflection();
		ref.startCollapseTimer(300000);

		handleDoors(ref, roomId, AFTER_SPAWN);

		int i = 0;
		while(i < ROOM_MONSTERS[roomId].length)
		{
			int monsterId = ROOM_MONSTERS[roomId][i++];
			int count = ROOM_MONSTERS[roomId][i++];

			for(int j = 0; j < count; j++)
			{
				int x = ROOM_MONSTERS[roomId][i++];
				int y = ROOM_MONSTERS[roomId][i++];
				int z = ROOM_MONSTERS[roomId][i++];

				Location loc = null;
				if(x == 0)
				{
					loc = getNpc().getSpawnedLoc().clone();
					loc.x = Rnd.get(loc.x - 150, loc.x + 150);
					loc.y = Rnd.get(loc.y - 150, loc.y + 150);
					//loc.z = z;
				}
				else
				{
					loc = new Location(x, y, z);
				}

				L2MonsterInstance mob = new L2MonsterInstance(IdFactory.getInstance().getNextId(), NpcTable.getTemplate(monsterId));
				mob.setSpawnedLoc(loc);
				mob.onSpawn();
				mob.spawnMe(loc);
				mob.setReflection(ref);
			}
		}

		if(_deadMonsterCheckTask != null)
			_deadMonsterCheckTask.cancel(true);

		_deadMonsterCheckTask = ThreadPoolManager.getInstance().scheduleAiAtFixedRate(new DeadMonsterCheckTask(ref, roomId), 1000, 1000, false);
	}

	public void finishRoomInvasion(Reflection ref, int roomId)
	{
		if(_deadMonsterCheckTask != null)
			_deadMonsterCheckTask.cancel(true);

		ref.stopCollapseTimer();

		handleDoors(ref, roomId, AFTER_KILL);

		if(roomId == 11)
		{
			for(L2Player player : ref.getPlayers())
			{
				player.setReflection(0);
			}
			DoorTable.getInstance().getDoor(18250024).openMe();
			DoorTable.getInstance().getDoor(18250025).openMe();
		}
	}

	private void handleDoors(Reflection ref, int roomId, int criterion)
	{
		for(int i = 0; i < ROOM_DOOR_MECHANICS[roomId].length; i += 3)
		{
			int doorId = ROOM_DOOR_MECHANICS[roomId][i];
			int order = ROOM_DOOR_MECHANICS[roomId][i + 1];
			int action = ROOM_DOOR_MECHANICS[roomId][i + 2];

			if(order == criterion)
			{
				for(L2DoorInstance door : ref.getDoors())
				{
					if(door.getDoorId() == doorId)
					{
						if(action == OPEN)
							door.openMe();
						else
							door.closeMe();
					}
				}
			}
		}
	}

	public void spawnSpores()
	{
		getNpc().doDie(null);

		for(int i = 0; i < SPORE_LOCATIONS.length; i++)
		{
			Location loc = new Location(SPORE_LOCATIONS[i][0], SPORE_LOCATIONS[i][1], SPORE_LOCATIONS[i][2]);

			int monsterId = 0;

			if(i % 2 == 0 && i % 4 != 0)
				monsterId = TowerOfNaiaManager.SPORE_EARTH_ID;
			else if(i % 3 == 0)
				monsterId = TowerOfNaiaManager.SPORE_WATER_ID;
			else if(i % 4 == 0)
				monsterId = TowerOfNaiaManager.SPORE_FIRE_ID;
			else
				monsterId = TowerOfNaiaManager.SPORE_WIND_ID;

			L2Spawn spawn;
			try
			{
				spawn = new L2Spawn(monsterId);
				spawn.setLoc(loc);
				spawn.setLocation(0);
				spawn.setAmount(1);
				spawn.setRespawnDelay(30);
				spawn.init();
			}
			catch(ClassNotFoundException e)
			{}
		}
	}

	private class DeadMonsterCheckTask implements Runnable
	{
		private int _roomId;
		private Reflection _ref;

		public DeadMonsterCheckTask(Reflection ref, int roomId)
		{
			_ref = ref;
			_roomId = roomId;
		}

		@Override
		public void run()
		{
			boolean alive = false;
			for(L2MonsterInstance monster : _ref.getMonsters())
			{
				if(monster != null && !monster.isDead())
				{
					alive = true;
					break;
				}
			}

			if(!alive)
				finishRoomInvasion(_ref, _roomId);
		}
	}

	public static boolean enterInstance(L2Player player, int id, boolean party)
	{
		if(player == null){ return false; }
		if(party)
		{
			if(player.getParty() == null)
			{
				player.sendMessage("Вы не в группе.");
				return false;
			}
			if(player.getParty().getPartyLeader() != player)
			{
				player.sendMessage("Вы не лидер группы.");
				return false;
			}
		}
		InstancedZoneManager izm = InstancedZoneManager.getInstance();
		FastMap<Integer, InstancedZone> izs = InstancedZoneManager.getInstance().getById(id);
		if(izs == null)
		{
			player.sendPacket(Msg.SYSTEM_ERROR);
			return false;
		}
		InstancedZone iz = izs.get(0);
		if(iz == null)
		{
			player.sendPacket(Msg.SYSTEM_ERROR);
			return false;
		}
		String name = iz.getName();
		if(izm.getTimeToNextEnterInstance(name, player) > 0)
		{
			player.sendPacket(new SystemMessage(2100).addName(player));
			return false;
		}
		int timelimit = iz.getTimelimit();
		if(party)
		{
			int minMembers = iz.getMinParty();
			int maxMembers = iz.getMaxParty();
			if(player.getParty().getMemberCount() < minMembers)
			{
				player.sendMessage("В группе должно быть не меньше " + minMembers + " человек.");
				return false;
			}
			if(player.getParty().getMemberCount() > maxMembers)
			{
				player.sendMessage("В группе должно быть не больше " + maxMembers + " человек.");
				return false;
			}
			for(L2Player member : player.getParty().getPartyMembers())
			{
				if(member.isCursedWeaponEquipped() || member.isInFlyingTransform() || member.isDead())
				{
					player.sendPacket(new SystemMessage(2098).addName(member));
					return false;
				}
				if(!player.isInRange(member, 500L))
				{
					member.sendPacket(Msg.ITS_TOO_FAR_FROM_THE_NPC_TO_WORK);
					player.sendPacket(Msg.ITS_TOO_FAR_FROM_THE_NPC_TO_WORK);
					return false;
				}
				if(izm.getTimeToNextEnterInstance(name, member) > 0)
				{
					player.getParty().broadcastToPartyMembers(new SystemMessage(2100).addName(member));
					return false;
				}
			}
		}
		Reflection r = new Reflection(name);
		r.setInstancedZoneId(id);
		for(InstancedZone i : izs.values())
		{
			if(r.getTeleportLoc() == null)
			{
				r.setTeleportLoc(i.getTeleportCoords());
			}
			r.FillSpawns(i.getSpawnsInfo());
			r.FillDoors(i.getDoors());
		}
		r.setCoreLoc(r.getReturnLoc());
		r.setReturnLoc(player.getLoc());
		if(party)
		{
			for(L2Player member : player.getParty().getPartyMembers())
			{
				member.setVar(name, String.valueOf(System.currentTimeMillis()));
				member.setVar("backCoords", r.getReturnLoc().toXYZString());
				member.teleToLocation(iz.getTeleportCoords(), r.getId());
			}
			player.getParty().setReflection(r);
		}
		else
		{
			player.setVar(name, String.valueOf(System.currentTimeMillis()));
			player.setVar("backCoords", r.getReturnLoc().toXYZString());
			player.teleToLocation(iz.getTeleportCoords(), r.getId());
			player.setReflection(r);
		}
		if(timelimit > 0)
		{
			r.startCollapseTimer(timelimit * 60 * 1000L);
		}
		return true;
	}

	@Override
	public void onLoad()
	{
		_log.info("Tower Of Naia Manager: Loaded.");
	}

	@Override
	public void onReload()
	{}

	@Override
	public void onShutdown()
	{}
}
