package commands.admin;

import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.Announcements;
import l2ft.gameserver.clientpackets.Say2C;
import l2ft.gameserver.handler.AdminCommandHandler;
import l2ft.gameserver.handler.IAdminCommandHandler;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.MailParcelController;
import l2ft.gameserver.model.items.MailParcelController.Letter;
import l2ft.gameserver.serverpackets.ExShowScreenMessage;
import l2ft.gameserver.serverpackets.ExShowScreenMessage.ScreenMessageAlign;

/**
 * This class handles following admin commands: - announce text = announces text
 * to all players - list_announcements = show menu - reload_announcements =
 * reloads announcements from txt file - announce_announcements = announce all
 * stored announcements to all players - add_announcement text = adds text to
 * startup announcements - del_announcement id = deletes announcement with
 * respective id
 */
public class AdminAnnouncements implements IAdminCommandHandler, ScriptFile
{
	private static enum Commands
	{
		admin_list_announcements,
		admin_announce_announcements,
		admin_add_announcement,
		admin_del_announcement,
		admin_announce,
		admin_a,
		admin_announce_menu,
		admin_crit_announce,
		admin_c,
		admin_toscreen,
		admin_s,
		admin_send_leter_online,
		admin_send_leter_all,
		admin_reload_announcements
	}

	@Override
	public boolean useAdminCommand(Enum comm, String[] wordList, String fullString, L2Player activeChar)
	{
		Commands command = (Commands) comm;

		if(!activeChar.getPlayerAccess().CanAnnounce)
			return false;

		Announcements a = Announcements.getInstance();

		switch(command)
		{
			case admin_list_announcements:
				a.listAnnouncements(activeChar);
				break;
			case admin_announce_menu:
				a.handleAnnounce(fullString, 20);
				a.listAnnouncements(activeChar);
				break;
			case admin_announce_announcements:
				for(L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
					a.showAnnouncements(player);
				a.listAnnouncements(activeChar);
				break;
			case admin_add_announcement:
				if(wordList.length < 2)
					return false;
				try
				{
					String val = fullString.substring(23);
					a.addAnnouncement(val);
					a.listAnnouncements(activeChar);
				}
				catch(StringIndexOutOfBoundsException e)
				{}
				break;
			case admin_del_announcement:
				if(wordList.length < 2)
					return false;
				try
				{
					int val = new Integer(fullString.substring(23));
					a.delAnnouncement(val);
					a.listAnnouncements(activeChar);
				}
				catch(StringIndexOutOfBoundsException e)
				{}
				break;
			case admin_announce:
				a.handleAnnounce(fullString, 15);
				break;
			case admin_a:
				a.handleAnnounce(fullString, 8);
				break;
			case admin_crit_announce:
			case admin_c:
				if(wordList.length < 2)
					return false;
				Announcements.getInstance().announceToAll(activeChar.getName() + ": " + fullString.replaceFirst("admin_crit_announce ", "").replaceFirst("admin_c ", ""), Say2C.CRITICAL_ANNOUNCEMENT);
				break;
			case admin_toscreen:
			case admin_s:
				if(wordList.length < 2)
					return false;
				String text = activeChar.getName() + ": " + fullString.replaceFirst("admin_toscreen ", "").replaceFirst("admin_s ", "");
				int time = 3000 + text.length() * 100; // 3 секунды + 100мс на символ
				ExShowScreenMessage sm = new ExShowScreenMessage(text, time, ScreenMessageAlign.TOP_CENTER, text.length() < 64);
				for(L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
					player.sendPacket(sm);
				break;
			case admin_send_leter_online:
				if(wordList.length < 2)
					return false;
				String val = fullString.substring(24);
				int expiretime = 360 * 3600 + (int) (System.currentTimeMillis() / 1000);
				for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
				if (player != null && player.isOnline() || player.isInOfflineMode() && !player.isDeleting())
				{
					Letter letter = new Letter();
					letter.receiverId = player.getObjectId();
					letter.receiverName = player.getName();
					letter.senderId = activeChar.getObjectId();
					letter.senderName = "GM";
					letter.topic = "From Administration";
					letter.body = val;
					letter.price = 0;
					letter.unread = 1;
					letter.validtime = expiretime;
					MailParcelController.getInstance().sendLetter(letter, null, null, activeChar);
				}
				activeChar.sendMessage("Letter Sended.");	
				break;
			case admin_send_leter_all:
				if(wordList.length < 2)
					return false;
				String val1 = fullString.substring(21);
				int expiretimer = 360 * 3600 + (int) (System.currentTimeMillis() / 1000);
				for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
				if(player != null)
				{
					Letter letter = new Letter();
					letter.receiverId = player.getObjectId();
					letter.receiverName = player.getName();
					letter.senderId = activeChar.getObjectId();
					letter.senderName = "GM";
					letter.topic = "From Administration";
					letter.body = val1;
					letter.price = 0;
					letter.unread = 1;
					letter.validtime = expiretimer;
					MailParcelController.getInstance().sendLetter(letter, null, null, activeChar);
				}
				activeChar.sendMessage("Letter Sended.");	
				break;
			case admin_reload_announcements:
				Announcements.getInstance().loadAnnouncements();
				Announcements.getInstance().listAnnouncements(activeChar);
				activeChar.sendMessage("Announcements reloaded.");
				break;
		}

		return true;
	}

	@Override
	public Enum[] getAdminCommandEnum()
	{
		return Commands.values();
	}

	@Override
	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	@Override
	public void onReload()
	{}

	@Override
	public void onShutdown()
	{}
}