package commands.admin;

import l2ft.Config;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.handler.AdminCommandHandler;
import l2ft.gameserver.handler.IAdminCommandHandler;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.serverpackets.ItemList;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.xml.ItemTemplates;
import l2ft.util.Log;
import l2ft.util.Rnd;

public class AdminCreateItem implements IAdminCommandHandler, ScriptFile
{
	private static enum Commands
	{
		admin_itemcreate,
		admin_create_item,
		admin_create_item_target,
		admin_create_item_all,
		admin_spreaditem
	}

	@Override
	public boolean useAdminCommand(Enum comm, String[] wordList, String fullString, L2Player activeChar)
	{
		Commands command = (Commands) comm;

		if(!activeChar.getPlayerAccess().UseGMShop)
			return false;

		switch(command)
		{
			case admin_itemcreate:
				AdminHelpPage.showHelpPage(activeChar, "itemcreation.htm");
				break;
			case admin_create_item:
				try
				{
					if(wordList.length == 3)
						createItem(activeChar, Integer.parseInt(wordList[1]), Long.parseLong(wordList[2]));
					else if(wordList.length == 2)
						createItem(activeChar, Integer.parseInt(wordList[1]), 1);
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Specify a valid number.");
				}
				catch(StringIndexOutOfBoundsException e)
				{
					activeChar.sendMessage("Can't create this item.");
				}
				AdminHelpPage.showHelpPage(activeChar, "itemcreation.htm");
				break;
			case admin_create_item_target:
				try
				{
					if(wordList.length == 3)
						createItem((L2Player)activeChar.getTarget(), Integer.parseInt(wordList[1]), Long.parseLong(wordList[2]));
					else if(wordList.length == 2)
						createItem((L2Player)activeChar.getTarget(), Integer.parseInt(wordList[1]), 1);
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Specify a valid number.");
				}
				catch(StringIndexOutOfBoundsException e)
				{
					activeChar.sendMessage("Can't create this item.");
				}
				AdminHelpPage.showHelpPage(activeChar, "itemcreation.htm");
				break;
			
			case admin_create_item_all:
				try
				{
					if(wordList.length == 3)
					{
						for(L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
							if (player != null)
								createItem(player, Integer.parseInt(wordList[1]), Long.parseLong(wordList[2]));
					}
					else if(wordList.length == 2)
					{
						for(L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
							if (player != null)
								createItem(player, Integer.parseInt(wordList[1]), 1);
					}
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Specify a valid number.");
				}
				catch(StringIndexOutOfBoundsException e)
				{
					activeChar.sendMessage("Can't create this item.");
				}
				AdminHelpPage.showHelpPage(activeChar, "itemcreation.htm");
				break;
				
			case admin_spreaditem:
				try
				{
					int id = Integer.parseInt(wordList[1]);
					int num = wordList.length > 2 ? Integer.parseInt(wordList[2]) : 1;
					long count = wordList.length > 3 ? Long.parseLong(wordList[3]) : 1;
					for(int i = 0; i < num; i++)
					{
						L2ItemInstance createditem = ItemTemplates.getInstance().createItem(id);
						createditem.setCount(count);
						createditem.dropToTheGround(activeChar, Rnd.coordsRandomize(activeChar, 100));
					}
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Specify a valid number.");
				}
				catch(StringIndexOutOfBoundsException e)
				{
					activeChar.sendMessage("Can't create this item.");
				}
				break;
		}

		return true;
	}

	@Override
	public Enum[] getAdminCommandEnum()
	{
		return Commands.values();
	}

	private void createItem(L2Player activeChar, int id, long num)
	{

		if(Config.DISABLE_CREATION_ID_FOR_ADMINS_LIST.contains(id)) 
			{
			activeChar.sendMessage("Admin forbidden to create id = " + id);
			return;
			}
		
		L2ItemInstance createditem = ItemTemplates.getInstance().createItem(id);
		createditem.setCount(num);
		activeChar.getInventory().addItem(createditem);
		Log.LogItem(activeChar, Log.Adm_AddItem, createditem);
		if(!createditem.isStackable())
			for(long i = 0; i < num - 1; i++)
			{
				createditem = ItemTemplates.getInstance().createItem(id);
				activeChar.getInventory().addItem(createditem);
				Log.LogItem(activeChar, Log.Adm_AddItem, createditem);
			}
		activeChar.sendPacket(new ItemList(activeChar, true), SystemMessage.obtainItems(id, num, 0));
	}

	@Override
	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	@Override
	public void onReload()
	{}

	@Override
	public void onShutdown()
	{}
}