package events.Mercenaries;

import java.sql.ResultSet;
import java.util.Map;
import java.util.logging.Logger;

import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.extensions.scripts.Functions;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.communitybbs.Manager.MercenariesManager;
import l2ft.gameserver.communitybbs.containers.Order;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Player;

public class Mercenaries extends Functions implements ScriptFile
{
	/**
	 * Эвент: Mercenaries (c) Ro0TT, 2011
	 */
	private static Logger _log = Logger.getLogger(Mercenaries.class.getName());

	public static Mercenaries _instance;

	public static Mercenaries getInstance()
	{
		if(_instance == null)
			_instance = new Mercenaries();
		return _instance;
	}

	public static void OnDie(L2Character self, L2Character killer)
	{
		if(killer != null)
			if(MercenariesManager.getInstance()._participants.contains(killer.getObjectId()))
			{
				Order order = MercenariesManager.getInstance()._orders.get(self.getObjectId());
				if(order != null)
					order.onKill((L2Player) killer);
			}
	}

	@Override
	public void onLoad()
	{
		restoreOrders();
		_log.info("Loaded Event: Mercenaries");
	}

	@Override
	public void onReload()
	{
		storeOrders();
	}

	@Override
	public void onShutdown()
	{
		storeOrders();
	}

	private void restoreOrders()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT * FROM `mercenaries_orders`");
			rset = statement.executeQuery();

			int target;
			String targetName;
			int client;
			String clientName;
			int countToKill;

			Order order;

			while(rset.next())
			{
				target = rset.getInt("target");
				targetName = rset.getString("targetName");
				client = rset.getInt("client");
				clientName = rset.getString("clientName");
				countToKill = rset.getInt("countToKill");

				order = new Order(client, clientName, target, targetName, countToKill);
				restoreRewards(order);
				restoreKills(order);
				MercenariesManager.getInstance()._orders.put(target, order);
			}
			_log.info("Restore Mercenaries completed.");
		}
		catch(final Exception e)
		{
			_log.warning("Could not restore orders!");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void restoreRewards(Order order)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT id,count FROM `mercenaries_rewards` WHERE `target`=?");
			statement.setInt(1, order.target);
			rset = statement.executeQuery();

			int id;
			long count;

			while(rset.next())
			{
				id = rset.getInt("id");
				count = rset.getLong("count");
				order.addReward(id, count);
			}
		}
		catch(final Exception e)
		{
			_log.warning("Could not restore orders!");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void restoreKills(Order order)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT killer,count FROM `mercenaries_kills` WHERE `target`=?");
			statement.setInt(1, order.target);
			rset = statement.executeQuery();

			int killer, count;

			while(rset.next())
			{
				killer = rset.getInt("killer");
				count = rset.getInt("count");
				order.kills.put(killer, count);
			}
		}
		catch(final Exception e)
		{
			_log.warning("Could not restore orders!");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	private void storeOrders()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement saveOrder = null;
		FiltredPreparedStatement saveOrderRewards = null;
		FiltredPreparedStatement saveOrderKills = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			saveOrder = con.prepareStatement("REPLACE INTO `mercenaries_orders` (target,targetName,client,clientName,countToKill) values(?,?,?,?,?)");
			saveOrderRewards = con.prepareStatement("REPLACE INTO `mercenaries_rewards` (target,id,count) values(?,?,?)");
			saveOrderKills = con.prepareStatement("REPLACE INTO `mercenaries_kills` (target,killer,count) values(?,?,?)");

			for(Order order : MercenariesManager.getInstance()._orders.values())
			{
				saveOrder.setInt(1, order.target);
				saveOrder.setString(2, order.targetName);
				saveOrder.setInt(3, order.client);
				saveOrder.setString(4, order.clientName);
				saveOrder.setInt(5, order.countToKill);
				saveOrder.execute();

				for(Map.Entry<Integer, Long> item : order.reward.entrySet())
				{
					saveOrderRewards.setInt(1, order.target);
					saveOrderRewards.setInt(2, item.getKey());
					saveOrderRewards.setLong(3, item.getValue());
					saveOrderRewards.execute();
				}

				for(Map.Entry<Integer, Integer> killer : order.kills.entrySet())
				{
					saveOrderKills.setInt(1, order.target);
					saveOrderKills.setInt(2, killer.getKey());
					saveOrderKills.setInt(3, killer.getValue());
					saveOrderKills.execute();
				}
			}

			_log.info("Store Mercenaries completed.");
		}
		catch(final Exception e)
		{
			_log.warning("Could not restore orders!");
			e.printStackTrace();
		}
		finally
		{
			con.close();
			saveOrder.close();
			saveOrderKills.close();
			saveOrderRewards.close();
		}
	}
}