package events.TvT;

import java.util.HashMap;
import java.util.Iterator;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2ft.common.ThreadPoolManager;
import l2ft.extensions.multilang.CustomMessage;
import l2ft.extensions.scripts.Functions;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.Announcements;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.entity.olympiad.Olympiad;
import l2ft.gameserver.model.events.GameEvent;
import l2ft.gameserver.model.events.GameEventManager;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.serverpackets.ExShowScreenMessage;
import l2ft.gameserver.serverpackets.Revive;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.xml.ItemTemplates;
import l2ft.util.Strings;

public class TvT extends GameEvent implements ScriptFile
{
	private int _state = 0;
	private static TvT _instance;
	private Logger _log = Logger.getLogger(TvT.class.getName());

	private FastMap<L2Player, Integer> _participants = new FastMap<L2Player, Integer>();

	private HashMap<L2Player, Integer> live_list = new HashMap<L2Player, Integer>();
	private int[] _score;
	private int curr_round = 1;

	public long startBattle = 0;
	private Reflection _ref;

	public TvT()
	{
		_instance = this;
	}

	public static TvT getInstance()
	{
		if (_instance == null)
			_instance = new TvT();
		return _instance;
	}

	public int getState()
	{
		return _state;
	}

	public String getName()
	{
		return "Team vs Team";
	}

	public long getNextTime()
	{
		long next_time = getConfigs().START_TIME;

		while (next_time <= System.currentTimeMillis() / 1000)
		{
			getConfigs().START_TIME += 86400;
			setNextEvent();
			next_time = getConfigs().START_TIME;
		}

		return next_time;
	}

	public void setNextEvent()
	{
		if (TvTConfig._configs != null && TvTConfig._configs.size() > 1)
		{
			TvTConfig._configs.sort();
		}
	}

	public Configs getConfigs()
	{
		return TvTConfig._configs == null ? null : TvTConfig._configs.get(0);
	}

	public boolean canUseItem(L2Player actor, L2ItemInstance item)
	{
		return _state == 0 || getConfigs().RESTRICT_ITEMS == null || getConfigs().RESTRICT_ITEMS.contains(item.getItemId());
	}

	@Override
	public void onLoad()
	{
		Iterator<?> i$;
		TvTConfig.load();
		GameEventManager.getInstance().registerEvent(getInstance());
		_log.fine("Loaded Event: TvT");
		_ref = new Reflection("TvT");
		try
		{
			for (i$ = getConfigs().DOORS.keySet().iterator(); i$.hasNext(); ) 
			{ 
				int id_door = ((Integer)i$.next()).intValue();
				if (!_ref.addDoor(id_door, getConfigs().DOORS.get(id_door)))
					_log.info("TvT: Door with ID: " + id_door + " is null...");
			}
		}
		catch (Exception e)
		{
			e.printStackTrace(); _log.info("TvT: Maby door not found...");
		}
		_state = 0;
	}

	@Override
	public void onReload()
	{
		if (_ref != null)
			_ref.clearReflection(1, false);
		if (TvTConfig._configs.size() > 0)
			TvTConfig._configs.clear();
		if (_state != 0)
			finish();
		onLoad();
	}

	@Override
	public void onShutdown()
	{
		_state = 0;
	}

	public boolean register(L2Player player)
	{
		if (!canRegister(player, true))
			return false;

		player.setTeam(TeamWithMinPlayers(), true);
		_participants.put(player, 0);
		player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouRegistred", player, new Object[0]));
		player._event = this;
		return true;
	}

	public void registerPlayer()
	{
		L2Player player = (L2Player) getSelf();
		GameEvent event = GameEventManager.getInstance().findEvent("Team vs Team");
		event.register(player);
	}
	
	public void unreg(L2Player player)
	{
		if (player == null) return;

		if (_state == 2 || !isParticipant(player))
		{
			player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouCancelRegistration", player, new Object[0]));
			return;
		}

		_participants.remove(player);
		player._event = null;
		player.setTeam(0, false);
		player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouRegistrationCanceled", player, new Object[0]));
	}

	public void remove(L2Player player)
	{
		if (player == null)
			return;

		if (_participants.containsKey(player))
			_participants.remove(player);
		if (live_list.containsKey(player))
			live_list.remove(player);

		player._event = null;
		player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouDisqualified", player, new Object[0]));
	}

	public boolean canRegister(L2Player player, boolean first)
	{
		if (getConfigs().ALLOW_TAKE_ITEM)
		{
			long take_item_count = Functions.getItemCount(player, getConfigs().TAKE_ITEM_ID);
			String name_take_items = ItemTemplates.getInstance().getTemplate(getConfigs().TAKE_ITEM_ID).getName();
			if(take_item_count > 0)
			{
				if((int)take_item_count < getConfigs().TAKE_COUNT)
				{
					player.sendMessage("Недостаточно" + name_take_items + "для участия.");
					return false;
				}
			}
			else
			{
				player.sendMessage("У Вас нет " + name_take_items + ", требуется для участия.");
				return false;
			}
		}
		if (first && _state != 1)
		{
			player.sendMessage("Процесс регистрации не активен.");
			return false;
		}
		if (first && isParticipant(player))
		{
			player.sendMessage("Вы уже являетесь участником этого эвента.");
			return false;
		}
		if (player.isMounted())
		{
			player.sendMessage("Отзовите питомца.");
			return false;
		}
		if (player.getDuel() != null)
		{
			player.sendMessage("Вы должны завершить дуель.");
			return false;
		}
		if ((player.getLevel() < getConfigs().MIN_LEVEL) || (player.getLevel() > getConfigs().MAX_LEVEL))
		{
			player.sendMessage("Вы не подходите для участия в эвенте с таким уровнем.");
			return false;
		}
		if ((first) && (player.getTeam() != 0))
		{
			player.sendMessage("Вы уже зарегестрированы на другом эвенте.");
			return false;
		}
		if (player.isInOlympiadMode() || player.isInZoneOlympiad() || ((first) && Olympiad.isRegistered(player)))
		{
			player.sendMessage("Вы уже зарегестрированы на Олимпиаде.");
			return false;
		}
		if ((player.isInParty()) && (player.getParty().isInDimensionalRift()))
		{
			player.sendMessage("Вы уже зарегестрированы на другом эвенте.");
			return false;
		}
		if (player.isTeleporting())
		{
			player.sendMessage("Вы находитесь в процессе телепортации.");
			return false;
		}
		if ((first) && (this._participants.size() >= getConfigs().MAX_PARTICIPANTS))
		{
			player.sendMessage("Достигнуто максимальное кол-во участников.");
			return false;
		}
		if (player.isCursedWeaponEquipped())
		{
			player.sendMessage("С проклятым оружием на эвент нельзя.");
			return false;
		}
		if (player.getKarma() > 0)
		{
			player.sendMessage("PK не может учавствовать в эвенте.");
			return false;
		}
		return true;
	}

	public void question()
	{
		for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
		{
			if (player != null && player.getLevel() >= getConfigs().MIN_LEVEL && player.getLevel() <= getConfigs().MAX_LEVEL && player.getReflection().getId() <= 0 && !player.isInOlympiadMode())
			{
				player.scriptRequest(new CustomMessage("scripts.events.TvT.TvT.AskPlayer", player).toString(), "events.TvT.TvT:registerPlayer", new Object[0]);
			}
		}
	}

	public int getCountPlayers()
	{
		return _participants.size();
	}

	public void canRegisters()
	{
		if (_participants != null)
			for (L2Player player : _participants.keySet())
				if (!canRegister(player, false))
					player.sendMessage("Если все условия не будут соблюдены - вы будите дисквалифицированы");
	}

	public boolean isParticipant(L2Player player)
	{
		return _participants.containsKey(player);
	}

	public int TeamWithMinPlayers()
	{
		int[] count = new int[getConfigs().TEAM_COUNTS + 1];

		for (L2Player player : _participants.keySet())
		{
			count[player.getTeam()] += 1;
		}
		int min = count[1];

		for (int i = 1; i < count.length; i++)
			min = Math.min(min, count[i]);
		for (int i = 1; i < count.length; i++)
		{
			if (count[i] != min) 
				continue;
			min = i;
		}
		return min;
	}

	public void sayToAll(String adress, String[] replacements, boolean all)
	{
		if (all)
			Announcements.getInstance().announceByCustomMessage(adress, replacements, 10);
		else
			for (L2Player player : _participants.keySet())
				Announcements.getInstance().announceToPlayerByCustomMessage(player, adress, replacements, 18);
	}

	public void startRegistration()
	{
		_state = 1;
		sayToAll("scripts.events.TvT.TvT.AnnounceRegistrationStarted", new String[] { getName() }, true);
		question();
		
		_score = new int[getConfigs().TEAM_COUNTS + 1];

		if (getConfigs().TIME_TO_START_BATTLE >= 30)
			ThreadPoolManager.getInstance().scheduleGeneral(new StartMessages("scripts.events.TvT.TvT.EventStartOver", new String[] { "30" }), (getConfigs().TIME_TO_START_BATTLE - 30) * 1000);
		if (getConfigs().TIME_TO_START_BATTLE >= 10)
			ThreadPoolManager.getInstance().scheduleGeneral(new StartMessages("scripts.events.TvT.TvT.EventStartOver", new String[] { "10" }), (getConfigs().TIME_TO_START_BATTLE - 10) * 1000);
		for (int i = 5; i >= 1; i--)
		{
			if (getConfigs().TIME_TO_START_BATTLE - i >= i)
				ThreadPoolManager.getInstance().scheduleGeneral(new StartMessages("scripts.events.TvT.TvT.EventStartOver", new String[] { Integer.toString(i) }), (getConfigs().TIME_TO_START_BATTLE - i) * 1000);
		}
		ThreadPoolManager.getInstance().scheduleGeneral(new TaskVoid("canRegisters", null), (getConfigs().TIME_TO_START_BATTLE - 10) * 1000);
		ThreadPoolManager.getInstance().scheduleGeneral(new TaskVoid("start", null), getConfigs().TIME_TO_START_BATTLE * 1000);
	}

	public void start()
	{
		if (_state == 0)
		{
			startRegistration();
		}
		else if (_state == 1)
		{
			if (getCountPlayers() >= getConfigs().MIN_PARTICIPANTS)
			{
				teleportPlayersToColiseum();
				ThreadPoolManager.getInstance().scheduleGeneral(new go(), getConfigs().PAUSE_TIME * 1000);
				sayToAll("scripts.events.TvT.TvT.AnnounceTeleportToColiseum", null, true);
				_state = 2;
			}
			else
			{
				sayToAll("scripts.events.TvT.TvT.AnnounceEventCanceled", new String[] { getName() }, true);
				_participants.clear();
				_state = 0;
				abort();
			}
		}
		else
		{
			sayToAll("scripts.events.TvT.TvT.AnnounceStartError", null, true);
		}
	}

	public void finish()
	{
		sayToAll("scripts.events.TvT.TvT.AnnounceEventEnd", null, false);

		if (_state == 2)
		{
			int WinTeam = -1;
			int count = 0;
			int max = 0;

			for (int i = 0; i < _score.length - 1; i++)
			{
				max = Math.max(_score[i], _score[(i + 1)]);
			}
			for (int i = 0; i < _score.length; i++)
			{
				if (_score[i] != max)
					continue;
				WinTeam = i;
				count++;
			}
			if (count != 1 || WinTeam == -1 || _score[WinTeam] == 0)
			{
				sayToAll("scripts.events.TvT.TvT.EventDraw", null, false);
			}
			else
			{
				rewardToWinTeam(WinTeam);
			

				ThreadPoolManager.getInstance().scheduleGeneral(new TaskVoid("restorePlayers", null), 1000L);
				ThreadPoolManager.getInstance().scheduleGeneral(new TaskVoid("teleportPlayersToSavedCoords", null), 2000L);
			}

			ThreadPoolManager.getInstance().scheduleGeneral(new TaskVoid("clearAll", null), 3500L);
			setNextEvent();
			GameEventManager.getInstance().nextEvent();
			_state = 0;
		}
	}

	public void abort()
	{
		finish();
		if (_state > 0)
			sayToAll("scripts.events.TvT.TvT.EventCompletedManually", new String[] { getName() }, true);
	}

	public void onLogout(L2Player player)
	{
		if ((player == null) || (player.getTeam() < 1))
		{
			return;
		}

		if ((_state == 1) && (_participants.containsKey(player)))
		{
			unreg(player);
			return;
		}

		if ((_state == 2) && (_participants.containsKey(player)))
		{
			remove(player);
			try
			{
				String var = player.getVar("backCoords");
				if ((var == null) || (var.equals("")))
				{
					return;
				}
				String[] coords = var.split(" ");
				if (coords.length != 4)
				{
					return;
				}
				player.teleToLocation(Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), Integer.parseInt(coords[2]), Integer.parseInt(coords[3]));
				player.unsetVar("backCoords");
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public void teleportPlayersToSavedCoords()
	{
		for (L2Player player : this._participants.keySet())
		{
			teleportPlayerToSavedCoords(player);
		}
	}

	public void teleportPlayerToSavedCoords(L2Player player)
	{
		try
		{
			String var = player.getVar("backCoords");
			if ((var == null) || (var.equals("")))
			{
				return;
			}

			String[] coords = var.split(",");
			if (coords.length != 3)
			{
				return;
			}
			player.teleToLocation(Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), Integer.parseInt(coords[2]), 0L);
			player.unsetVar("backCoords");
			player.unsetVar("reflection");
			player.setReflection(0L);
			player.setTeam(0, false);
			player._event = null;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void doDie(L2Character killer, L2Character self)
	{
		if (self == null || killer == null)
			return;
		if ((self instanceof L2Player) && (killer instanceof L2Player))
		{
			L2Player player = (L2Player)self;
			L2Player kill = (L2Player)killer;
			if (_participants.containsKey(player))
				_participants.put(player, _participants.get(player) + 1);
			if (_state == 2 && player.getTeam() > 0 && kill.getTeam() > 0 && _participants.containsKey(player) && _participants.containsKey(kill))
			{
				if (getConfigs().RESURRECTION_TIME == 0)
				{
					if (player != null) player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouDead", player, new Object[0]));
					live_list.remove(player);
				}
				else
				{
					if (killer != null)
						_score[killer.getTeam()] += 1;
					if (player != null)
					{
						player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouDead", player, new Object[0]).add(new Object[] { getConfigs().RESURRECTION_TIME }));
						ThreadPoolManager.getInstance().scheduleGeneral(new TaskVoid("ResurrectionPlayer", player), getConfigs().RESURRECTION_TIME * 1000);
					}
				}
			}
		}
	}

	public boolean checkRound(boolean finish)
	{
		if (!finish)
		{
			int liveTeamCount = 0;
			int team = 0;
			for (int i = 1; i <= getConfigs().TEAM_COUNTS; i++)
			{
				if (!live_list.containsValue(Integer.valueOf(i)))
					continue;
				liveTeamCount++;
				team = i;
				if (liveTeamCount > 1)
					return false;
			}

			_score[team] += 1;
		}

		if (curr_round >= getConfigs().NUMBER_OF_ROUNDS)
			finish();
		else
			nextRound();

		return true;
	}

	public void nextRound()
	{
		for (L2Player player : _participants.keySet())
		{
			restorePlayer(player);
			player.teleToLocation((getConfigs().TEAM_COORDS.get(player.getTeam() - 1)).rnd(100, 150, false), _ref.getId());
			if (getConfigs().NUMBER_OF_ROUNDS > 0)
				live_list.put(player, player.getTeam());
			player.sendPacket(new ExShowScreenMessage(new CustomMessage("scripts.events.TvT.TvT.StartBattle", player, new Object[0]).toString(), getConfigs().PAUSE_TIME * 700, ExShowScreenMessage.ScreenMessageAlign.MIDDLE_CENTER, true));
		}
		curr_round += 1;
		paralyzePlayers();
		ThreadPoolManager.getInstance().scheduleGeneral(new go(), getConfigs().PAUSE_TIME * 1000);
	}

	public void teleportPlayersToColiseum()
	{
		for (L2Player player : _participants.keySet())
		{
			if (!canRegister(player, false))
			{
				remove(player);
				player.sendMessage(new CustomMessage("scripts.events.TvT.TvT.YouDisqualified", player, new Object[0]));
				return;
			}
			if(getConfigs().ALLOW_TAKE_ITEM)
				Functions.removeItem(player, getConfigs().TAKE_ITEM_ID, (long)getConfigs().TAKE_COUNT);
			unRide(player);
			unSummonPet(player, true);
			if (getConfigs().STOP_ALL_EFFECTS) 
				player.getEffectList().stopAllEffects();
			player.setReflection(_ref.getId());
			player.setVar("backCoords", player.getLoc().toXYZString());
			player.teleToLocation((getConfigs().TEAM_COORDS.get(player.getTeam() - 1)).rnd(100, 150, false), _ref.getId());
			restorePlayer(player);
			if (getConfigs().NUMBER_OF_ROUNDS > 0) 
				live_list.put(player, Integer.valueOf(player.getTeam()));
			player.sendPacket(new ExShowScreenMessage(new CustomMessage("scripts.events.TvT.TvT.StartBattle", player, new Object[0]).toString(), getConfigs().PAUSE_TIME * 700, ExShowScreenMessage.ScreenMessageAlign.MIDDLE_CENTER, true));
		}

		paralyzePlayers();
	}

	public void paralyzePlayers()
	{
		L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
		for (L2Player player : _participants.keySet())
		{
			player.getEffectList().stopEffect(1411);
			revengeSkill.getEffects(player, player, false, false);
			if (player.getPet() != null)
			{
				revengeSkill.getEffects(player, player.getPet(), false, false);
			}
		}
	}

	public void unParalyzePlayers()
	{
		for (L2Player player : _participants.keySet())
		{
			player.getEffectList().stopEffect(4515);
			if (player.getPet() != null)
			{
				player.getPet().getEffectList().stopEffect(4515);
			}
			if (player.getParty() != null)
			{
				player.getParty().oustPartyMember(player);
			}
		}
	}

	public void restorePlayer(L2Player player)
	{
		player.setCurrentCp(player.getMaxCp());
		player.setCurrentMp(player.getMaxMp());

		if (player.isDead())
		{
			player.setCurrentHp(player.getMaxHp(), true);
			player.restoreExp();
			player.broadcastPacket(new Revive(player));
			player.doRevive();
		}
		else
		{
			player.setCurrentHp(player.getMaxHp(), false);
		}
	}

	public void restorePlayers()
	{
		for (L2Player player : this._participants.keySet())
		{
			player.setCurrentHpMp(player.getMaxHp(), player.getMaxMp());
			player.setCurrentCp(player.getMaxCp());
			if (player.isDead())
			{
				player.restoreExp();
				player.broadcastPacket(new Revive(player));
				player.doRevive();
			}
		}
	}

	public void ResurrectionPlayer(L2Player player)
	{
		if ((player._event == null) || (_state != 2) || (!_participants.containsKey(player)))
			return;
		restorePlayer(player);
		player.teleToLocation((getConfigs().TEAM_COORDS.get(player.getTeam() - 1)).rnd(100, 150, false), _ref.getId());
	}

	private void clearAll()
	{
		_participants.clear();
		live_list.clear();
	}

	public void rewardToWinTeam(int WinTeam)
	{
		int count = 0;
		for (L2Player player : _participants.keySet())
		{
			if (player != null && player.getTeam() == WinTeam)
				count++;
		}
		if (count < 1)
			return;

		for (L2Player player : _participants.keySet())
		{
			if (player != null && player.getTeam() == WinTeam && _participants.get(player) >= getConfigs().REWARD_FOR_KILL)
			{
				Functions.addItem(player, getConfigs().REWARD_ITEM_ID, getConfigs().REWARD_COUNT);
			}
		}
		sayToAll("scripts.events.TvT.TvT.EventWin", new String[] { getConfigs().TEAM_NAME.get(WinTeam - 1), getConfigs().NUMBER_OF_ROUNDS > 0 ? "Очков" : "Убийств", Integer.toString(_score[WinTeam]) }, false);
	}

	public StringBuffer getInformation(L2Player player)
	{
		int rounds = getConfigs().NUMBER_OF_ROUNDS;
		long min = (getConfigs().START_TIME - System.currentTimeMillis() / 1000L) / 60L;
		String time = min + " минут";
		String reward = getConfigs().REWARD_COUNT + " " + ItemTemplates.getInstance().getTemplate(getConfigs().REWARD_ITEM_ID).getName();

		StringBuffer content = new StringBuffer();
		content.append("<table width=425 cellspacing=0>");
		content.append("<tr><td align=center>Эвент: <font color=LEVEL>").append(getName()).append("</font></td></tr>");
		content.append("<tr><td align=center>Тип: <font color=LEVEL>").append(rounds > 0 ? "По раундам" : "По времени").append("</font></td></tr>");
		content.append("<tr><td align=center>Приз: <font color=LEVEL>").append(reward).append("</font></td></tr>");
		content.append("<tr><td align=center>Кол-во команд: <font color=LEVEL>").append(getConfigs().TEAM_COUNTS).append("</font></td></tr>");
		content.append("<tr><td align=center>Мин/Макс участников: <font color=LEVEL>").append(getConfigs().MIN_PARTICIPANTS).append("/").append(getConfigs().MAX_PARTICIPANTS).append("</font></td></tr>");
		if (_state == 1)
		{
			content.append("<tr><td align=center>");
			if (_participants == null || !_participants.containsKey(player))
				content.append(Strings.htmlButton("Зарегестрироваться", "bypass -h _bbseventreg;" + getName(), 120, 25));
			else 
				content.append(Strings.htmlButton("Отмена", "bypass -h _bbseventunreg;", 120, 25));
				content.append("</td></tr>");
		}
		else
		{
			content.append("<tr><td align=center>Начало через: <font color=LEVEL>").append(time).append("</font></td></tr>");
		}
		content.append("</table>");
		return content;
	}

	class TaskVoid implements Runnable
	{
		String _name;
		L2Player _player;

		TaskVoid(String name, L2Player player)
		{
			_name = name;
			_player = player;
		}

		public void run()
		{
			if (_name.equals("canRegisters"))
				canRegisters();
			else if (_name.equals("start"))
				start();
			else if (_name.equals("restorePlayers"))
				restorePlayers();
			else if (_name.equals("teleportPlayersToSavedCoords"))
				teleportPlayersToSavedCoords();
			else if (_name.equals("clearAll"))
				clearAll();
			else if (_name.equals("ResurrectionPlayer"))
				ResurrectionPlayer(_player);
		}
	}

	class StartMessages implements Runnable
	{
		String _adress;
		String[] _replacing;

		StartMessages(String adress, String[] replacing)
		{
			_adress = adress;
			_replacing = replacing;
		}

		public void run()
		{
			if (_state == 1)
				sayToAll(_adress, _replacing, true);
		}
	}

	public class go implements Runnable
	{
		public go()
		{
		}

		public void run()
		{
			unParalyzePlayers();
			int time = getConfigs().TIME_TO_END_BATTLE;

			sayToAll("scripts.events.TvT.TvT.RoundStarted", null, false);

			while (time >= 0 && _state == 2)
			{
				int sec = time - time / 60 * 60;

				for (L2Player player : _participants.keySet())
				{
					String message;
					if (TvT.this.getConfigs().NUMBER_OF_ROUNDS > 0)
					{
						message = "Раунд: " + curr_round + " из " + getConfigs().NUMBER_OF_ROUNDS;
						message = message + "\nКоманда: " + getConfigs().TEAM_NAME.get(player.getTeam() - 1);
					}
					else
					{
						message = "\nКоманда: " + getConfigs().TEAM_NAME.get(player.getTeam() - 1);
					}

					if (sec < 10)
						message = message + "\nОсталось: " + time / 60 + ":0" + sec;
					else
					{
						message = message + "\nОсталось: " + time / 60 + ":" + sec;
					}
					player.sendPacket(new ExShowScreenMessage(message, 2000, ExShowScreenMessage.ScreenMessageAlign.BOTTOM_RIGHT, false));
				}
				if (getConfigs().NUMBER_OF_ROUNDS > 0 && checkRound(false))
					return;
				try
				{
					Thread.sleep(1000);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
				time--;
			}

			if (getConfigs().NUMBER_OF_ROUNDS > 0)
				checkRound(true);
			else
				finish();
		}
	}
}