package npc.model;

import l2ft.extensions.scripts.Functions;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.serverpackets.NpcHtmlMessage;
import l2ft.gameserver.tables.NpcTable;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.util.Files;
import l2ft.util.Rnd;

public class DragonVortexInstance extends L2NpcInstance implements ScriptFile
{

	private static final int LARGE_DRAGON_BONE = 17248;

	private static final int[] _bossIds = {25718, 25719, 25720, 25721, 25722, 25723, 25724};

	public DragonVortexInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onLoad()
	{}

	@Override
	public void onReload()
	{}

	@Override
	public void onShutdown()
	{}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		if(!canBypassCheck(player, this))
			return;

		if(command.startsWith("summon"))
		{
			for (int mob = 0; mob < _bossIds.length; mob++)
				for (L2NpcInstance npc : L2ObjectsStorage.getAllByNpcId(_bossIds[mob], true))
					if (npc != null && npc.getRealDistance3D(npc) <= 2000)
					{
						showHtmlFile(player, "32871-2.htm");
						return;
					}
			spawn_boss(player, _bossIds[Rnd.get(0, 6)]);	
		}
		else
			super.onBypassFeedback(player, command);
	}

	public void spawn_boss(L2Player player, int bossId)
	{
		L2ItemInstance DragonBone = player.getInventory().getItemByItemId(LARGE_DRAGON_BONE);

		if(DragonBone != null && DragonBone.getCount() > 0)
		{
			Functions.removeItem(player, LARGE_DRAGON_BONE, 1);
			
			L2NpcInstance npc = NpcTable.getTemplate(bossId).getNewInstance();
			if(npc != null)
			{
				npc.setSpawnedLoc(this.getLoc());
				npc.onSpawn();
				npc.spawnMe(npc.getSpawnedLoc().rnd(60, 60, false));
			}
		}
		else
			showHtmlFile(player, "32871-1.htm");
	}
	
	public void showHtmlFile(L2Player player, String file)
	{
		NpcHtmlMessage html = new NpcHtmlMessage(player, this);
		if(!player.getLang().equalsIgnoreCase("ru"))
			html.setFile("data/html/default/" + file);
		else
			html.setFile("data/html-ru/default/" + file);

		player.sendPacket(html);
	}

	@Override
	public void showChatWindow(L2Player player, int val)
	{
		String html;

		if(player.getLang().equalsIgnoreCase("ru"))
			html = Files.read("data/html-ru/default/32871.htm", player);
		else
			html = Files.read("data/html/default/32871.htm", player);

		player.sendPacket(new NpcHtmlMessage(player, this, html, val));
	}
}