package quests._288_HandleWithCare;

import javolution.util.FastMap;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.instancemanager.InstancedZoneManager;
import l2ft.gameserver.model.entity.InstancedZone;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.quest.Quest;
import l2ft.gameserver.model.quest.QuestState;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.ReflectionTable;
import l2ft.util.Location;
import l2ft.util.Rnd;

public class _288_HandleWithCare extends Quest implements ScriptFile
{

	private static FastMap<Integer, Long> _instances = new FastMap<Integer, Long>();
	private static final Location TELEPORT_POSITION = new Location(103000, -124360, -2784);
	// NPC
	private static final int Ankumi = 32741;
	private static final int Batracos = 32740;
	// MOB
	private static final int Ugoros = 18863;
	
	public void onLoad()
	{}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _288_HandleWithCare()
	{
		super(true);
		addStartNpc(Ankumi);
		addTalkId(Batracos);
		addQuestItem(15496);
		addQuestItem(15497);
		addQuestItem(15498);
		addKillId(Ugoros);
	}

	@Override
	public String onEvent(String event, final QuestState st, L2NpcInstance npc)
	{
		final L2Player player = st.getPlayer();
		int cond = st.getCond();
		String htmltext = event;
		if(event.equalsIgnoreCase("angkumi_q0288_04.htm"))
		{
			st.setCond(1);
			st.setState(STARTED);
			st.playSound(SOUND_ACCEPT);
		}
		if(event.equalsIgnoreCase("enterinst"))
		{
			if(cond == 1) {
				if(st.getQuestItemsCount(15496) == 1)
				{
					st.takeItems(15496, 1);
					enterInstance(player);
					return null;
				} else
				{
					return "No item";
				}
			} else
			{
				return "No quest";
			}
		}
		if(event.equalsIgnoreCase("exit"))
		{
			returnToAden(player);
			return null;
		}
		if(event.equalsIgnoreCase("exitsGotovo"))
		{
			if(st.getQuestItemsCount(15497) == 1)
			{
				if(Rnd.chance(30)) {
					st.giveItems(9557, 1, false);
					st.giveItems(959, 1, false);
				} else
				{
					st.giveItems(9557, 1, false);
					st.giveItems(960, Rnd.get(1,3), false);
				}
			} 
			if(st.getQuestItemsCount(15498) == 1)
			{
					st.giveItems(9557, 1, false);
					st.giveItems(959, 1, false);
					st.giveItems(960, Rnd.get(1,3), false);
			}
			
			st.setState(COMPLETED);
			st.playSound(SOUND_FINISH);
			st.unset("cond");
			st.exitCurrentQuest(true);
			return null;
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int id = st.getState();
		int cond = st.getCond();
		final L2Player player = st.getPlayer();
		
		if(npcId == Ankumi)
		{
			if(id == CREATED)
			{
				if(st.getPlayer().getLevel() >= 82)
					return "angkumi_q0288_02.htm";
				else
					return "lvl.htm";			
			}
			if(id == STARTED)
			{
				if(cond == 1)
				return "angkumi_q0288_05.htm";
				if(cond == 2)
				return "angkumi_q0288_08.htm";
			}
			if(id == COMPLETED)
			{
				return "gotovo.htm";
			}
		}
		if(npcId == Batracos)
		{
	//		if(cond == 1)
	//		enterInstance(player);
			if(cond == 2)
			return "batracos005.htm";
		}
		
		return "noquest";
	}
	
	
	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		L2Player player = st.getPlayer();
		if(npcId == Ugoros && cond == 1)
		{
			st.set("cond", "2");
			st.playSound(SOUND_ITEMGET);
			long refId = player.getReflection().getId();
			addSpawnToInstance(Batracos, new Location(95912, 85688, -3757, 55000), 0, refId);
		}	
		return null;
	}
	
	
	private void enterInstance(L2Player player)
	{
		int instancedZoneId = 705;
		InstancedZoneManager ilm = InstancedZoneManager.getInstance();
		FastMap<Integer, InstancedZone> ils = ilm.getById(instancedZoneId);
		if(ils == null)
		{
			player.sendPacket(Msg.SYSTEM_ERROR);
			return;
		}

		InstancedZone il = ils.get(0);

		assert il != null;

		if(player.isInParty())
		{
			player.sendPacket(Msg.A_PARTY_CANNOT_BE_FORMED_IN_THIS_AREA);
			return;
		}

		if(player.isCursedWeaponEquipped())
		{
			player.sendPacket(new SystemMessage(SystemMessage.C1S_QUEST_REQUIREMENT_IS_NOT_SUFFICIENT_AND_CANNOT_BE_ENTERED).addName(player));
			return;
		}

		Long old = _instances.get(player.getObjectId());
		if(old != null)
		{
			Reflection old_r = ReflectionTable.getInstance().get(old);
			if(old_r != null)
			{
				player.setReflection(old_r);
				player.teleToLocation(il.getTeleportCoords());
				player.setVar("backCoords", old_r.getReturnLoc().toXYZString());
				return;
			}
		}

		Reflection r = new Reflection(il.getName());
		r.setInstancedZoneId(instancedZoneId);
		for(InstancedZone i : ils.values())
		{
			if(r.getReturnLoc() == null)
				r.setReturnLoc(i.getReturnCoords());
			if(r.getTeleportLoc() == null)
				r.setTeleportLoc(i.getTeleportCoords());
			r.FillSpawns(i.getSpawnsInfo());
		}

		int timelimit = il.getTimelimit();

		player.setReflection(r);
		player.teleToLocation(il.getTeleportCoords());
		player.setVar("backCoords", r.getReturnLoc().toXYZString());
		player.sendPacket(new SystemMessage(SystemMessage.THIS_DUNGEON_WILL_EXPIRE_IN_S1_MINUTES).addNumber(timelimit));

		r.setNotCollapseWithoutPlayers(true);
		r.startCollapseTimer(timelimit * 60 * 1000L);

		_instances.put(player.getObjectId(), r.getId());
	}
	
	private void returnToAden(L2Player player)
	{
		Reflection r = player.getReflection();
		if(r.getReturnLoc() != null)
		    player.teleToLocation(r.getReturnLoc(), 0);
		else
		    player.setReflection(0);
		    player.unsetVar("backCoords");
	}
}