/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package quests._423_TakeYourBestShot;

import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.quest.Quest;
import l2ft.gameserver.model.quest.QuestState;
import l2ft.util.Rnd;
import quests._249_PoisonedPlainsOfTheLizardmen._249_PoisonedPlainsOfTheLizardmen;

/**
 * @author Byldas
 * @date 15.06.11
 */

public class _423_TakeYourBestShot extends Quest implements ScriptFile
{
	// NPC
	private static final int BATRACOS = 32740;
	private static final int JOHNNY = 32744;
	// Item
	private static final int SEER_UGOROS_PASS = 15496;
	// Guard
	private static final int TANTA_GUARD = 18862;
	// Mobs
	private static final int MOB[] = { 22768, 22769, 22770, 22771, 22772, 22773, 22774 };

	public void onLoad() 
	{}

	public void onReload() 
	{}

	public void onShutdown() 
	{}
	
	public _423_TakeYourBestShot()
	{
		super(false);

		addStartNpc(JOHNNY);
		addTalkId(JOHNNY);
	//	addStartNpc(BATRACOS);
		addTalkId(BATRACOS);
	//	addFirstTalkId(BATRACOS);

		addKillId(TANTA_GUARD);
		for(int npcId : MOB)
			addKillId(npcId);
		addKillId(SEER_UGOROS_PASS);
	}
	
	@Override
	public String onEvent (String event, QuestState st, L2NpcInstance npc)
	{
		String htmltext = event;
		
		if (st == null)
			return htmltext;

		if (npc.getNpcId() == JOHNNY)
		{
			if (event.equalsIgnoreCase("32744-04.htm"))
			{
				st.setState(STARTED);
				st.set("cond", "1");
				st.playSound(SOUND_ACCEPT);
			}
			else if (event.equalsIgnoreCase("32744-quit.htm"))
			{
				st.exitCurrentQuest(true);
			}
		}
		return htmltext;
	}
	
	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "<html><body>You are either not on a quest that involves this NPC, or you don't meet this NPC's minimum quest requirements.</body></html>";
		if (st == null)
			return htmltext;

		if (npc.getNpcId() == JOHNNY)
		{
			switch(st.getState())
			{
				case CREATED :
					QuestState qs1 = st.getPlayer().getQuestState(_249_PoisonedPlainsOfTheLizardmen.class);
					if ((qs1 != null) && (qs1.getState() == COMPLETED) && (st.getPlayer().getLevel() >= 82))
						if (st.getPlayer().getInventory().getItemByItemId(SEER_UGOROS_PASS) !=null)
							htmltext = "32744-07.htm";
						else
							htmltext = "32744-01.htm";
					else
						htmltext = "32744-00.htm";
					break;
				case STARTED :
					if (st.getInt("cond") == 1)
						htmltext = "32744-05.htm";
					else if (st.getInt("cond") == 2)
						htmltext = "32744-06.htm";
					break;
			}
		}
		else if (npc.getNpcId() == BATRACOS)
		{
			if (st.getState() == CREATED)
				if (st.getPlayer().getInventory().getItemByItemId(SEER_UGOROS_PASS) !=null)
					htmltext = "32740-05.htm";
				else
					htmltext = "32740-00.htm";
			else if (st.getState() == STARTED && st.getInt("cond") == 1)
				htmltext = "32740-02.htm";
			else if (st.getState() == STARTED && st.getInt("cond") == 2)
			{
				st.giveItems(SEER_UGOROS_PASS, 1);
				st.playSound(SOUND_FINISH);
				st.unset("cond");
				st.exitCurrentQuest(true);
				htmltext = "32740-04.htm";
			}
		}
		return htmltext;
	}
/*
	@Override
	public String onFirstTalk(L2NpcInstance npc, L2Player player)
	{
		QuestState st = player.getQuestState(getName());
		if (st == null)
		{
			Quest q = QuestManager.getQuest(getName());
			st = q.newQuestState(player, 1);
		}
		
		return "32740.htm";
	}
	*/
	@Override
	public String onKill(L2NpcInstance npc, QuestState st) 
	{
		int npcId = npc.getNpcId();
		if (st == null)
			return null;
		
		if(arrayContains(MOB, npcId))
		{
			if(Rnd.chance(3))
			{
				st.addSpawn(TANTA_GUARD, st.getPlayer().getX() + 50, st.getPlayer().getY() + 50, st.getPlayer().getZ());	
			}
		}
		else if (npc.getNpcId() == TANTA_GUARD && st.getInt("cond") == 1)
		{
			st.set("cond", "2");
			st.playSound(SOUND_MIDDLE);
		}
		return null;
	}

	private boolean arrayContains(int[] array, int id)
	{
		for(int i : array)
			if(i == id)
				return true;
		return false;
	}		
	
	
}
