package quests._456_StoryNamelessSpirit;

import java.util.Calendar;

import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Spawn;
import l2ft.gameserver.model.quest.Quest;
import l2ft.gameserver.model.quest.QuestState;
import l2ft.util.Rnd;

/**
 * Квест Рассказ безымянного духа
 * Шанс выпадения квестового итема поставлен приблизительно и равен 50%
 * Призы выдаются рандомно при наличии всех 3х итемов в инвентаре 
 * Требуется тщательная проверка квеста (особенно то как он работает в пати)
 * Мертвых РБ (НПС) в базе не было. Добавил самостоятельно на свободные ID
 * В дальнейшем нужно поменять на нормальные ID
 * @author RuLeX
 */

public class _456_StoryNamelessSpirit extends Quest implements ScriptFile
{
	public void onLoad()
	{}

	public void onReload()
	{}

	public void onShutdown()
	{}

	// Quest Items
	private static final int DRAKE_LORD_ESSENCE  = 17251;
	private static final int BEHEMOTH_LEADER_ESSENCE  = 17252;
	private static final int DRAGON_BEAST_ESSENCE  = 17253;

	// NPC
	private static final int SEPARATED_SOUL1 = 32864;
	private static final int SEPARATED_SOUL2 = 32865;
	private static final int SEPARATED_SOUL3 = 32866;
	private static final int SEPARATED_SOUL4 = 32867;
	private static final int SEPARATED_SOUL5 = 32868;
	private static final int SEPARATED_SOUL6 = 32869;
	private static final int SEPARATED_SOUL7 = 32870;
	private static final int SEPARATED_SOUL8 = 32891;
	private static final int DRAKE_LEADER = 25725;
	private static final int BEHEMOTH_LEADER = 25726;
	private static final int DRAGON_BEAST = 25727;
	private static final int DRAKE_LEADER_NPC = 50848;
	private static final int BEHEMOTH_LEADER_NPC = 50726;
	private static final int DRAGON_BEAST_NPC = 50727;

	//REWARDS
	private static final int[] WEAPON = {15558, 15559, 15560, 15561, 15562, 15563, 15564, 15565, 15566, 15567, 15567, 15569, 15570, 15571};
	private static final int[] ARMOR = {15750, 15753, 15756, 15745, 15748, 15751, 15754, 15757, 15759, 15743, 15746, 15749, 15752, 15755, 15758, 15744, 15747};
	private static final int[] JEWEL = {15765, 15764, 15763};
	private static final int[] STUFF = {9552, 9553, 9554, 9555, 9557, 9556, 6577, 6578, 959, 2134};
	private static final int CHANCE = 5;
	private static long Last_Spawned_Pursuer = 0;

	//RESET
	private static final int RESET_HOUR = 6;
	private static final int RESET_MIN = 30;

	public _456_StoryNamelessSpirit()
	{
		super(true);
		addStartNpc(SEPARATED_SOUL1);
		addStartNpc(SEPARATED_SOUL2);
		addStartNpc(SEPARATED_SOUL3);
		addStartNpc(SEPARATED_SOUL4);
		addStartNpc(SEPARATED_SOUL5);
		addStartNpc(SEPARATED_SOUL6);
		addStartNpc(SEPARATED_SOUL7);
		addStartNpc(SEPARATED_SOUL8);

		addTalkId(DRAKE_LEADER_NPC);
		addTalkId(BEHEMOTH_LEADER_NPC);
		addTalkId(DRAGON_BEAST_NPC);

		addKillId(DRAKE_LEADER);
		addKillId(BEHEMOTH_LEADER);
		addKillId(DRAGON_BEAST);

	}

	@Override
	public String onEvent(String event, QuestState st, L2NpcInstance npc)
	{
		String htmltext = event;
		int cond = st.getInt("cond");
		int level = st.getPlayer().getLevel();
		if(event.equalsIgnoreCase("32864-04.htm"))
		{
			st.setState(STARTED);
			st.set("cond", "1");
			st.playSound(SOUND_ACCEPT);
		}

		else if(event.equalsIgnoreCase("32864-08.htm"))
		{
			st.exitCurrentQuest(true);
			st.playSound(SOUND_FINISH);
		}

		else if(event.equalsIgnoreCase("drake"))
		{
			if(st.getQuestItemsCount(DRAKE_LORD_ESSENCE) < 1 && st.getInt("cond") == 1)
			{
				if (st.getInt("KB456_DRAKE") != 1)
				{
					if(Rnd.getRnd(CHANCE))
					{	
						st.giveItems(DRAKE_LORD_ESSENCE, 1, false);
						htmltext = "32864-13.htm";
					}	else htmltext = "32864-14.htm";
					if((st.getQuestItemsCount(DRAKE_LORD_ESSENCE) >= 1) && (st.getQuestItemsCount(BEHEMOTH_LEADER_ESSENCE) >= 1) && (st.getQuestItemsCount(DRAGON_BEAST_ESSENCE) >= 1))
						{
							st.set("cond", "2");
							st.playSound(SOUND_MIDDLE);
						}
					st.set("KB456_DRAKE", 1);
				}
			}		
			else htmltext = "32864-10.htm";	
		}

		else if(event.equalsIgnoreCase("behemoth"))
		{
			if(st.getQuestItemsCount(BEHEMOTH_LEADER_ESSENCE) < 1 && st.getInt("cond") == 1)
			{
				if (st.getInt("KB456_BEHEMOTH") != 1)
				{
					if(Rnd.getRnd(CHANCE))
					{	
						st.giveItems(BEHEMOTH_LEADER_ESSENCE, 1, false);
						htmltext = "32864-13.htm";
					}	else htmltext = "32864-14.htm";
					if((st.getQuestItemsCount(DRAKE_LORD_ESSENCE) >= 1) && (st.getQuestItemsCount(BEHEMOTH_LEADER_ESSENCE) >= 1) && (st.getQuestItemsCount(DRAGON_BEAST_ESSENCE) >= 1))
						{
							st.set("cond", "2");
							st.playSound(SOUND_MIDDLE);
						}
					st.set("KB456_BEHEMOTH", 1);
				}
			}		
			else htmltext = "32864-10.htm";	
		}

		else if(event.equalsIgnoreCase("dragon"))
		{
			if(st.getQuestItemsCount(DRAGON_BEAST_ESSENCE) < 1 && st.getInt("cond") == 1)
			{
				if (st.getInt("KB456_DRAGON") != 1)
				{
					if(Rnd.getRnd(CHANCE))
					{	
						st.giveItems(DRAGON_BEAST_ESSENCE, 1, false);
						htmltext = "32864-13.htm";
					}	else htmltext = "32864-14.htm";
					if((st.getQuestItemsCount(DRAKE_LORD_ESSENCE) >= 1) && (st.getQuestItemsCount(BEHEMOTH_LEADER_ESSENCE) >= 1) && (st.getQuestItemsCount(DRAGON_BEAST_ESSENCE) >= 1))
						{
							st.set("cond", "2");
							st.playSound(SOUND_MIDDLE);
						}
					st.set("KB456_DRAGON", 1);
				}
			}		
			else htmltext = "32864-10.htm";	
		}

		else if(event.equalsIgnoreCase("32864-05"))
		{
			if(st.getQuestItemsCount(DRAKE_LORD_ESSENCE) >= 1 && st.getQuestItemsCount(BEHEMOTH_LEADER_ESSENCE) >= 1 && st.getQuestItemsCount(DRAGON_BEAST_ESSENCE) >= 1)
				htmltext = "32864-06.htm";
			else
				htmltext = "32864-09.htm";
		}
		else if(event.equalsIgnoreCase("32864-06"))
		{
			st.takeItems(DRAKE_LORD_ESSENCE, 1);
			st.takeItems(BEHEMOTH_LEADER_ESSENCE, 1);
			st.takeItems(DRAGON_BEAST_ESSENCE, 1);
			int priz = Rnd.get(3);
			switch(priz)
			{
				case 0:
					htmltext = "32864-11.htm";
					st.giveItems(WEAPON[Rnd.get(14)], 1, false);
					break;
				case 1:
					htmltext = "32864-11.htm";
					st.giveItems(ARMOR[Rnd.get(17)], 1, false);
					break;
				case 2:
					htmltext = "32864-11.htm";
					st.giveItems(JEWEL[Rnd.get(3)], 1, false);
					break;
				case 3:
					htmltext = "32864-11.htm";
					st.giveItems(STUFF[Rnd.get(10)], 1, false);
					st.giveItems(STUFF[Rnd.get(10)], 1, false);
					break;			
			}
			Calendar reDo = Calendar.getInstance();
			reDo.set(Calendar.MINUTE, RESET_MIN);
			if (reDo.get(Calendar.HOUR_OF_DAY) >= RESET_HOUR)
				reDo.add(Calendar.DATE, 1);
			reDo.set(Calendar.HOUR_OF_DAY, RESET_HOUR);
	        st.getPlayer().setVar("NextQuest456", "" + (reDo.getTimeInMillis()));
	        
	        st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		int id = st.getState();
		int npcId = npc.getNpcId();
		if((st.getQuestItemsCount(DRAKE_LORD_ESSENCE) >= 1) && (st.getQuestItemsCount(BEHEMOTH_LEADER_ESSENCE) >= 1) && (st.getQuestItemsCount(DRAGON_BEAST_ESSENCE) >= 1))
		{
			st.set("cond", "2");
			st.playSound(SOUND_MIDDLE);
		}
		if(npcId == SEPARATED_SOUL1 || npcId == SEPARATED_SOUL2 || npcId == SEPARATED_SOUL3 || npcId == SEPARATED_SOUL4 || npcId == SEPARATED_SOUL5 || npcId == SEPARATED_SOUL6 || npcId == SEPARATED_SOUL7 || npcId == SEPARATED_SOUL8)
		{		
			String req = st.getPlayer().getVar("NextQuest456") == null ? "0" : st.getPlayer().getVar("NextQuest456");
			if (Long.parseLong(req) > System.currentTimeMillis())
				return "32864-12.htm";
			if(id == CREATED)
			{
				if(st.getPlayer().getLevel() >= 80)
				{
					st.set("cond", "1");
					htmltext = "32864-02.htm";
				}	
				else
				{
					htmltext = "32864-01.htm";
				}
			}
			else if (st.getInt("cond") == 2)
			{
				htmltext = "32864-03.htm";
			}
		}
		else if (npcId == DRAKE_LEADER_NPC)
		{	
			if (st.getInt("cond") == 1)
			htmltext = "drake.htm";
			else 
			htmltext = "drake_no.htm";
		}

		else if (npcId == BEHEMOTH_LEADER_NPC)
		{	
			if (st.getInt("cond") == 1)
			htmltext = "behemoth.htm";
			else 
			htmltext = "behemoth_no.htm";
		}

		else if (npcId == DRAGON_BEAST_NPC)
		{	
			if (st.getInt("cond") == 1)
			htmltext = "dragon.htm";
			else 
			htmltext = "dragon_no.htm";
		}	
			
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		if(st.getInt("cond") == 1)
		{
			if(npcId == DRAKE_LEADER)
			{
				L2NpcInstance isQuest = L2ObjectsStorage.getByNpcId(DRAKE_LEADER_NPC);
				if(isQuest == null)
				{
					st.addSpawn(DRAKE_LEADER_NPC, 120000);
					st.playSound(SOUND_MIDDLE);
					st.startQuestTimer("DRAKE_LEADER_Fail", 120000);
					st.set("KB456_DRAKE", 0);
				}
			}		
			else if(npcId == BEHEMOTH_LEADER)
			{			
				L2NpcInstance isQuest = L2ObjectsStorage.getByNpcId(BEHEMOTH_LEADER_NPC);
				if(isQuest == null)
				{
					st.addSpawn(BEHEMOTH_LEADER_NPC, 120000);
					st.playSound(SOUND_MIDDLE);
					st.startQuestTimer("BEHEMOTH_LEADER_Fail", 120000);
					st.set("KB456_BEHEMOTH", 0);
				}
			}
			else if(npcId == DRAGON_BEAST)
			{
				L2NpcInstance isQuest = L2ObjectsStorage.getByNpcId(DRAGON_BEAST_NPC);
				if(isQuest == null)
				{
					st.addSpawn(DRAGON_BEAST_NPC, 120000);
					st.playSound(SOUND_MIDDLE);
					st.startQuestTimer("DRAGON_BEAST_Fail", 120000);
					st.set("KB456_DRAGON", 0);
				}
			}
			else
				st.playSound(SOUND_ITEMGET);
		}
		return null;
	}
}