package quests._552_OlympiadVeteran;

import java.util.Calendar;

import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.entity.olympiad.Olympiad;
import l2ft.gameserver.model.entity.olympiad.OlympiadGame;
import l2ft.gameserver.model.entity.olympiad.QuestEvent;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.quest.Quest;
import l2ft.gameserver.model.quest.QuestState;

public class _552_OlympiadVeteran extends Quest implements QuestEvent
{
	private static int OLYMPIAD_MANAGER = 31688;
	private static int OLYMPIAD_TREASURE_CHEST = 17169;
	private static int MEDAL_OF_GLORY = 21874;

	private static int[] PARTICIPATION_CERTIFICATE = { 17243, 17242, 17241 };

	public _552_OlympiadVeteran()
	{
		super(false);
		addStartNpc(OLYMPIAD_MANAGER);
		addQuestItem(PARTICIPATION_CERTIFICATE);
	}

	public String onEvent(String event, QuestState st, L2NpcInstance npc)
	{
		if (event.equalsIgnoreCase("quest_accept"))
		{
			st.setCond(1);
			st.setState(2);
			st.playSound("ItemSound.quest_accept");
			event = "31688-2.htm";
		}
		else if (event.equalsIgnoreCase("give_reward"))
		{
			event = addReward(st);
		}
		return event;
	}

	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		int cond = st.getCond();
		if (st.getState() == 1)
		{
			if (!canTake(st.getPlayer()))
			{
				st.exitCurrentQuest(true);
				return "31688-3.htm";
			}
			if (st.getPlayer().getLevel() >= 75)
			{
				return "31688.htm";
			}

		htmltext = "31688-lowlvl.htm";
		st.exitCurrentQuest(true);
		}
		else if (cond == 1)
		{
			htmltext = "31688-4.htm";
		} 
		else if (cond == 2)
		{
			htmltext = addReward(st);
		}
		return htmltext;
	}

	public void startMatch(L2Player player, QuestState qs, int side)
	{
	}

	public void endMatch(L2Player player, QuestState qs, int win)
	{
		if (player == null || qs.getCond() != 1)
			return;
		OlympiadGame game = Olympiad.getOlympiadGame(player.getOlympiadGameId());
		if (game == null)
			return;
		int count;
		switch(game.getType())
		{
			case CLASSED:
			{
				count = qs.getInt("count_classed");
				if(count >= 5)
					break;
				qs.set("count_classed", count + 1);
				if (count != 4)
					break;
				qs.giveItems(PARTICIPATION_CERTIFICATE[0], 1);
				qs.playSound("ItemSound.quest_itemget");
				break;
			}
			case NON_CLASSED:
			{
				count = qs.getInt("count_nonclassed");
				if (count >= 5)
					break;
				qs.set("count_nonclassed", count + 1);
				if (count != 4)
					break;
				qs.giveItems(PARTICIPATION_CERTIFICATE[1], 1);
				qs.playSound("ItemSound.quest_itemget");
				break;
			}
			case TEAM:
			case TEAM_RANDOM:
			{
				count = qs.getInt("count_team");
				if (count >= 5)
					break;
				qs.set("count_team", count + 1);
				if (count != 4)
					break;
				qs.giveItems(PARTICIPATION_CERTIFICATE[2], 1);
				qs.playSound("ItemSound.quest_itemget");
			}
		}

		for (int id : PARTICIPATION_CERTIFICATE)
		{
			if (qs.getQuestItemsCount(id) == 0)
				return;
			qs.setCond(2);
			qs.playSound("ItemSound.quest_middle");
		}
	}

	private String addReward(QuestState qs)
	{
		int count = 0;
		for (int i = 0; i <= 2; i++)
		{
			if (qs.getQuestItemsCount(PARTICIPATION_CERTIFICATE[i]) > 0)
				switch (i)
				{
					case 0:
						count++;
						break;
					case 1:
						count++;
						break;
					case 2:
						count++;
				}
		}
		if (count == 0)
			return "31688-notreward.htm";
		qs.giveItems(OLYMPIAD_TREASURE_CHEST, count);
		qs.giveItems(MEDAL_OF_GLORY, 1L);
		qs.playSound("ItemSound.quest_finish");
		qs.exitCurrentQuest(true);
		setDailyTimer(qs);
		return "31688-addreward.htm";
	}

	private void setDailyTimer(QuestState st)
	{		
		Calendar reDo = Calendar.getInstance();
		reDo.set(Calendar.MINUTE, 30);
		if (reDo.get(Calendar.HOUR_OF_DAY) >= 6)
			reDo.add(Calendar.DATE, 1);
		reDo.set(Calendar.HOUR_OF_DAY, 6);
        st.getPlayer().setVar(getName(), "" + (reDo.getTimeInMillis()));
	}
}