package quests._553_OlympiadUndefeated;

import java.util.Calendar;

import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.entity.olympiad.QuestEvent;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.quest.Quest;
import l2ft.gameserver.model.quest.QuestState;

public class _553_OlympiadUndefeated extends Quest implements QuestEvent
{
	private static int OLYMPIAD_MANAGER = 31688;

	private static int OLYMPIAD_TREASURE_CHEST = 17169;
	private static int MEDAL_OF_GLORY = 21874;

	private static int[] PARTICIPATION_CERTIFICATE = { 17244, 17245, 17246 };

	public _553_OlympiadUndefeated()
	{
		super(false);
		addStartNpc(OLYMPIAD_MANAGER);
		addQuestItem(PARTICIPATION_CERTIFICATE);
	}

	public String onEvent(String event, QuestState st, L2NpcInstance npc)
	{
		if (event.equalsIgnoreCase("quest_accept"))
		{
			st.setCond(1);
			st.setState(2);
			st.playSound("ItemSound.quest_accept");
			event = "31688-2.htm";
		}
		else if (event.equalsIgnoreCase("give_reward"))
		{
			event = addReward(st);
		}
		return event;
	}

	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		int cond = st.getCond();
		if (st.getState() == 1)
		{
			if (!canTake(st.getPlayer()))
			{
				st.exitCurrentQuest(true);
				return "31688-3.htm";
			}
			if (st.getPlayer().getLevel() >= 75)
			{
				return "31688.htm";
			}

			htmltext = "31688-lowlvl.htm";
			st.exitCurrentQuest(true);
		}
		else if (cond == 1)
		{
			htmltext = "31688-4.htm";
		}
		else if (cond == 2)
		{
			htmltext = addReward(st);
		}
		return htmltext;
	}

	public void startMatch(L2Player player, QuestState qs, int side)
	{
	}

	public void endMatch(L2Player player, QuestState qs, int win)
	{
		if ((player == null) || (qs.getCond() != 1))
			return;
		if (win == 1)
			incScore(qs);
		else if (win == 0)
			decScore(qs);
	}

	private String addReward(QuestState qs)
	{
		int count = 0;
			if (qs.getQuestItemsCount(PARTICIPATION_CERTIFICATE[2]) > 0)
				count = 6;
			else if (qs.getQuestItemsCount(PARTICIPATION_CERTIFICATE[1]) > 0)
				count = 3;
			else if (qs.getQuestItemsCount(PARTICIPATION_CERTIFICATE[0]) > 0)
				count = 1;
			else
				return "31688-notreward.htm";
		qs.giveItems(OLYMPIAD_TREASURE_CHEST, count);
		qs.giveItems(MEDAL_OF_GLORY, 1);
		qs.playSound("ItemSound.quest_finish");
		qs.exitCurrentQuest(true);
		setDailyTimer(qs);
		return "31688-addreward.htm";
	}

	public void incScore(QuestState qs)
	{
		int count = qs.getInt("count") + 1;
		qs.set("count", count);
		switch (count)
		{
			case 2:
				qs.giveItems(PARTICIPATION_CERTIFICATE[0], 1);
				qs.playSound("ItemSound.quest_itemget");
				break;
			case 5:
				qs.giveItems(PARTICIPATION_CERTIFICATE[1], 1);
				qs.playSound("ItemSound.quest_itemget");
				break;
			case 10:
				qs.giveItems(PARTICIPATION_CERTIFICATE[2], 1);
				qs.playSound("ItemSound.quest_itemget");
				qs.playSound("ItemSound.quest_middle");
				qs.setCond(2);
		}
	}

	private void setDailyTimer(QuestState st)
	{		
		Calendar reDo = Calendar.getInstance();
		reDo.set(Calendar.MINUTE, 30);
		if (reDo.get(Calendar.HOUR_OF_DAY) >= 6)
			reDo.add(Calendar.DATE, 1);
		reDo.set(Calendar.HOUR_OF_DAY, 6);
        st.getPlayer().setVar(getName(), "" + (reDo.getTimeInMillis()));
	}

	public void decScore(QuestState qs)
	{
		qs.set("count", 0);
		qs.takeAllItems(PARTICIPATION_CERTIFICATE);
	}
}