package services;

import javolution.util.FastMap;
import l2ft.extensions.scripts.Functions;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.instancemanager.InstancedZoneManager;
import l2ft.gameserver.model.entity.InstancedZone;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.util.GArray;
import l2ft.util.Location;

/**
 * Используется для Hellbound
 */
public class Caravan extends Functions implements ScriptFile
{
	public void onLoad()
	{
		System.out.println("Loaded Service: Tower");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	private static final int FieryDemonBloodSkill = 2357;
	private static Location TowerofInfinitumLocationPoint = new Location(-22204, 277056, -15045);
	private static Location TullyEntranceLocationPoint = new Location(17947, 283205, -9696);
	private static Location TullyFloor1LocationPoint = new Location(-13400, 272827, -15304);
	private static FastMap<Integer, Long> _instances = new FastMap<Integer, Long>();


	public void enterToInfinitumTower()
	{
		L2Player p = (L2Player) getSelf();
		L2NpcInstance n = getNpc();

		// Нету партии или не лидер партии
		if(p.getParty() == null || !p.getParty().isLeader(p))
		{
			n.onBypassFeedback(p, "Chat 1");
			return;
		}

		GArray<L2Player> members = p.getParty().getPartyMembers();

		// Далеко или нету эффекта херба Fiery Demon Blood
		for(L2Player member : members)
			if(member == null || !L2NpcInstance.canBypassCheck(member, n) || member.getEffectList().getEffectsBySkillId(FieryDemonBloodSkill) == null)
			{
				n.onBypassFeedback(p, "Chat 2");
				return;
			}

		// Телепортируем партию на 1 этаж Tower of Infinitum
		for(L2Player member : members)
			member.teleToLocation(TowerofInfinitumLocationPoint);
	}
	
	public void TeleportNew(String[] param)
	{
		if(param.length < 2)
			throw new IllegalArgumentException();
		
		L2Player p = (L2Player) getSelf();
		L2NpcInstance n = getNpc();

		// Нету партии или не лидер партии
		if(p.getParty() == null || !p.getParty().isLeader(p))
		{
			p.sendMessage("Вы не лидер!");
			return;
		}
		
		GArray<L2Player> members = p.getParty().getPartyMembers();
		
		int x = Integer.parseInt(param[0]);
		int y = Integer.parseInt(param[1]);
		int z = Integer.parseInt(param[2]);
		
		Location NewTeleport = new Location(x, y, z);
		
		for(L2Player member : members)
			member.teleToLocation(NewTeleport);
	}
	
	public void enterDemonPrince()
	{
		L2Player player = (L2Player) getSelf();
		if(player == null)
			return;

		int instancedZoneId = 142;
		InstancedZoneManager izm = InstancedZoneManager.getInstance();
		FastMap<Integer, InstancedZone> izs = izm.getById(instancedZoneId);
		if(izs == null)
		{
			player.sendPacket(Msg.SYSTEM_ERROR);
			return;
		}

		InstancedZone iz = izs.get(0);
		assert iz != null;

		String name = iz.getName();
		int timelimit = iz.getTimelimit();
		boolean dispellBuffs = iz.isDispellBuffs();
		int min_level = iz.getMinLevel();
		int max_level = iz.getMaxLevel();
		int minParty = iz.getMinParty();
		int maxParty = iz.getMaxParty();

		if(!player.isInParty())
		{
			player.sendPacket(Msg.YOU_ARE_NOT_CURRENTLY_IN_A_PARTY_SO_YOU_CANNOT_ENTER);
			return;
		}
		
		if(!player.getParty().isLeader(player))
		{
			player.sendPacket(Msg.ONLY_A_PARTY_LEADER_CAN_TRY_TO_ENTER);
			return;
		}

		if(player.getParty().getMemberCount() > maxParty)
		{
			player.sendPacket(Msg.YOU_CANNOT_ENTER_DUE_TO_THE_PARTY_HAVING_EXCEEDED_THE_LIMIT);
			return;
		}

		for(L2Player member : player.getParty().getPartyMembers())
		{
			if(member.getLevel() < min_level || member.getLevel() > max_level)
			{
				SystemMessage sm = new SystemMessage(SystemMessage.C1S_LEVEL_REQUIREMENT_IS_NOT_SUFFICIENT_AND_CANNOT_BE_ENTERED).addName(member);
				member.sendPacket(sm);
				player.sendPacket(sm);
				return;
			}
			if(!player.isInRange(member, 500))
			{
				member.sendPacket(Msg.ITS_TOO_FAR_FROM_THE_NPC_TO_WORK);
				player.sendPacket(Msg.ITS_TOO_FAR_FROM_THE_NPC_TO_WORK);
				return;
			}
		}

		Reflection r = new Reflection(iz);
		r.setInstancedZoneId(instancedZoneId);

		for(InstancedZone i : izs.values())
		{
			if(r.getReturnLoc() == null)
				r.setReturnLoc(i.getReturnCoords());
			if(r.getTeleportLoc() == null)
				r.setTeleportLoc(i.getTeleportCoords());
			r.FillSpawns(i.getSpawnsInfo());
		}

		for(L2Player member : player.getParty().getPartyMembers())
		{
			member.setVar("backCoords", r.getReturnLoc().toXYZString());
			member.teleToLocation(iz.getTeleportCoords(), r.getId());
			//TODO: должен брать какой-то итем
		}

		player.getParty().setReflection(r);
		r.setParty(player.getParty());
		if(timelimit > 0)
		{
			r.startCollapseTimer(timelimit * 60 * 1000L);
			player.getParty().broadcastToPartyMembers(new SystemMessage(SystemMessage.THIS_DUNGEON_WILL_EXPIRE_IN_S1_MINUTES).addNumber(timelimit));
		}
	}

	public void enterToTullyEntrance()
	{
		L2Player p = (L2Player) getSelf();
		L2NpcInstance n = getNpc();

		if(!L2NpcInstance.canBypassCheck(p, n))
			return;

		// Телепортируем чара в предбанник Tully's Workshop
		if(p.isQuestCompleted("_132_MatrasCuriosity"))
			p.teleToLocation(TullyEntranceLocationPoint);
		else
			n.onBypassFeedback(p, "Chat 1");
	}
	
	public void enterToTullyFloor1()
	{
		L2Player p = (L2Player) getSelf();
		L2NpcInstance n = getNpc();

		if(!L2NpcInstance.canBypassCheck(p, n))
			return;

		// Нету партии или не лидер партии
		if(p.getParty() == null || !p.getParty().isLeader(p))
		{
			n.onBypassFeedback(p, "Chat 2");
			return;
		}

		GArray<L2Player> members = p.getParty().getPartyMembers();

		// Далеко или не выполнен 132 квест
		for(L2Player member : members)
			if(member == null || !L2NpcInstance.canBypassCheck(member, n) || !member.isQuestCompleted("_132_MatrasCuriosity"))
			{
				n.onBypassFeedback(p, "Chat 1");
				return;
			}

		// Телепортируем партию на 1 этаж Tully's Workshop
		for(L2Player member : members)
			member.teleToLocation(TullyFloor1LocationPoint);
	}
}