package services;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.concurrent.ConcurrentHashMap;

import l2ft.Config;
import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.extensions.scripts.Functions;
import l2ft.extensions.scripts.ScriptFile;
import l2ft.gameserver.communitybbs.CommunityBoard;
import l2ft.gameserver.model.L2Clan;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.serverpackets.PledgeShowInfoUpdate;
import l2ft.gameserver.serverpackets.PledgeShowMemberListAdd;
import l2ft.gameserver.serverpackets.ShowBoard;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.util.Files;
import l2ft.util.Log;

public class CommunityAcademy extends Functions implements ScriptFile {

    private static final long ADENA2PLAYER = 10000000;
    private static final long ADENA_FROM_CLAN = 25000000;
    private static final int PLAYERS_IN_LIST = 90;

    private static ConcurrentHashMap<Integer, String> academic_list = new ConcurrentHashMap<Integer, String>();

    public void mainPage() {
        L2Player player = (L2Player) getSelf();

        if (player.isClanLeader()) {
            String html = Files.read(Config.COMMUNITYBOARD_HTML_ROOT + "2004.htm");
            String append = "<table>";
            int i = 0;
            int cell = 0;
            for (int id : academic_list.keySet()) {
                L2Player member = L2ObjectsStorage.getPlayer(id);
                if (member == null || !member.canJoinClan() || member.getClan() != null || member.getClassId().level() >= 2)
                    continue;
                cell++;
                append += "<a action=\"bypass -h _bbsscripts;index;community.CommunityAcademy:getAcademic " + id + "\"><font color=\"aadd77\">" + academic_list.get(id) + "</font></a>\n";
                i++;
                if (cell == 1)
                    append += "<tr>";
                append += "<td align=left valign=top fixwidth=120><a action=\"bypass -h _bbsscripts;index;community.CommunityAcademy:getAcademic " + id + "\"><font color=\"aadd77\">" + academic_list.get(id) + "</font></a></td> ";
                if (cell == Config.NAME_PER_ROW_COMMUNITYBOARD) {
                    cell = 0;
                    append += "</tr>";
                }
                if (i >= PLAYERS_IN_LIST)
                    break;
            }
            append += "</table>";
            html = html.replaceFirst("%append%", append);
            ShowBoard.separateAndSend(html, player);
        } else if (player.getClassId().level() < 2) {
            if (!academic_list.containsKey(player.getObjectId())) {
                CommunityBoard.getInstance().handleCommands(player.getNetConnection(), "_bbstop;2007");
            } else {
                CommunityBoard.getInstance().handleCommands(player.getNetConnection(), "_bbstop;2001");
            }
        } else {
            if (academic_list.containsKey(player.getObjectId())) {
                CommunityBoard.getInstance().handleCommands(player.getNetConnection(), "_bbstop;2002");
            } else {
                String html = Files.read(Config.COMMUNITYBOARD_HTML_ROOT + "2003.htm");
                String supports = "<table width=640 border=1>";
                supports += "<tr><td> Имя/Name: </td><td> Статус/Status: </td><td> Рейтинг/Rating: </td><td> Голосовать/Vote: </td></tr>";
                ThreadConnection con = null;
                FiltredPreparedStatement st1 = null;
                ResultSet rs1 = null;
                try {
                    con = L2DatabaseFactory.getInstance().getConnection();
                    st1 = con.prepareStatement("SELECT * FROM `supports` ORDER BY `rating` DESC");
                    rs1 = st1.executeQuery();
                    while (rs1.next()) {
                        String obj_Id = rs1.getString("id");
                        String rating = rs1.getString("rating");
                        FiltredPreparedStatement st2 = con.prepareStatement("SELECT char_name, online FROM characters WHERE obj_Id=" + obj_Id);
                        ResultSet rs2 = st2.executeQuery();
                        if(!rs2.next()) {
                            DatabaseUtils.closeDatabaseSR(st2, rs2);
                            continue;
                        }
                        String name = rs2.getString("char_name");
                        String status = rs2.getBoolean("online") ? "<font color=\"LEVEL\">Онлайн</font>" : "Оффлайн";
                        supports += "<tr><td>" + name + "</td><td>" + status + "</td><td>" + rating + "</td><td>"
                                + "<table><tr><td><button value=\"+\" action=\"bypass -h _bbsscripts;2008;community.CommunityAcademy:increaseRating "
                                + obj_Id + "\" width=21 height=21 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_CT1.Button_DF\" /></td><td>/</td> <td>"
                                + "<button value=\"-\" action=\"bypass -h _bbsscripts;2008;community.CommunityAcademy:decreaseRating "
                                + obj_Id + "\" width=21 height=21 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_CT1.Button_DF\" /></td></tr></table>"
                                + "</td></tr>";
                    }
                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {
                    DatabaseUtils.closeDatabaseCSR(con, st1, rs1);
                }
                supports += "</table>";
                html = html.replaceFirst("%supports%", supports);
                ShowBoard.separateAndSend(html, player);
            }
        }
    }

    public void joinAcademy() {
        L2Player player = (L2Player) getSelf();
        if (player.getClassId().level() >= 2 || player.getClan() != null) {
            player.sendMessage("Для вступления вы не должны состоять в клане и ваша профессия должна быть ниже второй");
            return;
        }
        player.sendMessage("Загляни на эту же страницу после получения второй профессии");
        academic_list.put(player.getObjectId(), player.getName());

        ThreadConnection con = null;
        FiltredPreparedStatement st = null;
        try {
            con = L2DatabaseFactory.getInstance().getConnection();
            st = con.prepareStatement("REPLACE INTO community_academy (objId, name, param) values (?,?,?)");
            st.setInt(1, player.getObjectId());
            st.setString(2, player.getName());
            st.setInt(3, 0);
            st.execute();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DatabaseUtils.closeDatabaseCS(con, st);
        }
    }

    public void getReward() {
        L2Player player = (L2Player) getSelf();
        academic_list.remove(player.getObjectId());
        player.addAdena(ADENA2PLAYER);
        ThreadConnection con = null;
        FiltredPreparedStatement st = null;
        try {
            con = L2DatabaseFactory.getInstance().getConnection();
            st = con.prepareStatement("DELETE FROM community_academy WHERE objId=?");
            st.setInt(1, player.getObjectId());
            st.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DatabaseUtils.closeDatabaseCS(con, st);
        }
        Log.add("Character " + player.getName() + " getReward " + ADENA2PLAYER + " adena", "Academy");
    }

    public void getAcademic(String[] args) {
        L2Player player = (L2Player) getSelf();
        int id = Integer.parseInt(args[0]);
        if (player.getAdena() < ADENA_FROM_CLAN) {
            player.sendMessage("Недостаточно адены.");
            return;
        }
        L2Player member;
        if ((member = L2ObjectsStorage.getPlayer(id)) == null)
            member = L2Player.restore(id);
        L2Clan clan = player.getClan();
        if (member == null || clan == null || !player.isClanLeader())
            return;
        if (clan.getAvailablePledgeTypes(-1) != 0 || clan.getSubPledgeMembersCount(-1) >= 20) {
            player.sendMessage("У вас нет академии, либо она уже полна!");
            return;
        }
        if (member.canJoinClan() && member.getClan() == null && member.getClassId().level() < 2) {
            member.setPledgeType(-1);
            clan.addClanMember(member);
            member.setClan(clan);
            clan.getClanMember(member.getName()).setPlayerInstance(member);
            member.setLvlJoinedAcademy(member.getLevel());
            clan.getClanMember(member.getName()).setPowerGrade(clan.getAffiliationRank(member.getPledgeType()));
            clan.broadcastToOnlineMembers(new SystemMessage(SystemMessage.S1_HAS_JOINED_THE_CLAN).addString(member.getName()));
            clan.broadcastToOtherOnlineMembers(new PledgeShowMemberListAdd(clan.getClanMember(member.getName())), member);
            clan.broadcastToOnlineMembers(new PledgeShowInfoUpdate(clan));

            member.setLeaveClanTime(0);
            member.updatePledgeClass();
            member.store(false);
            player.reduceAdena(ADENA_FROM_CLAN, true);
            Log.add("ClanLeader " + player.getName() + " getAcademic " + member.getName() + "Reduce adena:" + ADENA_FROM_CLAN, "Academy");
        } else {
            player.sendMessage("Данного персонажа невозможно принять в академию");
        }
    }

    public void increaseRating(String[] args) {
        if(args == null || args.length < 1)
            return;
        L2Player player = (L2Player) getSelf();
        ThreadConnection con = null;
        FiltredPreparedStatement st = null;
        ResultSet rs = null;
        String id = args[0];
        try {
            con = L2DatabaseFactory.getInstance().getConnection();
            st = con.prepareStatement("SELECT * FROM supports_vote WHERE hash=" + player.getAccountName().hashCode() + " LIMIT 1");
            rs = st.executeQuery();
            if(rs.next()) {
                player.sendMessage("Вы уже голосовали");
                DatabaseUtils.closeDatabaseSR(st, rs);
            } else {
                st = con.prepareStatement("INSERT INTO supports_vote (hash, account_name) VALUES (?,?)");
                st.setInt(1, player.getAccountName().hashCode());
                st.setString(2, player.getAccountName());
                st.execute();
                DatabaseUtils.closeStatement(st);
                st = con.prepareStatement("UPDATE supports SET rating=rating+1 WHERE id=" + id);
                st.executeUpdate();
                DatabaseUtils.closeDatabaseSR(st, rs);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            DatabaseUtils.closeDatabaseCSR(con, st, rs);
        }
    }

    public void decreaseRating(String[] args) {
        if(args == null || args.length < 1)
            return;
        L2Player player = (L2Player) getSelf();
        ThreadConnection con = null;
        FiltredPreparedStatement st = null;
        ResultSet rs = null;
        String id = args[0];
        try {
            con = L2DatabaseFactory.getInstance().getConnection();
            st = con.prepareStatement("SELECT * FROM supports_vote WHERE hash=" + player.getAccountName().hashCode() + " LIMIT 1");
            rs = st.executeQuery();
            if(rs.next()) {
                player.sendMessage("Вы уже голосовали");
                DatabaseUtils.closeDatabaseSR(st, rs);
            } else {
                st = con.prepareStatement("INSERT INTO supports_vote (hash, account_name) VALUES (?,?)");
                st.setInt(1, player.getAccountName().hashCode());
                st.setString(2, player.getAccountName());
                st.execute();
                DatabaseUtils.closeStatement(st);
                st = con.prepareStatement("UPDATE supports SET rating=rating-1 WHERE id=" + id);
                st.executeUpdate();
                DatabaseUtils.closeDatabaseSR(st, rs);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            DatabaseUtils.closeDatabaseCSR(con, st, rs);
        }
    }

    @Override
    public void onLoad() {
        ThreadConnection con = null;
        FiltredPreparedStatement st = null;
        ResultSet rs = null;
        try {
            con = L2DatabaseFactory.getInstance().getConnection();
            st = con.prepareStatement("SELECT * FROM community_academy WHERE objId > -1 ORDER BY id DESC");
            rs = st.executeQuery();
            while (rs.next())
                if (rs.getInt(4) == 0)
                    academic_list.put(rs.getInt(2), rs.getString(3));
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DatabaseUtils.closeDatabaseCSR(con, st, rs);
        }
    }

    @Override
    public void onReload() {

    }

    @Override
    public void onShutdown() {

    }
}
