package l2ft.database;

import java.sql.SQLException;
import java.util.AbstractMap.SimpleEntry;
import java.util.Arrays;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.logging.Logger;
import l2ft.Config;

public class DatabaseExecutor
{
	static final Logger _log = Logger.getLogger(DatabaseExecutor.class.getName());
	private static volatile boolean isInited;

	public static void init()
	{
		if (isInited)
		{
			_log.warning("Double init for DatabaseExecutor");
			return;
		}
		isInited = true;
		try
		{
			if (Config.SQL_ITEM_TRANSACTIONS_LOG_DATABASE != null)
				Table.log_items.setDb(Config.SQL_ITEM_TRANSACTIONS_LOG_DATABASE);
			for (Table t : Table.values())
				t.init();
			for (Query q : Query.values())
				q.init();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public static void shutdown()
	{
		try
		{
			for (Table t : Table.values())
			{
				int counter = 0;
				while ((!t.tasks.isEmpty()) && (counter < 200))
				{
					Thread.sleep(10L);
					counter++;
				}
				if (!t.tasks.isEmpty())
					_log.warning("Timeout for executor shutdown: " + t + ", tasks: " + t.tasks.size());
				t.executor.interrupt();
				t.con.close();
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public static enum Query
	{
		updateItem(Table.items, "UPDATE items SET owner_id=?,count=?,loc=?,loc_data=?,enchant_level=?,shadow_life_time=?,item_id=?,flags=? WHERE object_id = ? LIMIT 1"), 

		insertItem(Table.items, "REPLACE INTO items (owner_id,item_id,count,loc,loc_data,enchant_level,object_id,custom_type1,custom_type2,shadow_life_time,name,class,flags) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)"), 

		removeItem(Table.items, "DELETE FROM items WHERE object_id = ? LIMIT 1"), 

		removeItemAugAndAttribute(Table.item_attributes, "DELETE FROM item_attributes WHERE itemId = ? LIMIT 1"), 
		removeItemAug(Table.item_attributes, "UPDATE item_attributes SET augAttributes = -1, augSkillId = -1, augSkillLevel = -1 WHERE itemId = ? LIMIT 1"), 

		updateItemAttribute(Table.item_attributes, "REPLACE INTO item_attributes VALUES(?,?,?,?,?,?)"), 

		updateOnlineStatus(Table.characters, "UPDATE characters SET online=?, lastAccess=? WHERE obj_id=? LIMIT 1"), 
		updatePlayer(Table.characters, "UPDATE characters SET face=?,hairStyle=?,hairColor=?,heading=?,x=?,y=?,z=?,karma=?,pvpkills=?,pkkills=?,rec_have=?,rec_left=?,clanid=?,deletetime=?,title=?,accesslevel=?,online=?,leaveclan=?,deleteclan=?,nochannel=?,onlinetime=?,pledge_type=?,pledge_rank=?,lvl_joined_academy=?,apprentice=?,key_bindings=?,pcBangPoints=?,char_name=?,vitality=?,fame=?,bookmarks=?,navitPoints=?,navitBonus=?,rec_timeleft=? WHERE obj_Id=? LIMIT 1"), 

		addSubclass(Table.character_subclasses, "INSERT INTO character_subclasses (char_obj_id, class_id, exp, sp, curHp, curMp, curCp, maxHp, maxMp, maxCp, level, active, isBase, death_penalty, skills) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"), 

		storeSubclass(Table.character_subclasses, "UPDATE character_subclasses SET exp=?,sp=?,curHp=?,curMp=?,curCp=?,level=?,active=?,isBase=?,death_penalty=?,skills=? WHERE char_obj_id=? AND class_id=? LIMIT 1"), 

		updateSubclassInfo(Table.character_subclasses, "UPDATE character_subclasses SET maxHp=?,maxMp=?,maxCp=? WHERE char_obj_id=? AND active=1 LIMIT 1"), 

		storeVar(Table.character_variables, "REPLACE INTO character_variables (obj_id, name, value) VALUES (?,?,?)"), 
		deleteVar(Table.character_variables, "DELETE FROM `character_variables` WHERE `obj_id`=? AND `name`=? LIMIT 1"), 

		questUpdate(Table.character_quests, "REPLACE INTO character_quests (char_id,name,var,value) VALUES (?,?,?,?)"), 
		questDelete(Table.character_quests, "DELETE FROM character_quests WHERE char_id=? AND name=? AND var=?"), 
		questDeleteAll(Table.character_quests, "DELETE FROM character_quests WHERE char_id=? AND name=?"), 

		deleteContactFromList(Table.character_contacts, "DELETE FROM `character_contacts` WHERE `char_id` = ? AND `contact_id` = ?"), 

		addContactToList(Table.character_contacts, "REPLACE INTO `character_contacts` (`char_id`, `contact_id`) VALUES (?, ?)"), 

		removeBookmarks(Table.character_bookmarks, "DELETE FROM `character_bookmarks` WHERE char_Id=?"), 
		storeBookmarks(Table.character_bookmarks, "INSERT INTO `character_bookmarks` VALUES(?,?,?,?,?,?,?,?)"), 

		shortCutAdd(Table.character_shortcuts, "REPLACE INTO character_shortcuts SET char_obj_id=?,slot=?,page=?,type=?,shortcut_id=?,level=?,class_index=?"), 

		shortCutRemove(Table.character_shortcuts, "DELETE FROM character_shortcuts WHERE char_obj_id=? AND slot=? AND page=? AND class_index=?"), 

		storeSkill(Table.character_skills, "REPLACE INTO character_skills (char_obj_id,skill_id,skill_level,skill_name,class_index) values(?,?,?,?,?)"), 

		removeSkill(Table.character_skills, "DELETE FROM character_skills WHERE skill_id=? AND char_obj_id=? AND class_index=?"), 

		clearSkillsSave(Table.character_skills_save, "DELETE FROM character_skills_save WHERE char_obj_id = ? AND class_index=? AND `end_time` < ?"), 

		storeSkillsSave(Table.character_skills_save, "REPLACE INTO `character_skills_save` (`char_obj_id`,`skill_id`,`class_index`,`end_time`,`reuse_delay_org`) VALUES (?,?,?,?,?)"), 

		clearEffectsSave(Table.character_effects_save, "DELETE FROM character_effects_save WHERE char_obj_id = ? AND class_index=?"), 

		storeEffectsSave(Table.character_effects_save, "INSERT IGNORE INTO `character_effects_save` (`char_obj_id`,`skill_id`,`skill_level`,`effect_count`,`effect_cur_time`,`duration`,`order`,`class_index`) VALUES (?,?,?,?,?,?,?,?)"), 

		deleteFromBlocklist(Table.character_blocklist, "DELETE FROM character_blocklist WHERE obj_Id = ?"), 
		addToBlocklist(Table.character_blocklist, "INSERT IGNORE INTO `character_blocklist` (`obj_Id`,`target_Id`) VALUES (?,?)"), 

		petsUpdate(Table.pets, "UPDATE pets SET name=?,level=?,curHp=?,curMp=?,exp=?,sp=?,fed=?,objId=? WHERE item_obj_id = ?"), 

		petsInsert(Table.pets, "INSERT INTO pets (name,level,curHp,curMp,exp,sp,fed,objId,item_obj_id) VALUES (?,?,?,?,?,?,?,?,?)"), 

		petsDelete(Table.pets, "DELETE FROM pets WHERE item_obj_id=?"), 

		itemsLogAdd(Table.log_items, "INSERT IGNORE INTO `log_items` (`op_id`,`op_type`,`actor`,`target`,`item_obj_id`,`item_id`,`item_count`) VALUES (?,?,?,?,?,?,?)"), 

		itemsLogUpdate(Table.log_items, "UPDATE `log_items` SET `target`=? WHERE `op_id`=? LIMIT 1");

		private final Table _table;
		private final String _query;
		private FiltredPreparedStatement statement;

		private Query(Table table, String query)
		{
			_table = table;
			_query = query;
		}

		public Table getTable()
		{
			return _table;
		}

		public FiltredPreparedStatement getStatement()
		{
			return statement;
		}

		public void init()
		{
			if (_query == null)
			{
				DatabaseExecutor._log.warning("Query for " + toString() + " is not defined.");
				return;
			}
			try
			{
				statement = _table.getConnection().prepareStatement(_query);
			}
			catch (Exception e)
			{
				DatabaseExecutor._log.warning("Unable to prepare statement for query " + toString());
				e.printStackTrace();
			}
		}

		public void execute(Object[] param)
		{
			_table.addTask(this, param);
		}

		public void executeNow(Object[] param)
		{
			_table.addUrgentTask(this, param);
		}
	}

	public static enum Table
	{
		items, 
		item_attributes, 
		characters, 
		character_subclasses, 
		character_contacts, 
		character_bookmarks, 
		character_skills, 
		character_skills_save, 
		character_effects_save, 
		character_variables, 
		character_blocklist, 
		character_quests, 
		character_shortcuts, 
		pets, 
		hwidLog, 
		log_items, 
		hwidBonus, 
		hwidBans;

		private DatabaseConnectionProperties dbProps;
		private ThreadConnection con;
		private LinkedBlockingDeque<SimpleEntry<DatabaseExecutor.Query, Object[]>> tasks = new LinkedBlockingDeque<SimpleEntry<DatabaseExecutor.Query, Object[]>>();
		private Thread executor;

		private Table()
		{
			dbProps = Config.DATABASE;
		}

		private void setDb(DatabaseConnectionProperties database)
		{
			dbProps = database;
		}

		public ThreadConnection getConnection()
		{
			return con;
		}

		public void init() throws SQLException
		{
			con = dbProps.instance.getNewConnection();
			executor = new Thread(new Runnable()
			{
				public void run()
				{
					SimpleEntry task = null;
					Query query = null;
					while(true)
					try
					{
						task = (SimpleEntry)Table.this.tasks.take();
						query = (Query)task.getKey();
						query.getStatement().setVars((Object[])task.getValue());
						query.getStatement().executeUpdate();
						continue; 
					}
					catch (InterruptedException ie)
					{
						continue;
					}
					catch (Exception e)
					{
						if ((task != null) && (task.getKey() != null))
							_log.warning("Exception in query: " + (Query)task.getKey() + "; Vars: " + Arrays.toString((Object[])task.getValue()));
						e.printStackTrace();
					}
				}
			});
			executor.setDaemon(true);
			executor.start();
		}

		public void addTask(Query query, Object[] param)
		{
			tasks.offer(new SimpleEntry<Query, Object[]>(query, param));
		}

		public void addUrgentTask(Query query, Object[] param)
		{
			tasks.offerFirst(new SimpleEntry<Query, Object[]>(query, param));
		}
	}
}