package l2ft.gameserver;

import java.io.File;
import java.sql.ResultSet;
import java.util.Calendar;
import java.util.HashMap;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2ft.Config;
import l2ft.Server;
import l2ft.common.ThreadPoolManager;
import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.database.mysql;
import l2ft.extensions.Stat;
import l2ft.extensions.network.MMOConnection;
import l2ft.extensions.network.MMOSocket;
import l2ft.extensions.network.SelectorConfig;
import l2ft.extensions.network.SelectorThread;
import l2ft.extensions.scripts.Events;
import l2ft.extensions.scripts.ScriptObject;
import l2ft.gameserver.cache.CrestCache;
import l2ft.gameserver.geodata.GeoEngine;
import l2ft.gameserver.handler.AdminCommandHandler;
import l2ft.gameserver.handler.ItemHandler;
import l2ft.gameserver.handler.UserCommandHandler;
import l2ft.gameserver.handler.VoicedCommandHandler;
import l2ft.gameserver.idfactory.IdFactory;
import l2ft.gameserver.instancemanager.AuctionManager;
import l2ft.gameserver.instancemanager.CastleManager;
import l2ft.gameserver.instancemanager.CastleManorManager;
import l2ft.gameserver.instancemanager.CastleSiegeManager;
import l2ft.gameserver.instancemanager.ClanHallManager;
import l2ft.gameserver.instancemanager.ClanHallSiegeManager;
import l2ft.gameserver.instancemanager.CoupleManager;
import l2ft.gameserver.instancemanager.CursedWeaponsManager;
import l2ft.gameserver.instancemanager.DelaySpawnManager;
import l2ft.gameserver.instancemanager.DimensionalRiftManager;
import l2ft.gameserver.instancemanager.ExtractableItemsManager;
import l2ft.gameserver.instancemanager.FortressManager;
import l2ft.gameserver.instancemanager.FortressSiegeManager;
import l2ft.gameserver.instancemanager.GameHandyManager;
import l2ft.gameserver.instancemanager.HellboundManager;
import l2ft.gameserver.instancemanager.InstancedZoneManager;
import l2ft.gameserver.instancemanager.ItemAuctionManager;
import l2ft.gameserver.instancemanager.ItemsForOnline;
import l2ft.gameserver.instancemanager.KrateisCubeManager;
import l2ft.gameserver.instancemanager.L2TopManager;
import l2ft.gameserver.instancemanager.LotteryManager;
import l2ft.gameserver.instancemanager.MMOTopManager;
import l2ft.gameserver.instancemanager.MercTicketManager;
import l2ft.gameserver.instancemanager.PartyRoomManager;
import l2ft.gameserver.instancemanager.PlayerMessageStack;
import l2ft.gameserver.instancemanager.RaidBossSpawnManager;
import l2ft.gameserver.instancemanager.RewardForOnline;
import l2ft.gameserver.instancemanager.TownManager;
import l2ft.gameserver.instancemanager.UnderGroundColliseumManager;
import l2ft.gameserver.itemmall.ItemMall;
import l2ft.gameserver.loginservercon.LSConnection;
import l2ft.gameserver.model.AutoChatHandler;
import l2ft.gameserver.model.AutoSpawnHandler;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.entity.Hero;
import l2ft.gameserver.model.entity.MonsterRace;
import l2ft.gameserver.model.entity.SevenSigns;
import l2ft.gameserver.model.entity.SevenSignsFestival.SevenSignsFestival;
import l2ft.gameserver.model.entity.olympiad.Olympiad;
import l2ft.gameserver.model.entity.siege.territory.TerritorySiege;
import l2ft.gameserver.model.entity.vehicle.L2VehicleManager;
import l2ft.gameserver.model.items.MailParcelController;
import l2ft.gameserver.network.L2GameClient;
import l2ft.gameserver.network.L2GamePacketHandler;
import l2ft.gameserver.tables.AirShipDocksTable;
import l2ft.gameserver.tables.AugmentationData;
import l2ft.gameserver.tables.CharNameTable;
import l2ft.gameserver.tables.CharTemplateTable;
import l2ft.gameserver.tables.ClanTable;
import l2ft.gameserver.tables.DoorTable;
import l2ft.gameserver.tables.FakePlayersTable;
import l2ft.gameserver.tables.HennaTable;
import l2ft.gameserver.tables.HennaTreeTable;
import l2ft.gameserver.tables.LevelUpTable;
import l2ft.gameserver.tables.MapRegion;
import l2ft.gameserver.tables.NpcTable;
import l2ft.gameserver.tables.PetSkillsTable;
import l2ft.gameserver.tables.SkillSpellbookTable;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.tables.SkillTreeTable;
import l2ft.gameserver.tables.SpawnTable;
import l2ft.gameserver.tables.StaticObjectsTable;
import l2ft.gameserver.tables.TeleportTable;
import l2ft.gameserver.taskmanager.ItemsAutoDestroy;
import l2ft.gameserver.taskmanager.MemoryWatchDog;
import l2ft.gameserver.taskmanager.TaskManager;
import l2ft.gameserver.xml.ItemTemplates;
import l2ft.status.Status;
import l2ft.util.FirstTeam;
import l2ft.util.GArray;
import l2ft.util.HWID;
import l2ft.util.RRDTools;
import l2ft.util.Strings;

@SuppressWarnings({ "nls", "unqualified-field-access", "boxing" })
public class GameServer
{
	private static final Logger _log = Logger.getLogger(GameServer.class.getName());

	private final SelectorThread<L2GameClient> _selectorThreads[];
	public static GameServer gameServer;

	public static Status statusServer;

	public static Events events;

	public static FastMap<String, ScriptObject> scriptsObjects = new FastMap<String, ScriptObject>().setShared(true);

	private static int _serverStarted;

	protected static boolean serverLoaded = false;
	
	protected static final String LOG_FOLDER = "log"; // Name of folder for log file
	protected static final String LOG_NAME = "./config/main/log.ini"; // Name of log file
	protected static final String LOG_TEXT = "Telnet server is currently disabled.";
	protected static final String Lines = "=================================================";

	public SelectorThread<L2GameClient>[] getSelectorThreads()
	{
		return _selectorThreads;
	}

	public static int time()
	{
		return (int) (System.currentTimeMillis() / 1000);
	}

	public static int uptime()
	{
		return time() - _serverStarted;
	}

	@SuppressWarnings("unchecked")
	public GameServer() throws Exception
	{
		Server.gameServer = this;

		_serverStarted = time();
		_log.finest("used mem:" + MemoryWatchDog.getMemUsedMb());

		Strings.reload();

		IdFactory _idFactory = IdFactory.getInstance();
		if(!_idFactory.isInitialized())
		{
			_log.severe("Could not read object IDs from DB. Please Check Your Data.");
			throw new Exception("Could not initialize the ID factory");
		}

		ThreadPoolManager.getInstance();

		if(Config.DEADLOCKCHECK_INTERVAL > 0)
			new DeadlockDetector().start();

		CrestCache.load();
		// start game time control early
		GameTimeController.getInstance();

		// keep the references of Singletons to prevent garbage collection
		CharNameTable.getInstance();

		AuctionManager.getInstance();

		ClanTable.getInstance();

		FakePlayersTable.getInstance();

		SkillTable.getInstance();

		PetSkillsTable.getInstance();

        ItemTemplates.getInstance();

		events = new Events();

		TradeController.getInstance();
		ExtractableItemsManager.getInstance();
		RecipeController.getInstance();
		_log.config("RecipeController: Loaded " + RecipeController.getInstance().GetSize() + " Recipes.");
		
		
		ItemAuctionManager.getInstance();

		SkillTreeTable.getInstance();
		SkillSpellbookTable.getInstance();
		CharTemplateTable.getInstance();

		NpcTable.getInstance();
		if(!NpcTable.isInitialized())
		{
			_log.severe("Could not find the extraced files. Please Check Your Data.");
			throw new Exception("Could not initialize the npc table");
		}

		HennaTable _hennaTable = HennaTable.getInstance();
		if(!_hennaTable.isInitialized())
			throw new Exception("Could not initialize the Henna Table");
		HennaTreeTable.getInstance();
		if(!_hennaTable.isInitialized())
			throw new Exception("Could not initialize the Henna Tree Table");

		LevelUpTable.getInstance();

		GeoEngine.loadGeo();

		DoorTable.getInstance();

		UnderGroundColliseumManager.getInstance();

		TownManager.getInstance();

		CastleManager.getInstance();
		CastleSiegeManager.load();

		FortressManager.getInstance();
		FortressSiegeManager.load();

		ClanHallManager.getInstance();
		ClanHallSiegeManager.load();

		TerritorySiege.load();

		CastleManorManager.getInstance();

		SpawnTable.getInstance();

		RaidBossSpawnManager.getInstance();

		DimensionalRiftManager.getInstance();

		InstancedZoneManager.getInstance();

		Announcements.getInstance();

		LotteryManager.getInstance();

		MapRegion.getInstance();

		AugmentationData.getInstance();

		PlayerMessageStack.getInstance();

		if(Config.AUTODESTROY_ITEM_AFTER > 0)
			ItemsAutoDestroy.getInstance();

		MonsterRace.getInstance();

		StaticObjectsTable.getInstance();

		SevenSigns _sevenSignsEngine = SevenSigns.getInstance();
		SevenSignsFestival.getInstance();
		_sevenSignsEngine.updateFestivalScore();

		AutoSpawnHandler _autoSpawnHandler = AutoSpawnHandler.getInstance();
		_log.config("AutoSpawnHandler: Loaded " + _autoSpawnHandler.size() + " handlers in total.");

		AutoChatHandler _autoChatHandler = AutoChatHandler.getInstance();
		_log.config("AutoChatHandler: Loaded " + _autoChatHandler.size() + " handlers in total.");

		_sevenSignsEngine.spawnSevenSignsNPC();

		if(Config.ENABLE_OLYMPIAD)
		{
			Olympiad.load();
			Hero.getInstance();
		}

		CursedWeaponsManager.getInstance();
		
		HellboundManager.getInstance(); 		

		if(!Config.ALLOW_WEDDING)
		{
			CoupleManager.getInstance();
			_log.config("CoupleManager initialized");
		}

		ItemHandler _itemHandler = ItemHandler.getInstance();
		_log.config("ItemHandler: Loaded " + _itemHandler.size() + " handlers.");

		AdminCommandHandler _adminCommandHandler = AdminCommandHandler.getInstance();
		_log.config("AdminCommandHandler: Loaded " + _adminCommandHandler.size() + " handlers.");

		UserCommandHandler _userCommandHandler = UserCommandHandler.getInstance();
		_log.config("UserCommandHandler: Loaded " + _userCommandHandler.size() + " handlers.");

		VoicedCommandHandler _voicedCommandHandler = VoicedCommandHandler.getInstance();
		_log.config("VoicedCommandHandler: Loaded " + _voicedCommandHandler.size() + " handlers.");

		TaskManager.getInstance();

		MercTicketManager.getInstance();

		L2VehicleManager.getInstance();
		AirShipDocksTable.getInstance();
		
		KrateisCubeManager.getInstance().init();
		GameHandyManager.getInstance();
		
		Shutdown _shutdownHandler = Shutdown.getInstance();
		Runtime.getRuntime().addShutdownHook(_shutdownHandler);

		try
		{
			// Colosseum doors
			DoorTable.getInstance().getDoor(24190001).openMe();
			DoorTable.getInstance().getDoor(24190002).openMe();
			DoorTable.getInstance().getDoor(24190003).openMe();
			DoorTable.getInstance().getDoor(24190004).openMe();

			// TOI doors
			DoorTable.getInstance().getDoor(23180001).openMe();
			DoorTable.getInstance().getDoor(23180002).openMe();
			DoorTable.getInstance().getDoor(23180003).openMe();
			DoorTable.getInstance().getDoor(23180004).openMe();
			DoorTable.getInstance().getDoor(23180005).openMe();
			DoorTable.getInstance().getDoor(23180006).openMe();

			// Эти двери, похоже выполняют декоративную функцию,
			// находятся во Frozen Labyrinth над мостом по пути к снежной королеве.
			DoorTable.getInstance().getDoor(23140001).openMe();
			DoorTable.getInstance().getDoor(23140002).openMe();

			DoorTable.getInstance().checkAutoOpen();
		}
		catch(NullPointerException e)
		{
			_log.warning("Doors table does not contain the right door info. Update doors.");
			e.printStackTrace();
		}

		_log.config("IdFactory: Free ObjectID's remaining: " + IdFactory.getInstance().size());

		TeleportTable.getInstance();

		PartyRoomManager.getInstance();
        ItemMall.getInstance();

		new File("./log/game").mkdirs();

		int restartTime = 0;
		int restartAt = 0;

		// Время запланированного на определенное время суток рестарта
		if(Config.RESTART_AT_TIME > -1)
		{
			Calendar calendarRestartAt = Calendar.getInstance();
			calendarRestartAt.set(Calendar.HOUR_OF_DAY, Config.RESTART_AT_TIME);
			calendarRestartAt.set(Calendar.MINUTE, 0);

			// Если запланированное время уже прошло, то берем +24 часа
			if(calendarRestartAt.getTimeInMillis() < System.currentTimeMillis())
				calendarRestartAt.add(Calendar.HOUR_OF_DAY, 24);

			restartAt = (int) (calendarRestartAt.getTimeInMillis() - System.currentTimeMillis()) / 1000;
		}

		// Время регулярного рестарта (через определенное время)
		restartTime = Config.RESTART_TIME * 60 * 60;

		// Проверяем какой рестарт раньше, регулярный или запланированный
		if(restartTime < restartAt && restartTime > 0 || restartTime > restartAt && restartAt == 0)
			Shutdown.getInstance().setAutoRestart(restartTime);
		else if(restartAt > 0)
			Shutdown.getInstance().setAutoRestart(restartAt);

		MailParcelController.getInstance();

		MMOTopManager.getInstance();
		L2TopManager.getInstance();
		ItemsForOnline.getInstance();
		if(Config.show_team)
		{
			RewardForOnline.getInstance();
		}
		DelaySpawnManager.getInstance().spawnDelayMobs();
		_log.info("GameServer Started");
		FirstTeam.info();
		//Проверка лицензии
		//LicenseManager.getInstance();
		_log.config("Maximum Numbers of Connected Players: " + Config.MAXIMUM_ONLINE_USERS);

		Stat.init();
		

		if(Config.RRD_ENABLED)
			RRDTools.init();

		if(Config.PROTECT_ENABLE && Config.PROTECT_GS_ENABLE_HWID_BANS)
			HWID.reloadBannedHWIDs();

		if(Config.PROTECT_ENABLE && Config.PROTECT_GS_ENABLE_HWID_BONUS)
			HWID.reloadBonusHWIDs();

		MMOSocket.getInstance();
		if (Config.HOW_LONG_SLEEP > 0)
		{
			GameServer._log.info("Sleeping on " + Config.HOW_LONG_SLEEP +" seconds");
			Thread.sleep(Config.HOW_LONG_SLEEP*1000);
		}
		LSConnection.getInstance().start();

		SelectorThread.setAntiFlood(Config.ANTIFLOOD_ENABLE);
		SelectorThread.setAntiFloodSocketsConf(Config.MAX_UNHANDLED_SOCKETS_PER_IP, Config.UNHANDLED_SOCKET_MIN_TTL);
		L2GamePacketHandler gph = new L2GamePacketHandler();
		SelectorConfig<L2GameClient> sc = new SelectorConfig<L2GameClient>(gph);
		sc.setMaxSendPerPass(30);
		sc.setSelectorSleepTime(1);
		SelectorThread.setGlobalReadLock(Config.PORTS_GAME.length > 1);
		_selectorThreads = new SelectorThread[Config.PORTS_GAME.length];
		for(int i = 0; i < Config.PORTS_GAME.length; i++)
		{
			_selectorThreads[i] = new SelectorThread<L2GameClient>(sc, gph, gph, gph, null);
			_selectorThreads[i].openServerSocket(null, Config.PORTS_GAME[i]);
			_selectorThreads[i].start();
		}
		
		_log.info("Load time: "+ Integer.toString(time() - _serverStarted)+" second.");
		
		if(Config.SERVICES_OFFLINE_TRADE_RESTORE_AFTER_RESTART)
			// Это довольно тяжелая задача поэтому пусть идет отдельным тридом
			new Thread(new Runnable(){
				@Override
				public void run()
				{
					if(Config.SERVICES_OFFLINE_TRADE_SECONDS_TO_KICK > 0)
					{
						int min_offline_restore = (int) (System.currentTimeMillis() / 1000 - Config.SERVICES_OFFLINE_TRADE_SECONDS_TO_KICK);
						mysql.set("DELETE FROM character_variables WHERE `name` = 'offline' AND `value` < " + min_offline_restore);
					}
					mysql.set("DELETE FROM character_variables WHERE `name` = 'offline' AND `obj_id` IN (SELECT `obj_id` FROM `characters` WHERE `accessLevel` < 0)");

					ThreadConnection con = null;
					FiltredPreparedStatement st = null;
					ResultSet rs = null;

					try
					{
						GArray<Object> logins = mysql.get_array(Config.ACCOUNTS_DATABASE.instance, "SELECT `login` FROM `accounts` WHERE `access_level` < 0");
						if(logins.size() > 0)
						{
							con = L2DatabaseFactory.getInstance().getConnection();
							st = con.prepareStatement("DELETE FROM character_variables WHERE `name` = 'offline' AND `obj_id` IN (SELECT `obj_id` FROM `characters` WHERE `account_name`=?)");
							for(Object login : logins)
							{
								st.setString(1, (String) login);
								st.executeUpdate();
							}
						}
					}
					catch(Exception e)
					{
						e.printStackTrace();
					}
					finally
					{
						DatabaseUtils.closeDatabaseCSR(con, st, rs);
					}

					GArray<HashMap<String, Object>> list = mysql.getAll("SELECT `obj_id`, `value`, (SELECT `account_name` FROM `characters` WHERE `characters`.`obj_Id` = `character_variables`.`obj_id` LIMIT 1) AS `account_name` FROM `character_variables` WHERE name LIKE 'offline'");
					for(HashMap<String, Object> e : list)
					{
						L2GameClient client = new L2GameClient(new MMOConnection<L2GameClient>(null), true);
						client.setCharSelection((Integer) e.get("obj_id"));
						L2Player p = client.loadCharFromDisk(0);
						if(p == null || p.isDead())
							continue;
						client.setLoginName((String) e.get("account_name") == null ? "OfflineTrader_" + p.getName() : (String) e.get("account_name"));
						client.OnOfflineTrade();
						p.restoreBonus();
						p.spawnMe();
						p.updateTerritories();
						p.setOnlineStatus(true);
						p.setOfflineMode(true);
						p.setConnected(false);
						p.setNameColor(Config.SERVICES_OFFLINE_TRADE_NAME_COLOR);
						p.restoreEffects();
						p.restoreDisableSkills();
						p.broadcastUserInfo(true);
						if(p.getClan() != null && p.getClan().getClanMember(p.getObjectId()) != null)
							p.getClan().getClanMember(p.getObjectId()).setPlayerInstance(p);
						if(Config.SERVICES_OFFLINE_TRADE_SECONDS_TO_KICK > 0)
							p.startKickTask((Config.SERVICES_OFFLINE_TRADE_SECONDS_TO_KICK + Integer.parseInt(e.get("value").toString())) * 1000L - System.currentTimeMillis());
					}
					_log.info("Restored " + list.size() + " offline traders");
				}
			}).start();
	}

	public static boolean isLoaded()
	{
		return serverLoaded;
	}
}