package l2ft.gameserver.cache;

import l2ft.gameserver.serverpackets.ActionFail;
import l2ft.gameserver.serverpackets.CharacterCreateFail;
import l2ft.gameserver.serverpackets.CharacterCreateSuccess;
import l2ft.gameserver.serverpackets.ClientSetTime;
import l2ft.gameserver.serverpackets.ExDominionWarEnd;
import l2ft.gameserver.serverpackets.ExMPCCClose;
import l2ft.gameserver.serverpackets.ExMPCCOpen;
import l2ft.gameserver.serverpackets.ExNotifyBirthDay;
import l2ft.gameserver.serverpackets.ExRestartClient;
import l2ft.gameserver.serverpackets.ExShowQuestInfo;
import l2ft.gameserver.serverpackets.ExShowVariationCancelWindow;
import l2ft.gameserver.serverpackets.ExShowVariationMakeWindow;
import l2ft.gameserver.serverpackets.GameGuardQuery;
import l2ft.gameserver.serverpackets.L2GameServerPacket;
import l2ft.gameserver.serverpackets.LeaveWorld;
import l2ft.gameserver.serverpackets.PartySmallWindowDeleteAll;
import l2ft.gameserver.serverpackets.PledgeShowMemberListDeleteAll;
import l2ft.gameserver.serverpackets.ServerClose;
import l2ft.gameserver.serverpackets.SunRise;
import l2ft.gameserver.serverpackets.SunSet;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.serverpackets.TradePressOtherOk;
import l2ft.gameserver.serverpackets.TutorialCloseHtml;

public abstract class Msg
{
	public static final SystemMessage YOU_CAREFULLY_NOCK_AN_ARROW = new SystemMessage(SystemMessage.YOU_CAREFULLY_NOCK_AN_ARROW);
	public static final SystemMessage PETS_CRITICAL_HIT = new SystemMessage(SystemMessage.PETS_CRITICAL_HIT);
	public static final SystemMessage THE_PET_IS_TOO_HIGH_LEVEL_TO_CONTROL = new SystemMessage(SystemMessage.THE_PET_IS_TOO_HIGH_LEVEL_TO_CONTROL);
	public static final SystemMessage SUMMONED_MONSTERS_CRITICAL_HIT = new SystemMessage(SystemMessage.SUMMONED_MONSTERS_CRITICAL_HIT);
	public static final SystemMessage LETHAL_STRIKE = new SystemMessage(SystemMessage.LETHAL_STRIKE);
	public static final SystemMessage YOUR_LETHAL_STRIKE_WAS_SUCCESSFUL = new SystemMessage(SystemMessage.YOUR_LETHAL_STRIKE_WAS_SUCCESSFUL);
	public static final SystemMessage YOUR_SHIELD_DEFENSE_HAS_SUCCEEDED = new SystemMessage(SystemMessage.YOUR_SHIELD_DEFENSE_HAS_SUCCEEDED);
	public static final SystemMessage YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS = new SystemMessage(SystemMessage.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
	public static final SystemMessage CANNOT_SEE_TARGET = new SystemMessage(SystemMessage.CANNOT_SEE_TARGET);
	public static final SystemMessage THAT_IS_THE_INCORRECT_TARGET = new SystemMessage(SystemMessage.THAT_IS_THE_INCORRECT_TARGET);
	public static final SystemMessage THE_TARGET_IS_UNAVAILABLE_FOR_SEEDING = new SystemMessage(SystemMessage.THE_TARGET_IS_UNAVAILABLE_FOR_SEEDING);
	public static final SystemMessage THE_SEED_HAS_BEEN_SOWN = new SystemMessage(SystemMessage.THE_SEED_HAS_BEEN_SOWN);
	public static final SystemMessage INCORRECT_ITEM_COUNT = new SystemMessage(SystemMessage.INCORRECT_ITEM_COUNT);
	public static final SystemMessage THIS_SEED_MAY_NOT_BE_SOWN_HERE = new SystemMessage(SystemMessage.THIS_SEED_MAY_NOT_BE_SOWN_HERE);
	public static final SystemMessage THE_HARVEST_HAS_FAILED = new SystemMessage(SystemMessage.THE_HARVEST_HAS_FAILED);
	public static final SystemMessage THE_HARVEST_FAILED_BECAUSE_THE_SEED_WAS_NOT_SOWN = new SystemMessage(SystemMessage.THE_HARVEST_FAILED_BECAUSE_THE_SEED_WAS_NOT_SOWN);
	public static final SystemMessage INVALID_TARGET = new SystemMessage(SystemMessage.INVALID_TARGET);
	public static final SystemMessage YOUR_TARGET_IS_OUT_OF_RANGE = new SystemMessage(SystemMessage.YOUR_TARGET_IS_OUT_OF_RANGE);
	public static final SystemMessage THERE_IS_NOT_ENOUGH_SPACE_TO_MOVE_THE_SKILL_CANNOT_BE_USED = new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_SPACE_TO_MOVE_THE_SKILL_CANNOT_BE_USED);
	public static final SystemMessage YOU_CANNOT_MOVE_WHILE_SITTING = new SystemMessage(SystemMessage.YOU_CANNOT_MOVE_WHILE_SITTING);
	public static final SystemMessage YOU_CANNOT_TRANSFORM_WHILE_SITTING = new SystemMessage(SystemMessage.YOU_CANNOT_TRANSFORM_WHILE_SITTING);
	public static final SystemMessage YOU_MAY_NOT_ATTACK_IN_A_PEACEFUL_ZONE = new SystemMessage(SystemMessage.YOU_MAY_NOT_ATTACK_IN_A_PEACEFUL_ZONE);
	public static final SystemMessage ONLY_LEVEL_SUP_10_CAN_RECOMMEND = new SystemMessage(SystemMessage.ONLY_CHARACTERS_OF_LEVEL_10_OR_ABOVE_ARE_AUTHORIZED_TO_MAKE_RECOMMENDATIONS);
	public static final SystemMessage YOU_CANNOT_RECOMMEND_YOURSELF = new SystemMessage(SystemMessage.YOU_CANNOT_RECOMMEND_YOURSELF);
	public static final SystemMessage NO_MORE_RECOMMENDATIONS_TO_HAVE = new SystemMessage(SystemMessage.YOU_ARE_NOT_AUTHORIZED_TO_MAKE_FURTHER_RECOMMENDATIONS_AT_THIS_TIME_YOU_WILL_RECEIVE_MORE_RECOMMENDATION_CREDITS_EACH_DAY_AT_1_PM);
	public static final SystemMessage YOU_NO_LONGER_RECIVE_A_RECOMMENDATION = new SystemMessage(SystemMessage.YOUR_SELECTED_TARGET_CAN_NO_LONGER_RECEIVE_A_RECOMMENDATION);
	public static final SystemMessage THAT_CHARACTER_IS_RECOMMENDED = new SystemMessage(SystemMessage.THAT_CHARACTER_HAS_ALREADY_BEEN_RECOMMENDED);
	public static final SystemMessage NOT_WORKING_PLEASE_TRY_AGAIN_LATER = new SystemMessage(SystemMessage.NOT_WORKING_PLEASE_TRY_AGAIN_LATER);
	public static final SystemMessage YOU_MAY_NOT_ATTACK_THIS_TARGET_IN_A_PEACEFUL_ZONE = new SystemMessage(SystemMessage.YOU_MAY_NOT_ATTACK_THIS_TARGET_IN_A_PEACEFUL_ZONE);
	public static final SystemMessage ONLY_FISHING_SKILLS_ARE_AVAILABLE = new SystemMessage(SystemMessage.ONLY_FISHING_SKILLS_ARE_AVAILABLE);
	public static final SystemMessage YOU_CANNOT_SUMMON_A_BASE_BECAUSE_YOU_ARE_NOT_IN_BATTLE = new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_A_BASE_BECAUSE_YOU_ARE_NOT_IN_BATTLE);
	public static final SystemMessage THIS_ITEM_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT = new SystemMessage(SystemMessage.THIS_ITEM_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT);
	public static final SystemMessage NOT_ENOUGH_HP = new SystemMessage(SystemMessage.NOT_ENOUGH_HP);
	public static final SystemMessage NOT_ENOUGH_MP = new SystemMessage(SystemMessage.NOT_ENOUGH_MP);
	public static final SystemMessage YOU_HAVE_RUN_OUT_OF_ARROWS = new SystemMessage(SystemMessage.YOU_HAVE_RUN_OUT_OF_ARROWS);
	public static final SystemMessage NOT_ENOUGH_BOLTS = new SystemMessage(SystemMessage.NOT_ENOUGH_BOLTS);
	public static final SystemMessage THE_RECIPE_IS_INCORRECT = new SystemMessage(SystemMessage.THE_RECIPE_IS_INCORRECT);
	public static final SystemMessage PLEASE_REGISTER_A_RECIPE = new SystemMessage(SystemMessage.PLEASE_REGISTER_A_RECIPE);
	public static final SystemMessage YOU_DO_NOT_HAVE_ENOUGH_ADENA = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
	public static final SystemMessage YOUR_INVENTORY_IS_FULL = new SystemMessage(SystemMessage.YOUR_INVENTORY_IS_FULL);
	public static final SystemMessage YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT = new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT);
	public static final SystemMessage NOT_ENOUGH_MATERIALS = new SystemMessage(SystemMessage.NOT_ENOUGH_MATERIALS);
	public static final SystemMessage YOUR_PROTOCOL_VERSION_IS_DIFFERENT_PLEASE_RESTART_YOUR_CLIENT_AND_RUN_A_FULL_CHECK = new SystemMessage(SystemMessage.YOUR_PROTOCOL_VERSION_IS_DIFFERENT_PLEASE_RESTART_YOUR_CLIENT_AND_RUN_A_FULL_CHECK);
	public static final SystemMessage PROGRESS_IN_A_QUEST_IS_POSSIBLE_ONLY_WHEN_YOUR_INVENTORYS_WEIGHT_AND_VOLUME_ARE_LESS_THAN_80_PERCENT_OF_CAPACITY = new SystemMessage(SystemMessage.PROGRESS_IN_A_QUEST_IS_POSSIBLE_ONLY_WHEN_YOUR_INVENTORYS_WEIGHT_AND_VOLUME_ARE_LESS_THAN_80_PERCENT_OF_CAPACITY);
	public static final SystemMessage NO_FURTHER_RECIPES_MAY_BE_REGISTERED = new SystemMessage(SystemMessage.NO_FURTHER_RECIPES_MAY_BE_REGISTERED);
	public static final SystemMessage CREATE_ITEM_LEVEL_IS_TOO_LOW_TO_REGISTER_THIS_RECIPE = new SystemMessage(SystemMessage.CREATE_ITEM_LEVEL_IS_TOO_LOW_TO_REGISTER_THIS_RECIPE);
	public static final SystemMessage THAT_RECIPE_IS_ALREADY_REGISTERED = new SystemMessage(SystemMessage.THAT_RECIPE_IS_ALREADY_REGISTERED);
	public static final SystemMessage YOU_ARE_NOT_AUTHORIZED_TO_REGISTER_A_RECIPE = new SystemMessage(SystemMessage.YOU_ARE_NOT_AUTHORIZED_TO_REGISTER_A_RECIPE);
	public static final SystemMessage ITEMS_LEFT_AT_THE_CLAN_HALL_WAREHOUSE_CAN_ONLY_BE_RETRIEVED_BY_THE_CLAN_LEADER_DO_YOU_WANT_TO_CONTINUE = new SystemMessage(SystemMessage.ITEMS_LEFT_AT_THE_CLAN_HALL_WAREHOUSE_CAN_ONLY_BE_RETRIEVED_BY_THE_CLAN_LEADER_DO_YOU_WANT_TO_CONTINUE);
	public static final SystemMessage YOU_ARE_NOT_AUTHORIZED_TO_HARVEST = new SystemMessage(SystemMessage.YOU_ARE_NOT_AUTHORIZED_TO_HARVEST);
	public static final SystemMessage THERE_ARE_NO_PRIORITY_RIGHTS_ON_A_SWEEPER = new SystemMessage(SystemMessage.THERE_ARE_NO_PRIORITY_RIGHTS_ON_A_SWEEPER);
	public static final SystemMessage THE_MANOR_SYSTEM_IS_CURRENTLY_UNDER_MAINTENANCE = new SystemMessage(SystemMessage.THE_MANOR_SYSTEM_IS_CURRENTLY_UNDER_MAINTENANCE);
	public static final SystemMessage WAREHOUSE_IS_TOO_FAR = new SystemMessage(SystemMessage.WAREHOUSE_IS_TOO_FAR);
	public static final SystemMessage YOUR_WAREHOUSE_IS_FULL = new SystemMessage(SystemMessage.YOUR_WAREHOUSE_IS_FULL);
	public static final SystemMessage YOU_LACK_THE_FUNDS_NEEDED_TO_PAY_FOR_THIS_TRANSACTION = new SystemMessage(SystemMessage.YOU_LACK_THE_FUNDS_NEEDED_TO_PAY_FOR_THIS_TRANSACTION);
	public static final SystemMessage THAT_PLAYER_IS_NOT_CURRENTLY_ONLINE = new SystemMessage(SystemMessage.THAT_PLAYER_IS_NOT_CURRENTLY_ONLINE);
	public static final SystemMessage ONLY_THE_CREATOR_OF_A_CHANNEL_CAN_ISSUE_A_GLOBAL_COMMAND = new SystemMessage(SystemMessage.ONLY_THE_CREATOR_OF_A_CHANNEL_CAN_ISSUE_A_GLOBAL_COMMAND);
	public static final SystemMessage YOU_WERE_DISMISSED_FROM_THE_COMMAND_CHANNEL = new SystemMessage(SystemMessage.YOU_WERE_DISMISSED_FROM_THE_COMMAND_CHANNEL);
	public static final SystemMessage CONGRATULATIONS_YOU_WILL_NOW_GRADUATE_FROM_THE_CLAN_ACADEMY_AND_LEAVE_YOUR_CURRENT_CLAN_AS_A_GRADUATE_OF_THE_ACADEMY_YOU_CAN_IMMEDIATELY_JOIN_A_CLAN_AS_A_REGULAR_MEMBER_WITHOUT_BEING_SUBJECT_TO_ANY_PENALTIES = new SystemMessage(SystemMessage.CONGRATULATIONS_YOU_WILL_NOW_GRADUATE_FROM_THE_CLAN_ACADEMY_AND_LEAVE_YOUR_CURRENT_CLAN_AS_A_GRADUATE_OF_THE_ACADEMY_YOU_CAN_IMMEDIATELY_JOIN_A_CLAN_AS_A_REGULAR_MEMBER_WITHOUT_BEING_SUBJECT_TO_ANY_PENALTIES);
	public static final SystemMessage SINCE_THE_CLAN_REPUTATION_SCORE_HAS_DROPPED_TO_0_OR_LOWER_YOUR_CLAN_SKILLS_WILL_BE_DE_ACTIVATED = new SystemMessage(SystemMessage.SINCE_THE_CLAN_REPUTATION_SCORE_HAS_DROPPED_TO_0_OR_LOWER_YOUR_CLAN_SKILLS_WILL_BE_DE_ACTIVATED);
	public static final SystemMessage THE_CLAN_SKILL_WILL_BE_ACTIVATED_BECAUSE_THE_CLANS_REPUTATION_SCORE_HAS_REACHED_TO_0_OR_HIGHER = new SystemMessage(SystemMessage.THE_CLAN_SKILL_WILL_BE_ACTIVATED_BECAUSE_THE_CLANS_REPUTATION_SCORE_HAS_REACHED_TO_0_OR_HIGHER);
	public static final SystemMessage YOUR_CLAN_HAS_ALREADY_ESTABLISHED_A_CLAN_ACADEMY = new SystemMessage(SystemMessage.YOUR_CLAN_HAS_ALREADY_ESTABLISHED_A_CLAN_ACADEMY);
	public static final SystemMessage THE_CLAN_REPUTATION_SCORE_IS_TOO_LOW = new SystemMessage(SystemMessage.THE_CLAN_REPUTATION_SCORE_IS_TOO_LOW);
	public static final SystemMessage YOU_HAVE_INCREASED_YOUR_LEVEL = new SystemMessage(SystemMessage.YOUR_LEVEL_HAS_INCREASED);
	public static final SystemMessage NO_SLOT_EXISTS_TO_DRAW_THE_SYMBOL = new SystemMessage(SystemMessage.NO_SLOT_EXISTS_TO_DRAW_THE_SYMBOL);
	public static final SystemMessage BETTER_RESURRECTION_HAS_BEEN_ALREADY_PROPOSED = new SystemMessage(SystemMessage.BETTER_RESURRECTION_HAS_BEEN_ALREADY_PROPOSED);
	public static final SystemMessage SINCE_THE_MASTER_WAS_IN_THE_PROCESS_OF_BEING_RESURRECTED_THE_ATTEMPT_TO_RESURRECT_THE_PET_HAS_BEEN_CANCELLED = new SystemMessage(SystemMessage.SINCE_THE_MASTER_WAS_IN_THE_PROCESS_OF_BEING_RESURRECTED_THE_ATTEMPT_TO_RESURRECT_THE_PET_HAS_BEEN_CANCELLED);
	public static final SystemMessage WHILE_A_PET_IS_ATTEMPTING_TO_RESURRECT_IT_CANNOT_HELP_IN_RESURRECTING_ITS_MASTER = new SystemMessage(SystemMessage.WHILE_A_PET_IS_ATTEMPTING_TO_RESURRECT_IT_CANNOT_HELP_IN_RESURRECTING_ITS_MASTER);
	public static final SystemMessage IT_IS_IMPOSSIBLE_TO_BE_RESSURECTED_IN_BATTLEFIELDS_WHERE_SIEGE_WARS_ARE_IN_PROCESS = new SystemMessage(SystemMessage.IT_IS_IMPOSSIBLE_TO_BE_RESSURECTED_IN_BATTLEFIELDS_WHERE_SIEGE_WARS_ARE_IN_PROCESS);
	public static final SystemMessage IF_A_BASE_CAMP_DOES_NOT_EXIST_RESURRECTION_IS_NOT_POSSIBLE = new SystemMessage(SystemMessage.IF_A_BASE_CAMP_DOES_NOT_EXIST_RESURRECTION_IS_NOT_POSSIBLE);
	public static final SystemMessage THE_GUARDIAN_TOWER_HAS_BEEN_DESTROYED_AND_RESURRECTION_IS_NOT_POSSIBLE = new SystemMessage(SystemMessage.THE_GUARDIAN_TOWER_HAS_BEEN_DESTROYED_AND_RESURRECTION_IS_NOT_POSSIBLE);
	public static final SystemMessage BAITS_HAVE_BEEN_LOST_BECAUSE_THE_FISH_GOT_AWAY = new SystemMessage(SystemMessage.BAITS_HAVE_BEEN_LOST_BECAUSE_THE_FISH_GOT_AWAY);
	public static final SystemMessage ENDS_FISHING = new SystemMessage(SystemMessage.ENDS_FISHING);
	public static final SystemMessage SOUL_CANNOT_BE_ABSORBED_ANY_MORE = new SystemMessage(SystemMessage.SOUL_CANNOT_BE_ABSORBED_ANY_MORE);
	public static final SystemMessage CASTING_HAS_BEEN_INTERRUPTED = new SystemMessage(SystemMessage.YOUR_CASTING_HAS_BEEN_INTERRUPTED);
	public static final SystemMessage YOU_HAVE_BEEN_PLAYING_FOR_AN_EXTENDED_PERIOD_OF_TIME_PLEASE_CONSIDER_TAKING_A_BREAK = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_PLAYING_FOR_AN_EXTENDED_PERIOD_OF_TIME_PLEASE_CONSIDER_TAKING_A_BREAK);
	public static final SystemMessage ONLY_THE_LEADER_CAN_GIVE_OUT_INVITATIONS = new SystemMessage(SystemMessage.ONLY_THE_LEADER_CAN_GIVE_OUT_INVITATIONS);
	public static final SystemMessage WAITING_FOR_ANOTHER_REPLY = new SystemMessage(SystemMessage.WAITING_FOR_ANOTHER_REPLY);
	public static final SystemMessage THE_PERSON_IS_IN_A_MESSAGE_REFUSAL_MODE = new SystemMessage(SystemMessage.THE_PERSON_IS_IN_A_MESSAGE_REFUSAL_MODE);
	public static final SystemMessage THE_USER_WHO_REQUESTED_TO_BECOME_FRIENDS_IS_NOT_FOUND_IN_THE_GAME = new SystemMessage(SystemMessage.THE_USER_WHO_REQUESTED_TO_BECOME_FRIENDS_IS_NOT_FOUND_IN_THE_GAME);
	public static final SystemMessage YOU_CANNOT_ADD_YOURSELF_TO_YOUR_OWN_FRIEND_LIST = new SystemMessage(SystemMessage.YOU_CANNOT_ADD_YOURSELF_TO_YOUR_OWN_FRIEND_LIST);
	public static final SystemMessage WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM = new SystemMessage(SystemMessage.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM);
	public static final SystemMessage NOTHING_HAPPENED = new SystemMessage(SystemMessage.NOTHING_HAPPENED);
	public static final SystemMessage YOU_CANNOT_DO_THAT_WHILE_FISHING = new SystemMessage(SystemMessage.YOU_CANNOT_DO_THAT_WHILE_FISHING);
	public static final SystemMessage THAT_ITEM_CANNOT_BE_DISCARDED = new SystemMessage(SystemMessage.THAT_ITEM_CANNOT_BE_DISCARDED);
	public static final SystemMessage THIS_ITEM_CANNOT_BE_DISCARDED = new SystemMessage(SystemMessage.THIS_ITEM_CANNOT_BE_DISCARDED);
	public static final SystemMessage THAT_IS_TOO_FAR_FROM_YOU_TO_DISCARD = new SystemMessage(SystemMessage.THAT_IS_TOO_FAR_FROM_YOU_TO_DISCARD);
	public static final SystemMessage HERO_WEAPONS_CANNOT_BE_DESTROYED = new SystemMessage(SystemMessage.HERO_WEAPONS_CANNOT_BE_DESTROYED);
	public static final SystemMessage YOU_CANNOT_DESTROY_IT_BECAUSE_THE_NUMBER_IS_INCORRECT = new SystemMessage(SystemMessage.YOU_CANNOT_DESTROY_IT_BECAUSE_THE_NUMBER_IS_INCORRECT);
	public static final SystemMessage THE_PET_HAS_BEEN_SUMMONED_AND_CANNOT_BE_DELETED = new SystemMessage(SystemMessage.THE_PET_HAS_BEEN_SUMMONED_AND_CANNOT_BE_DELETED);
	public static final SystemMessage THE_ITEM_HAS_BEEN_SUCCESSFULLY_CRYSTALLIZED = new SystemMessage(SystemMessage.THE_ITEM_HAS_BEEN_SUCCESSFULLY_CRYSTALLIZED);
	public static final SystemMessage THIS_ACCOUNT_CANOT_USE_PRIVATE_STORES = new SystemMessage(SystemMessage.THIS_ACCOUNT_CANOT_USE_PRIVATE_STORES);
	public static final SystemMessage A_PRIVATE_STORE_MAY_NOT_BE_OPENED_IN_THIS_AREA = new SystemMessage(SystemMessage.A_PRIVATE_STORE_MAY_NOT_BE_OPENED_IN_THIS_AREA);
	public static final SystemMessage A_PRIVATE_STORE_MAY_NOT_BE_OPENED_WHILE_USING_A_SKILL = new SystemMessage(SystemMessage.A_PRIVATE_STORE_MAY_NOT_BE_OPENED_WHILE_USING_A_SKILL);
	public static final SystemMessage WHILE_YOU_ARE_ENGAGED_IN_COMBAT_YOU_CANNOT_OPERATE_A_PRIVATE_STORE_OR_PRIVATE_WORKSHOP = new SystemMessage(SystemMessage.WHILE_YOU_ARE_ENGAGED_IN_COMBAT_YOU_CANNOT_OPERATE_A_PRIVATE_STORE_OR_PRIVATE_WORKSHOP);
	public static final SystemMessage A_DEAD_PET_CANNOT_BE_SENT_BACK = new SystemMessage(SystemMessage.A_DEAD_PET_CANNOT_BE_SENT_BACK);
	public static final SystemMessage A_PET_CANNOT_BE_SENT_BACK_DURING_BATTLE = new SystemMessage(SystemMessage.A_PET_CANNOT_BE_SENT_BACK_DURING_BATTLE);
	public static final SystemMessage YOU_CANNOT_RESTORE_HUNGRY_PETS = new SystemMessage(SystemMessage.YOU_CANNOT_RESTORE_HUNGRY_PETS);
	public static final SystemMessage A_DEAD_PET_CANNOT_BE_RIDDEN = new SystemMessage(SystemMessage.A_DEAD_PET_CANNOT_BE_RIDDEN);
	public static final SystemMessage YOU_ARE_NOT_ALLOWED_TO_DISMOUNT_AT_THIS_LOCATION = new SystemMessage(SystemMessage.YOU_ARE_NOT_ALLOWED_TO_DISMOUNT_AT_THIS_LOCATION);
	public static final SystemMessage THIS_ACCOUNT_CANOT_TRADE_ITEMS = new SystemMessage(SystemMessage.THIS_ACCOUNT_CANOT_TRADE_ITEMS);
	public static final SystemMessage YOU_HAVE_BEEN_BLOCKED_FROM_THE_CONTACT_YOU_SELECTED = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_BLOCKED_FROM_THE_CONTACT_YOU_SELECTED);
	public static final SystemMessage YOU_ARE_ALREADY_TRADING_WITH_SOMEONE = new SystemMessage(SystemMessage.YOU_ARE_ALREADY_TRADING_WITH_SOMEONE);
	public static final SystemMessage INAPPROPRIATE_ENCHANT_CONDITIONS = new SystemMessage(SystemMessage.INAPPROPRIATE_ENCHANT_CONDITIONS);
	public static final SystemMessage YOU_CANNOT_PRACTICE_ENCHANTING_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_MANUFACTURING_WORKSHOP = new SystemMessage(SystemMessage.YOU_CANNOT_PRACTICE_ENCHANTING_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_MANUFACTURING_WORKSHOP);
	public static final SystemMessage YOU_CANNOT_ADD_ELEMENTAL_POWER_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_WORKSHOP = new SystemMessage(SystemMessage.YOU_CANNOT_ADD_ELEMENTAL_POWER_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_WORKSHOP);
	public static final SystemMessage PLEASE_SELECT_ITEM_TO_ADD_ELEMENTAL_POWER = new SystemMessage(SystemMessage.PLEASE_SELECT_ITEM_TO_ADD_ELEMENTAL_POWER);
	public static final SystemMessage FAILED_IN_BLESSED_ENCHANT_THE_ENCHANT_VALUE_OF_THE_ITEM_BECAME_0 = new SystemMessage(SystemMessage.FAILED_IN_BLESSED_ENCHANT_THE_ENCHANT_VALUE_OF_THE_ITEM_BECAME_0);
	public static final SystemMessage YOUR_TITLE_CANNOT_EXCEED_16_CHARACTERS_IN_LENGHT = new SystemMessage(SystemMessage.YOUR_TITLE_CANNOT_EXCEED_16_CHARACTERS_IN_LENGTH_PLEASE_TRY_AGAIN);
	public static final SystemMessage TITLE_HAS_CHANGED = new SystemMessage(SystemMessage.TITLE_HAS_CHANGED);
	public static final SystemMessage TITLE_ENDOWMENT_IS_ONLY_POSSIBLE_WHEN_CLANS_SKILL_LEVELS_ARE_ABOVE_3 = new SystemMessage(SystemMessage.TITLE_ENDOWMENT_IS_ONLY_POSSIBLE_WHEN_CLANS_SKILL_LEVELS_ARE_ABOVE_3);
	public static final SystemMessage SYSTEM_ERROR = new SystemMessage(SystemMessage.SYSTEM_ERROR);
	public static final SystemMessage THIS_ITEM_CANNOT_BE_TRADED_OR_SOLD = new SystemMessage(SystemMessage.THIS_ITEM_CANNOT_BE_TRADED_OR_SOLD);
	public static final SystemMessage YOU_CANNOT_MOVE_ADDITIONAL_ITEMS_BECAUSE_TRADE_HAS_BEEN_CONFIRMED = new SystemMessage(SystemMessage.YOU_MAY_NO_LONGER_ADJUST_ITEMS_IN_THE_TRADE_BECAUSE_THE_TRADE_HAS_BEEN_CONFIRMED);
	public static final SystemMessage THAT_PLAYER_IS_NOT_ONLINE = new SystemMessage(SystemMessage.THAT_PLAYER_IS_NOT_ONLINE);
	public static final SystemMessage MAGIC_CRITICAL_HIT = new SystemMessage(SystemMessage.MAGIC_CRITICAL_HIT);
	public static final SystemMessage THERE_IS_NOT_ENOUGHT_SOUL = new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGHT_SOUL);
	public static final SystemMessage YOU_FAILED_AT_ITEM_MIXING = new SystemMessage(SystemMessage.YOU_FAILED_AT_ITEM_MIXING);
	public static final SystemMessage YOU_HAVE_GAINED_VITALITY_POINTS = new SystemMessage(SystemMessage.YOU_HAVE_GAINED_VITALITY_POINTS);
	public static final SystemMessage VITALITY_HAS_INCREASED = new SystemMessage(SystemMessage.VITALITY_HAS_INCREASED);
	public static final SystemMessage VITALITY_HAS_DECREASED = new SystemMessage(SystemMessage.VITALITY_HAS_DECREASED);
	public static final SystemMessage YOUR_VITALITY_IS_AT_MAXIMUM = new SystemMessage(SystemMessage.YOUR_VITALITY_IS_AT_MAXIMUM);
	public static final SystemMessage VITALITY_IS_FULLY_EXHAUSTED = new SystemMessage(SystemMessage.VITALITY_IS_FULLY_EXHAUSTED);
	public static final SystemMessage YOUR_TRADE_IS_SUCCESSFUL = new SystemMessage(SystemMessage.YOUR_TRADE_IS_SUCCESSFUL);
	public static final SystemMessage THE_ATTEMPT_TO_TRADE_HAS_FAILED = new SystemMessage(SystemMessage.THE_ATTEMPT_TO_TRADE_HAS_FAILED);
	public static final SystemMessage THE_SEED_WAS_SUCCESSFULLY_SOWN = new SystemMessage(SystemMessage.THE_SEED_WAS_SUCCESSFULLY_SOWN);
	public static final SystemMessage THE_SEED_WAS_NOT_SOWN = new SystemMessage(SystemMessage.THE_SEED_WAS_NOT_SOWN);
	public static final SystemMessage HALF_KILL = new SystemMessage(SystemMessage.HALF_KILL);
	public static final SystemMessage CP_DISAPPEARS_WHEN_HIT_WITH_A_HALF_KILL_SKILL = new SystemMessage(SystemMessage.CP_DISAPPEARS_WHEN_HIT_WITH_A_HALF_KILL_SKILL);
	public static final SystemMessage ITS_TOO_FAR_FROM_THE_NPC_TO_WORK = new SystemMessage(SystemMessage.ITS_TOO_FAR_FROM_THE_NPC_TO_WORK);
	public static final SystemMessage YOU_CANNOT_ENTER_DUE_TO_THE_PARTY_HAVING_EXCEEDED_THE_LIMIT = new SystemMessage(SystemMessage.YOU_CANNOT_ENTER_DUE_TO_THE_PARTY_HAVING_EXCEEDED_THE_LIMIT);
	public static final SystemMessage YOU_CANNOT_ENTER_BECAUSE_YOU_ARE_NOT_IN_A_CURRENT_COMMAND_CHANNEL = new SystemMessage(SystemMessage.YOU_CANNOT_ENTER_BECAUSE_YOU_ARE_NOT_IN_A_CURRENT_COMMAND_CHANNEL);
	public static final SystemMessage ONLY_A_PARTY_LEADER_CAN_TRY_TO_ENTER = new SystemMessage(SystemMessage.ONLY_A_PARTY_LEADER_CAN_TRY_TO_ENTER);
	public static final SystemMessage YOU_ARE_NOT_CURRENTLY_IN_A_PARTY_SO_YOU_CANNOT_ENTER = new SystemMessage(SystemMessage.YOU_ARE_NOT_CURRENTLY_IN_A_PARTY_SO_YOU_CANNOT_ENTER);
	public static final SystemMessage DOES_NOT_FIT_STRENGTHENING_CONDITIONS_OF_THE_SCROLL = new SystemMessage(SystemMessage.DOES_NOT_FIT_STRENGTHENING_CONDITIONS_OF_THE_SCROLL);
	public static final SystemMessage THIS_IS_AN_AREA_WHERE_YOU_CANNOT_USE_THE_MINI_MAP_THE_MINI_MAP_WILL_NOT_BE_OPENED = new SystemMessage(SystemMessage.THIS_IS_AN_AREA_WHERE_YOU_CANNOT_USE_THE_MINI_MAP_THE_MINI_MAP_WILL_NOT_BE_OPENED);
	public static final SystemMessage YOU_MAY_NOT_SUMMON_FROM_YOUR_CURRENT_LOCATION = new SystemMessage(SystemMessage.YOU_MAY_NOT_SUMMON_FROM_YOUR_CURRENT_LOCATION);
	public static final SystemMessage YOU_CANNOT_SUMMON_DURING_COMBAT = new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_DURING_COMBAT);
	public static final SystemMessage YOU_CANNOT_SUMMON_DURING_A_TRADE_OR_WHILE_USING_THE_PRIVATE_SHOPS = new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_DURING_A_TRADE_OR_WHILE_USING_THE_PRIVATE_SHOPS);
	public static final SystemMessage YOU_CANNOT_SUMMON_PLAYERS_WHO_ARE_CURRENTLY_PARTICIPATING_IN_THE_GRAND_OLYMPIAD = new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_PLAYERS_WHO_ARE_CURRENTLY_PARTICIPATING_IN_THE_GRAND_OLYMPIAD);
	public static final SystemMessage YOUR_TARGET_IS_IN_AN_AREA_WHICH_BLOCKS_SUMMONING = new SystemMessage(SystemMessage.YOUR_TARGET_IS_IN_AN_AREA_WHICH_BLOCKS_SUMMONING);
	public static final SystemMessage A_PARTY_CANNOT_BE_FORMED_IN_THIS_AREA = new SystemMessage(SystemMessage.A_PARTY_CANNOT_BE_FORMED_IN_THIS_AREA);
	public static final SystemMessage PARTY_IS_FULL = new SystemMessage(SystemMessage.PARTY_IS_FULL);
	public static final SystemMessage THE_PLAYER_DECLINED_TO_JOIN_YOUR_PARTY = new SystemMessage(SystemMessage.THE_PLAYER_DECLINED_TO_JOIN_YOUR_PARTY);
	public static final SystemMessage THERE_WAS_NOTHING_FOUND_INSIDE_OF_THAT = new SystemMessage(SystemMessage.THERE_WAS_NOTHING_FOUND_INSIDE_OF_THAT);

	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_TRANSFORMED = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_TRANSFORMED);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_PETRIFIED = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_PETRIFIED);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_DEAD = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_DEAD);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_FISHING = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_FISHING);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_IN_BATTLE = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_IN_BATTLE);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_IN_A_DUEL = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_IN_A_DUEL);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_SITTING = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_SITTING);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_SKILL_CASTING = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_SKILL_CASTING);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_A_CURSED_WEAPON_IS_EQUPPED = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_A_CURSED_WEAPON_IS_EQUPPED);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_HOLDING_A_FLAG = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_HOLDING_A_FLAG);
	public static final SystemMessage YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_A_PET_OR_A_SERVITOR_IS_SUMMONED = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_AN_AIRSHIP_WHILE_A_PET_OR_A_SERVITOR_IS_SUMMONED);
	public static final SystemMessage YOU_HAVE_ALREADY_BOARDED_ANOTHER_AIRSHIP = new SystemMessage(SystemMessage.YOU_HAVE_ALREADY_BOARDED_ANOTHER_AIRSHIP);
	public static final SystemMessage NO_MORE_AIRSHIPS_CAN_BE_SUMMONED_AS_THE_MAXIMUM_AIRSHIP_LIMIT_HAS_BEEN_MET = new SystemMessage(SystemMessage.NO_MORE_AIRSHIPS_CAN_BE_SUMMONED_AS_THE_MAXIMUM_AIRSHIP_LIMIT_HAS_BEEN_MET);
	public static final SystemMessage ANOTHER_AIRSHIP_HAS_ALREADY_BEEN_SUMMONED_AT_THE_WHARF_PLEASE_TRY_AGAIN_LATER = new SystemMessage(SystemMessage.ANOTHER_AIRSHIP_HAS_ALREADY_BEEN_SUMMONED_AT_THE_WHARF_PLEASE_TRY_AGAIN_LATER);
	public static final SystemMessage BOARDING_OR_CANCELLATION_OF_BOARDING_ON_AIRSHIPS_IS_NOT_ALLOWED_IN_THE_CURRENT_AREA = new SystemMessage(SystemMessage.BOARDING_OR_CANCELLATION_OF_BOARDING_ON_AIRSHIPS_IS_NOT_ALLOWED_IN_THE_CURRENT_AREA);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_TRANSFORMED = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_TRANSFORMED);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_YOU_ARE_PETRIFIED = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_YOU_ARE_PETRIFIED);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHEN_YOU_ARE_DEAD = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHEN_YOU_ARE_DEAD);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_FISHING = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_FISHING);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_IN_A_BATTLE = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_IN_A_BATTLE);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_IN_A_DUEL = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_IN_A_DUEL);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_IN_A_SITTING_POSITION = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_IN_A_SITTING_POSITION);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_USING_A_SKILL = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_USING_A_SKILL);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_A_CURSED_WEAPON_IS_EQUIPPED = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_A_CURSED_WEAPON_IS_EQUIPPED);
	public static final SystemMessage YOU_CANNOT_CONTROL_THE_TARGET_WHILE_HOLDING_A_FLAG = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_THE_TARGET_WHILE_HOLDING_A_FLAG);
	public static final SystemMessage YOU_CANNOT_CONTROL_BECAUSE_YOU_DO_NOT_MEET_THE_REQUIREMENTS = new SystemMessage(SystemMessage.YOU_CANNOT_CONTROL_BECAUSE_YOU_DO_NOT_MEET_THE_REQUIREMENTS);
	public static final SystemMessage IN_ORDER_TO_ACQUIRE_AN_AIRSHIP_THE_CLAN_S_LEVEL_MUST_BE_LEVEL_5_OR_HIGHER = new SystemMessage(SystemMessage.IN_ORDER_TO_ACQUIRE_AN_AIRSHIP_THE_CLAN_S_LEVEL_MUST_BE_LEVEL_5_OR_HIGHER);
	public static final SystemMessage AN_AIRSHIP_CANNOT_BE_SUMMONED_BECAUSE_EITHER_YOU_HAVE_NOT_REGISTERED_YOUR_AIRSHIP_LICENSE_OR_THE = new SystemMessage(SystemMessage.AN_AIRSHIP_CANNOT_BE_SUMMONED_BECAUSE_EITHER_YOU_HAVE_NOT_REGISTERED_YOUR_AIRSHIP_LICENSE_OR_THE);
	public static final SystemMessage THE_AIRSHIP_OWNED_BY_THE_CLAN_IS_ALREADY_BEING_USED_BY_ANOTHER_CLAN_MEMBER = new SystemMessage(SystemMessage.THE_AIRSHIP_OWNED_BY_THE_CLAN_IS_ALREADY_BEING_USED_BY_ANOTHER_CLAN_MEMBER);
	public static final SystemMessage THE_CLAN_OWNED_AIRSHIP_ALREADY_EXISTS = new SystemMessage(SystemMessage.THE_CLAN_OWNED_AIRSHIP_ALREADY_EXISTS);
	public static final SystemMessage THE_AIRSHIP_SUMMON_LICENSE_HAS_ALREADY_BEEN_ACQUIRED = new SystemMessage(SystemMessage.THE_AIRSHIP_SUMMON_LICENSE_HAS_ALREADY_BEEN_ACQUIRED);
	public static final SystemMessage THE_AIRSHIP_SUMMON_LICENSE_HAS_BEEN_ENTERED_YOUR_CLAN_CAN_NOW_SUMMON_THE_AIRSHIP = new SystemMessage(SystemMessage.THE_AIRSHIP_SUMMON_LICENSE_HAS_BEEN_ENTERED_YOUR_CLAN_CAN_NOW_SUMMON_THE_AIRSHIP);

	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_DURING_A_BATTLE = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_DURING_A_BATTLE);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_WHILE_PARTICIPATING_A_LARGE_SCALE_BATTLE_SUCH_AS_A_CASTLE_SIEGE = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_WHILE_PARTICIPATING_A_LARGE_SCALE_BATTLE_SUCH_AS_A_CASTLE_SIEGE);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_DURING_A_DUEL = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_DURING_A_DUEL);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_WHILE_FLYING = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_WHILE_FLYING);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_WHILE_PARTICIPATING_IN_AN_OLYMPIAD_MATCH = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_WHILE_PARTICIPATING_IN_AN_OLYMPIAD_MATCH);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_WHILE_YOU_ARE_IN_A_FLINT_OR_PARALYZED_STATE = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_WHILE_YOU_ARE_IN_A_FLINT_OR_PARALYZED_STATE);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_WHILE_YOU_ARE_DEAD = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_WHILE_YOU_ARE_DEAD);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_IN_THIS_AREA = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_IN_THIS_AREA);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_UNDERWATER = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_UNDERWATER);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_IN_AN_INSTANT_ZONE = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_IN_AN_INSTANT_ZONE);
	public static final SystemMessage YOU_HAVE_NO_SPACE_TO_SAVE_THE_TELEPORT_LOCATION = new SystemMessage(SystemMessage.YOU_HAVE_NO_SPACE_TO_SAVE_THE_TELEPORT_LOCATION);
	public static final SystemMessage YOU_CANNOT_TELEPORT_BECAUSE_YOU_DO_NOT_HAVE_A_TELEPORT_ITEM = new SystemMessage(SystemMessage.YOU_CANNOT_TELEPORT_BECAUSE_YOU_DO_NOT_HAVE_A_TELEPORT_ITEM);
	public static final SystemMessage THE_SAVED_TELEPORT_LOCATION_WILL_BE_DELETED_DO_YOU_WISH_TO_CONTINUE = new SystemMessage(SystemMessage.THE_SAVED_TELEPORT_LOCATION_WILL_BE_DELETED_DO_YOU_WISH_TO_CONTINUE);
	public static final SystemMessage THE_NUMBER_OF_MY_TELEPORTS_SLOTS_HAS_BEEN_INCREASED = new SystemMessage(SystemMessage.THE_NUMBER_OF_MY_TELEPORTS_SLOTS_HAS_BEEN_INCREASED);
	public static final SystemMessage YOU_CANNOT_USE_MY_TELEPORTS_TO_REACH_THIS_AREA = new SystemMessage(SystemMessage.YOU_CANNOT_USE_MY_TELEPORTS_TO_REACH_THIS_AREA);
	public static final SystemMessage YOU_CANNOT_BOOKMARK_THIS_LOCATION_BECAUSE_YOU_DO_NOT_HAVE_A_MY_TELEPORT_FLAG = new SystemMessage(SystemMessage.YOU_CANNOT_BOOKMARK_THIS_LOCATION_BECAUSE_YOU_DO_NOT_HAVE_A_MY_TELEPORT_FLAG);

	public static final SystemMessage THE_SOUL_CRYSTAL_WAS_NOT_ABLE_TO_ABSORB_A_SOUL = new SystemMessage(SystemMessage.THE_SOUL_CRYSTAL_WAS_NOT_ABLE_TO_ABSORB_A_SOUL);
	public static final SystemMessage WELCOME = new SystemMessage(SystemMessage.WELCOME_TO_THE_WORLD_OF_LINEAGE_II);
	public static final SystemMessage YOU_CANNOT_EXIT_WHILE_IN_COMBAT = new SystemMessage(SystemMessage.YOU_CANNOT_EXIT_WHILE_IN_COMBAT);
	public static final SystemMessage YOU_CANNOT_RESTART_WHILE_IN_COMBAT = new SystemMessage(SystemMessage.YOU_CANNOT_RESTART_WHILE_IN_COMBAT);
	public static final SystemMessage YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING = new SystemMessage(SystemMessage.YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING);
	public static final SystemMessage FIVE_YEARS_HAVE_PASSED_SINCE_THIS_CHARACTERS_CREATION = new SystemMessage(SystemMessage.FIVE_YEARS_HAVE_PASSED_SINCE_THIS_CHARACTERS_CREATION);
	public static final SystemMessage YOUR_BIRTHDAY_GIFT_HAS_ARRIVED_YOU_CAN_OBTAIN_IT_FROM_THE_GATEKEEPER_IN_ANY_VILLAGE = new SystemMessage(SystemMessage.YOUR_BIRTHDAY_GIFT_HAS_ARRIVED_YOU_CAN_OBTAIN_IT_FROM_THE_GATEKEEPER_IN_ANY_VILLAGE);
	public static final SystemMessage MESSAGE_REFUSAL_MODE = new SystemMessage(SystemMessage.MESSAGE_REFUSAL_MODE);
	public static final SystemMessage OBSERVERS_CANNOT_PARTICIPATE = new SystemMessage(SystemMessage.OBSERVERS_CANNOT_PARTICIPATE);
	public static final SystemMessage YOU_MAY_NOT_USE_ITEMS_IN_A_PRIVATE_STORE_OR_PRIVATE_WORK_SHOP = new SystemMessage(SystemMessage.YOU_MAY_NOT_USE_ITEMS_IN_A_PRIVATE_STORE_OR_PRIVATE_WORK_SHOP);
	public static final SystemMessage YOU_MAY_NOT_EQUIP_ITEMS_WHILE_CASTING_OR_PERFORMING_A_SKILL = new SystemMessage(SystemMessage.YOU_MAY_NOT_EQUIP_ITEMS_WHILE_CASTING_OR_PERFORMING_A_SKILL);
	public static final SystemMessage YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS);
	public static final SystemMessage YOU_DO_NOT_HAVE_ENOUGH_SP_TO_LEARN_SKILLS = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_SP_TO_LEARN_SKILLS);
	public static final SystemMessage ONLY_THE_CLAN_LEADER_IS_ENABLED = new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED);
	public static final SystemMessage YOU_HAVE_EXCEEDED_THE_QUANTITY_THAT_CAN_BE_INPUTTED = new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_QUANTITY_THAT_CAN_BE_INPUTTED);
	public static final SystemMessage AUGMENTATION_REMOVAL_CAN_ONLY_BE_DONE_ON_AN_AUGMENTED_ITEM = new SystemMessage(SystemMessage.AUGMENTATION_REMOVAL_CAN_ONLY_BE_DONE_ON_AN_AUGMENTED_ITEM);
	public static final SystemMessage FEATURE_AVAILABLE_TO_ALLIANCE_LEADERS_ONLY = new SystemMessage(SystemMessage.FEATURE_AVAILABLE_TO_ALLIANCE_LEADERS_ONLY);
	public static final SystemMessage YOU_ARE_NOT_CURRENTLY_ALLIED_WITH_ANY_CLANS = new SystemMessage(SystemMessage.YOU_ARE_NOT_CURRENTLY_ALLIED_WITH_ANY_CLANS);

	public static final SystemMessage YOUR_BID_PRICE_MUST_BE_HIGHER_THAN_THE_MINIMUM_PRICE_THAT_CAN_BE_BID = new SystemMessage(SystemMessage.YOUR_BID_PRICE_MUST_BE_HIGHER_THAN_THE_MINIMUM_PRICE_THAT_CAN_BE_BID);
	public static final SystemMessage YOUR_BID_CANNOT_EXCEED_100_BILLION = new SystemMessage(SystemMessage.YOUR_BID_CANNOT_EXCEED_100_BILLION);
	public static final SystemMessage YOUR_BID_MUST_BE_HIGHER_THAN_THE_CURRENT_HIGHEST_BID = new SystemMessage(SystemMessage.YOUR_BID_MUST_BE_HIGHER_THAN_THE_CURRENT_HIGHEST_BID);
	public static final SystemMessage THE_SECOND_BID_AMOUNT_MUST_BE_HIGHER_THAN_THE_ORIGINAL = new SystemMessage(SystemMessage.THE_SECOND_BID_AMOUNT_MUST_BE_HIGHER_THAN_THE_ORIGINAL);
	public static final SystemMessage YOU_CANNOT_PARTICIPATE_IN_AN_AUCTION = new SystemMessage(SystemMessage.YOU_CANNOT_PARTICIPATE_IN_AN_AUCTION);
	public static final SystemMessage BIDDER_EXISTS__THE_AUCTION_TIME_HAS_BEEN_EXTENDED_BY_5_MINUTES = new SystemMessage(SystemMessage.BIDDER_EXISTS__THE_AUCTION_TIME_HAS_BEEN_EXTENDED_BY_5_MINUTES);
	public static final SystemMessage BIDDER_EXISTS__AUCTION_TIME_HAS_BEEN_EXTENDED_BY_3_MINUTES = new SystemMessage(SystemMessage.BIDDER_EXISTS__AUCTION_TIME_HAS_BEEN_EXTENDED_BY_3_MINUTES);
	public static final SystemMessage YOU_DO_NOT_HAVE_ENOUGH_ADENA_FOR_THIS_BID = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ADENA_FOR_THIS_BID);
	public static final SystemMessage ONLY_A_CLAN_LEADER_WHOSE_CLAN_IS_OF_LEVEL_2_OR_HIGHER_IS_ALLOWED_TO_PARTICIPATE_IN_A_CLAN_HALL_AUCTION = new SystemMessage(SystemMessage.ONLY_A_CLAN_LEADER_WHOSE_CLAN_IS_OF_LEVEL_2_OR_HIGHER_IS_ALLOWED_TO_PARTICIPATE_IN_A_CLAN_HALL_AUCTION);
	public static final SystemMessage SINCE_YOU_HAVE_ALREADY_SUBMITTED_A_BID_YOU_ARE_NOT_ALLOWED_TO_PARTICIPATE_IN_ANOTHER_AUCTION_AT_THIS_TIME = new SystemMessage(SystemMessage.SINCE_YOU_HAVE_ALREADY_SUBMITTED_A_BID_YOU_ARE_NOT_ALLOWED_TO_PARTICIPATE_IN_ANOTHER_AUCTION_AT_THIS_TIME);
	public static final SystemMessage YOUR_FORCE_HAS_REACHED_MAXIMUM_CAPACITY_ = new SystemMessage(SystemMessage.YOUR_FORCE_HAS_REACHED_MAXIMUM_CAPACITY_);
	public static final SystemMessage A_SUB_CLASS_MAY_NOT_BE_CREATED_OR_CHANGED_WHILE_A_SERVITOR_OR_PET_IS_SUMMONED = new SystemMessage(SystemMessage.A_SUB_CLASS_MAY_NOT_BE_CREATED_OR_CHANGED_WHILE_A_SERVITOR_OR_PET_IS_SUMMONED);
	public static final SystemMessage SUB_CLASSES_MAY_NOT_BE_CREATED_OR_CHANGED_WHILE_A_SKILL_IS_IN_USE = new SystemMessage(SystemMessage.SUB_CLASSES_MAY_NOT_BE_CREATED_OR_CHANGED_WHILE_A_SKILL_IS_IN_USE);
	public static final SystemMessage A_SUB_CLASS_CANNOT_BE_CREATED_OR_CHANGED_WHILE_YOU_ARE_OVER_YOUR_WEIGHT_LIMIT = new SystemMessage(SystemMessage.A_SUB_CLASS_CANNOT_BE_CREATED_OR_CHANGED_WHILE_YOU_ARE_OVER_YOUR_WEIGHT_LIMIT);
	public static final SystemMessage A_SUB_CLASS_CANNOT_BE_CREATED_OR_CHANGED_BECAUSE_YOU_HAVE_EXCEEDED_YOUR_INVENTORY_LIMIT = new SystemMessage(SystemMessage.A_SUB_CLASS_CANNOT_BE_CREATED_OR_CHANGED_BECAUSE_YOU_HAVE_EXCEEDED_YOUR_INVENTORY_LIMIT);
	public static final SystemMessage SINCE_YOU_HAVE_CHANGED_YOUR_CLASS_INTO_A_SUB_JOB_YOU_CANNOT_PARTICIPATE_IN_THE_OLYMPIAD = new SystemMessage(SystemMessage.SINCE_YOU_HAVE_CHANGED_YOUR_CLASS_INTO_A_SUB_JOB_YOU_CANNOT_PARTICIPATE_IN_THE_OLYMPIAD);
	public static final SystemMessage CONGRATULATIONS_YOU_HAVE_TRANSFERRED_TO_A_NEW_CLASS = new SystemMessage(SystemMessage.CONGRATULATIONS_YOU_HAVE_TRANSFERRED_TO_A_NEW_CLASS);
	public static final SystemMessage THE_TRANSFER_OF_SUB_CLASS_HAS_BEEN_COMPLETED = new SystemMessage(SystemMessage.THE_TRANSFER_OF_SUB_CLASS_HAS_BEEN_COMPLETED);
	public static final SystemMessage THE_NEW_SUB_CLASS_HAS_BEEN_ADDED = new SystemMessage(SystemMessage.THE_NEW_SUB_CLASS_HAS_BEEN_ADDED);
	public static final SystemMessage YOU_ARE_NOT_QUALIFIED_TO_CREATE_A_CLAN = new SystemMessage(SystemMessage.YOU_ARE_NOT_QUALIFIED_TO_CREATE_A_CLAN);
	public static final SystemMessage YOU_HAVE_FAILED_TO_CREATE_A_CLAN = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_CREATE_A_CLAN);
	public static final SystemMessage YOU_MUST_WAIT_10_DAYS_BEFORE_CREATING_A_NEW_CLAN = new SystemMessage(SystemMessage.YOU_MUST_WAIT_10_DAYS_BEFORE_CREATING_A_NEW_CLAN);
	public static final SystemMessage CLAN_NAMES_LENGTH_IS_INCORRECT = new SystemMessage(SystemMessage.CLAN_NAMES_LENGTH_IS_INCORRECT);
	public static final SystemMessage CLAN_NAME_IS_INCORRECT = new SystemMessage(SystemMessage.CLAN_NAME_IS_INCORRECT);
	public static final SystemMessage THIS_NAME_ALREADY_EXISTS = new SystemMessage(SystemMessage.THIS_NAME_ALREADY_EXISTS);
	public static final SystemMessage CLAN_HAS_BEEN_CREATED = new SystemMessage(SystemMessage.CLAN_HAS_BEEN_CREATED);
	public static final SystemMessage ANOTHER_MILITARY_UNIT_IS_ALREADY_USING_THAT_NAME_PLEASE_ENTER_A_DIFFERENT_NAME = new SystemMessage(SystemMessage.ANOTHER_MILITARY_UNIT_IS_ALREADY_USING_THAT_NAME_PLEASE_ENTER_A_DIFFERENT_NAME);
	public static final SystemMessage THE_CONDITIONS_NECESSARY_TO_CREATE_A_MILITARY_UNIT_HAVE_NOT_BEEN_MET = new SystemMessage(SystemMessage.THE_CONDITIONS_NECESSARY_TO_CREATE_A_MILITARY_UNIT_HAVE_NOT_BEEN_MET);
	public static final SystemMessage YOU_CANNOT_DISPERSE_THE_CLANS_IN_YOUR_ALLIANCE = new SystemMessage(SystemMessage.YOU_CANNOT_DISPERSE_THE_CLANS_IN_YOUR_ALLIANCE);
	public static final SystemMessage YOU_CANNOT_DISSOLVE_A_CLAN_WHILE_ENGAGED_IN_A_WAR = new SystemMessage(SystemMessage.YOU_CANNOT_DISSOLVE_A_CLAN_WHILE_ENGAGED_IN_A_WAR);
	public static final SystemMessage UNABLE_TO_DISPERSE_YOUR_CLAN_OWNS_ONE_OR_MORE_CASTLES_OR_HIDEOUTS = new SystemMessage(SystemMessage.UNABLE_TO_DISPERSE_YOUR_CLAN_OWNS_ONE_OR_MORE_CASTLES_OR_HIDEOUTS);
	public static final SystemMessage UNABLE_TO_DISPERSE_YOUR_CLAN_HAS_REQUESTED_TO_PARTICIPATE_IN_A_CASTLE_SIEGE = new SystemMessage(SystemMessage.UNABLE_TO_DISPERSE_YOUR_CLAN_HAS_REQUESTED_TO_PARTICIPATE_IN_A_CASTLE_SIEGE);
	public static final SystemMessage NOW_THAT_YOUR_CLAN_LEVEL_IS_ABOVE_LEVEL_5_IT_CAN_ACCUMULATE_CLAN_REPUTATION_POINTS = new SystemMessage(SystemMessage.NOW_THAT_YOUR_CLAN_LEVEL_IS_ABOVE_LEVEL_5_IT_CAN_ACCUMULATE_CLAN_REPUTATION_POINTS);
	public static final SystemMessage CLAN_HAS_FAILED_TO_INCREASE_SKILL_LEVEL = new SystemMessage(SystemMessage.CLAN_HAS_FAILED_TO_INCREASE_SKILL_LEVEL);
	public static final SystemMessage ONLY_CLAN_LEADERS_MAY_CREATE_ALLIANCES = new SystemMessage(SystemMessage.ONLY_CLAN_LEADERS_MAY_CREATE_ALLIANCES);
	public static final SystemMessage YOU_ALREADY_BELONG_TO_ANOTHER_ALLIANCE = new SystemMessage(SystemMessage.YOU_ALREADY_BELONG_TO_ANOTHER_ALLIANCE);
	public static final SystemMessage INCORRECT_LENGTH_FOR_AN_ALLIANCE_NAME = new SystemMessage(SystemMessage.INCORRECT_LENGTH_FOR_AN_ALLIANCE_NAME);
	public static final SystemMessage INCORRECT_ALLIANCE_NAME = new SystemMessage(SystemMessage.INCORRECT_ALLIANCE_NAME);
	public static final SystemMessage TO_CREATE_AN_ALLIANCE_YOUR_CLAN_MUST_BE_LEVEL_5_OR_HIGHER = new SystemMessage(SystemMessage.TO_CREATE_AN_ALLIANCE_YOUR_CLAN_MUST_BE_LEVEL_5_OR_HIGHER);
	public static final SystemMessage THIS_ALLIANCE_NAME_ALREADY_EXISTS = new SystemMessage(SystemMessage.THIS_ALLIANCE_NAME_ALREADY_EXISTS);
	public static final SystemMessage YOU_CANNOT_CREATE_A_NEW_ALLIANCE_WITHIN_1_DAY_AFTER_DISSOLUTION = new SystemMessage(SystemMessage.YOU_CANNOT_CREATE_A_NEW_ALLIANCE_WITHIN_1_DAY_AFTER_DISSOLUTION);
	public static final SystemMessage YOU_HAVE_FAILED_TO_DISSOLVE_THE_ALLIANCE = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_DISSOLVE_THE_ALLIANCE);
	public static final SystemMessage CANCELS_FISHING = new SystemMessage(SystemMessage.CANCELS_FISHING);
	public static final SystemMessage YOU_CANT_FISH_WHILE_YOU_ARE_ON_BOARD = new SystemMessage(SystemMessage.YOU_CANT_FISH_WHILE_YOU_ARE_ON_BOARD);
	public static final SystemMessage YOU_CANNOT_FISH_WHILE_USING_A_RECIPE_BOOK_PRIVATE_MANUFACTURE_OR_PRIVATE_STORE = new SystemMessage(SystemMessage.YOU_CANNOT_FISH_WHILE_USING_A_RECIPE_BOOK_PRIVATE_MANUFACTURE_OR_PRIVATE_STORE);
	public static final SystemMessage YOU_CANT_FISH_HERE = new SystemMessage(SystemMessage.YOU_CANT_FISH_HERE);
	public static final SystemMessage FISHING_POLES_ARE_NOT_INSTALLED = new SystemMessage(SystemMessage.FISHING_POLES_ARE_NOT_INSTALLED);
	public static final SystemMessage BAITS_ARE_NOT_PUT_ON_A_HOOK = new SystemMessage(SystemMessage.BAITS_ARE_NOT_PUT_ON_A_HOOK);
	public static final SystemMessage NOT_ENOUGH_BAIT = new SystemMessage(SystemMessage.NOT_ENOUGH_BAIT);
	public static final SystemMessage STARTS_FISHING = new SystemMessage(SystemMessage.STARTS_FISHING);
	public static final SystemMessage THE_OLYMPIAD_GAME_IS_NOT_CURRENTLY_IN_PROGRESS = new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_IS_NOT_CURRENTLY_IN_PROGRESS);
	public static final SystemMessage ONLY_NOBLESS_CAN_PARTICIPATE_IN_THE_OLYMPIAD = new SystemMessage(SystemMessage.ONLY_NOBLESS_CAN_PARTICIPATE_IN_THE_OLYMPIAD);
	public static final SystemMessage YOU_CANT_JOIN_THE_OLYMPIAD_WITH_A_SUB_JOB_CHARACTER = new SystemMessage(SystemMessage.YOU_CANT_JOIN_THE_OLYMPIAD_WITH_A_SUB_JOB_CHARACTER);
	public static final SystemMessage YOU_HAVE_ALREADY_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_AN_EVENT = new SystemMessage(SystemMessage.YOU_HAVE_ALREADY_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_AN_EVENT);
	public static final SystemMessage YOU_ARE_ALREADY_ON_THE_WAITING_LIST_FOR_ALL_CLASSES_WAITING_TO_PARTICIPATE_IN_THE_GAME = new SystemMessage(SystemMessage.YOU_ARE_ALREADY_ON_THE_WAITING_LIST_FOR_ALL_CLASSES_WAITING_TO_PARTICIPATE_IN_THE_GAME);
	public static final SystemMessage YOU_ARE_ALREADY_ON_THE_WAITING_LIST_TO_PARTICIPATE_IN_THE_GAME_FOR_YOUR_CLASS = new SystemMessage(SystemMessage.YOU_ARE_ALREADY_ON_THE_WAITING_LIST_TO_PARTICIPATE_IN_THE_GAME_FOR_YOUR_CLASS);
	public static final SystemMessage YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_CLASSIFIED_GAMES = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_CLASSIFIED_GAMES);
	public static final SystemMessage YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_NO_CLASS_GAMES = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_NO_CLASS_GAMES);
	public static final SystemMessage YOU_HAVE_REGISTERED_ON_THE_WAITING_LIST_FOR_THE_NON_CLASS_LIMITED_TEAM_MATCH_EVENT = new SystemMessage(SystemMessage.YOU_HAVE_REGISTERED_ON_THE_WAITING_LIST_FOR_THE_NON_CLASS_LIMITED_TEAM_MATCH_EVENT);
	public static final SystemMessage YOU_HAVE_NOT_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_A_GAME = new SystemMessage(SystemMessage.YOU_HAVE_NOT_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_A_GAME);
	public static final SystemMessage YOU_HAVE_BEEN_DELETED_FROM_THE_WAITING_LIST_OF_A_GAME = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_DELETED_FROM_THE_WAITING_LIST_OF_A_GAME);
	public static final SystemMessage WHILE_YOU_ARE_ON_THE_WAITING_LIST_YOU_ARE_NOT_ALLOWED_TO_WATCH_THE_GAME = new SystemMessage(SystemMessage.WHILE_YOU_ARE_ON_THE_WAITING_LIST_YOU_ARE_NOT_ALLOWED_TO_WATCH_THE_GAME);
	public static final SystemMessage YOU_DO_NOT_MEET_THE_REQUIRED_CONDITION_TO_EQUIP_THAT_ITEM = new SystemMessage(SystemMessage.YOU_DO_NOT_MEET_THE_REQUIRED_CONDITION_TO_EQUIP_THAT_ITEM);
	public static final SystemMessage THIS_ITEM_CAN_ONLY_BE_WORN_BY_A_MEMBER_OF_THE_CLAN_ACADEMY = new SystemMessage(SystemMessage.THIS_ITEM_CAN_ONLY_BE_WORN_BY_A_MEMBER_OF_THE_CLAN_ACADEMY);
	public static final SystemMessage THE_FISH_GOT_AWAY = new SystemMessage(SystemMessage.THE_FISH_GOT_AWAY);
	public static final SystemMessage TIME_IS_UP_SO_THAT_FISH_GOT_AWAY = new SystemMessage(SystemMessage.TIME_IS_UP_SO_THAT_FISH_GOT_AWAY);
	public static final SystemMessage SUCCEEDED_IN_GETTING_A_BITE = new SystemMessage(SystemMessage.SUCCEEDED_IN_GETTING_A_BITE);
	public static final SystemMessage SUCCEEDED_IN_FISHING = new SystemMessage(SystemMessage.SUCCEEDED_IN_FISHING);
	public static final SystemMessage FISH_HAS_RESISTED = new SystemMessage(SystemMessage.FISH_HAS_RESISTED);
	public static final SystemMessage YOU_HAVE_CAUGHT_A_MONSTER = new SystemMessage(SystemMessage.YOU_HAVE_CAUGHT_A_MONSTER);
	public static final SystemMessage ONCE_AN_ITEM_IS_AUGMENTED_IT_CANNOT_BE_AUGMENTED_AGAIN = new SystemMessage(SystemMessage.ONCE_AN_ITEM_IS_AUGMENTED_IT_CANNOT_BE_AUGMENTED_AGAIN);
	public static final SystemMessage THIS_IS_NOT_A_SUITABLE_ITEM = new SystemMessage(SystemMessage.THIS_IS_NOT_A_SUITABLE_ITEM);
	public static final SystemMessage YOU_CANNOT_AUGMENT_ITEMS_WHILE_A_PRIVATE_STORE_OR_PRIVATE_WORKSHOP_IS_IN_OPERATION = new SystemMessage(SystemMessage.YOU_CANNOT_AUGMENT_ITEMS_WHILE_A_PRIVATE_STORE_OR_PRIVATE_WORKSHOP_IS_IN_OPERATION);
	public static final SystemMessage YOU_CANNOT_AUGMENT_ITEMS_WHILE_DEAD = new SystemMessage(SystemMessage.YOU_CANNOT_AUGMENT_ITEMS_WHILE_DEAD);
	public static final SystemMessage YOU_CANNOT_AUGMENT_ITEMS_WHILE_PARALYZED = new SystemMessage(SystemMessage.YOU_CANNOT_AUGMENT_ITEMS_WHILE_PARALYZED);
	public static final SystemMessage YOU_CANNOT_AUGMENT_ITEMS_WHILE_FISHING = new SystemMessage(SystemMessage.YOU_CANNOT_AUGMENT_ITEMS_WHILE_FISHING);
	public static final SystemMessage YOU_CANNOT_AUGMENT_ITEMS_WHILE_SITTING_DOWN = new SystemMessage(SystemMessage.YOU_CANNOT_AUGMENT_ITEMS_WHILE_SITTING_DOWN);
	public static final SystemMessage SELECT_THE_CATALYST_FOR_AUGMENTATION = new SystemMessage(SystemMessage.SELECT_THE_CATALYST_FOR_AUGMENTATION);
	public static final SystemMessage OVER_HIT = new SystemMessage(SystemMessage.OVER_HIT);
	public static final SystemMessage THE_SOUL_CRYSTALS_CAUSED_RESONATION_AND_FAILED_AT_ABSORBING_A_SOUL = new SystemMessage(SystemMessage.THE_SOUL_CRYSTALS_CAUSED_RESONATION_AND_FAILED_AT_ABSORBING_A_SOUL);
	public static final SystemMessage THE_SOUL_CRYSTAL_IS_REFUSING_TO_ABSORB_A_SOUL = new SystemMessage(SystemMessage.THE_SOUL_CRYSTAL_IS_REFUSING_TO_ABSORB_A_SOUL);
	public static final SystemMessage THE_SOUL_CRYSTAL_SUCCEEDED_IN_ABSORBING_A_SOUL = new SystemMessage(SystemMessage.THE_SOUL_CRYSTAL_SUCCEEDED_IN_ABSORBING_A_SOUL);
	public static final SystemMessage GEMSTONE_QUANTITY_IS_INCORRECT = new SystemMessage(SystemMessage.GEMSTONE_QUANTITY_IS_INCORRECT);
	public static final SystemMessage PRESS_THE_AUGMENT_BUTTON_TO_BEGIN = new SystemMessage(SystemMessage.PRESS_THE_AUGMENT_BUTTON_TO_BEGIN);
	public static final SystemMessage YOU_DO_NOT_HAVE_AUTHORITY_TO_INVITE_SOMEONE_TO_THE_COMMAND_CHANNEL = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_AUTHORITY_TO_INVITE_SOMEONE_TO_THE_COMMAND_CHANNEL);
	public static final SystemMessage YOU_HAVE_INVITED_WRONG_TARGET = new SystemMessage(SystemMessage.YOU_HAVE_INVITED_WRONG_TARGET);
	public static final SystemMessage THERE_IS_NO_OPPONENT_TO_RECEIVE_YOUR_CHALLENGE_FOR_A_DUEL = new SystemMessage(SystemMessage.THERE_IS_NO_OPPONENT_TO_RECEIVE_YOUR_CHALLENGE_FOR_A_DUEL);
	public static final SystemMessage SINCE_THE_PERSON_YOU_CHALLENGED_IS_NOT_CURRENTLY_IN_A_PARTY_THEY_CANNOT_DUEL_AGAINST_YOUR_PARTY = new SystemMessage(SystemMessage.SINCE_THE_PERSON_YOU_CHALLENGED_IS_NOT_CURRENTLY_IN_A_PARTY_THEY_CANNOT_DUEL_AGAINST_YOUR_PARTY);
	public static final SystemMessage PARTY_DUEL_CANNOT_BE_INITIATED_DUEL_TO_A_POLYMORPHED_PARTY_MEMBER = new SystemMessage(SystemMessage.PARTY_DUEL_CANNOT_BE_INITIATED_DUEL_TO_A_POLYMORPHED_PARTY_MEMBER);
	public static final SystemMessage THE_OPPOSING_PARTY_IS_CURRENTLY_UNABLE_TO_ACCEPT_A_CHALLENGE_TO_A_DUEL = new SystemMessage(SystemMessage.THE_OPPOSING_PARTY_IS_CURRENTLY_UNABLE_TO_ACCEPT_A_CHALLENGE_TO_A_DUEL);
	public static final SystemMessage IT_IS_NOT_AN_AUCTION_PERIOD = new SystemMessage(SystemMessage.IT_IS_NOT_AN_AUCTION_PERIOD);
	public static final SystemMessage YOU_HAVE_CANCELED_YOUR_BID = new SystemMessage(SystemMessage.YOU_HAVE_CANCELED_YOUR_BID);
	public static final SystemMessage IT_HAS_NOT_YET_BEEN_SEVEN_DAYS_SINCE_CANCELING_AN_AUCTION = new SystemMessage(SystemMessage.IT_HAS_NOT_YET_BEEN_SEVEN_DAYS_SINCE_CANCELING_AN_AUCTION);
	public static final SystemMessage THERE_ARE_NO_COMMUNITIES_IN_MY_CLAN_CLAN_COMMUNITIES_ARE_ALLOWED_FOR_CLANS_WITH_SKILL_LEVELS_OF_2_AND_HIGHER = new SystemMessage(SystemMessage.THERE_ARE_NO_COMMUNITIES_IN_MY_CLAN_CLAN_COMMUNITIES_ARE_ALLOWED_FOR_CLANS_WITH_SKILL_LEVELS_OF_2_AND_HIGHER);
	public static final SystemMessage YOU_HAVE_RECENTLY_BEEN_DISMISSED_FROM_A_CLAN_YOU_ARE_NOT_ALLOWED_TO_JOIN_ANOTHER_CLAN_FOR_24_HOURS = new SystemMessage(SystemMessage.YOU_HAVE_RECENTLY_BEEN_DISMISSED_FROM_A_CLAN_YOU_ARE_NOT_ALLOWED_TO_JOIN_ANOTHER_CLAN_FOR_24_HOURS);
	public static final SystemMessage CLAN_HAS_DISPERSED = new SystemMessage(SystemMessage.CLAN_HAS_DISPERSED);
	public static final SystemMessage THE_ALLIANCE_HAS_BEEN_DISSOLVED = new SystemMessage(SystemMessage.THE_ALLIANCE_HAS_BEEN_DISSOLVED);
	public static final SystemMessage YOU_HAVE_WITHDRAWN_FROM_THE_ALLIANCE = new SystemMessage(SystemMessage.YOU_HAVE_WITHDRAWN_FROM_THE_ALLIANCE);
	public static final SystemMessage THE_COMMUNITY_SERVER_IS_CURRENTLY_OFFLINE = new SystemMessage(SystemMessage.THE_COMMUNITY_SERVER_IS_CURRENTLY_OFFLINE);
	public static final SystemMessage THE_DEATH_PENALTY_HAS_BEEN_LIFTED = new SystemMessage(SystemMessage.THE_DEATH_PENALTY_HAS_BEEN_LIFTED);
	public static final SystemMessage THE_REBEL_ARMY_RECAPTURED_THE_FORTRESS = new SystemMessage(SystemMessage.THE_REBEL_ARMY_RECAPTURED_THE_FORTRESS);
	public static final SystemMessage YOU_ALREADY_HAVE_A_PET = new SystemMessage(SystemMessage.YOU_ALREADY_HAVE_A_PET);
	public static final SystemMessage THERE_ARE_NOT_ANY_GMS_THAT_ARE_PROVIDING_CUSTOMER_SERVICE_CURRENTLY = new SystemMessage(SystemMessage.THERE_ARE_NOT_ANY_GMS_THAT_ARE_PROVIDING_CUSTOMER_SERVICE_CURRENTLY);
	public static final SystemMessage _GM_LIST_ = new SystemMessage(SystemMessage._GM_LIST_);
	public static final SystemMessage REJUVENATING_HP = new SystemMessage(SystemMessage.REJUVENATING_HP);
	public static final SystemMessage YOU_CANNOT_CONVERT_THIS_ITEM = new SystemMessage(SystemMessage.YOU_CANNOT_CONVERT_THIS_ITEM);
	public static final SystemMessage BECAUSE_PET_OR_SERVITOR_MAY_BE_DROWNED_WHILE_THE_BOAT_MOVES_PLEASE_RELEASE_THE_SUMMON_BEFORE_DEPARTURE = new SystemMessage(SystemMessage.BECAUSE_PET_OR_SERVITOR_MAY_BE_DROWNED_WHILE_THE_BOAT_MOVES_PLEASE_RELEASE_THE_SUMMON_BEFORE_DEPARTURE);
	public static final SystemMessage YOU_CANNOT_MOUNT_BECAUSE_YOU_DO_NOT_MEET_THE_REQUIREMENTS = new SystemMessage(SystemMessage.YOU_CANNOT_MOUNT_BECAUSE_YOU_DO_NOT_MEET_THE_REQUIREMENTS);
	public static final SystemMessage THE_CASTLE_GATE_HAS_BEEN_BROKEN_DOWN = new SystemMessage(SystemMessage.THE_CASTLE_GATE_HAS_BEEN_BROKEN_DOWN);
	public static final SystemMessage THE_CLAN_HALL_FEE_IS_ONE_WEEK_OVERDUE_THEREFORE_THE_CLAN_HALL_OWNERSHIP_HAS_BEEN_REVOKED = new SystemMessage(SystemMessage.THE_CLAN_HALL_FEE_IS_ONE_WEEK_OVERDUE_THEREFORE_THE_CLAN_HALL_OWNERSHIP_HAS_BEEN_REVOKED);
	public static final SystemMessage ACQUIRED_50_CLAN_FAME_POINTS = new SystemMessage(SystemMessage.ACQUIRED_50_CLAN_FAME_POINTS);
	public static final SystemMessage ANOTHER_PERSON_HAS_LOGGED_IN_WITH_THE_SAME_ACCOUNT = new SystemMessage(SystemMessage.ANOTHER_PERSON_HAS_LOGGED_IN_WITH_THE_SAME_ACCOUNT);
	public static final SystemMessage YOU_DO_NOT_HAVE_THE_AUTHORITY_TO_CANCEL_MERCENARY_POSITIONING = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_THE_AUTHORITY_TO_CANCEL_MERCENARY_POSITIONING);
	public static final SystemMessage CANNOT_CRYSTALLIZE_CRYSTALLIZATION_SKILL_LEVEL_TOO_LOW = new SystemMessage(SystemMessage.CANNOT_CRYSTALLIZE_CRYSTALLIZATION_SKILL_LEVEL_TOO_LOW);
	public static final SystemMessage CONGRATULATIONS_YOUR_RAID_WAS_SUCCESSFUL = new SystemMessage(SystemMessage.CONGRATULATIONS_YOUR_RAID_WAS_SUCCESSFUL);
	public static final SystemMessage YOU_MAY_NOT_GET_ON_BOARD_WITHOUT_A_PASS = new SystemMessage(SystemMessage.YOU_MAY_NOT_GET_ON_BOARD_WITHOUT_A_PASS);
	public static final SystemMessage THAT_CHARACTER_DOES_NOT_EXIST = new SystemMessage(SystemMessage.THAT_CHARACTER_DOES_NOT_EXIST);
	public static final SystemMessage THIS_SKILL_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT = new SystemMessage(SystemMessage.THIS_SKILL_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT);
	public static final SystemMessage PUMPING_SKILL_IS_AVAILABLE_ONLY_WHILE_FISHING = new SystemMessage(SystemMessage.PUMPING_SKILL_IS_AVAILABLE_ONLY_WHILE_FISHING);
	public static final SystemMessage REELING_SKILL_IS_AVAILABLE_ONLY_WHILE_FISHING = new SystemMessage(SystemMessage.REELING_SKILL_IS_AVAILABLE_ONLY_WHILE_FISHING);
	public static final SystemMessage SINCE_THE_SKILL_LEVEL_OF_REELING_PUMPING_IS_HIGHER_THAN_THE_LEVEL_OF_YOUR_FISHING_MASTERY_A_PENALTY_OF_S1_WILL_BE_APPLIED = new SystemMessage(SystemMessage.SINCE_THE_SKILL_LEVEL_OF_REELING_PUMPING_IS_HIGHER_THAN_THE_LEVEL_OF_YOUR_FISHING_MASTERY_A_PENALTY_OF_S1_WILL_BE_APPLIED);
	public static final SystemMessage YOU_HAVE_ACCEPTED_THE_ALLIANCE = new SystemMessage(SystemMessage.YOU_HAVE_ACCEPTED_THE_ALLIANCE);
	public static final SystemMessage YOU_HAVE_FAILED_TO_INVITE_A_CLAN_INTO_THE_ALLIANCE = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_INVITE_A_CLAN_INTO_THE_ALLIANCE);
	public static final SystemMessage ENTERED_THE_CLAN = new SystemMessage(SystemMessage.ENTERED_THE_CLAN);
	public static final SystemMessage AFTER_A_CLAN_MEMBER_IS_DISMISSED_FROM_A_CLAN_THE_CLAN_MUST_WAIT_AT_LEAST_A_DAY_BEFORE_ACCEPTING_A_NEW_MEMBER = new SystemMessage(SystemMessage.AFTER_A_CLAN_MEMBER_IS_DISMISSED_FROM_A_CLAN_THE_CLAN_MUST_WAIT_AT_LEAST_A_DAY_BEFORE_ACCEPTING_A_NEW_MEMBER);
	public static final SystemMessage AFTER_LEAVING_OR_HAVING_BEEN_DISMISSED_FROM_A_CLAN_YOU_MUST_WAIT_AT_LEAST_A_DAY_BEFORE_JOINING_ANOTHER_CLAN = new SystemMessage(SystemMessage.AFTER_LEAVING_OR_HAVING_BEEN_DISMISSED_FROM_A_CLAN_YOU_MUST_WAIT_AT_LEAST_A_DAY_BEFORE_JOINING_ANOTHER_CLAN);
	public static final SystemMessage _IGNORE_LIST_ = new SystemMessage(SystemMessage._IGNORE_LIST_);
	public static final SystemMessage YOU_ARE_NOW_BLOCKING_EVERYTHING = new SystemMessage(SystemMessage.YOU_ARE_NOW_BLOCKING_EVERYTHING);
	public static final SystemMessage YOU_ARE_NO_LONGER_BLOCKING_EVERYTHING = new SystemMessage(SystemMessage.YOU_ARE_NO_LONGER_BLOCKING_EVERYTHING);
	public static final SystemMessage YOUR_PETS_NAME_CAN_BE_UP_TO_8_CHARACTERS = new SystemMessage(SystemMessage.YOUR_PETS_NAME_CAN_BE_UP_TO_8_CHARACTERS);
	public static final SystemMessage ITEMS_REQUIRED_FOR_SKILL_ENCHANT_ARE_INSUFFICIENT = new SystemMessage(SystemMessage.ITEMS_REQUIRED_FOR_SKILL_ENCHANT_ARE_INSUFFICIENT);
	public static final SystemMessage EXP_REQUIRED_FOR_SKILL_ENCHANT_IS_INSUFFICIENT = new SystemMessage(SystemMessage.EXP_REQUIRED_FOR_SKILL_ENCHANT_IS_INSUFFICIENT);
	public static final SystemMessage SP_REQUIRED_FOR_SKILL_ENCHANT_IS_INSUFFICIENT = new SystemMessage(SystemMessage.SP_REQUIRED_FOR_SKILL_ENCHANT_IS_INSUFFICIENT);
	public static final SystemMessage NO_USER_HAS_BEEN_INVITED_TO_THE_COMMAND_CHANNEL = new SystemMessage(SystemMessage.NO_USER_HAS_BEEN_INVITED_TO_THE_COMMAND_CHANNEL);
	public static final SystemMessage YOU_CANNOT_JOIN_A_COMMAND_CHANNEL_WHILE_TELEPORTING = new SystemMessage(SystemMessage.YOU_CANNOT_JOIN_A_COMMAND_CHANNEL_WHILE_TELEPORTING);
	public static final SystemMessage YOU_HAVE_SUCCEEDED_IN_INVITING_A_FRIEND = new SystemMessage(SystemMessage.YOU_HAVE_SUCCEEDED_IN_INVITING_A_FRIEND);
	public static final SystemMessage YOU_HAVE_FAILED_TO_INVITE_A_FRIEND = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_INVITE_A_FRIEND);
	public static final SystemMessage CANNOT_GIVE_ITEMS_TO_A_DEAD_PET = new SystemMessage(SystemMessage.CANNOT_GIVE_ITEMS_TO_A_DEAD_PET);
	public static final SystemMessage EXCEEDED_PET_INVENTORYS_WEIGHT_LIMIT = new SystemMessage(SystemMessage.EXCEEDED_PET_INVENTORYS_WEIGHT_LIMIT);
	public static final SystemMessage THE_SYMBOL_HAS_BEEN_ADDED = new SystemMessage(SystemMessage.THE_SYMBOL_HAS_BEEN_ADDED);
	public static final SystemMessage THE_SYMBOL_CANNOT_BE_DRAWN = new SystemMessage(SystemMessage.THE_SYMBOL_CANNOT_BE_DRAWN);
	public static final SystemMessage YOU_MAY_CREATE_UP_TO_48_MACROS = new SystemMessage(SystemMessage.YOU_MAY_CREATE_UP_TO_48_MACROS);
	public static final SystemMessage ENTER_THE_NAME_OF_THE_MACRO = new SystemMessage(SystemMessage.ENTER_THE_NAME_OF_THE_MACRO);
	public static final SystemMessage MACRO_DESCRIPTIONS_MAY_CONTAIN_UP_TO_32_CHARACTERS = new SystemMessage(SystemMessage.MACRO_DESCRIPTIONS_MAY_CONTAIN_UP_TO_32_CHARACTERS);
	public static final SystemMessage YOU_CANNOT_BOARD_A_SHIP_WHILE_YOU_ARE_POLYMORPHED = new SystemMessage(SystemMessage.YOU_CANNOT_BOARD_A_SHIP_WHILE_YOU_ARE_POLYMORPHED);
	public static final SystemMessage YOU_HAVE_FAILED_TO_WITHDRAW_FROM_THE_ALLIANCE = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_WITHDRAW_FROM_THE_ALLIANCE);
	public static final SystemMessage A_CLAN_MEMBER_MAY_NOT_BE_DISMISSED_DURING_COMBAT = new SystemMessage(SystemMessage.A_CLAN_MEMBER_MAY_NOT_BE_DISMISSED_DURING_COMBAT);
	public static final SystemMessage THE_TARGET_MUST_BE_A_CLAN_MEMBER = new SystemMessage(SystemMessage.THE_TARGET_MUST_BE_A_CLAN_MEMBER);
	public static final SystemMessage ITEM_NOT_AVAILABLE_FOR_PETS = new SystemMessage(SystemMessage.ITEM_NOT_AVAILABLE_FOR_PETS);
	public static final SystemMessage THE_ACADEMY_ROYAL_GUARD_ORDER_OF_KNIGHTS_IS_FULL_AND_CANNOT_ACCEPT_NEW_MEMBERS_AT_THIS_TIME = new SystemMessage(SystemMessage.THE_ACADEMY_ROYAL_GUARD_ORDER_OF_KNIGHTS_IS_FULL_AND_CANNOT_ACCEPT_NEW_MEMBERS_AT_THIS_TIME);
	public static final SystemMessage THE_ATTEMPT_TO_SELL_HAS_FAILED = new SystemMessage(SystemMessage.THE_ATTEMPT_TO_SELL_HAS_FAILED);
	public static final SystemMessage AUGMENTATION_FAILED_DUE_TO_INAPPROPRIATE_CONDITIONS = new SystemMessage(SystemMessage.AUGMENTATION_FAILED_DUE_TO_INAPPROPRIATE_CONDITIONS);
	public static final SystemMessage THE_ITEM_WAS_SUCCESSFULLY_AUGMENTED = new SystemMessage(SystemMessage.THE_ITEM_WAS_SUCCESSFULLY_AUGMENTED);
	public static final SystemMessage YOU_ARE_NOT_AUTHORIZED_TO_DO_THAT = new SystemMessage(SystemMessage.YOU_ARE_NOT_AUTHORIZED_TO_DO_THAT);
	public static final SystemMessage YOU_HAVE_NOT_DECLARED_A_CLAN_WAR_TO_S1_CLAN = new SystemMessage(SystemMessage.YOU_HAVE_NOT_DECLARED_A_CLAN_WAR_TO_S1_CLAN);
	public static final SystemMessage A_CEASE_FIRE_DURING_A_CLAN_WAR_CAN_NOT_BE_CALLED_WHILE_MEMBERS_OF_YOUR_CLAN_ARE_ENGAGED_IN_BATTLE = new SystemMessage(SystemMessage.A_CEASE_FIRE_DURING_A_CLAN_WAR_CAN_NOT_BE_CALLED_WHILE_MEMBERS_OF_YOUR_CLAN_ARE_ENGAGED_IN_BATTLE);
	public static final SystemMessage CLAN_CREST_REGISTRATION_IS_ONLY_POSSIBLE_WHEN_CLANS_SKILL_LEVELS_ARE_ABOVE_3 = new SystemMessage(SystemMessage.CLAN_CREST_REGISTRATION_IS_ONLY_POSSIBLE_WHEN_CLANS_SKILL_LEVELS_ARE_ABOVE_3);
	public static final SystemMessage THE_CLANS_EMBLEM_WAS_SUCCESSFULLY_REGISTERED__ONLY_A_CLAN_THAT_OWNS_A_CLAN_HALL_OR_A_CASTLE_CAN_GET_THEIR_EMBLEM_DISPLAYED_ON_CLAN_RELATED_ITEMS = new SystemMessage(SystemMessage.THE_CLANS_EMBLEM_WAS_SUCCESSFULLY_REGISTERED__ONLY_A_CLAN_THAT_OWNS_A_CLAN_HALL_OR_A_CASTLE_CAN_GET_THEIR_EMBLEM_DISPLAYED_ON_CLAN_RELATED_ITEMS);
	public static final SystemMessage A_DECLARATION_OF_WAR_AGAINST_MORE_THAN_30_CLANS_CANT_BE_MADE_AT_THE_SAME_TIME = new SystemMessage(SystemMessage.A_DECLARATION_OF_WAR_AGAINST_MORE_THAN_30_CLANS_CANT_BE_MADE_AT_THE_SAME_TIME);
	public static final SystemMessage A_CLAN_WAR_CAN_BE_DECLARED_ONLY_IF_THE_CLAN_IS_LEVEL_THREE_OR_ABOVE_AND_THE_NUMBER_OF_CLAN_MEMBERS_IS_FIFTEEN_OR_GREATER = new SystemMessage(SystemMessage.A_CLAN_WAR_CAN_BE_DECLARED_ONLY_IF_THE_CLAN_IS_LEVEL_THREE_OR_ABOVE_AND_THE_NUMBER_OF_CLAN_MEMBERS_IS_FIFTEEN_OR_GREATER);
	public static final SystemMessage A_DECLARATION_OF_CLAN_WAR_AGAINST_AN_ALLIED_CLAN_CANT_BE_MADE = new SystemMessage(SystemMessage.A_DECLARATION_OF_CLAN_WAR_AGAINST_AN_ALLIED_CLAN_CANT_BE_MADE);
	public static final SystemMessage THE_DECLARATION_OF_WAR_HAS_BEEN_ALREADY_MADE_TO_THE_CLAN = new SystemMessage(SystemMessage.THE_DECLARATION_OF_WAR_HAS_BEEN_ALREADY_MADE_TO_THE_CLAN);
	public static final SystemMessage FOOL_YOU_CANNOT_DECLARE_WAR_AGAINST_YOUR_OWN_CLAN = new SystemMessage(SystemMessage.FOOL_YOU_CANNOT_DECLARE_WAR_AGAINST_YOUR_OWN_CLAN);
	public static final SystemMessage THE_DECLARATION_OF_WAR_CANT_BE_MADE_BECAUSE_THE_CLAN_DOES_NOT_EXIST_OR_ACT_FOR_A_LONG_PERIOD = new SystemMessage(SystemMessage.THE_DECLARATION_OF_WAR_CANT_BE_MADE_BECAUSE_THE_CLAN_DOES_NOT_EXIST_OR_ACT_FOR_A_LONG_PERIOD);
	public static final SystemMessage ONLY_THE_CLAN_LEADER_MAY_APPLY_FOR_WITHDRAWAL_FROM_THE_ALLIANCE = new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_MAY_APPLY_FOR_WITHDRAWAL_FROM_THE_ALLIANCE);
	public static final SystemMessage ALLIANCE_LEADERS_CANNOT_WITHDRAW = new SystemMessage(SystemMessage.ALLIANCE_LEADERS_CANNOT_WITHDRAW);
	public static final SystemMessage A_CLAN_THAT_OWNS_A_CASTLE_CANNOT_PARTICIPATE_IN_ANOTHER_SIEGE = new SystemMessage(SystemMessage.A_CLAN_THAT_OWNS_A_CASTLE_CANNOT_PARTICIPATE_IN_ANOTHER_SIEGE);
	public static final SystemMessage THE_CLAN_THAT_OWNS_THE_CASTLE_IS_AUTOMATICALLY_REGISTERED_ON_THE_DEFENDING_SIDE = new SystemMessage(SystemMessage.THE_CLAN_THAT_OWNS_THE_CASTLE_IS_AUTOMATICALLY_REGISTERED_ON_THE_DEFENDING_SIDE);
	public static final SystemMessage YOU_CANNOT_REGISTER_ON_THE_ATTACKING_SIDE_BECAUSE_YOU_ARE_PART_OF_AN_ALLIANCE_WITH_THE_CLAN_THAT_OWNS_THE_CASTLE = new SystemMessage(SystemMessage.YOU_CANNOT_REGISTER_ON_THE_ATTACKING_SIDE_BECAUSE_YOU_ARE_PART_OF_AN_ALLIANCE_WITH_THE_CLAN_THAT_OWNS_THE_CASTLE);
	public static final SystemMessage SIEGE_REGISTRATION_IS_NOT_POSSIBLE_DUE_TO_A_CONTRACT_WITH_A_HIGHER_CASTLE = new SystemMessage(SystemMessage.SIEGE_REGISTRATION_IS_NOT_POSSIBLE_DUE_TO_A_CONTRACT_WITH_A_HIGHER_CASTLE);
	public static final SystemMessage ONE_CANNOT_LEAVE_ONES_CLAN_DURING_COMBAT = new SystemMessage(SystemMessage.ONE_CANNOT_LEAVE_ONES_CLAN_DURING_COMBAT);
	public static final SystemMessage THERE_ARE_NO_MORE_ITEMS_IN_THE_SHORTCUT = new SystemMessage(SystemMessage.THERE_ARE_NO_MORE_ITEMS_IN_THE_SHORTCUT);
	public static final SystemMessage IT_IS_A_CHARACTER_THAT_CANNOT_BE_SPOILED = new SystemMessage(SystemMessage.IT_IS_A_CHARACTER_THAT_CANNOT_BE_SPOILED);
	public static final SystemMessage ALREADY_SPOILED = new SystemMessage(SystemMessage.ALREADY_SPOILED);
	public static final SystemMessage THE_SPOIL_CONDITION_HAS_BEEN_ACTIVATED = new SystemMessage(SystemMessage.THE_SPOIL_CONDITION_HAS_BEEN_ACTIVATED);
	public static final SystemMessage AGATHION_SKILLS_CAN_BE_USED_ONLY_WHEN_AGATHION_IS_SUMMONED = new SystemMessage(SystemMessage.AGATHION_SKILLS_CAN_BE_USED_ONLY_WHEN_AGATHION_IS_SUMMONED);
	public static final SystemMessage A_MALICIOUS_SKILL_CANNOT_BE_USED_IN_A_PEACE_ZONE = new SystemMessage(SystemMessage.A_MALICIOUS_SKILL_CANNOT_BE_USED_IN_A_PEACE_ZONE);
	public static final SystemMessage CUBIC_SUMMONING_FAILED = new SystemMessage(SystemMessage.CUBIC_SUMMONING_FAILED);
	public static final SystemMessage THE_OPPONENT_CLAN_HAS_BEGUN_TO_ENGRAVE_THE_RULER = new SystemMessage(SystemMessage.THE_OPPONENT_CLAN_HAS_BEGUN_TO_ENGRAVE_THE_RULER);
	public static final SystemMessage YOU_HAVE_NOT_DEPOSITED_ANY_ITEMS_IN_YOUR_WAREHOUSE = new SystemMessage(SystemMessage.YOU_HAVE_NOT_DEPOSITED_ANY_ITEMS_IN_YOUR_WAREHOUSE);
	public static final SystemMessage THIS_AREA_CANNOT_BE_ENTERED_WHILE_MOUNTED_ATOP_OF_A_WYVERN_YOU_WILL_BE_DISMOUNTED_FROM_YOUR_WYVERN_IF_YOU_DO_NOT_LEAVE = new SystemMessage(SystemMessage.THIS_AREA_CANNOT_BE_ENTERED_WHILE_MOUNTED_ATOP_OF_A_WYVERN_YOU_WILL_BE_DISMOUNTED_FROM_YOUR_WYVERN_IF_YOU_DO_NOT_LEAVE);
	public static final SystemMessage _HERE_YOU_CAN_BUY_ONLY_SEEDS_OF_S1_MANOR = new SystemMessage(SystemMessage._HERE_YOU_CAN_BUY_ONLY_SEEDS_OF_S1_MANOR);
	public static final SystemMessage A_MANOR_CANNOT_BE_SET_UP_BETWEEN_6_AM_AND_8_PM = new SystemMessage(SystemMessage.A_MANOR_CANNOT_BE_SET_UP_BETWEEN_6_AM_AND_8_PM);
	public static final SystemMessage ANOTHER_ELEMENTAL_POWER_HAS_ALREADY_BEEN_ADDED_THIS_ELEMENTAL_POWER_CANNOT_BE_ADDED = new SystemMessage(SystemMessage.ANOTHER_ELEMENTAL_POWER_HAS_ALREADY_BEEN_ADDED_THIS_ELEMENTAL_POWER_CANNOT_BE_ADDED);
	public static final SystemMessage ELEMENTAL_POWER_ENCHANCER_USAGE_HAS_BEEN_CANCELLED = new SystemMessage(SystemMessage.ELEMENTAL_POWER_ENCHANCER_USAGE_HAS_BEEN_CANCELLED);
	public static final SystemMessage YOU_HAVE_FAILED_TO_ADD_ELEMENTAL_POWER = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_ADD_ELEMENTAL_POWER);
	public static final SystemMessage CAN_BE_USED_ONLY_BY_THE_REVOLUTIONARIES_OF_DUSK = new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_REVOLUTIONARIES_OF_DUSK);
	public static final SystemMessage CAN_BE_USED_ONLY_BY_THE_LORDS_OF_DAWN = new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_LORDS_OF_DAWN);
	public static final SystemMessage CONTRIBUTION_LEVEL_HAS_EXCEEDED_THE_LIMIT_YOU_MAY_NOT_CONTINUE = new SystemMessage(SystemMessage.CONTRIBUTION_LEVEL_HAS_EXCEEDED_THE_LIMIT_YOU_MAY_NOT_CONTINUE);
	public static final SystemMessage YOUVE_CHOSEN_TO_FIGHT_FOR_THE_SEAL_OF_STRIFE_DURING_THIS_QUEST_EVENT_PERIOD = new SystemMessage(SystemMessage.YOUVE_CHOSEN_TO_FIGHT_FOR_THE_SEAL_OF_STRIFE_DURING_THIS_QUEST_EVENT_PERIOD);
	public static final SystemMessage YOUVE_CHOSEN_TO_FIGHT_FOR_THE_SEAL_OF_GNOSIS_DURING_THIS_QUEST_EVENT_PERIOD = new SystemMessage(SystemMessage.YOUVE_CHOSEN_TO_FIGHT_FOR_THE_SEAL_OF_GNOSIS_DURING_THIS_QUEST_EVENT_PERIOD);
	public static final SystemMessage YOUVE_CHOSEN_TO_FIGHT_FOR_THE_SEAL_OF_AVARICE_DURING_THIS_QUEST_EVENT_PERIOD = new SystemMessage(SystemMessage.YOUVE_CHOSEN_TO_FIGHT_FOR_THE_SEAL_OF_AVARICE_DURING_THIS_QUEST_EVENT_PERIOD);
	public static final SystemMessage YOU_WILL_PARTICIPATE_IN_THE_SEVEN_SIGNS_AS_A_MEMBER_OF_THE_REVOLUTIONARIES_OF_DUSK = new SystemMessage(SystemMessage.YOU_WILL_PARTICIPATE_IN_THE_SEVEN_SIGNS_AS_A_MEMBER_OF_THE_REVOLUTIONARIES_OF_DUSK);
	public static final SystemMessage YOU_WILL_PARTICIPATE_IN_THE_SEVEN_SIGNS_AS_A_MEMBER_OF_THE_LORDS_OF_DAWN = new SystemMessage(SystemMessage.YOU_WILL_PARTICIPATE_IN_THE_SEVEN_SIGNS_AS_A_MEMBER_OF_THE_LORDS_OF_DAWN);
	public static final SystemMessage CHATTING_IS_CURRENTLY_PROHIBITED_IF_YOU_TRY_TO_CHAT_BEFORE_THE_PROHIBITION_IS_REMOVED_THE_PROHIBITION_TIME_WILL_BECOME_EVEN_LONGER = new SystemMessage(SystemMessage.CHATTING_IS_CURRENTLY_PROHIBITED_IF_YOU_TRY_TO_CHAT_BEFORE_THE_PROHIBITION_IS_REMOVED_THE_PROHIBITION_TIME_WILL_BECOME_EVEN_LONGER);
	public static final SystemMessage FAILED_TO_JOIN_THE_CLAN = new SystemMessage(SystemMessage.FAILED_TO_JOIN_THE_CLAN);
	public static final SystemMessage TO_JOIN_A_CLAN_ACADEMY_CHARACTERS_MUST_BE_LEVEL_40_OR_BELOW_NOT_BELONG_ANOTHER_CLAN_AND_NOT_YET_COMPLETED_THEIR_2ND_CLASS_TRANSFER = new SystemMessage(SystemMessage.TO_JOIN_A_CLAN_ACADEMY_CHARACTERS_MUST_BE_LEVEL_40_OR_BELOW_NOT_BELONG_ANOTHER_CLAN_AND_NOT_YET_COMPLETED_THEIR_2ND_CLASS_TRANSFER);
	public static final SystemMessage YOU_CANNOT_ASK_YOURSELF_TO_APPLY_TO_A_CLAN = new SystemMessage(SystemMessage.YOU_CANNOT_ASK_YOURSELF_TO_APPLY_TO_A_CLAN);
	public static final SystemMessage HP_WAS_FULLY_RECOVERED_AND_SKILL_WAS_REMOVED = new SystemMessage(SystemMessage.HP_WAS_FULLY_RECOVERED_AND_SKILL_WAS_REMOVED);
	public static final SystemMessage IT_IS_NOT_LOCKED = new SystemMessage(SystemMessage.IT_IS_NOT_LOCKED);
	public static final SystemMessage YOU_ARE_UNABLE_TO_UNLOCK_THE_DOOR = new SystemMessage(SystemMessage.YOU_ARE_UNABLE_TO_UNLOCK_THE_DOOR);
	public static final SystemMessage YOU_HAVE_FAILED_TO_UNLOCK_THE_DOOR = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_UNLOCK_THE_DOOR);
	public static final SystemMessage LOTTERY_TICKETS_ARE_NOT_CURRENTLY_BEING_SOLD = new SystemMessage(SystemMessage.LOTTERY_TICKETS_ARE_NOT_CURRENTLY_BEING_SOLD);
	public static final SystemMessage TICKETS_FOR_THE_CURRENT_LOTTERY_ARE_NO_LONGER_AVAILABLE = new SystemMessage(SystemMessage.TICKETS_FOR_THE_CURRENT_LOTTERY_ARE_NO_LONGER_AVAILABLE);
	public static final SystemMessage SELECT_THE_ITEM_TO_BE_AUGMENTED = new SystemMessage(SystemMessage.SELECT_THE_ITEM_TO_BE_AUGMENTED);
	public static final SystemMessage SELECT_THE_ITEM_FROM_WHICH_YOU_WISH_TO_REMOVE_AUGMENTATION = new SystemMessage(SystemMessage.SELECT_THE_ITEM_FROM_WHICH_YOU_WISH_TO_REMOVE_AUGMENTATION);
	public static final SystemMessage THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT = new SystemMessage(SystemMessage.THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT);
	public static final SystemMessage MONSTER_RACE_TICKETS_ARE_NO_LONGER_AVAILABLE = new SystemMessage(SystemMessage.MONSTER_RACE_TICKETS_ARE_NO_LONGER_AVAILABLE);
	public static final SystemMessage MONSTER_RACE_PAYOUT_INFORMATION_IS_NOT_AVAILABLE_WHILE_TICKETS_ARE_BEING_SOLD = new SystemMessage(SystemMessage.MONSTER_RACE_PAYOUT_INFORMATION_IS_NOT_AVAILABLE_WHILE_TICKETS_ARE_BEING_SOLD);
	public static final SystemMessage NOT_ENOUGH_FAME_POINTS = new SystemMessage(SystemMessage.NOT_ENOUGH_FAME_POINTS);
	public static final SystemMessage YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS);
	public static final SystemMessage THE_WEIGHT_AND_VOLUME_LIMIT_OF_INVENTORY_MUST_NOT_BE_EXCEEDED = new SystemMessage(SystemMessage.THE_WEIGHT_AND_VOLUME_LIMIT_OF_INVENTORY_MUST_NOT_BE_EXCEEDED);
	public static final SystemMessage YOU_ARE_NOT_A_CLAN_MEMBER = new SystemMessage(SystemMessage.YOU_ARE_NOT_A_CLAN_MEMBER);
	public static final SystemMessage YOU_ARE_SHORT_OF_ACCUMULATED_POINTS = new SystemMessage(SystemMessage.YOU_ARE_SHORT_OF_ACCUMULATED_POINTS);
	public static final SystemMessage OBSERVATION_IS_ONLY_POSSIBLE_DURING_A_SIEGE = new SystemMessage(SystemMessage.OBSERVATION_IS_ONLY_POSSIBLE_DURING_A_SIEGE);
	public static final SystemMessage ONLY_A_PARTY_LEADER_CAN_ACCESS_THE_COMMAND_CHANNEL = new SystemMessage(SystemMessage.ONLY_A_PARTY_LEADER_CAN_ACCESS_THE_COMMAND_CHANNEL);
	public static final SystemMessage YOU_DO_NOT_HAVE_AUTHORITY_TO_USE_THE_COMMAND_CHANNEL = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_AUTHORITY_TO_USE_THE_COMMAND_CHANNEL);
	public static final SystemMessage ONLY_CHANNEL_OPENER_CAN_GIVE_ALL_COMMAND = new SystemMessage(SystemMessage.ONLY_CHANNEL_OPENER_CAN_GIVE_ALL_COMMAND);
	public static final SystemMessage YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING = new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING);
	public static final SystemMessage SHOUT_AND_TRADE_CHATING_CANNOT_BE_USED_SHILE_POSSESSING_A_CURSED_WEAPON = new SystemMessage(SystemMessage.SHOUT_AND_TRADE_CHATING_CANNOT_BE_USED_SHILE_POSSESSING_A_CURSED_WEAPON);
	public static final SystemMessage ONLY_CLANS_OF_CLAN_LEVEL_1_OR_HIGHER_CAN_USE_A_CLAN_WAREHOUSE = new SystemMessage(SystemMessage.ONLY_CLANS_OF_CLAN_LEVEL_1_OR_HIGHER_CAN_USE_A_CLAN_WAREHOUSE);
	public static final SystemMessage YOU_DO_NOT_HAVE_THE_RIGHT_TO_USE_THE_CLAN_WAREHOUSE = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_THE_RIGHT_TO_USE_THE_CLAN_WAREHOUSE);
	public static final SystemMessage S1_IS_BUSY_PLEASE_TRY_AGAIN_LATER = new SystemMessage(SystemMessage.S1_IS_BUSY_PLEASE_TRY_AGAIN_LATER);
	public static final SystemMessage SERVITOR_DISAPPEASR_BECAUSE_THE_SUMMONING_TIME_IS_OVER = new SystemMessage(SystemMessage.SERVITOR_DISAPPEASR_BECAUSE_THE_SUMMONING_TIME_IS_OVER);
	public static final SystemMessage SINCE_YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_MAINTAIN_THE_SERVITORS_STAY_THE_SERVITOR_WILL_DISAPPEAR = new SystemMessage(SystemMessage.SINCE_YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_MAINTAIN_THE_SERVITORS_STAY_THE_SERVITOR_WILL_DISAPPEAR);
	public static final SystemMessage SINCE_YOU_DO_NOT_MEET_THE_REQUIREMENTS_YOU_ARE_NOT_ALLOWED_TO_ENTER_THE_PARTY_ROOM = new SystemMessage(SystemMessage.SINCE_YOU_DO_NOT_MEET_THE_REQUIREMENTS_YOU_ARE_NOT_ALLOWED_TO_ENTER_THE_PARTY_ROOM);
	public static final SystemMessage YOU_ARE_NOT_ALLOWED_TO_ENTER_THE_PARTY_ROOM = new SystemMessage(SystemMessage.YOU_ARE_NOT_ALLOWED_TO_ENTER_THE_PARTY_ROOM);
	public static final SystemMessage SWEEPER_FAILED_TARGET_NOT_SPOILED = new SystemMessage(SystemMessage.SWEEPER_FAILED_TARGET_NOT_SPOILED);
	public static final SystemMessage THE_ATTACK_HAS_BEEN_BLOCKED = new SystemMessage(SystemMessage.THE_ATTACK_HAS_BEEN_BLOCKED);
	public static final SystemMessage THE_CAPACITY_OF_THE_WAREHOUSE_HAS_BEEN_EXCEEDED = new SystemMessage(SystemMessage.THE_CAPACITY_OF_THE_WAREHOUSE_HAS_BEEN_EXCEEDED);
	public static final SystemMessage THE_TRANSACTION_IS_COMPLETE = new SystemMessage(SystemMessage.THE_TRANSACTION_IS_COMPLETE);
	public static final SystemMessage YOU_FAILED_AT_SENDING_THE_PACKAGE_BECAUSE_YOU_ARE_TOO_FAR_FROM_THE_WAREHOUSE = new SystemMessage(SystemMessage.YOU_FAILED_AT_SENDING_THE_PACKAGE_BECAUSE_YOU_ARE_TOO_FAR_FROM_THE_WAREHOUSE);
	public static final SystemMessage THE_PURCHASE_PRICE_IS_HIGHER_THAN_THE_AMOUNT_OF_MONEY_THAT_YOU_HAVE_AND_SO_YOU_CANNOT_OPEN_A_PERSONAL_STORE = new SystemMessage(SystemMessage.THE_PURCHASE_PRICE_IS_HIGHER_THAN_THE_AMOUNT_OF_MONEY_THAT_YOU_HAVE_AND_SO_YOU_CANNOT_OPEN_A_PERSONAL_STORE);
	public static final SystemMessage THE_PET_HAS_BEEN_KILLED_IF_YOU_DO_NOT_RESURRECT_IT_WITHIN_24_HOURS_THE_PETS_BODY_WILL_DISAPPEAR_ALONG_WITH_ALL_THE_PETS_ITEMS = new SystemMessage(SystemMessage.THE_PET_HAS_BEEN_KILLED_IF_YOU_DO_NOT_RESURRECT_IT_WITHIN_24_HOURS_THE_PETS_BODY_WILL_DISAPPEAR_ALONG_WITH_ALL_THE_PETS_ITEMS);
	public static final SystemMessage THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE = new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE);
	public static final SystemMessage TRYING_ON_MODE_HAS_ENDED = new SystemMessage(SystemMessage.TRYING_ON_MODE_HAS_ENDED);
	public static final SystemMessage YOU_ALREADY_POLYMORPHED_AND_CANNOT_POLYMORPH_AGAIN = new SystemMessage(SystemMessage.YOU_ALREADY_POLYMORPHED_AND_CANNOT_POLYMORPH_AGAIN);
	public static final SystemMessage YOU_CANNOT_POLYMORPH_INTO_THE_DESIRED_FORM_IN_WATER = new SystemMessage(SystemMessage.YOU_CANNOT_POLYMORPH_INTO_THE_DESIRED_FORM_IN_WATER);
	public static final SystemMessage YOU_CANNOT_POLYMORPH_WHILE_RIDING_A_PET = new SystemMessage(SystemMessage.YOU_CANNOT_POLYMORPH_WHILE_RIDING_A_PET);
	public static final SystemMessage YOU_CANNOT_POLYMORPH_WHILE_UNDER_THE_EFFECT_OF_A_SPECIAL_SKILL = new SystemMessage(SystemMessage.YOU_CANNOT_POLYMORPH_WHILE_UNDER_THE_EFFECT_OF_A_SPECIAL_SKILL);
	public static final SystemMessage YOU_CANNOT_POLYMORPH_WHILE_RIDING_A_BOAT = new SystemMessage(SystemMessage.YOU_CANNOT_POLYMORPH_WHILE_RIDING_A_BOAT);
	public static final SystemMessage YOU_CANNOT_POLYMORPH_WHEN_YOU_HAVE_SUMMONED_A_SERVITOR_PET = new SystemMessage(SystemMessage.YOU_CANNOT_POLYMORPH_WHEN_YOU_HAVE_SUMMONED_A_SERVITOR_PET);
	public static final SystemMessage YOU_HAVE_WITHDRAWN_FROM_THE_PARTY = new SystemMessage(SystemMessage.YOU_HAVE_WITHDRAWN_FROM_THE_PARTY);
	public static final SystemMessage YOU_CANNOT_TRANSFER_RIGHTS_TO_YOURSELF = new SystemMessage(SystemMessage.YOU_CANNOT_TRANSFER_RIGHTS_TO_YOURSELF);
	public static final SystemMessage YOU_CAN_TRANSFER_RIGHTS_ONLY_TO_ANOTHER_PARTY_MEMBER = new SystemMessage(SystemMessage.YOU_CAN_TRANSFER_RIGHTS_ONLY_TO_ANOTHER_PARTY_MEMBER);
	public static final SystemMessage YOU_ARE_CURRENTLY_BANNED_FROM_ACTIVITIES_RELATED_TO_THE_PRIVATE_STORE_AND_PRIVATE_WORKSHOP = new SystemMessage(SystemMessage.YOU_ARE_CURRENTLY_BANNED_FROM_ACTIVITIES_RELATED_TO_THE_PRIVATE_STORE_AND_PRIVATE_WORKSHOP);
	public static final SystemMessage YOU_HAVE_ENTERED_A_COMBAT_ZONE = new SystemMessage(SystemMessage.YOU_HAVE_ENTERED_A_COMBAT_ZONE);
	public static final SystemMessage YOU_HAVE_LEFT_A_COMBAT_ZONE = new SystemMessage(SystemMessage.YOU_HAVE_LEFT_A_COMBAT_ZONE);
	public static final SystemMessage YOU_HAVE_FAILED_TO_DELETE_THE_CHARACTER_FROM_IGNORE_LIST = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_DELETE_THE_CHARACTER_FROM_IGNORE_LIST);
	public static final SystemMessage YOU_HAVE_FAILED_TO_REGISTER_THE_USER_TO_YOUR_IGNORE_LIST = new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_REGISTER_THE_USER_TO_YOUR_IGNORE_LIST);
	public static final SystemMessage YOU_MAY_NOT_IMPOSE_A_BLOCK_ON_A_GM = new SystemMessage(SystemMessage.YOU_MAY_NOT_IMPOSE_A_BLOCK_ON_A_GM);
	public static final SystemMessage YOU_MAY_NOT_ALLY_WITH_A_CLAN_YOU_ARE_AT_BATTLE_WITH = new SystemMessage(SystemMessage.YOU_MAY_NOT_ALLY_WITH_A_CLAN_YOU_ARE_AT_BATTLE_WITH);
	public static final SystemMessage YOUR_TARGET_CANNOT_BE_FOUND = new SystemMessage(SystemMessage.YOUR_TARGET_CANNOT_BE_FOUND);
	public static final SystemMessage NOT_JOINED_IN_ANY_CLAN = new SystemMessage(SystemMessage.NOT_JOINED_IN_ANY_CLAN);
	public static final SystemMessage _ATTACK_LIST_ = new SystemMessage(SystemMessage._ATTACK_LIST_);
	public static final SystemMessage _UNDER_ATTACK_LIST_ = new SystemMessage(SystemMessage._UNDER_ATTACK_LIST_);
	public static final SystemMessage _WAR_LIST_ = new SystemMessage(SystemMessage._WAR_LIST_);
	public static final SystemMessage _PARTY_INFORMATION_ = new SystemMessage(SystemMessage._PARTY_INFORMATION_);
	public static final SystemMessage LOOTING_METHOD_FINDERS_KEEPERS = new SystemMessage(SystemMessage.LOOTING_METHOD_FINDERS_KEEPERS);
	public static final SystemMessage LOOTING_METHOD_BY_TURN = new SystemMessage(SystemMessage.LOOTING_METHOD_BY_TURN);
	public static final SystemMessage LOOTING_METHOD_BY_TURN_INCLUDING_SPOIL = new SystemMessage(SystemMessage.LOOTING_METHOD_BY_TURN_INCLUDING_SPOIL);
	public static final SystemMessage LOOTING_METHOD_RANDOM = new SystemMessage(SystemMessage.LOOTING_METHOD_RANDOM);
	public static final SystemMessage LOOTING_METHOD_RANDOM_INCLUDING_SPOIL = new SystemMessage(SystemMessage.LOOTING_METHOD_RANDOM_INCLUDING_SPOIL);
	public static final SystemMessage A_STRIDER_CAN_BE_RIDDEN_ONLY_WHEN_STANDING = new SystemMessage(SystemMessage.A_STRIDER_CAN_BE_RIDDEN_ONLY_WHEN_STANDING);
	public static final SystemMessage INSTANCE_ZONE_TIME_LIMIT = new SystemMessage(SystemMessage.INSTANCE_ZONE_TIME_LIMIT);
	public static final SystemMessage THERE_IS_NO_INSTANCE_ZONE_UNDER_A_TIME_LIMIT = new SystemMessage(SystemMessage.THERE_IS_NO_INSTANCE_ZONE_UNDER_A_TIME_LIMIT);
	public static final SystemMessage ONLY_THE_ALLIANCE_CHANNEL_LEADER_CAN_ATTEMPT_ENTRY = new SystemMessage(SystemMessage.ONLY_THE_ALLIANCE_CHANNEL_LEADER_CAN_ATTEMPT_ENTRY);
	public static final SystemMessage YOU_CANNOT_ENTER_BECAUSE_THE_CORRESPONDING_ALLIANCE_CHANNEL_S_MAXIMUM_NUMBER_OF_ENTRANTS_HAS_BEEN_REACHED = new SystemMessage(SystemMessage.YOU_CANNOT_ENTER_BECAUSE_THE_CORRESPONDING_ALLIANCE_CHANNEL_S_MAXIMUM_NUMBER_OF_ENTRANTS_HAS);
	public static final SystemMessage YOU_HAVE_ENTERED_ANOTHER_INSTANCE_ZONE_THEREFORE_YOU_CANNOT_ENTER_CORRESPONDING_DUNGEON = new SystemMessage(SystemMessage.YOU_HAVE_ENTERED_ANOTHER_INSTANCE_ZONE_THEREFORE_YOU_CANNOT_ENTER_CORRESPONDING_DUNGEON);
	public static final SystemMessage ONLY_THE_CREATOR_OF_A_CHANNEL_CAN_USE_THE_CHANNEL_DISMISS_COMMAND = new SystemMessage(SystemMessage.ONLY_THE_CREATOR_OF_A_CHANNEL_CAN_USE_THE_CHANNEL_DISMISS_COMMAND);
	public static final SystemMessage ONLY_A_PARTY_LEADER_CAN_CHOOSE_THE_OPTION_TO_LEAVE_A_CHANNEL = new SystemMessage(SystemMessage.ONLY_A_PARTY_LEADER_CAN_CHOOSE_THE_OPTION_TO_LEAVE_A_CHANNEL);
	public static final SystemMessage THE_COMMAND_CHANNEL_HAS_BEEN_FORMED = new SystemMessage(SystemMessage.THE_COMMAND_CHANNEL_HAS_BEEN_FORMED);
	public static final SystemMessage MESSAGE_ACCEPTANCE_MODE = new SystemMessage(SystemMessage.MESSAGE_ACCEPTANCE_MODE);
	public static final SystemMessage SELECT_TARGET = new SystemMessage(SystemMessage.SELECT_TARGET);
	public static final SystemMessage THE_NPC_SERVER_IS_NOT_OPERATING = new SystemMessage(SystemMessage.THE_NPC_SERVER_IS_NOT_OPERATING);
	public static final SystemMessage USED_ONLY_DURING_A_QUEST_EVENT_PERIOD = new SystemMessage(SystemMessage.USED_ONLY_DURING_A_QUEST_EVENT_PERIOD);
	public static final SystemMessage YOU_CANNOT_TELEPORT_TO_A_VILLAGE_THAT_IS_IN_A_SIEGE = new SystemMessage(SystemMessage.YOU_CANNOT_TELEPORT_TO_A_VILLAGE_THAT_IS_IN_A_SIEGE);
	public static final SystemMessage THEYRE_OFF = new SystemMessage(SystemMessage.THEYRE_OFF);
	public static final SystemMessage YOU_HAVE_BEEN_DISCONNECTED_FROM_THE_SERVER_PLEASE_LOGIN_AGAIN = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_DISCONNECTED_FROM_THE_SERVER_PLEASE_LOGIN_AGAIN);
	public static final SystemMessage YOU_HAVE_COMPLETED_THE_QUEST_FOR_3RD_OCCUPATION_CHANGE_AND_MOVED_TO_ANOTHER_CLASS_CONGRATULATIONS = new SystemMessage(SystemMessage.YOU_HAVE_COMPLETED_THE_QUEST_FOR_3RD_OCCUPATION_CHANGE_AND_MOVED_TO_ANOTHER_CLASS_CONGRATULATIONS);
	public static final SystemMessage YOU_HAVE_QUIT_THE_COMMAND_CHANNEL = new SystemMessage(SystemMessage.YOU_HAVE_QUIT_THE_COMMAND_CHANNEL);
	public static final SystemMessage CLANS_SKILL_LEVEL_HAS_INCREASED = new SystemMessage(SystemMessage.CLANS_SKILL_LEVEL_HAS_INCREASED);
	public static final SystemMessage THE_OLYMPIAD_GAME_HAS_STARTED = new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_HAS_STARTED);
	public static final SystemMessage THIS_COMMAND_CAN_ONLY_BE_USED_BY_A_NOBLESSE = new SystemMessage(SystemMessage.THIS_COMMAND_CAN_ONLY_BE_USED_BY_A_NOBLESSE);
	public static final SystemMessage PETS_AND_SERVITORS_ARE_NOT_AVAILABLE_AT_THIS_TIME = new SystemMessage(SystemMessage.PETS_AND_SERVITORS_ARE_NOT_AVAILABLE_AT_THIS_TIME);
	public static final SystemMessage WHEN_PET_OR_SERVITOR_IS_DEAD_SOULSHOTS_OR_SPIRITSHOTS_FOR_PET_OR_SERVITOR_ARE_NOT_AVAILABLE = new SystemMessage(SystemMessage.WHEN_PET_OR_SERVITOR_IS_DEAD_SOULSHOTS_OR_SPIRITSHOTS_FOR_PET_OR_SERVITOR_ARE_NOT_AVAILABLE);
	public static final SystemMessage YOU_DONT_HAVE_ENOUGH_SOULSHOTS_NEEDED_FOR_A_PET_SERVITOR = new SystemMessage(SystemMessage.YOU_DONT_HAVE_ENOUGH_SOULSHOTS_NEEDED_FOR_A_PET_SERVITOR);
	public static final SystemMessage YOU_DONT_HAVE_ENOUGH_SPIRITSHOTS_NEEDED_FOR_A_PET_SERVITOR = new SystemMessage(SystemMessage.YOU_DONT_HAVE_ENOUGH_SPIRITSHOTS_NEEDED_FOR_A_PET_SERVITOR);
	public static final SystemMessage THE_OPPOSING_PARTY_HAS_DECLINED_YOUR_CHALLENGE_TO_A_DUEL = new SystemMessage(SystemMessage.THE_OPPOSING_PARTY_HAS_DECLINED_YOUR_CHALLENGE_TO_A_DUEL);
	public static final SystemMessage POWER_OF_MANA_ENABLED = new SystemMessage(SystemMessage.POWER_OF_MANA_ENABLED);
	public static final SystemMessage NOT_ENOUGH_SPIRITSHOTS = new SystemMessage(SystemMessage.NOT_ENOUGH_SPIRITSHOTS);
	public static final SystemMessage SPIRITSHOT_DOES_NOT_MATCH_WEAPON_GRADE = new SystemMessage(SystemMessage.SPIRITSHOT_DOES_NOT_MATCH_WEAPON_GRADE);
	public static final SystemMessage CANNOT_USE_SPIRITSHOTS = new SystemMessage(SystemMessage.CANNOT_USE_SPIRITSHOTS);
	public static final SystemMessage SELECT_ITEM_TO_ENCHANT = new SystemMessage(SystemMessage.SELECT_ITEM_TO_ENCHANT);
	public static final SystemMessage THIS_FISHING_SHOT_IS_NOT_FIT_FOR_THE_FISHING_POLE_CRYSTAL = new SystemMessage(SystemMessage.THIS_FISHING_SHOT_IS_NOT_FIT_FOR_THE_FISHING_POLE_CRYSTAL);
	public static final SystemMessage YOU_MAY_NOT_USE_MULTIPLE_PETS_OR_SERVITORS_AT_THE_SAME_TIME = new SystemMessage(SystemMessage.YOU_MAY_NOT_USE_MULTIPLE_PETS_OR_SERVITORS_AT_THE_SAME_TIME);
	public static final SystemMessage A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE = new SystemMessage(SystemMessage.A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE);
	public static final SystemMessage SUMMON_A_PET = new SystemMessage(SystemMessage.SUMMON_A_PET);
	public static final SystemMessage YOU_MAY_NOT_THROW_THE_DICE_AT_THIS_TIMETRY_AGAIN_LATER = new SystemMessage(SystemMessage.YOU_MAY_NOT_THROW_THE_DICE_AT_THIS_TIMETRY_AGAIN_LATER);
	public static final SystemMessage POWER_OF_THE_SPIRITS_ENABLED = new SystemMessage(SystemMessage.POWER_OF_THE_SPIRITS_ENABLED);
	public static final SystemMessage NOT_ENOUGH_SOULSHOTS = new SystemMessage(SystemMessage.NOT_ENOUGH_SOULSHOTS);
	public static final SystemMessage SOULSHOT_DOES_NOT_MATCH_WEAPON_GRADE = new SystemMessage(SystemMessage.SOULSHOT_DOES_NOT_MATCH_WEAPON_GRADE);
	public static final SystemMessage CANNOT_USE_SOULSHOTS = new SystemMessage(SystemMessage.CANNOT_USE_SOULSHOTS);
	public static final SystemMessage A_CLAN_THAT_HAS_WITHDRAWN_OR_BEEN_EXPELLED_CANNOT_ENTER_INTO_AN_ALLIANCE_WITHIN_ONE_DAY_OF_WITHDRAWAL_OR_EXPULSION = new SystemMessage(SystemMessage.A_CLAN_THAT_HAS_WITHDRAWN_OR_BEEN_EXPELLED_CANNOT_ENTER_INTO_AN_ALLIANCE_WITHIN_ONE_DAY_OF_WITHDRAWAL_OR_EXPULSION);
	public static final SystemMessage THE_AMOUNT_IS_NOT_SUFFICIENT_AND_SO_THE_MANOR_IS_NOT_IN_OPERATION = new SystemMessage(SystemMessage.THE_AMOUNT_IS_NOT_SUFFICIENT_AND_SO_THE_MANOR_IS_NOT_IN_OPERATION);
	public static final SystemMessage THE_MANOR_INFORMATION_HAS_BEEN_UPDATED = new SystemMessage(SystemMessage.THE_MANOR_INFORMATION_HAS_BEEN_UPDATED);
	public static final SystemMessage LOTTERY_TICKET_SALES_HAVE_BEEN_TEMPORARILY_SUSPENDED = new SystemMessage(SystemMessage.LOTTERY_TICKET_SALES_HAVE_BEEN_TEMPORARILY_SUSPENDED);
	public static final SystemMessage THE_COMMAND_CHANNEL_HAS_BEEN_DISBANDED = new SystemMessage(SystemMessage.THE_COMMAND_CHANNEL_HAS_BEEN_DISBANDED);
	public static final SystemMessage YOU_ARE_NOT_IN_SIEGE = new SystemMessage(SystemMessage.YOU_ARE_NOT_IN_SIEGE);
	public static final SystemMessage YOU_HAVE_BEEN_OUSTED_FROM_THE_PARTY_ROOM = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_OUSTED_FROM_THE_PARTY_ROOM);
	public static final SystemMessage YOU_HAVE_EXITED_FROM_THE_PARTY_ROOM = new SystemMessage(SystemMessage.YOU_HAVE_EXITED_FROM_THE_PARTY_ROOM);
	public static final SystemMessage YOU_HAVE_BEEN_OUTBID = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_OUTBID);
	public static final SystemMessage THE_CLAN_HALL_WHICH_HAD_BEEN_PUT_UP_FOR_AUCTION_WAS_NOT_SOLD_AND_THEREFORE_HAS_BEEN_RELISTED = new SystemMessage(SystemMessage.THE_CLAN_HALL_WHICH_HAD_BEEN_PUT_UP_FOR_AUCTION_WAS_NOT_SOLD_AND_THEREFORE_HAS_BEEN_RELISTED);
	public static final SystemMessage YOU_HAVE_BID_IN_A_CLAN_HALL_AUCTION = new SystemMessage(SystemMessage.YOU_HAVE_BID_IN_A_CLAN_HALL_AUCTION);
	public static final SystemMessage IN_A_MOMENT_YOU_WILL_BE_TRANSPORTED_TO_THE_SITE_WHERE_THE_DUEL_WILL_TAKE_PLACE = new SystemMessage(SystemMessage.IN_A_MOMENT_YOU_WILL_BE_TRANSPORTED_TO_THE_SITE_WHERE_THE_DUEL_WILL_TAKE_PLACE);
	public static final SystemMessage LET_THE_DUEL_BEGIN = new SystemMessage(SystemMessage.LET_THE_DUEL_BEGIN);
	public static final SystemMessage THE_DUEL_HAS_ENDED_IN_A_TIE = new SystemMessage(SystemMessage.THE_DUEL_HAS_ENDED_IN_A_TIE);
	public static final SystemMessage YOU_ARE_UNABLE_TO_REQUEST_A_DUEL_AT_THIS_TIME = new SystemMessage(SystemMessage.YOU_ARE_UNABLE_TO_REQUEST_A_DUEL_AT_THIS_TIME);
	public static final SystemMessage SEVEN_SIGNS_PREPARATIONS_HAVE_BEGUN_FOR_THE_NEXT_QUEST_EVENT = new SystemMessage(SystemMessage.SEVEN_SIGNS_PREPARATIONS_HAVE_BEGUN_FOR_THE_NEXT_QUEST_EVENT);
	public static final SystemMessage SEVEN_SIGNS_THE_QUEST_EVENT_PERIOD_HAS_BEGUN_SPEAK_WITH_A_PRIEST_OF_DAWN_OR_DUSK_PRIESTESS_IF_YOU_WISH_TO_PARTICIPATE_IN_THE_EVENT = new SystemMessage(SystemMessage.SEVEN_SIGNS_THE_QUEST_EVENT_PERIOD_HAS_BEGUN_SPEAK_WITH_A_PRIEST_OF_DAWN_OR_DUSK_PRIESTESS_IF_YOU_WISH_TO_PARTICIPATE_IN_THE_EVENT);
	public static final SystemMessage SEVEN_SIGNS_QUEST_EVENT_HAS_ENDED_RESULTS_ARE_BEING_TALLIED = new SystemMessage(SystemMessage.SEVEN_SIGNS_QUEST_EVENT_HAS_ENDED_RESULTS_ARE_BEING_TALLIED);
	public static final SystemMessage SEVEN_SIGNS_THIS_IS_THE_SEAL_VALIDATION_PERIOD_A_NEW_QUEST_EVENT_PERIOD_BEGINS_NEXT_MONDAY = new SystemMessage(SystemMessage.SEVEN_SIGNS_THIS_IS_THE_SEAL_VALIDATION_PERIOD_A_NEW_QUEST_EVENT_PERIOD_BEGINS_NEXT_MONDAY);
	public static final SystemMessage THE_OLYMPIAD_GAME_HAS_ENDED = new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_HAS_ENDED);
	public static final SystemMessage THE_GAME_HAS_BEEN_CANCELLED_BECAUSE_THE_OTHER_PARTY_ENDS_THE_GAME = new SystemMessage(SystemMessage.THE_GAME_HAS_BEEN_CANCELLED_BECAUSE_THE_OTHER_PARTY_ENDS_THE_GAME);
	public static final SystemMessage THE_GAME_HAS_BEEN_CANCELLED_BECAUSE_THE_OTHER_PARTY_DOES_NOT_MEET_THE_REQUIREMENTS_FOR_JOINING_THE_GAME = new SystemMessage(SystemMessage.THE_GAME_HAS_BEEN_CANCELLED_BECAUSE_THE_OTHER_PARTY_DOES_NOT_MEET_THE_REQUIREMENTS_FOR_JOINING_THE_GAME);
	public static final SystemMessage THE_GAME_ENDED_IN_A_TIE = new SystemMessage(SystemMessage.THE_GAME_ENDED_IN_A_TIE);
	public static final SystemMessage STARTS_THE_GAME = new SystemMessage(SystemMessage.STARTS_THE_GAME);
	public static final SystemMessage THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_IS_IN_EFFECT_IT_WILL_BE_DISSOLVED_WHEN_THE_CASTLE_LORD_IS_REPLACED = new SystemMessage(SystemMessage.THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_IS_IN_EFFECT_IT_WILL_BE_DISSOLVED_WHEN_THE_CASTLE_LORD_IS_REPLACED);
	public static final SystemMessage THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_HAS_BEEN_DISSOLVED = new SystemMessage(SystemMessage.THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_HAS_BEEN_DISSOLVED);
	public static final SystemMessage THE_SIEGE_OF_THE_CLAN_HALL_IS_FINISHED = new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_IS_FINISHED);
	public static final SystemMessage THE_SIEGE_OF_THE_CLAN_HALL_HAS_BEGUN = new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_HAS_BEGUN);
	public static final SystemMessage YOUR_BASE_IS_BEING_ATTACKED = new SystemMessage(SystemMessage.YOUR_BASE_IS_BEING_ATTACKED);
	public static final SystemMessage THE_CLOAK_CANNOT_BE_EQUIPPED_BECAUSE_A_NECESSARY_ITEM_IS_NOT_EQUIPPED = new SystemMessage(SystemMessage.THE_CLOAK_CANNOT_BE_EQUIPPED_BECAUSE_A_NECESSARY_ITEM_IS_NOT_EQUIPPED);
	public static final SystemMessage THE_CLOAK_EQUIP_HAS_BEEN_REMOVED_BECAUSE_THE_ARMOR_SET_EQUIP_HAS_BEEN_REMOVED = new SystemMessage(SystemMessage.THE_CLOAK_EQUIP_HAS_BEEN_REMOVED_BECAUSE_THE_ARMOR_SET_EQUIP_HAS_BEEN_REMOVED);
	public static final SystemMessage FAILED_TO_CHANGE_ATTACK_TARGET = new SystemMessage(SystemMessage.FAILED_TO_CHANGE_ATTACK_TARGET);
	public static final SystemMessage THE_COLLECTION_HAS_FAILED = new SystemMessage(SystemMessage.THE_COLLECTION_HAS_FAILED);
	public static final SystemMessage TIME_EXPIRED = new SystemMessage(SystemMessage.TIME_EXPIRED);
	public static final SystemMessage YOU_CANNOT_FORWARD_BECAUSE_THE_PRIVATE_SHOP_OR_WORKSHOP_IS_IN_PROGRESS = new SystemMessage(SystemMessage.YOU_CANNOT_FORWARD_BECAUSE_THE_PRIVATE_SHOP_OR_WORKSHOP_IS_IN_PROGRESS);
	public static final SystemMessage YOU_CANNOT_FORWARD_DURING_AN_EXCHANGE = new SystemMessage(SystemMessage.YOU_CANNOT_FORWARD_DURING_AN_EXCHANGE);
	public static final SystemMessage YOU_CANNOT_FORWARD_IN_A_NON_PEACE_ZONE_LOCATION = new SystemMessage(SystemMessage.YOU_CANNOT_FORWARD_IN_A_NON_PEACE_ZONE_LOCATION);
	public static final SystemMessage YOU_CANNOT_FORWARD_DURING_AN_ITEM_ENHANCEMENT_OR_ATTRIBUTE_ENHANCEMENT = new SystemMessage(SystemMessage.YOU_CANNOT_FORWARD_DURING_AN_ITEM_ENHANCEMENT_OR_ATTRIBUTE_ENHANCEMENT);
	public static final SystemMessage YOU_CANNOT_SEND_A_MAIL_TO_YOURSELF = new SystemMessage(SystemMessage.YOU_CANNOT_SEND_A_MAIL_TO_YOURSELF);
	public static final SystemMessage WHEN_THE_RECIPIENT_DOESN_T_EXIST_OR_THE_CHARACTER_HAS_BEEN_DELETED_SENDING_MAIL_IS_NOT_POSSIBLE = new SystemMessage(SystemMessage.WHEN_THE_RECIPIENT_DOESN_T_EXIST_OR_THE_CHARACTER_HAS_BEEN_DELETED_SENDING_MAIL_IS_NOT_POSSIBLE);
	public static final SystemMessage THE_ITEM_THAT_YOU_RE_TRYING_TO_SEND_CANNOT_BE_FORWARDED_BECAUSE_IT_ISN_T_PROPER = new SystemMessage(SystemMessage.THE_ITEM_THAT_YOU_RE_TRYING_TO_SEND_CANNOT_BE_FORWARDED_BECAUSE_IT_ISN_T_PROPER);
	public static final SystemMessage YOU_CANNOT_FORWARD_BECAUSE_YOU_DON_T_HAVE_ENOUGH_ADENA = new SystemMessage(SystemMessage.YOU_CANNOT_FORWARD_BECAUSE_YOU_DON_T_HAVE_ENOUGH_ADENA);
	public static final SystemMessage YOU_CANNOT_RECEIVE_BECAUSE_THE_PRIVATE_SHOP_OR_WORKSHOP_IS_IN_PROGRESS = new SystemMessage(SystemMessage.YOU_CANNOT_RECEIVE_BECAUSE_THE_PRIVATE_SHOP_OR_WORKSHOP_IS_IN_PROGRESS);
	public static final SystemMessage YOU_CANNOT_RECEIVE_DURING_AN_EXCHANGE = new SystemMessage(SystemMessage.YOU_CANNOT_RECEIVE_DURING_AN_EXCHANGE);
	public static final SystemMessage YOU_CANNOT_RECEIVE_IN_A_NON_PEACE_ZONE_LOCATION = new SystemMessage(SystemMessage.YOU_CANNOT_RECEIVE_IN_A_NON_PEACE_ZONE_LOCATION);
	public static final SystemMessage YOU_CANNOT_RECEIVE_DURING_AN_ITEM_ENHANCEMENT_OR_ATTRIBUTE_ENHANCEMENT = new SystemMessage(SystemMessage.YOU_CANNOT_RECEIVE_DURING_AN_ITEM_ENHANCEMENT_OR_ATTRIBUTE_ENHANCEMENT);
	public static final SystemMessage YOU_CANNOT_RECEIVE_BECAUSE_YOU_DON_T_HAVE_ENOUGH_ADENA = new SystemMessage(SystemMessage.YOU_CANNOT_RECEIVE_BECAUSE_YOU_DON_T_HAVE_ENOUGH_ADENA);

	public static final SystemMessage _ALLIANCE_INFORMATION_ = new SystemMessage(SystemMessage._ALLIANCE_INFORMATION_);
	public static final SystemMessage _CLAN_INFORMATION_ = new SystemMessage(SystemMessage._CLAN_INFORMATION_);
	public static final SystemMessage __EQUALS__ = new SystemMessage(SystemMessage.__EQUALS__);
	public static final SystemMessage __DASHES__ = new SystemMessage(SystemMessage.__DASHES__);
	public static final SystemMessage _FRIENDS_LIST_ = new SystemMessage(SystemMessage._FRIENDS_LIST_);

	public static final SystemMessage THE_TERRITORY_WAR_WILL_BEGIN_IN_20_MINUTES_TERRITORY_RELATED_FUNCTIONS_IE__BATTLEFIELD_CHANNEL = new SystemMessage(SystemMessage.THE_TERRITORY_WAR_WILL_BEGIN_IN_20_MINUTES_TERRITORY_RELATED_FUNCTIONS_IE__BATTLEFIELD_CHANNEL);
	public static final SystemMessage TERRITORY_WAR_BEGINS_IN_10_MINUTES = new SystemMessage(SystemMessage.TERRITORY_WAR_BEGINS_IN_10_MINUTES);
	public static final SystemMessage TERRITORY_WAR_BEGINS_IN_5_MINUTES = new SystemMessage(SystemMessage.TERRITORY_WAR_BEGINS_IN_5_MINUTES);
	public static final SystemMessage TERRITORY_WAR_BEGINS_IN_1_MINUTE = new SystemMessage(SystemMessage.TERRITORY_WAR_BEGINS_IN_1_MINUTE);
	public static final SystemMessage THE_TERRITORY_WAR_REQUEST_PERIOD_HAS_ENDED = new SystemMessage(SystemMessage.THE_TERRITORY_WAR_REQUEST_PERIOD_HAS_ENDED);
	public static final SystemMessage IT_IS_NOT_A_TERRITORY_WAR_REGISTRATION_PERIOD_SO_A_REQUEST_CANNOT_BE_MADE_AT_THIS_TIME = new SystemMessage(SystemMessage.IT_IS_NOT_A_TERRITORY_WAR_REGISTRATION_PERIOD_SO_A_REQUEST_CANNOT_BE_MADE_AT_THIS_TIME);
	public static final SystemMessage YOU_VE_ALREADY_REQUESTED_A_TERRITORY_WAR_IN_ANOTHER_TERRITORY_ELSEWHERE = new SystemMessage(SystemMessage.YOU_VE_ALREADY_REQUESTED_A_TERRITORY_WAR_IN_ANOTHER_TERRITORY_ELSEWHERE);
	public static final SystemMessage TERRITORY_WAR_HAS_BEGUN = new SystemMessage(SystemMessage.TERRITORY_WAR_HAS_BEGUN);
	public static final SystemMessage TERRITORY_WAR_HAS_ENDED = new SystemMessage(SystemMessage.TERRITORY_WAR_HAS_ENDED);
	public static final SystemMessage THE_TERRITORY_WAR_CHANNEL_AND_FUNCTIONS_WILL_NOW_BE_DEACTIVATED = new SystemMessage(SystemMessage.THE_TERRITORY_WAR_CHANNEL_AND_FUNCTIONS_WILL_NOW_BE_DEACTIVATED);
	public static final SystemMessage YOU_VE_ACQUIRED_THE_WARD_MOVE_QUICKLY_TO_YOUR_FORCES__OUTPOST = new SystemMessage(SystemMessage.YOU_VE_ACQUIRED_THE_WARD_MOVE_QUICKLY_TO_YOUR_FORCES__OUTPOST);
	public static final SystemMessage THE_EFFECT_OF_TERRITORY_WARD_IS_DISAPPEARING = new SystemMessage(SystemMessage.THE_EFFECT_OF_TERRITORY_WARD_IS_DISAPPEARING);
	public static final SystemMessage YOU_CANNOT_TELEPORT_WHILE_IN_POSSESSION_OF_A_WARD = new SystemMessage(SystemMessage.YOU_CANNOT_TELEPORT_WHILE_IN_POSSESSION_OF_A_WARD);
	public static final SystemMessage ONLY_CHARACTERS_WHO_ARE_LEVEL_40_OR_ABOVE_WHO_HAVE_COMPLETED_THEIR_SECOND_CLASS_TRANSFER_CAN = new SystemMessage(SystemMessage.ONLY_CHARACTERS_WHO_ARE_LEVEL_40_OR_ABOVE_WHO_HAVE_COMPLETED_THEIR_SECOND_CLASS_TRANSFER_CAN);
	public static final SystemMessage ANOTHER_PLAYER_IS_PROBABLY_CONTROLLING_THE_TARGET = new SystemMessage(SystemMessage.ANOTHER_PLAYER_IS_PROBABLY_CONTROLLING_THE_TARGET);
	public static final SystemMessage YOU_CANNOT_ENTER_BECAUSE_YOU_ARE_NOT_ASSOCIATED_WITH_THE_CURRENT_COMMAND_CHANNEL = new SystemMessage(SystemMessage.YOU_CANNOT_ENTER_BECAUSE_YOU_ARE_NOT_ASSOCIATED_WITH_THE_CURRENT_COMMAND_CHANNEL);
	public static final SystemMessage THE_ATTEMPT_TO_ACQUIRE_THE_SKILL_HAS_FAILED_BECAUSE_OF_AN_INSUFFICIENT_CLAN_REPUTATION_SCORE = new SystemMessage(SystemMessage.THE_ATTEMPT_TO_ACQUIRE_THE_SKILL_HAS_FAILED_BECAUSE_OF_AN_INSUFFICIENT_CLAN_REPUTATION_SCORE);
	public static final SystemMessage YOUR_SELECTED_TARGET_CAN_NO_LONGER_RECEIVE_A_RECOMMENDATION = new SystemMessage(SystemMessage.YOUR_SELECTED_TARGET_CAN_NO_LONGER_RECEIVE_A_RECOMMENDATION);
	public static final SystemMessage YOU_OBTAINED_S1_RECOMMENDATIONS = new SystemMessage(SystemMessage.YOU_OBTAINED_S1_RECOMMENDATIONS);
	public static final SystemMessage FROM_NOW_ON_ANGEL_NEVIT_ABIDE_WITH_YOU = new SystemMessage(SystemMessage.FROM_NOW_ON_ANGEL_NEVIT_ABIDE_WITH_YOU);
	
	// Это статичные пакеты-триггеры
	public static final L2GameServerPacket ActionFail = new ActionFail();
	public static final L2GameServerPacket ExMPCCOpen = new ExMPCCOpen();
	public static final L2GameServerPacket ExMPCCClose = new ExMPCCClose();
	public static final L2GameServerPacket ExRestartClient = new ExRestartClient();
	public static final L2GameServerPacket ExShowQuestInfo = new ExShowQuestInfo();
	public static final L2GameServerPacket ExShowVariationCancelWindow = new ExShowVariationCancelWindow();
	public static final L2GameServerPacket ExShowVariationMakeWindow = new ExShowVariationMakeWindow();
	public static final L2GameServerPacket GameGuardQuery = new GameGuardQuery();
	public static final L2GameServerPacket LeaveWorld = new LeaveWorld();
	public static final L2GameServerPacket ServerClose = new ServerClose();
	public static final L2GameServerPacket SunRise = new SunRise();
	public static final L2GameServerPacket SunSet = new SunSet();
	public static final L2GameServerPacket ClientSetTime = new ClientSetTime();
	public static final L2GameServerPacket TradePressOtherOk = new TradePressOtherOk();
	public static final L2GameServerPacket TutorialCloseHtml = new TutorialCloseHtml();
	public static final L2GameServerPacket PartySmallWindowDeleteAll = new PartySmallWindowDeleteAll();
	public static final L2GameServerPacket ExNotifyBirthDay = new ExNotifyBirthDay();
	public static final L2GameServerPacket PledgeShowMemberListDeleteAll = new PledgeShowMemberListDeleteAll();

	public static final L2GameServerPacket ExDominionWarEnd = new ExDominionWarEnd();
	public static final L2GameServerPacket CharacterCreateSuccess = new CharacterCreateSuccess();
	public static final L2GameServerPacket CharacterCreateFail_REASON_TOO_MANY_CHARACTERS = new CharacterCreateFail(CharacterCreateFail.REASON_TOO_MANY_CHARACTERS);
	public static final L2GameServerPacket CharacterCreateFail_REASON_16_ENG_CHARS = new CharacterCreateFail(CharacterCreateFail.REASON_16_ENG_CHARS);
	public static final L2GameServerPacket CharacterCreateFail_REASON_NAME_ALREADY_EXISTS = new CharacterCreateFail(CharacterCreateFail.REASON_NAME_ALREADY_EXISTS);
	public static final SystemMessage MP_BECAME_0_ARCANE_SHIELD_DISAPPEARING = new SystemMessage(SystemMessage.MP_BECAME_0_ARCANE_SHIELD_DISAPPEARING);
	public static final SystemMessage ARCANE_SHIELD_DECREASED_YOUR_MP_BY_S1_INSTEAD_OF_HP = new SystemMessage(SystemMessage.ARCANE_SHIELD_DECREASED_YOUR_MP_BY_S1_INSTEAD_OF_HP);
	public static final SystemMessage THE_MAXIMUM_MATCHES_YOU_CAN_PARTICIPATE_IN_1_WEEK_IS_70 = new SystemMessage(3224);
	public static final SystemMessage THE_TOTAL_NUMBER_OF_MATCHES_THAT_CAN_BE_ENTERED_IN_1_WEEK_IS_60_CLASS_IRRELEVANT_INDIVIDUAL = new SystemMessage(3225);
	public static final SystemMessage THE_REQUEST_CANNOT_BE_COMPLETED_BECAUSE_THE_REQUIREMENTS_ARE_NOT_MET_IN_ORDER_TO_PARTICIPATE_IN = new SystemMessage(2941);
	public static final SystemMessage YOU_CANNOT_REGISTER_WHILE_POSSESSING_A_CURSED_WEAPON = new SystemMessage(2708);
	
	public static final SystemMessage NAME_ALREADY_EXIST_ON_THE_ADDED_LIST = new SystemMessage(3216);
	public static final SystemMessage THE_NAME_IS_NOT_CURRENTLY_REGISTERED = new SystemMessage(3217);
	public static final SystemMessage YOU_CANNOT_ADD_YOU_OWN_NAME = new SystemMessage(3221);
	public static final SystemMessage CONTACT_LIST_LIMIT_REACHED = new SystemMessage(3222);
	public static final SystemMessage YOU_CANNOT_RECEIVE_A_VITAMIN_ITEM_DURING_AN_EXCHANGE = new SystemMessage(2376);
	public static final SystemMessage YOU_CANNOT_RECEIVE_THE_VITAMIN_ITEM_BECAUSE_YOU_HAVE_EXCEED_YOUR_INVENTORY_WEIGHT_QUANTITY_LIMIT = new SystemMessage(2333);
	public static final SystemMessage THERE_ARE_NO_MORE_VITAMIN_ITEMS_TO_BE_FOUND = new SystemMessage(2335);  
	public static final SystemMessage PARTY_LOOT_CHANGE_WAS_CANCELLED = new SystemMessage(3137);
	public static final L2GameServerPacket THERE_ARE_NO_OTHER_SKILLS_TO_LEARN = new SystemMessage(750);
	
	

}