package l2ft.gameserver.clientpackets;

import l2ft.Config;
import l2ft.extensions.scripts.Functions;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Clan;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2ShortCut;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.L2SkillLearn;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.serverpackets.AcquireSkillList;
import l2ft.gameserver.serverpackets.ExStorageMaxCount;
import l2ft.gameserver.serverpackets.ShortCutRegister;
import l2ft.gameserver.serverpackets.SkillList;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.SkillSpellbookTable;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.tables.SkillTable.SubclassSkills;
import l2ft.gameserver.tables.SkillTreeTable;
import l2ft.util.Util;

public class RequestAquireSkill extends L2GameClientPacket
{
	// format: cddd(d)
	private int _id, _level, _skillType, _clanUnit;

	@Override
	public void readImpl()
	{
		_id = readD();
		_level = readD();
		_skillType = readD();
		if(_skillType == 3)
			_clanUnit = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null || activeChar.getTransformation() != 0)
			return;
		L2NpcInstance trainer = activeChar.getLastNpc();
		if((trainer == null || activeChar.getDistance(trainer.getX(), trainer.getY()) > L2Character.INTERACTION_DISTANCE) && !activeChar.isGM())
			return;

		if(SubclassSkills.isSubclassSkill(_id))
		{
			Functions.callScripts("services.SubclassSkills", "learnSkill", new Object[] { activeChar, new Integer(_id) });
			return;
		}

		activeChar.setSkillLearningClassId(activeChar.getClassId());

		L2Skill skill = SkillTable.getInstance().getInfo(_id, _level);
		if(skill == null)
			return;

		if(activeChar.getSkillLevel(_id) >= _level)
			return; // already knows the skill with this level

		boolean isClanAdditionalSkill = _skillType == AcquireSkillList.CLAN_ADDITIONAL;

	    if ((!isClanAdditionalSkill) && (activeChar.getSkillLevel(Integer.valueOf(_id)) >= _level)) 
	    	return;
	    
		
		boolean isTransferSkill = _skillType == AcquireSkillList.TRANSFER;

		if(isTransferSkill && (activeChar.getLevel() < 76 || activeChar.getClassId().getLevel() < 4))
		{
			activeChar.sendMessage("You must have 3rd class change quest completed.");
			return;
		}

		if(!isTransferSkill && !isClanAdditionalSkill && _level > 1 && activeChar.getSkillLevel(_id) != _level - 1)
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[58]", "tried to increase skill " + _id + " level to " + _level + " while having it's level " + activeChar.getSkillLevel(_id), 1);
			return;
		}

		if(!(skill.isCommon() || isTransferSkill || SkillTreeTable.getInstance().isSkillPossible(activeChar, _id, _level)))
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[64]", "tried to learn skill " + _id + " while on class " + activeChar.getActiveClass(), 1);
			return;
		}

		L2Clan clan = (_skillType == 2) || (_skillType == 3) ? activeChar.getClan() : null;
	    L2SkillLearn SkillLearn = SkillTreeTable.getSkillLearn(_id, _level, activeChar.getClassId(), clan, isTransferSkill);

	    if (SkillLearn.minLevel > activeChar.getLevel())
	    {
	    	Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[94]", "tried to learn skill " + _id + "/" + _level + " with min learn level " + SkillLearn.minLevel + " while having level " + activeChar.getLevel(), 1);
	    	return;
	    }

	    int itemCount = SkillLearn.getItemCount();
	    if (itemCount == -1)
	    {
	    	activeChar.sendActionFailed();
	    	return;
	    }

		if(_skillType == AcquireSkillList.CLAN)
			learnClanSkill(skill, activeChar.getClan(), -2);
		if (_skillType == AcquireSkillList.CLAN_ADDITIONAL)
		      learnClanSkill(skill, activeChar.getClan(), _clanUnit);
		else if(_skillType == AcquireSkillList.TRANSFER)
		{
			if(isTransferSkill)
			{
				int item_id = 0;
				switch(activeChar.getClassId())
				{
					case cardinal:
						item_id = 15307;
						break;
					case evaSaint:
						item_id = 15308;
						break;
					case shillienSaint:
						item_id = 15309;
						break;
					default:
						activeChar.sendMessage("There is no skills for your class.");
						return;
				}

				L2ItemInstance spb = activeChar.getInventory().getItemByItemId(item_id);
				if(spb == null || spb.getCount() < 1)
				{
					activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS);
					return;
				}

				L2ItemInstance ri = activeChar.getInventory().destroyItem(spb, 1, true);
				activeChar.sendPacket(SystemMessage.removeItems(ri.getItemId(), 1));

				String var = activeChar.getVar("TransferSkills" + item_id);
				if(var == null)
					var = "";
				if(!var.isEmpty())
					var += ";";
				var += skill.getId();
				activeChar.setVar("TransferSkills" + item_id, var);
			}

			activeChar.addSkill(skill, true);
			activeChar.updateStats();
			activeChar.sendUserInfo(true);
		}
		else
		{
			int _requiredSp = SkillTreeTable.getInstance().getSkillCost(activeChar, skill);

			if(activeChar.getSp() >= _requiredSp || SkillLearn.common || SkillLearn.transformation)
			{
				Integer spb_id = SkillSpellbookTable.getSkillSpellbooks().get(SkillSpellbookTable.hashCode(new int[] {
						skill.getId(), skill.getLevel() }));
				if(spb_id != null)
				{
					L2ItemInstance spb = activeChar.getInventory().getItemByItemId(spb_id);
					if(spb == null || spb.getCount() < itemCount)
					{
						activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS);
						return;
					}
					L2ItemInstance ri = activeChar.getInventory().destroyItem(spb, itemCount, true);
					activeChar.sendPacket(SystemMessage.removeItems(ri.getItemId(), itemCount));
				}

				activeChar.addSkill(skill, true);
				if(!SkillLearn.common && !SkillLearn.transformation)
					activeChar.setSp(activeChar.getSp() - _requiredSp);

				activeChar.updateStats();
				activeChar.sendUserInfo(true);

				//update all the shortcuts to this skill
				if(_level > 1)
					for(L2ShortCut sc : activeChar.getAllShortCuts())
						if(sc.id == _id && sc.type == L2ShortCut.TYPE_SKILL)
						{
							L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, _level);
							activeChar.sendPacket(new ShortCutRegister(newsc));
							activeChar.registerShortCut(newsc);
						}
			}
			else
			{
				activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_SP_TO_LEARN_SKILLS);
				return;
			}
		}

		if(SkillLearn.common)
			activeChar.sendPacket(new ExStorageMaxCount(activeChar));
		activeChar.sendPacket(new SkillList(activeChar));

		if(trainer != null)
			if(_skillType == AcquireSkillList.USUAL)
				trainer.showSkillList(activeChar);
			else if(_skillType == AcquireSkillList.FISHING)
				trainer.showFishingSkillList(activeChar);
			else if(_skillType == AcquireSkillList.CLAN)
				trainer.showClanSkillList(activeChar, false);
			else if (_skillType == AcquireSkillList.CLAN_ADDITIONAL)
		        trainer.showClanSkillList(activeChar, true);
			else if(_skillType == AcquireSkillList.TRANSFORMATION)
				trainer.showTransformationSkillList(activeChar);
			else if(_skillType == AcquireSkillList.TRANSFER)
				trainer.showTransferSkillList(activeChar);
	}

	private void learnClanSkill(L2Skill skill, L2Clan clan, int unit)
	{
		L2Player player = getClient().getActiveChar();
		if(player == null || skill == null || clan == null)
			return;
		L2NpcInstance trainer = player.getLastNpc();
		if(trainer == null)
			return;
		
		if(!player.isClanLeader())
		{
			player.sendPacket(Msg.ONLY_THE_CLAN_LEADER_IS_ENABLED);
			return;
		}
		
		if (((unit >= 0) && (!clan.isSubPledgeExist(unit))) || (skill.getMinPledgeClass() > clan.getLevel()))
	    {
			player.sendMessage("Your clan doesn't meet requirements to learn this skill.");
			return;
	    }
		
		L2SkillLearn skillLearn = SkillTreeTable.getSkillLearn(_id, _level, null, clan, false);
	    int requiredRep = SkillTreeTable.getInstance().getSkillRepCost(clan, skill, unit);
	    if (clan.getReputationScore() < requiredRep)
	    {
	    	player.sendMessage("Your clan doesn't have enough reputation points to learn this skill.");
	    	return;
	    }
	    
		int itemId = Config.ALT_DISABLE_SPELLBOOKS ? 0 : skillLearn.itemId;
	    if (itemId > 0)
	    {
	    	L2ItemInstance spb = player.getInventory().getItemByItemId(itemId);
	    	if(spb == null)
	    	{
				player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS);
	    		return;
	    	}
	    	L2ItemInstance ri = player.getInventory().destroyItem(spb, skillLearn.itemCount, true);
	    	player.sendPacket(SystemMessage.removeItems(ri.getItemId(), skillLearn.itemCount));
	    }
	    clan.incReputation(-requiredRep, false, "AquireSkill: " + _id + ", lvl " + _level);
	    if (unit < 0) 
	    	clan.addNewSkill(skill, true);
	    else
	    {
	    	L2Clan.SubPledge sp = clan.getSubPledge(unit);
	    	if (sp == null)
	    	{
	    		System.out.println("WTF? unit: " + unit + " not available?");
	    		return;
	    	}
	    	sp.addSkill(skill);
	    	sp.storeSkills();
	    }
			
	    player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EARNED_S1).addSkillName(_id, _level));
	    clan.boarcastSkillListToOnlineMembers();
	    trainer.showClanSkillList(player, unit >= 0);
	}	
}