package l2ft.gameserver.clientpackets;

import java.util.logging.Logger;

import l2ft.Config;
import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.model.items.PcInventory;
import l2ft.gameserver.serverpackets.EnchantResult;
import l2ft.gameserver.serverpackets.InventoryUpdate;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.templates.L2Item;
import l2ft.gameserver.templates.L2Weapon;
import l2ft.gameserver.xml.ItemTemplates;
import l2ft.util.Log;
import l2ft.util.Rnd;

public class RequestEnchantItem extends L2GameClientPacket
{
	protected static Logger _log = Logger.getLogger(RequestEnchantItem.class.getName());

	// Format: cd
	private int _objectId, _catalystObjId;

	@Override
	public void readImpl()
	{
		_objectId = readD();
		_catalystObjId = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isOutOfControl() || activeChar.isActionsDisabled())
		{
			activeChar.sendActionFailed();
			return;
		}

		PcInventory inventory = activeChar.getInventory();
		L2ItemInstance itemToEnchant = inventory.getItemByObjectId(_objectId);
		L2ItemInstance catalyst = _catalystObjId > 0 ? inventory.getItemByObjectId(_catalystObjId) : null;
		L2ItemInstance scroll = activeChar.getEnchantScroll();
		activeChar.setEnchantScroll(null);

		if(itemToEnchant == null || scroll == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		// Затычка, ибо клиент криво обрабатывает RequestExTryToPutEnchantSupportItem
		if(!RequestExTryToPutEnchantSupportItem.checkCatalyst(itemToEnchant, catalyst))
			catalyst = null;

		switch(scroll.getItemId())
		{
			case 13540:
				// С помощью Master Yogi's Scroll: Enchant Weapon можно точить только Staff of Master Yogi
				if(scroll.getItemId() == 13540 && itemToEnchant.getItemId() != 13539 || itemToEnchant.getItemId() == 13539 && scroll.getItemId() != 13540)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 21581:
				// Рубашка ольфа
				if(scroll.getItemId() == 21581 && itemToEnchant.getItemId() != 21580 || itemToEnchant.getItemId() == 21580 && scroll.getItemId() != 21581)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 21582:
				// Рубашка ольфа
				if(scroll.getItemId() == 21582 && itemToEnchant.getItemId() != 21580 || itemToEnchant.getItemId() == 21580 && scroll.getItemId() != 21582)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 21707:
				// Рубашка ольфа ивент
				if(scroll.getItemId() == 21707 && itemToEnchant.getItemId() != 21706 || itemToEnchant.getItemId() == 21706 && scroll.getItemId() != 21707)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 15350:
				// PC Bang Scroll D-Grade
				if(scroll.getItemId() == 15350 && !itemToEnchant.isPcBangD() || itemToEnchant.isPcBangD() && scroll.getItemId() != 15350)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 15349:
				// PC Bang Scroll C-Grade
				if(scroll.getItemId() == 15349 && !itemToEnchant.isPcBangC() || itemToEnchant.isPcBangC() && scroll.getItemId() != 15349)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 15348:
				// PC Bang Scroll B-Grade
				if(scroll.getItemId() == 15348 && !itemToEnchant.isPcBangB() || itemToEnchant.isPcBangB() && scroll.getItemId() != 15348)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 15347:
				// PC Bang Scroll A-Grade
				if(scroll.getItemId() == 15347 && !itemToEnchant.isPcBangA() || itemToEnchant.isPcBangA() && scroll.getItemId() != 15347)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			case 15346:
				// PC Bang Scroll S-Grade
				if(scroll.getItemId() == 15346 && !itemToEnchant.isPcBangS() || itemToEnchant.isPcBangS() && scroll.getItemId() != 15346)
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
			default:
				if(itemToEnchant.getItemId() == 13539 || itemToEnchant.isPcBangD() || itemToEnchant.isPcBangC() || itemToEnchant.isPcBangB() || itemToEnchant.isPcBangA() || itemToEnchant.isPcBangS() || itemToEnchant.isOlfsShirt())
				{
					activeChar.sendPacket(EnchantResult.CANCEL);
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					activeChar.sendActionFailed();
					return;
				}
				break;
		}

		Log.add(activeChar.getName() + "|Trying to enchant|" + itemToEnchant.getItemId() + "|+" + itemToEnchant.getEnchantLevel() + "|" + itemToEnchant.getObjectId(), "enchants");

		// Затычка, разрешающая точить Staff of Master Yogi
		if(!itemToEnchant.canBeEnchanted() && !isYogiStaffEnchanting(scroll, itemToEnchant))
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
			activeChar.sendActionFailed();
			return;
		}

		if(itemToEnchant.getLocation() != L2ItemInstance.ItemLocation.INVENTORY && itemToEnchant.getLocation() != L2ItemInstance.ItemLocation.PAPERDOLL)
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendPacket(Msg.YOU_CANNOT_PRACTICE_ENCHANTING_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_MANUFACTURING_WORKSHOP);
			activeChar.sendActionFailed();
			return;
		}

		if(itemToEnchant.isStackable() || (scroll = inventory.getItemByObjectId(scroll.getObjectId())) == null)
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendActionFailed();
			return;
		}

		int crystalId = itemToEnchant.getEnchantCrystalId(scroll, catalyst);

		// Затычка, разрешающая точить Staff of Master Yogi
		if(crystalId == 0 && !isYogiStaffEnchanting(scroll, itemToEnchant))
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
			activeChar.sendActionFailed();
			return;
		}

		// Staff of Master Yogi можно точить до 23
		if(!isYogiStaffEnchanting(scroll, itemToEnchant) && !isValid(itemToEnchant) || isYogiStaffEnchanting(scroll, itemToEnchant) && itemToEnchant.getEnchantLevel() >= 23)
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
			activeChar.sendActionFailed();
			return;
		}

		// Запрет на заточку чужих вещей, баг может вылезти на серверных лагах
		if(itemToEnchant.getOwnerId() != activeChar.getObjectId())
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
			activeChar.sendActionFailed();
			return;
		}

		L2ItemInstance removedScroll, removedCatalyst = null;
		synchronized (inventory)
		{
			removedScroll = inventory.destroyItem(scroll.getObjectId(), 1, true);
			if(catalyst != null)
				removedCatalyst = inventory.destroyItem(catalyst.getObjectId(), 1, true);
		}

		//tries enchant without scrolls
		if(removedScroll == null || catalyst != null && removedCatalyst == null)
		{
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendActionFailed();
			return;
		}

		int itemType = itemToEnchant.getItem().getType2();
		int safeEnchantLevel = itemToEnchant.getItem().getBodyPart() == L2Item.SLOT_FULL_ARMOR ? Config.SAFE_ENCHANT_FULL_BODY : Config.SAFE_ENCHANT_COMMON;

		double chance;
		if(itemToEnchant.getEnchantLevel() < safeEnchantLevel)
			chance = 100;
		else if(itemType == L2Item.TYPE2_WEAPON)
			if(Config.USE_ALT_ENCHANT)
			{
				if(removedScroll.isCrystallEnchantScroll())
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_WEAPON_FIGHT_CRYSTAL.size() ? Config.ENCHANT_WEAPON_FIGHT_CRYSTAL.get(Config.ENCHANT_WEAPON_FIGHT_CRYSTAL.size() - 1) : Config.ENCHANT_WEAPON_FIGHT_CRYSTAL.get(itemToEnchant.getEnchantLevel());
				else if(removedScroll.isBlessedEnchantScroll())
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_WEAPON_FIGHT_BLESSED.size() ? Config.ENCHANT_WEAPON_FIGHT_BLESSED.get(Config.ENCHANT_WEAPON_FIGHT_BLESSED.size() - 1) : Config.ENCHANT_WEAPON_FIGHT_BLESSED.get(itemToEnchant.getEnchantLevel());
				else
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_WEAPON_FIGHT.size() ? Config.ENCHANT_WEAPON_FIGHT.get(Config.ENCHANT_WEAPON_FIGHT.size() - 1) : Config.ENCHANT_WEAPON_FIGHT.get(itemToEnchant.getEnchantLevel());
			}
			else if(Config.SERVICES_RATE_BONUS_ENABLED && activeChar.getNetConnection().getBonus() > 1)
			{
				if (removedScroll.isBlessedEnchantScroll()) chance = Config.ENCHANT_CHANCE_WEAPON_BLESS_PA;
				else chance = removedScroll.isCrystallEnchantScroll() ? Config.ENCHANT_CHANCE_CRYSTAL_WEAPON_PA : Config.ENCHANT_CHANCE_WEAPON_PA;
			}
			else if (removedScroll.isBlessedEnchantScroll()) chance = Config.ENCHANT_CHANCE_WEAPON_BLESS;
			else chance = removedScroll.isCrystallEnchantScroll() ? Config.ENCHANT_CHANCE_CRYSTAL_WEAPON : Config.ENCHANT_CHANCE_WEAPON;
		else if(itemType == L2Item.TYPE2_SHIELD_ARMOR)
			if(Config.USE_ALT_ENCHANT)
			{
				if(removedScroll.isCrystallEnchantScroll())
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_ARMOR_CRYSTAL.size() ? Config.ENCHANT_ARMOR_CRYSTAL.get(Config.ENCHANT_ARMOR_CRYSTAL.size() - 1) : Config.ENCHANT_ARMOR_CRYSTAL.get(itemToEnchant.getEnchantLevel());
				else if(removedScroll.isBlessedEnchantScroll())
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_ARMOR_BLESSED.size() ? Config.ENCHANT_ARMOR_BLESSED.get(Config.ENCHANT_ARMOR_BLESSED.size() - 1) : Config.ENCHANT_ARMOR_BLESSED.get(itemToEnchant.getEnchantLevel());
				else
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_ARMOR.size() ? Config.ENCHANT_ARMOR.get(Config.ENCHANT_ARMOR.size() - 1) : Config.ENCHANT_ARMOR.get(itemToEnchant.getEnchantLevel());
			}
			else if(Config.SERVICES_RATE_BONUS_ENABLED && activeChar.getNetConnection().getBonus() > 1)
			{
				if (removedScroll.isBlessedEnchantScroll()) chance = Config.ENCHANT_CHANCE_ARMOR_BLESS_PA;
				else chance = removedScroll.isCrystallEnchantScroll() ? Config.ENCHANT_CHANCE_CRYSTAL_ARMOR_PA : Config.ENCHANT_CHANCE_ARMOR_PA;
			}
			else if (removedScroll.isBlessedEnchantScroll()) chance = Config.ENCHANT_CHANCE_ARMOR_BLESS;
			else chance = removedScroll.isCrystallEnchantScroll() ? Config.ENCHANT_CHANCE_CRYSTAL_ARMOR : Config.ENCHANT_CHANCE_ARMOR;
		else if(itemType == L2Item.TYPE2_ACCESSORY)
			if(Config.USE_ALT_ENCHANT)
			{
				if(removedScroll.isCrystallEnchantScroll())
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_ARMOR_JEWELRY_CRYSTAL.size() ? Config.ENCHANT_ARMOR_JEWELRY_CRYSTAL.get(Config.ENCHANT_ARMOR_JEWELRY_CRYSTAL.size() - 1) : Config.ENCHANT_ARMOR_JEWELRY_CRYSTAL.get(itemToEnchant.getEnchantLevel());
				else if(removedScroll.isBlessedEnchantScroll())
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_ARMOR_JEWELRY_BLESSED.size() ? Config.ENCHANT_ARMOR_JEWELRY_BLESSED.get(Config.ENCHANT_ARMOR_JEWELRY_BLESSED.size() - 1) : Config.ENCHANT_ARMOR_JEWELRY_BLESSED.get(itemToEnchant.getEnchantLevel());
				else
					chance = itemToEnchant.getEnchantLevel() > Config.ENCHANT_ARMOR_JEWELRY.size() ? Config.ENCHANT_ARMOR_JEWELRY.get(Config.ENCHANT_ARMOR_JEWELRY.size() - 1) : Config.ENCHANT_ARMOR_JEWELRY.get(itemToEnchant.getEnchantLevel());
			}
			else if(Config.SERVICES_RATE_BONUS_ENABLED && activeChar.getNetConnection().getBonus() > 1)
			{
				if (removedScroll.isBlessedEnchantScroll()) chance = Config.ENCHANT_CHANCE_ACCESSORY_BLESS_PA;
				else chance = removedScroll.isCrystallEnchantScroll() ? Config.ENCHANT_CHANCE_CRYSTAL_ACCESSORY_PA : Config.ENCHANT_CHANCE_ACCESSORY_PA;
			}
			else if (removedScroll.isBlessedEnchantScroll()) chance = Config.ENCHANT_CHANCE_ACCESSORY_BLESS;
			else chance = removedScroll.isCrystallEnchantScroll() ? Config.ENCHANT_CHANCE_CRYSTAL_ACCESSORY : Config.ENCHANT_CHANCE_ACCESSORY;
		else
		{
			System.out.println("WTF? Request to enchant " + itemToEnchant.getItemId());
			activeChar.sendPacket(EnchantResult.CANCEL);
			activeChar.sendActionFailed();
			activeChar.sendPacket(Msg.SYSTEM_ERROR);
			inventory.addItem(removedScroll);
			return;
		}

		if(scroll.isDivineEnchantScroll()) // Item Mall divine
			chance = 100;
		else if(scroll.isItemMallEnchantScroll()) // Item Mall normal/ancient
			chance += 10;

		else if(scroll.isItemMallWeaponEnhanceBackupStoneD()) // Item Mall Item 12362
			chance += 20;
		else if(scroll.isItemMallWeaponEnhanceBackupStoneC()) // Item Mall Item 12363
			chance += 18;
		else if(scroll.isItemMallWeaponEnhanceBackupStoneB()) // Item Mall Item 12364
			chance += 15;
		else if(scroll.isItemMallWeaponEnhanceBackupStoneA()) // Item Mall Item 12365
			chance += 12;
		else if(scroll.isItemMallWeaponEnhanceBackupStoneS()) // Item Mall Item 12366
			chance += 10;
		else if(scroll.isItemMallArmorEnhanceBackupStoneD()) // Item Mall Item 12367
			chance += 35;
		else if(scroll.isItemMallArmorEnhanceBackupStoneC()) // Item Mall Item 12368
			chance += 27;
		else if(scroll.isItemMallArmorEnhanceBackupStoneB()) // Item Mall Item 12369
			chance += 23;
		else if(scroll.isItemMallArmorEnhanceBackupStoneA()) // Item Mall Item 12370
			chance += 18;
		else if(scroll.isItemMallArmorEnhanceBackupStoneS()) // Item Mall Item 12371
			chance += 15;
		if(removedCatalyst != null)
			chance += removedCatalyst.getCatalystPower();

        if (Config.ALT_ENCHANT_FORMULA_FOR_PVP_SERVER) {
            int enchlvl = itemToEnchant.getEnchantLevel();
            L2Item.Grade crystaltype = itemToEnchant.getItem().getCrystalType();

            //для уровнения шансов дуальщиков и остальных на победу в PvP вставка SA в дули халявная
            if (itemType == L2Item.TYPE2_WEAPON && itemToEnchant.getItemType() == L2Weapon.WeaponType.DUAL)
                safeEnchantLevel += 1;

            if (enchlvl < safeEnchantLevel)
                chance = 100;
            else if (enchlvl > 11)
                chance = 1;
            else {
                // Выборка базового шанса
                if (itemType == L2Item.TYPE2_WEAPON) {
                    L2Weapon wepToEnchant = (L2Weapon) itemToEnchant.getItem();
                    boolean magewep = itemType == L2Item.TYPE2_WEAPON && crystaltype.cry >= L2Item.CRYSTAL_C && wepToEnchant.getPDamage() - wepToEnchant.getMDamage() <= wepToEnchant.getPDamage() * 0.4;
                    chance = !magewep ? Config.ECHANCE_WEAPON : Config.ECHANCE_MAGE_WEAPON;

                    // Штраф на двуручное оружие(немагическое)
                    if (itemToEnchant.getItem().getBodyPart() == L2Item.SLOT_LR_HAND && itemToEnchant.getItem().getItemType() == L2Weapon.WeaponType.BLUNT && !magewep)
                        chance -= Config.PENALTY_FOR_ECHANCE_TO_HAND_BLANT;
                } else
                    chance = Config.ECHANCE_ARMOR;

                int DeltaChance = 15;

                // Основная прогрессия
                for (int i = safeEnchantLevel; i < enchlvl; i++) {
                    if (i == safeEnchantLevel + 2)
                        DeltaChance -= 5;
                    if (i == safeEnchantLevel + 6)
                        DeltaChance -= 5;
                    chance -= DeltaChance;
                }

                // Учёт грейда
                int Delta2 = 5;
                for (int in = 0x00; in < crystaltype.ordinal(); in++) {
                    if (in == L2Item.CRYSTAL_C)
                        Delta2 -= 5;
                    if (in == L2Item.CRYSTAL_B)
                        Delta2 -= 5;
                    if (in == L2Item.CRYSTAL_A)
                        Delta2 -= 2;
                    if (in == L2Item.CRYSTAL_S)
                        Delta2 -= 1;
                }
                chance += Delta2;

                if (scroll.isBlessedEnchantScroll())
                    chance += 2;
                if (chance < 1)
                    chance = 1;
				if(scroll.isDivineEnchantScroll()) // Item Mall divine
					chance = 100;
				if(scroll.isItemMallEnchantScroll()) // Item Mall normal/ancient
					chance += 10;
			}
        }
		
		if(Rnd.chance(chance))
		{
			itemToEnchant.setEnchantLevel(itemToEnchant.getEnchantLevel() + 1);
			itemToEnchant.updateDatabase();

			activeChar.sendPacket(new InventoryUpdate().addModifiedItem(itemToEnchant));

			Log.add(activeChar.getName() + "|Successfully enchanted|" + itemToEnchant.getItemId() + "|to+" + itemToEnchant.getEnchantLevel() + "|" + chance, "enchants");
			Log.LogItem(activeChar, Log.EnchantItem, itemToEnchant);
			activeChar.sendPacket(EnchantResult.SUCESS);

			if(itemToEnchant.getEnchantLevel() >= (itemType == L2Item.TYPE2_WEAPON ? 6 : 5) && Config.ENCHANT_6_ANNOUNCE_TO_ALL)
			{
				activeChar.altUseSkill(SkillTable.getInstance().getInfo(21006, 1), activeChar);
				activeChar.broadcastPacket(new SystemMessage(SystemMessage.C1_HAS_SUCCESSFULY_ENCHANTED_A__S2_S3).addName(activeChar).addNumber(itemToEnchant.getEnchantLevel()).addItemName(itemToEnchant.getItemId()));
			}
		}
		else
		{
			Log.add(activeChar.getName() + "|Failed to enchant|" + itemToEnchant.getItemId() + "|+" + itemToEnchant.getEnchantLevel() + "|" + chance, "enchants");

			if(scroll.isBlessedEnchantScroll()) // фейл, но заточка блесед
			{
				itemToEnchant.setEnchantLevel(Config.SAFE_ENCHANT_LVL);
				activeChar.sendPacket(new InventoryUpdate().addModifiedItem(itemToEnchant));
				activeChar.sendPacket(Msg.FAILED_IN_BLESSED_ENCHANT_THE_ENCHANT_VALUE_OF_THE_ITEM_BECAME_0);
				activeChar.sendPacket(EnchantResult.BLESSED_FAILED);
			}
			else if(scroll.isAncientEnchantScroll()) // фейл, но заточка ancient
				activeChar.sendPacket(EnchantResult.ANCIENT_FAILED);
			else if(scroll.isDestructionEnchantScroll()) // фейл, но заточка ancient 
				activeChar.sendPacket(EnchantResult.ANCIENT_FAILED);
			else
			// фейл, разбиваем вещь
			{
				if(itemToEnchant.isEquipped())
					inventory.unEquipItemInSlot(itemToEnchant.getEquipSlot());

				L2ItemInstance destroyedItem = inventory.destroyItem(itemToEnchant.getObjectId(), 1, true);
				if(destroyedItem == null)
				{
					_log.warning("failed to destroy " + itemToEnchant.getObjectId() + " after unsuccessful enchant attempt by char " + activeChar.getName());
					activeChar.sendActionFailed();
					return;
				}

				Log.LogItem(activeChar, Log.EnchantItemFail, itemToEnchant);

				if(crystalId > 0)
				{
					L2ItemInstance crystalsToAdd = ItemTemplates.getInstance().createItem(crystalId);

					int count = (int) (itemToEnchant.getItem().getCrystalCount() * 0.87);
					if(destroyedItem.getEnchantLevel() > 3)
						count += itemToEnchant.getItem().getCrystalCount() * 0.25 * (destroyedItem.getEnchantLevel() - 3);
					if(count < 1)
						count = 1;
					crystalsToAdd.setCount(count);

					inventory.addItem(crystalsToAdd);
					Log.LogItem(activeChar, Log.Sys_GetItem, crystalsToAdd);

					activeChar.sendPacket(new EnchantResult(1, crystalsToAdd.getItemId(), count), SystemMessage.obtainItems(crystalId, count, 0));
				}
				else
					activeChar.sendPacket(EnchantResult.FAILED_NO_CRYSTALS);

				activeChar.validateItemExpertisePenalties(false, destroyedItem.getItem().getType2() == L2Item.TYPE2_SHIELD_ARMOR || destroyedItem.getItem().getType2() == L2Item.TYPE2_ACCESSORY, destroyedItem.getItem().getType2() == L2Item.TYPE2_WEAPON);
			}
		}

		activeChar.refreshOverloaded();
		activeChar.setEnchantScroll(null);
		activeChar.sendChanges();
	}

	private static boolean isYogiStaffEnchanting(L2ItemInstance scroll, L2ItemInstance itemToEnchant)
	{
		switch(scroll.getItemId())
		{
			case 13540:
				if(scroll.getItemId() == 13540 && itemToEnchant.getItemId() == 13539)
					return true;
				break;
			case 21581:
				if(scroll.getItemId() == 21581 && itemToEnchant.getItemId() == 21580)
					return true;
				break;
			case 21582:
				if(scroll.getItemId() == 21582 && itemToEnchant.getItemId() == 21580)
					return true;
				break;
			case 21707:
				if(scroll.getItemId() == 21707 && itemToEnchant.getItemId() == 21706)
					return true;
				break;
			case 15350:
				if(scroll.getItemId() == 15350 && itemToEnchant.isPcBangD())
					return true;
				break;
			case 15349:
				if(scroll.getItemId() == 15349 && itemToEnchant.isPcBangC())
					return true;
				break;
			case 15348:
				if(scroll.getItemId() == 15348 && itemToEnchant.isPcBangB())
					return true;
				break;
			case 15347:
				if(scroll.getItemId() == 15347 && itemToEnchant.isPcBangA())
					return true;
				break;
			case 15346:
				if(scroll.getItemId() == 15346 && itemToEnchant.isPcBangS())
					return true;
				break;
			default:
				if(itemToEnchant.getItemId() == 13539 || itemToEnchant.isPcBangD() || itemToEnchant.isPcBangC() || itemToEnchant.isPcBangB() || itemToEnchant.isPcBangA() || itemToEnchant.isPcBangS() || itemToEnchant.isOlfsShirt())
					return false;
				break;
		}
		return false;
	}

	private final boolean isValid(L2ItemInstance enchantItem)
	{
		if(enchantItem == null)
			return false;
		int type2 = enchantItem.getItem().getType2();
		switch(type2)
		{
			case L2Item.TYPE2_WEAPON:
				if(Config.ENCHANT_MAX_WEAPON > 0 && enchantItem.getEnchantLevel() >= Config.ENCHANT_MAX_WEAPON)
					return false;
				break;
			case L2Item.TYPE2_SHIELD_ARMOR:
				if(Config.ENCHANT_MAX_ARMOR > 0 && enchantItem.getEnchantLevel() >= Config.ENCHANT_MAX_ARMOR)
					return false;
				break;
			case L2Item.TYPE2_ACCESSORY:
				if(Config.ENCHANT_MAX_JEWELRY > 0 && enchantItem.getEnchantLevel() >= Config.ENCHANT_MAX_JEWELRY)
					return false;
				break;
			default:
				return false;
		}
		return true;
	}
}