package l2ft.gameserver.clientpackets;

import l2ft.Config;
import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.model.items.PcInventory;
import l2ft.gameserver.serverpackets.ExPutEnchantTargetItemResult;
import l2ft.gameserver.templates.L2Item;
import l2ft.util.Log;

public class RequestExTryToPutEnchantTargetItem extends L2GameClientPacket
{

	private int _objectId;

	@Override
	protected void readImpl()
	{
		_objectId = readD();
	}

	@Override
	protected void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isOutOfControl() || activeChar.isActionsDisabled())
		{
			activeChar.sendPacket(new ExPutEnchantTargetItemResult(0, 0, 0));
			return;
		}

		PcInventory inventory = activeChar.getInventory();
		L2ItemInstance itemToEnchant = inventory.getItemByObjectId(_objectId);
		L2ItemInstance scroll = activeChar.getEnchantScroll();

		if(itemToEnchant == null || scroll == null)
		{
			activeChar.sendPacket(new ExPutEnchantTargetItemResult(0, 0, 0));
			return;
		}

		switch(scroll.getItemId())
		{
			case 13540:
				// С помощью Master Yogi's Scroll: Enchant Weapon можно точить только Staff of Master Yogi
				if(scroll.getItemId() == 13540 && itemToEnchant.getItemId() != 13539 || itemToEnchant.getItemId() == 13539 && scroll.getItemId() != 13540)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 21581:
				// Рубашка ольфа
				if(scroll.getItemId() == 21581 && itemToEnchant.getItemId() != 21580 || itemToEnchant.getItemId() == 21580 && scroll.getItemId() != 21581)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 21582:
				// Рубашка ольфа
				if(scroll.getItemId() == 21582 && itemToEnchant.getItemId() != 21580 || itemToEnchant.getItemId() == 21580 && scroll.getItemId() != 21582)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 21707:
				// Рубашка ольфа ивент
				if(scroll.getItemId() == 21707 && itemToEnchant.getItemId() != 21706 || itemToEnchant.getItemId() == 21706 && scroll.getItemId() != 21707)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 15350:
				// PC Bang Scroll D-Grade
				if(scroll.getItemId() == 15350 && !itemToEnchant.isPcBangD() || itemToEnchant.isPcBangD() && scroll.getItemId() != 15350)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 15349:
				// PC Bang Scroll C-Grade
				if(scroll.getItemId() == 15349 && !itemToEnchant.isPcBangC() || itemToEnchant.isPcBangC() && scroll.getItemId() != 15349)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 15348:
				// PC Bang Scroll B-Grade
				if(scroll.getItemId() == 15348 && !itemToEnchant.isPcBangB() || itemToEnchant.isPcBangB() && scroll.getItemId() != 15348)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 15347:
				// PC Bang Scroll A-Grade
				if(scroll.getItemId() == 15347 && !itemToEnchant.isPcBangA() || itemToEnchant.isPcBangA() && scroll.getItemId() != 15347)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			case 15346:
				// PC Bang Scroll S-Grade
				if(scroll.getItemId() == 15346 && !itemToEnchant.isPcBangS() || itemToEnchant.isPcBangS() && scroll.getItemId() != 15346)
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
			default:
				if(itemToEnchant.getItemId() == 13539 || itemToEnchant.isPcBangD() || itemToEnchant.isPcBangC() || itemToEnchant.isPcBangB() || itemToEnchant.isPcBangA() || itemToEnchant.isPcBangS() || itemToEnchant.isOlfsShirt())
				{
					activeChar.sendActionFailed();
					activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
					return;
				}
				break;
		}

		Log.add(activeChar.getName() + "|Trying to put enchant|" + itemToEnchant.getItemId() + "|+" + itemToEnchant.getEnchantLevel() + "|" + itemToEnchant.getObjectId(), "enchants");

		// Затычка, разрешающая точить Staff of Master Yogi и оружие PC Bang + рубашку Ольфа
		if(!itemToEnchant.canBeEnchanted() && itemToEnchant.getItemId() != 13539  && !itemToEnchant.isPcBangD() && !itemToEnchant.isPcBangC() && !itemToEnchant.isPcBangB() && !itemToEnchant.isPcBangA() && !itemToEnchant.isPcBangS() && !itemToEnchant.isOlfsShirt() || itemToEnchant.isStackable())
		{
			activeChar.sendActionFailed();
			activeChar.sendPacket(Msg.DOES_NOT_FIT_STRENGTHENING_CONDITIONS_OF_THE_SCROLL);
			return;
		}

		if(itemToEnchant.getLocation() != L2ItemInstance.ItemLocation.INVENTORY && itemToEnchant.getLocation() != L2ItemInstance.ItemLocation.PAPERDOLL)
		{
			activeChar.sendActionFailed();
			activeChar.sendPacket(Msg.INAPPROPRIATE_ENCHANT_CONDITIONS);
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new ExPutEnchantTargetItemResult(0, 0, 0));
			activeChar.sendPacket(Msg.YOU_CANNOT_PRACTICE_ENCHANTING_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_MANUFACTURING_WORKSHOP);
			return;
		}

		if((scroll = inventory.getItemByObjectId(scroll.getObjectId())) == null)
		{
			activeChar.setEnchantScroll(null);
			activeChar.sendPacket(new ExPutEnchantTargetItemResult(0, 0, 0));
			return;
		}

		int crystalId = itemToEnchant.getEnchantCrystalId(scroll, null);

		// Затычка, разрешающая точить Staff of Master Yogi и оружие PC Bang + рубашку Ольфа
		if(crystalId == 0 && itemToEnchant.getItemId() != 13539 && !itemToEnchant.isPcBangD() && !itemToEnchant.isPcBangC() && !itemToEnchant.isPcBangB() && !itemToEnchant.isPcBangA() && !itemToEnchant.isPcBangS() && !itemToEnchant.isOlfsShirt())
		{
			activeChar.sendActionFailed();
			activeChar.sendPacket(Msg.DOES_NOT_FIT_STRENGTHENING_CONDITIONS_OF_THE_SCROLL);
			return;
		}

		int type2 = itemToEnchant.getItem().getType2();
		switch(itemToEnchant.getItemId())
		{
			case 13539:
				if(itemToEnchant.getEnchantLevel() >= 23)
				{
					activeChar.sendActionFailed();
					activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
					return;
				}
				break;
				
			case 21580:
				if(itemToEnchant.getEnchantLevel() > 10)
				{
					activeChar.sendActionFailed();
					activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
					return;
				}
				break;
			case 21706:
				if(itemToEnchant.getEnchantLevel() > 10)
				{
					activeChar.sendActionFailed();
					activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
					return;
				}
				break;
			default:
				switch(type2)
				{
				case L2Item.TYPE2_WEAPON:
					if(itemToEnchant.getEnchantLevel() >= Config.ENCHANT_MAX_WEAPON)
					{
						activeChar.sendActionFailed();
						activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
						return;
					}
					break;
				case L2Item.TYPE2_SHIELD_ARMOR:
					if(itemToEnchant.getEnchantLevel() >= Config.ENCHANT_MAX_ARMOR)
					{
						activeChar.sendActionFailed();
						activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
						return;
					}
					break;
				case L2Item.TYPE2_ACCESSORY:
					if(itemToEnchant.getEnchantLevel() >= Config.ENCHANT_MAX_JEWELRY)
					{
						activeChar.sendActionFailed();
						activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
						return;
					}
					break;
				default:
					if(itemToEnchant.getEnchantLevel() >= Config.ENCHANT_MAX)
					{
						activeChar.sendActionFailed();
						activeChar.sendMessage(new CustomMessage("l2ft.gameserver.clientpackets.RequestEnchantItem.MaxLevel", activeChar));
						return;
					}
				}
		}

		// Запрет на заточку чужих вещей, баг может вылезти на серверных лагах
		if(itemToEnchant.getOwnerId() != activeChar.getObjectId())
		{
			activeChar.sendPacket(new ExPutEnchantTargetItemResult(0, 0, 0));
			return;
		}

		activeChar.sendPacket(new ExPutEnchantTargetItemResult(1, 0, 0));
	}
}