package l2ft.gameserver.communitybbs.Manager;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.logging.Logger;

import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.gameserver.idfactory.IdFactory;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.serverpackets.ExMailArrived;
import l2ft.gameserver.serverpackets.InventoryUpdate;
import l2ft.gameserver.serverpackets.NpcHtmlMessage;
import l2ft.gameserver.serverpackets.ShowBoard;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.templates.L2Item;
import l2ft.gameserver.xml.ItemTemplates;
import l2ft.util.Util;

/**
 * @author Vital 
 * 
 */
public class AuctionBBSManager extends BaseBBSManager
{
	private static final Logger _log = Logger.getLogger(AuctionBBSManager.class.getName());
	private static ArrayList<Integer> _lotsBidded = new ArrayList<Integer>();

	public static AuctionBBSManager getInstance()
	{
		return SingletonHolder._instance;
	}

	private class LotList
	{
		private int lotId;
		private int ownerId;
		private int itemId;
		private int objectId;
		private long count;
		private int enchantLevel;
		private int currency;
		private int startingBid;
		private int bidIncrement;
		private int buyNow;
		private long endDate;
		private String endDateFormated;
		private boolean isProcessed;
	}

	private class BidList
	{
		private Integer bidderId;
		private Long bidAmount;
		private String bidDateFormated;
	}

	public ArrayList<LotList> getLots()
	{
		ArrayList<LotList> _lots = new ArrayList<LotList>();
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT * FROM auction_lots ORDER BY endDate");
			result = statement.executeQuery();
			while (result.next())
			{
				LotList lot = new LotList();
				lot.lotId = result.getInt("lotId");
				lot.ownerId = result.getInt("ownerId");
				lot.itemId = result.getInt("itemId");
				lot.objectId = result.getInt("objectId");
				lot.count = result.getLong("count");
				lot.enchantLevel = result.getInt("enchantLevel");
				lot.currency = result.getInt("currency");
				lot.startingBid = result.getInt("startingBid");
				lot.bidIncrement = result.getInt("bidIncrement");
				lot.buyNow = result.getInt("buyNow");
				lot.endDate = result.getLong("endDate");
				lot.endDateFormated = new SimpleDateFormat("MMM dd, HH:mm").format(new Date(result.getLong("endDate")));
				lot.isProcessed = result.getBoolean("processed");
				_lots.add(lot);
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
		return _lots;
	}

	public ArrayList<BidList> getBids(int lotId)
	{
		ArrayList<BidList> _bids = new ArrayList<BidList>();
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT * FROM auction_bids WHERE lotId = ? ORDER BY bidDate DESC");
			statement.setInt(1, lotId);
			result = statement.executeQuery();
			while (result.next())
			{
				BidList bid = new BidList();
				bid.bidderId = result.getInt("bidderId");
				bid.bidAmount = result.getLong("bidAmount");
				bid.bidDateFormated = new SimpleDateFormat("MMM dd, HH:mm:ss").format(new Date(result.getLong("bidDate")));
				_bids.add(bid);
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
		return _bids;
	}

	private LotList getLot(int lotId)
	{
		for (LotList temp : getLots())
			if (temp.lotId == lotId)
				return temp;
		return new LotList();
	}

	/*
	 * (non-Javadoc)
	 * @see com.l2jfree.gameserver.communitybbs.Manager.BaseBBSManager#parsecmd(java .lang.String, com.l2jfree.gameserver.model.actor.instance.L2Player)
	 */
	@Override
	public void parsecmd(String command, L2Player activeChar)
	{
		if (command.equals("_bbsauction"))
			showAuctionPage(activeChar, 1, "All", false);
		else if (command.startsWith("_bbsauction;"))
		{
			String[] params;
			params = command.split(";");
			showAuctionPage(activeChar, Integer.parseInt(params[1]), params[2], Boolean.parseBoolean(params[3]));
		}
		else if (command.startsWith("_bbsauction_view "))
			showLotPage(activeChar, Integer.parseInt(command.substring(17)));
		else if (command.startsWith("_bbsauction_bid "))
		{
			String[] params;
			params = command.split(" ");
			if (params.length > 2)
			{
				try
				{
					addBid(activeChar, Integer.parseInt(params[1]), Long.valueOf(params[2]), Integer.parseInt(params[3]), Long.valueOf(params[4].replaceAll(",", "")));
				}
				catch (NumberFormatException e)
				{
					activeChar.sendMessage("Your must enter a number to bid!");
				}
			}
			showLotPage(activeChar, Integer.parseInt(params[1]));
			showBidPage(activeChar, Integer.parseInt(params[1]));
		}
		else if (command.equals("_bbsauction_new"))
			showNewAuctionPage(activeChar);
		else if (command.startsWith("_bbsauction_new "))
		{
			int hours, currency, objectId;
			Long startingBid, increment, buyNow, count;
			String[] params;
			params = command.split(" ");
			hours = Integer.parseInt(params[1]);
			currency = Integer.parseInt(params[2]);
			startingBid = Long.parseLong(params[3]);
			increment = Long.parseLong(params[4]);
			buyNow = Long.parseLong(params[5]);
			try
			{
				count = Long.parseLong(params[6].replaceAll(",", ""));
				objectId = Integer.parseInt(params[7]);
			}
			catch (NumberFormatException e)
			{
				activeChar.sendMessage("Error: Please enter how many of the item you would like to auction!");
				showInventoryPage(activeChar, 1, hours, currency, startingBid, increment, buyNow);
				return;
			}
			if (hours > 168 || startingBid > Long.valueOf("500000000") || increment > Long.valueOf("1000000") || buyNow > Long.valueOf("2000000000") || count > Long.valueOf("2000000000") || hours < 1 || startingBid < Long.valueOf("0") || increment < Long.valueOf("0") || buyNow < Long.valueOf("0") || count < Long.valueOf("1") || (buyNow <= startingBid && buyNow > Long.valueOf("0")))
			{
				activeChar.sendMessage("Error: One of your fields was incorrect!");
				return;
			}
			addAuction(activeChar, hours, currency, startingBid, increment, buyNow, count, objectId);
			showAuctionPage(activeChar, 1, "All", false);
			showComfirmationPage(activeChar);
		}
		else if (command.startsWith("_bbsauction_new_next "))
		{
			int hours, currency, page;
			Long startingBid, increment, buyNow;
			String[] params;
			params = command.split(" ");
			try
			{
				page = Integer.parseInt(params[1].replaceAll(",", ""));
				hours = Integer.parseInt(params[2].replaceAll(",", ""));
				currency = (params[3].equals("Adena") ? 57 : 5575);
				startingBid = Long.parseLong(params[4].replaceAll(",", ""));
				increment = Long.parseLong(params[5].replaceAll(",", ""));
				buyNow = Long.parseLong(params[6].replaceAll(",", ""));
				if (hours > 168 || startingBid > Long.valueOf("500000000") || increment > Long.valueOf("1000000") || buyNow > Long.valueOf("2000000000") || hours < 1 || startingBid < Long.valueOf("0") || increment < Long.valueOf("0") || buyNow < Long.valueOf("0") || (buyNow <= startingBid && buyNow > Long.valueOf("0")))
				{
					activeChar.sendMessage("Error: One of your fields was incorrect!");
					return;
				}
				showInventoryPage(activeChar, page, hours, currency, startingBid, increment, buyNow);
			}
			catch (Exception e)
			{
				activeChar.sendMessage("Error: One of your fields was incorrect!");
			}
		}
		else if (command.startsWith("_bbsauction_buy_now "))
			showBuyNowPage(activeChar, Integer.parseInt(command.substring(20)));
		else if (command.startsWith("_bbsauction_buy_now_confirm "))
		{
			LotList lot = getLot(Integer.parseInt(command.substring(28)));
			if (addBid(activeChar, lot.lotId, 0, lot.currency, lot.buyNow))
			{
				endAuction(lot.lotId);
				showBuyNowComfirmationPage(activeChar);
			}
		}
		else
			notImplementedYet(activeChar, command);
	}

	private void showAuctionPage(L2Player activeChar, int page, String viewOnly, boolean showEnded)
	{
		int index = 0, minIndex, maxIndex;
		maxIndex = (page == 1 ? page * 9 : (page * 10) - 1);
		minIndex = maxIndex - 9;
		getLotsBiddedOn(activeChar);
		final StringBuilder html = new StringBuilder(8000);
		html.append("<html>");
		html.append("<body><br><br>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=810><tr><td width=10></td><td width=800 height=30 align=left>");
		html.append("<a action=\"bypass _bbshome\">HOME</a>&nbsp;&gt;&nbsp;<a action=\"bypass _bbsauction;1;" + viewOnly + ";" + showEnded + "\">" + (showEnded ? "Ended" : "Active") + " Auctions</a>");
		html.append("</td></tr>");
		html.append("</table>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=770 bgcolor=808080>");
		html.append("<tr><td height=10></td>");
		html.append("</tr><tr>");
		html.append("<td fixWIDTH=5></td>");
		html.append("<td><a action=\"bypass _bbsauction_new\">[New Auction]</a></td>");
		html.append("<td><a action=\"bypass _bbsauction;1;" + viewOnly + ";" + (showEnded ? "false" : "true") + "\">[" + (showEnded ? "Active" : "Ended") + " Auctions]</a></td>");
		html.append("<td fixWIDTH=350></td>");
		html.append("<td>View Only:</td>");
		html.append("<td fixWIDTH=125><combobox width=115 var=Combo list=\"All;Ancient;Big Blunt;Big Sword;Blunt;Bow;Crossbow;Dagger;Dual Fist;Dual Sword;Etc;Fist;Heavy Armor;Light Armor;Robe Armor;Material;Other;Other Armor;PetCollar;Pet;Pole;Potion;Quest;Rapier;Receipe;Scroll;Seed;Shield;Shot;Sword\"></td>");
		html.append("<td><button value=\"Go\" action=\"Write Auction Sort _ Combo Combo Combo\" width=20 height=16 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></td>");
		html.append("<td fixWIDTH=5></td>");
		html.append("</tr>");
		html.append("<tr><td height=10></td>");
		html.append("</tr></table>");
		html.append("<br>");
		html.append("<table border=0 cellspacing=0 cellpadding=2 bgcolor=808080 width=770>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=5></td>");
		html.append("<td FIXWIDTH=380 align=center>Item Up For Auction</td>");
		html.append("<td FIXWIDTH=100 align=center>Current Bid</td>");
		html.append("<td FIXWIDTH=100 align=center>Buy Now</td>");
		html.append("<td FIXWIDTH=60 align=center>Currency</td>");
		html.append("<td FIXWIDTH=120 align=center>Ending Date</td>");
		html.append("<td FIXWIDTH=5></td>");
		html.append("</tr></table>");
		for (LotList lot : getLots())
		{
			long highestBid = getHighestBid(lot.lotId);
			long currentBid = (lot.startingBid > highestBid ? lot.startingBid : highestBid);
			L2Item item = ItemTemplates.getInstance().getTemplate(lot.itemId);
			if (viewOnly.equals("All") || viewOnly.equals(item.getItemType().toString()))
			{
				if ((lot.endDate > Calendar.getInstance().getTimeInMillis() && !showEnded) || (lot.endDate < Calendar.getInstance().getTimeInMillis() && showEnded))
				{
					if (index < minIndex)
					{
						index++;
						continue;
					}
					if (index > maxIndex)
						break;
					html.append("<table border=0 cellspacing=0 cellpadding=2 width=770 " + (_lotsBidded.contains(lot.lotId) ? "bgcolor=\"333388\"" : (activeChar.getObjectId() == lot.ownerId) ? "bgcolor=\"337722\"" : "") + ">");
					html.append("<tr>");
					html.append("<td FIXWIDTH=5></td>");
					if (item.isEquipable())
						html.append("<td FIXWIDTH=380 align=left valign=top><a action=\"bypass _bbsauction_view " + lot.lotId + "\">" + "+" + lot.enchantLevel + " " + item.getName() + "</a></td>");
					else if (lot.enchantLevel != 0)
						html.append("<td FIXWIDTH=380 align=left valign=top><a action=\"bypass _bbsauction_view " + lot.lotId + "\">" + "Level " + lot.enchantLevel + " " + item.getName() + "</a></td>");
					else if (lot.count != 1l)
						html.append("<td FIXWIDTH=380 align=left valign=top><a action=\"bypass _bbsauction_view " + lot.lotId + "\">" + "(" + Util.formatAdena(lot.count) + ") " + item.getName() + "</a></td>");
					else
						html.append("<td FIXWIDTH=380 align=left valign=top><a action=\"bypass _bbsauction_view " + lot.lotId + "\">" + item.getName() + "</a></td>");
					html.append("<td FIXWIDTH=100 align=center valign=center>" + Util.formatAdena((int) currentBid) + "</td>");
					html.append("<td FIXWIDTH=100 align=center valign=center>" + (lot.buyNow == 0 || currentBid >= lot.buyNow ? "-" : (lot.ownerId == activeChar.getObjectId() || showEnded ? "Buy Now!" : "<a action=\"bypass _bbsauction_buy_now " + lot.lotId + "\">Buy Now!</a>")) + "</td>");
					html.append("<td FIXWIDTH=60 align=center valign=center><img src=icon.");
					html.append((lot.currency == 57 ? "etc_adena_i00" : "etc_ancient_adena_i00"));
					html.append(" width=32 height=20></td>");
					html.append("<td FIXWIDTH=120 align=center valign=center>" + lot.endDateFormated + "</td>");
					html.append("<td FIXWIDTH=5></td>");
					html.append("</tr></table>");
					html.append("<img src=\"L2UI.SquareBlank\" width=\"770\" height=\"3\">");
					html.append("<img src=\"L2UI.SquareGrey\" width=\"770\" height=\"1\">");
					index++;
				}
			}
		}
		html.append("<center><table width=770><tr>");
		html.append("<td align=right><button action=\"bypass _bbsauction;" + (page == 1 ? page : page - 1) + ";" + viewOnly + ";" + showEnded + "\" width=16 height=16 back=\"L2UI_ct1.button_df_left_down\" fore=\"L2UI_ct1.button_df_left\"></td>");
		for (int i = 1; i <= 10; i++)
			html.append("<td align=center fixedwidth=10><a action=\"bypass _bbsauction;" + i + ";" + viewOnly + ";" + showEnded + "\">" + i + "</a></td>");
		html.append("<td align=left><button action=\"bypass _bbsauction;" + (page + 1) + ";" + viewOnly + ";" + showEnded + "\" width=16 height=16 back=\"L2UI_ct1.button_df_right_down\" fore=\"L2UI_ct1.button_df_right\"></td>");
		html.append("</tr></table>");
		html.append("</body></html>");
		separateAndSend(html.toString(), activeChar);
	}

	private void showLotPage(L2Player activeChar, int lotId)
	{
		int count = 0;
		int currentBid;
		int bidCount;
		LotList lot;
		try
		{
			currentBid = getHighestBid(lotId);
			lot = getLot(lotId);
		}
		catch (Exception e)
		{
			activeChar.sendMessage("This auction lot does not exist anymore.");
			_log.warning(e.getMessage());
			return;
		}
		bidCount = countBids(lot.lotId);
		L2Item item = ItemTemplates.getInstance().getTemplate(lot.itemId);
		if (currentBid == 0)
			currentBid = lot.startingBid;
		final StringBuilder html = new StringBuilder();
		html.append("<html>");
		html.append("<body><br><br>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=810><tr><td width=10></td><td width=800 height=30 align=left>");
		html.append("<a action=\"bypass _bbshome\">HOME</a>&nbsp;&gt;&nbsp;<a action=\"bypass _bbsauction\">Active Auctions</a>");
		html.append("</td></tr>");
		html.append("</table>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=770 bgcolor=808080>");
		html.append("<tr><td height=10></td></tr>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=5 height=20></td>");
		html.append("<td FIXWIDTH=100 height=20 align=right>Item:&nbsp;</td>");
		if (item.isEquipable())
			html.append("<td FIXWIDTH=360 height=20 align=left>" + "+" + lot.enchantLevel + " " + item.getName() + "</td>");
		else if (lot.enchantLevel != 0)
			html.append("<td FIXWIDTH=360 height=20 align=left>" + "Level " + lot.enchantLevel + " " + item.getName() + "</td>");
		else if (lot.count != 1l)
			html.append("<td FIXWIDTH=360 height=20 align=left>(" + Util.formatAdena(lot.count) + ") " + item.getName() + "</td>");
		else
			html.append("<td FIXWIDTH=360 height=20 align=left>" + item.getName() + "</td>");
		html.append("<td FIXWIDTH=150 height=20 align=right>End Date:&nbsp;</td>");
		html.append("<td FIXWIDTH=150 height=20 align=left>" + lot.endDateFormated + "</td>");//
		html.append("<td fixWIDTH=5 height=20></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=5 height=20></td>");
		html.append("<td FIXWIDTH=100 height=20 align=right>Owner:&nbsp;</td>");
		html.append("<td FIXWIDTH=360 height=20 align=left>" + getCharName(lot.ownerId) + "</td>");//
		html.append("<td FIXWIDTH=150 height=20 align=right>Status:&nbsp;</td>");
		if (lot.endDate > Calendar.getInstance().getTimeInMillis())
			html.append("<td FIXWIDTH=150 height=20 align=left><font color=00FF00>Active</font></td>");//
		else
			html.append("<td FIXWIDTH=150 height=20 align=left><font color=FF0000>Ended</font></td>");//
		html.append("<td fixWIDTH=5 height=20></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=5 height=20></td>");
		html.append("<td FIXWIDTH=100 height=20 align=right>Bids:&nbsp;</td>");
		html.append("<td FIXWIDTH=360 height=20 align=left>" + Util.formatAdena(bidCount) + "</td>");//
		html.append("<td FIXWIDTH=150 height=20 align=right>Buy Now Price:&nbsp;</td>");
		if (lot.buyNow == 0)
			html.append("<td FIXWIDTH=150 height=20 align=left>None</td>");
		else
			html.append("<td FIXWIDTH=150 height=20 align=left>" + lot.buyNow + "</td>");
		html.append("<td fixWIDTH=5 height=20></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=5 height=20></td>");
		html.append("<td FIXWIDTH=100 height=20 align=right>Current Bid:&nbsp;</td>");
		html.append("<td FIXWIDTH=360 height=20 align=left>" + Util.formatAdena(getHighestBid(lotId)) + "</td>");//
		html.append("<td FIXWIDTH=150 height=20 align=right>Currency:&nbsp;</td>");
		html.append("<td FIXWIDTH=150 height=20 align=left><img src=icon.");
		html.append(lot.currency == 57 ? "etc_adena_i00" : "etc_ancient_adena_i00");
		html.append(" width=32 height=16></td>");
		html.append("<td fixWIDTH=5 height=20></td>");
		html.append("</tr>");
		html.append("</table>");
		html.append("<br>&nbsp;&nbsp;&nbsp;Last 10 bids<br>");
		html.append("<table border=0 cellspacing=0 cellpadding=2 bgcolor=808080 width=770>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=85></td>");
		html.append("<td FIXWIDTH=200 align=center>Bidder</td>");
		html.append("<td FIXWIDTH=200 align=center>Bid</td>");
		html.append("<td FIXWIDTH=200 align=center>Date</td>");
		html.append("<td FIXWIDTH=85></td>");
		html.append("</tr></table>");
		for (BidList bid : getBids(lot.lotId))
		{
			count++;
			if (count > 10)
				break;
			html.append("<table border=0 cellspacing=0 cellpadding=2 width=770><tr>");
			html.append("<td FIXWIDTH=85></td>");
			html.append("<td FIXWIDTH=200 align=center>" + getCharName(bid.bidderId) + "</td>");
			html.append("<td FIXWIDTH=200 align=center>(" + Util.formatAdena(bid.bidAmount.intValue()) + ") ");
			html.append(lot.currency == 57 ? "Adena" : "Ancient Adena");
			html.append("</td><td FIXWIDTH=200 align=center>" + bid.bidDateFormated + "</td>");
			html.append("<td FIXWIDTH=85></td>");
			html.append("</tr></table>");
			html.append("<img src=\"L2UI.SquareBlank\" width=\"770\" height=\"3\">");
			html.append("<img src=\"L2UI.SquareGrey\" width=\"770\" height=\"1\">");
		}
		html.append("<table border=0 cellspacing=0 cellpadding=2 width=770><tr>");
		if (lot.ownerId == activeChar.getObjectId())
			html.append("<td>You may not place a bid on your own auction lot.</td>");
		else if (lot.endDate < Calendar.getInstance().getTimeInMillis())
			html.append("<td>This auction lot has ended.</td>");
		else
		{
			html.append("<td>Please enter a bid more than " + Util.formatAdena((bidCount != 0 ? (currentBid + lot.bidIncrement) : currentBid)) + " ");
			html.append(lot.currency == 57 ? "Adena" : "Ancient Adena");
			html.append("</td></tr><tr>");
			html.append("<td><button value=\"Bid Window\" action=\"bypass _bbsauction_bid " + lot.lotId + "\" width=100 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></td>");
		}
		html.append("</tr></table>");
		html.append("</body></html>");
		separateAndSend(html.toString(), activeChar);
	}

	private void showBidPage(L2Player activeChar, int lotId)
	{
		LotList lot = null;
		try
		{
			lot = getLot(lotId);
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		L2Item item = ItemTemplates.getInstance().getTemplate(lot.itemId);
		int currentBid = getHighestBid(lotId);
		int bidCount = countBids(lot.lotId);
		if (currentBid == 0)
			currentBid = lot.startingBid;
		NpcHtmlMessage nhm = new NpcHtmlMessage(5);
		StringBuilder html = new StringBuilder("<html><body>");
		html.append("<title>Bidding Window</title>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280 bgcolor=808080>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=100 align=right>Item:</td>");
		html.append("<td FIXWIDTH=180>" + item.getName() + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Enchant Level:</td>");
		html.append("<td FIXWIDTH=180 valign=top>" + lot.enchantLevel + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Count:</td>");
		html.append("<td FIXWIDTH=180>" + Util.formatAdena(lot.count) + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Currency:</td>");
		html.append("<td FIXWIDTH=180><img src=icon." + (lot.currency == 57 ? "etc_adena_i00" : "etc_ancient_adena_i00") + " width=32 height=16></td>");
		html.append("</tr></table>");
		html.append("<br>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280><tr><td FIXWIDTH=280 height=20></td></tr></table>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280 bgcolor=808080>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=100 align=right>Current Bid:</td>");
		html.append("<td FIXWIDTH=180>" + Util.formatAdena(currentBid) + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Buy Now Price:</td>");
		html.append("<td FIXWIDTH=180 valign=top>" + lot.buyNow + "</td>");
		html.append("</tr></table>");
		html.append("<br><br><br>");
		if (lot.ownerId == activeChar.getObjectId())
			html.append("You may not place a bid on your own auction lot.");
		else if (lot.endDate < Calendar.getInstance().getTimeInMillis())
			html.append("The auction has ended.");
		else
		{
			html.append("<td>Please enter a bid more then " + Util.formatAdena((bidCount != 0 ? (currentBid + lot.bidIncrement) : currentBid)) + " " + (lot.currency == 57 ? "Adena" : "Ancient Adena") + "</td>");
			html.append("<edit var=\"Value\" width=150 height=11 length=\"13\"><br>");
			html.append("<button value=\"Place Bid\" action=\"bypass -h _bbsauction_bid " + lot.lotId + " " + lot.bidIncrement + " " + lot.currency + " $Value\" width=80 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\">");
		}
		html.append("</body></html>");
		nhm.setHtml(html.toString());
		activeChar.sendPacket(nhm);
		// Send a Server->Client ActionFailed to the L2Player in order to
		// avoid that the client wait another packet
		activeChar.sendActionFailed();
	}

	private void showNewAuctionPage(L2Player activeChar)
	{
		NpcHtmlMessage nhm = new NpcHtmlMessage(5);
		StringBuilder html = new StringBuilder("<html><body>");
		html.append("<title>New Auction Lot Window</title>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=280 height=30>Please enter how many hours you want the auction to last. (1 - 168)</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30><edit var=\"Time\" width=50 height=11 length=\"3\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30>Please select a currency.</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30><combobox width=200 var=Currency list=\"Adena;Ancient_Adena\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30>Please enter a minimum starting bid price. (0 - 500,000,000)</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30><edit var=\"StartingBid\" width=150 height=11 length=\"11\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30>Please enter a bid increment. (1 - 1,000,000)</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30><edit var=\"Increment\" width=150 height=11 length=\"9\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30>Please enter a buy now price. (0 = Off , 1 - 2,000,000,000)</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30><edit var=\"BuyNow\" width=150 height=11 length=\"13\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30 align=right><button value=\"Continue to next page\" action=\"bypass _bbsauction_new_next 1 $Time $Currency $StartingBid $Increment $BuyNow\" width=175 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=20></td>");
		html.append("</tr>");
		html.append("</table>");
		html.append("</body></html>");
		nhm.setHtml(html.toString());
		activeChar.sendPacket(nhm);
		// Send a Server->Client ActionFailed to the L2Player in order to
		// avoid that the client wait another packet
		activeChar.sendActionFailed();
	}

	private void showInventoryPage(L2Player activeChar, int page, int time, int currency, long startingBid, long increment, long buyNow)
	{
		int index = 0, minIndex, maxIndex;
		maxIndex = (page == 1 ? page * 19 : (page * 20) - 1);
		minIndex = maxIndex - 19;
		NpcHtmlMessage nhm = new NpcHtmlMessage(5);
		StringBuilder html = new StringBuilder("<html><body>");
		html.append("<title>New Auction Lot Window</title>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280><tr>");
		html.append("<td FIXWIDTH=280 height=20 align=left><button value=\"<< Prev\" action=\"bypass _bbsauction_new_next " + (page == 1 ? page : page - 1) + " " + time + " " + currency + " " + startingBid + " " + increment + " " + buyNow + "\" width=75 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></td>");
		html.append("<td FIXWIDTH=280 height=20 align=right><button value=\"Next >>\" action=\"bypass _bbsauction_new_next " + (page + 1) + " " + time + " " + currency + " " + startingBid + " " + increment + " " + buyNow + "\" width=75 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></td>");
		html.append("</tr></table>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=280 height=30>Please enter how many of the item your going to put up for auction. If the item is not stackable, this field will not be read. (1 - 2,000,000,000)</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30><edit var=\"Count\" width=150 height=11 length=\"13\"></td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=280 height=30>Please select an item that you wish to put up for auction. If items are not listed on this page, please use the buttons at the top to help find them.</td>");
		html.append("</tr>");
		for (L2ItemInstance item : activeChar.getInventory().getItems())
		{
			if (((item.isTradeable() && !item.isAugmented() && !item.isShadowItem())) && !item.isEquipped())
			{
				if (index < minIndex)
				{
					index++;
					continue;
				}
				if (index > maxIndex)
					break;
				if (item.isStackable())
				{
					html.append("<tr>");
					html.append("<td FIXWIDTH=280><a action=\"bypass _bbsauction_new " + time + " " + currency + " " + startingBid + " " + increment + " " + buyNow + " $Count " + item.getObjectId() + "\">(" + Util.formatAdena(item.getCount()) + ") " + item.getName() + "</a></td>");
					html.append("</tr>");
				}
				else
				{
					html.append("<tr>");
					if (item.getEnchantLevel() > 0)
						html.append("<td FIXWIDTH=280><a action=\"bypass _bbsauction_new " + time + " " + currency + " " + startingBid + " " + increment + " " + buyNow + " " + 1 + " " + item.getObjectId() + "\">" + (item.isEquipable() ? "+" : "Level ") + item.getEnchantLevel() + " " + item.getName() + "</a></td>");
					else
						html.append("<td FIXWIDTH=280><a action=\"bypass _bbsauction_new " + time + " " + currency + " " + startingBid + " " + increment + " " + buyNow + " " + 1 + " " + item.getObjectId() + "\">" + item.getName() + "</a></td>");
					html.append("</tr>");
				}
				html.append("<tr>");
				html.append("<td FIXWIDTH=280 height=15></td>");
				html.append("</tr>");
				index++;
			}
		}
		html.append("<tr>");
		html.append("<td FIXWIDTH=280 height=20></td>");
		html.append("</tr>");
		html.append("</table>");
		html.append("</body></html>");
		nhm.setHtml(html.toString());
		activeChar.sendPacket(nhm);
		// Send a Server->Client ActionFailed to the L2Player in order to
		// avoid that the client wait another packet
		activeChar.sendActionFailed();
	}

	private void showBuyNowPage(L2Player activeChar, int lotId)
	{
		LotList lot = null;
		try
		{
			lot = getLot(lotId);
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		L2Item item = ItemTemplates.getInstance().getTemplate(lot.itemId);
		int currentBid = getHighestBid(lotId);
		if (currentBid == 0)
			currentBid = lot.startingBid;
		NpcHtmlMessage nhm = new NpcHtmlMessage(5);
		StringBuilder html = new StringBuilder("<html><body>");
		html.append("<title>Buy Now Comfirmation Window</title>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280 bgcolor=808080>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=100 align=right>Item:</td>");
		html.append("<td FIXWIDTH=180>" + item.getName() + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Enchant Level:</td>");
		html.append("<td FIXWIDTH=180 valign=top>" + lot.enchantLevel + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Count:</td>");
		html.append("<td FIXWIDTH=180>" + Util.formatAdena(lot.count) + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Currency:</td>");
		html.append("<td FIXWIDTH=180><img src=icon." + (lot.currency == 57 ? "etc_adena_i00" : "etc_ancient_adena_i00") + " width=32 height=16></td>");
		html.append("</tr></table>");
		html.append("<br>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280><tr><td FIXWIDTH=280 height=20></td></tr></table>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280 bgcolor=808080>");
		html.append("<tr>");
		html.append("<td FIXWIDTH=100 align=right>Current Bid:</td>");
		html.append("<td FIXWIDTH=180>" + Util.formatAdena(currentBid) + "</td>");
		html.append("</tr><tr>");
		html.append("<td FIXWIDTH=100 align=right>Buy Now Price:</td>");
		html.append("<td FIXWIDTH=180 valign=top>" + Util.formatAdena(lot.buyNow) + "</td>");
		html.append("</tr></table>");
		html.append("<table border=0 cellspacing=0 cellpadding=0 width=280>");
		html.append("<tr>");
		if (lot.ownerId == activeChar.getObjectId())
			html.append("<td>You may not place a bid on your own auction lot.</td>");
		else if (lot.endDate < Calendar.getInstance().getTimeInMillis())
			html.append("<td>This auction lot has ended.</td>");
		else
		{
			html.append("<td FIXWIDTH=280>Are you sure you wish to buy now?</td>");
			html.append("</tr><tr>");
			html.append("<td FIXWIDTH=280 align=center><button value=\"BUY NOW!\" action=\"bypass _bbsauction_buy_now_confirm " + lot.lotId + "\" width=100 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></td>");
		}
		html.append("</tr>");
		html.append("</table>");
		html.append("</body></html>");
		nhm.setHtml(html.toString());
		activeChar.sendPacket(nhm);
		// Send a Server->Client ActionFailed to the L2Player in order to
		// avoid that the client wait another packet
		activeChar.sendActionFailed();
	}

	private void showComfirmationPage(L2Player activeChar)
	{
		NpcHtmlMessage nhm = new NpcHtmlMessage(5);
		StringBuilder html = new StringBuilder("<html><body>");
		html.append("<title>Auction Comfirmation Window</title>");
		html.append("Congrats! Your item is now up for auction!");
		html.append("</body></html>");
		nhm.setHtml(html.toString());
		activeChar.sendPacket(nhm);
		// Send a Server->Client ActionFailed to the L2Player in order to
		// avoid that the client wait another packet
		activeChar.sendActionFailed();
	}

	private void showBuyNowComfirmationPage(L2Player activeChar)
	{
		NpcHtmlMessage nhm = new NpcHtmlMessage(5);
		StringBuilder html = new StringBuilder("<html><body>");
		html.append("<title>Auction Buy Now Comfirmation Window</title>");
		html.append("Congrats! You have won the auction by default, the auction will soon be processed.");
		html.append("</body></html>");
		nhm.setHtml(html.toString());
		activeChar.sendPacket(nhm);
		// Send a Server->Client ActionFailed to the L2Player in order to
		// avoid that the client wait another packet
		activeChar.sendActionFailed();
	}

	private void addAuction(L2Player activeChar, int hours, int currency, long statingBid, long increment, long buyNow, long count, int objectId)
	{
		long epochHours = hours * 60 * 60 * 1000;
		L2ItemInstance item = activeChar.getInventory().getItemByObjectId(objectId);
		if (activeChar.getInventory().destroyItem(objectId, (int) count, true) != null)
		{
			ThreadConnection con = null;
			FiltredPreparedStatement statement = null;
			ResultSet rs = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("INSERT INTO auction_lots (ownerId, itemId, objectId, count, enchantLevel, currency, startingBid, bidIncrement, buyNow, endDate) VALUES (?,?,?,?,?,?,?,?,?,?)");
				statement.setInt(1, activeChar.getObjectId());
				statement.setInt(2, item.getItemId());
				statement.setInt(3, (count == 1 ? objectId : IdFactory.getInstance().getNextId()));
				statement.setLong(4, count);
				statement.setInt(5, item.getEnchantLevel());
				statement.setInt(6, currency);
				statement.setLong(7, statingBid);
				statement.setLong(8, increment);
				statement.setLong(9, buyNow);
				statement.setLong(10, Calendar.getInstance().getTimeInMillis() + epochHours);
				statement.execute();
				statement.close();
				/* TODO
				if (activeChar.isGM())
					com.l2emu.gameserver.util.GMAudit.auditGMAction(activeChar.getName(), "auction", "_bbsauction_new", count + " - " + item.getEnchantLevel() + " - " + item.getItemId() + " - " + item.getName() + " - " + objectId);
				*/
			}
			catch (Exception e)
			{
				_log.warning(e.getMessage());
			}
			finally
			{
				DatabaseUtils.closeDatabaseCSR(con, statement, rs);
			}
		}
	}

	public void processAuctions()
	{
		boolean playerWasOnline = false;
		BidList bid;
		for (LotList lot : getLots())
		{
			if (lot.endDate < Calendar.getInstance().getTimeInMillis() && !lot.isProcessed)
			{
				bid = getHighestBidder(lot.lotId);
				L2Item itemWon = ItemTemplates.getInstance().getTemplate(lot.itemId);
				if (countBids(lot.lotId) != 0)
				{
					if (bid != null)
					{
						ThreadConnection con = null;
						FiltredPreparedStatement statement = null;
						ResultSet rs = null;
						try
						{
							con = L2DatabaseFactory.getInstance().getConnection();
							for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
								if (player.getObjectId() == lot.ownerId)
								{
									L2ItemInstance item = player.getInventory().addItem(lot.currency, bid.bidAmount.intValue());
									InventoryUpdate iu = new InventoryUpdate();
									iu.addItem(item);
									player.sendPacket(iu);
									player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1).addItemName(item.getItemId()).addNumber(bid.bidAmount.intValue()));
									player.sendPacket(new SystemMessage(SystemMessage.YOUVE_GOT_MAIL));
									player.sendPacket(ExMailArrived.STATIC_PACKET);
									playerWasOnline = true;
									break;
								}
							if (!playerWasOnline)
								addItemToInventory(con, lot.ownerId, IdFactory.getInstance().getNextId(), lot.currency, bid.bidAmount, 0);
							sendMail(con, lot.ownerId, "Auctioneer", "Your auction was success!", "Item Sold: " + (lot.count != 1 ? "(" + Util.formatAdena(lot.count) + ")" : (itemWon.isEquipable() ? "+" : "Level ") + lot.enchantLevel) + " " + itemWon.getName() + "<br>Check your inventory for " + Util.formatAdena(bid.bidAmount.intValue()) + " " + ItemTemplates.getInstance().getTemplate(lot.currency));
							for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
								if (player.getObjectId() == bid.bidderId)
								{
									player.sendPacket(new SystemMessage(SystemMessage.YOUVE_GOT_MAIL));
									player.sendPacket(ExMailArrived.STATIC_PACKET);
									break;
								}
							addItemToInventory(con, bid.bidderId, (lot.count == 1 ? lot.objectId : IdFactory.getInstance().getNextId()), lot.itemId, lot.count, lot.enchantLevel);
							sendMail(con, bid.bidderId, "Bidder", "You won an auction!!", "Please relog to see your " + (lot.count != 1 ? "(" + Util.formatAdena(lot.count) + ")" : (itemWon.isEquipable() ? "+" : "Level ") + lot.enchantLevel) + " " + itemWon.getName());
							statement = con.prepareStatement("UPDATE auction_lots SET processed = 'true' WHERE lotId = ?");
							statement.setInt(1, lot.lotId);
							statement.execute();
							statement.close();
						}
						catch (Exception e)
						{
							_log.warning(e.getMessage());
						}
						finally
						{
							DatabaseUtils.closeDatabaseCSR(con, statement, rs);
						}
					}
				}
				else
				{
					ThreadConnection con = null;
					FiltredPreparedStatement statement = null;
					ResultSet rs = null;
					try
					{
						con = L2DatabaseFactory.getInstance().getConnection();
						for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
							if (player.getObjectId() == bid.bidderId)
							{
								player.sendPacket(new SystemMessage(SystemMessage.YOUVE_GOT_MAIL));
								player.sendPacket(ExMailArrived.STATIC_PACKET);
								break;
							}
						addItemToInventory(con, lot.ownerId, (lot.count == 1 ? lot.objectId : IdFactory.getInstance().getNextId()), lot.itemId, lot.count, lot.enchantLevel);
						sendMail(con, lot.ownerId, "Auctioneer", "Your auction did not have any bids", "Please relog to see your " + (lot.count != 1 ? "(" + Util.formatAdena(lot.count) + ")" : (itemWon.isEquipable() ? "+" : "Level ") + lot.enchantLevel) + " " + itemWon.getName());
						statement = con.prepareStatement("UPDATE auction_lots SET processed = 'true' WHERE lotId = ?");
						statement.setInt(1, lot.lotId);
						statement.execute();
						statement.close();
					}
					catch (Exception e)
					{
						_log.warning(e.getMessage());
					}
					finally
					{
						DatabaseUtils.closeDatabaseCSR(con, statement, rs);
					}
				}
			}
		}
	}

	public void removeOldAuctions()
	{
		for (LotList lot : getLots())
		{
			if ((lot.endDate + Long.valueOf("604800000")) < Calendar.getInstance().getTimeInMillis() && lot.isProcessed)
			{
				ThreadConnection con = null;
				FiltredPreparedStatement statement = null;
				ResultSet rs = null;
				try
				{
					con = L2DatabaseFactory.getInstance().getConnection();
					statement = con.prepareStatement("DELETE FROM auction_bids WHERE lotId = ?");
					statement.setInt(1, lot.lotId);
					statement.execute();
					statement.close();
					statement = con.prepareStatement("DELETE FROM auction_lots WHERE lotId = ?");
					statement.setInt(1, lot.lotId);
					statement.execute();
					statement.close();
				}
				catch (Exception e)
				{
					_log.warning(e.getMessage());
				}
				finally
				{
					DatabaseUtils.closeDatabaseCSR(con, statement, rs);
				}
			}
		}
	}

	private boolean addBid(L2Player activeChar, int lotId, long bidIncrement, int currency, long bidAmount)
	{
		boolean playerWasOnline = false;
		long currentBid = getHighestBid(lotId);
		int prevBidderId, prevBidAmount;
		LotList lot = getLot(lotId);
		int bidCount = countBids(lot.lotId);
	if (currentBid == 0)
			currentBid = lot.startingBid;
		if (lot.endDate < Calendar.getInstance().getTimeInMillis())
		{
			activeChar.sendMessage("This auction lot has already ended and is no longer taking bids.");
			return false;
		}
		if (bidAmount <= activeChar.getInventory().getCountOf(currency))
		{
			if (bidAmount > (bidCount != 0 ? currentBid + bidIncrement :  currentBid))
			{
				ThreadConnection con = null;
				FiltredPreparedStatement statement = null;
				ResultSet rs = null;
				try
				{
					try
					{
						con = L2DatabaseFactory.getInstance().getConnection();
						statement = con.prepareStatement("SELECT * FROM auction_bids WHERE lotId = ? ORDER BY bidId DESC");
						statement.setInt(1, lotId);
						ResultSet result = statement.executeQuery();
						result.next();
						prevBidderId = result.getInt("bidderId");
						prevBidAmount = result.getInt("bidAmount");
						statement.close();
						for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
							if (player.getObjectId() == prevBidderId)
							{
								L2ItemInstance item = player.getInventory().addItem(currency, prevBidAmount);
								InventoryUpdate iu = new InventoryUpdate();
								iu.addItem(item);
								player.sendPacket(iu);
								player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1).addItemName(item.getItemId()).addNumber(prevBidAmount));
								player.sendPacket(new SystemMessage(SystemMessage.YOUVE_GOT_MAIL));
								player.sendPacket(ExMailArrived.STATIC_PACKET);
								playerWasOnline = true;
								break;
							}
						if (!playerWasOnline)
							addItemToInventory(con, prevBidderId, IdFactory.getInstance().getNextId(), currency, prevBidAmount, 0);
						sendMail(con, prevBidderId, "Bidder", "You've been out bidded!", "Someone has out bidded you!<br1><a action=\"bypass _bbsauction_bid " + lotId + "\">Click here to bid fast!</a>");
					}
					catch (Exception e)
					{
						_log.warning(e.getMessage());
					}
					con = L2DatabaseFactory.getInstance().getConnection();
					statement = con.prepareStatement("INSERT INTO auction_bids (lotId, bidderId, bidAmount, bidDate) VALUES (?,?,?,?)");
					statement.setInt(1, lotId);
					statement.setInt(2, activeChar.getObjectId());
					statement.setLong(3, bidAmount);
					statement.setLong(4, Calendar.getInstance().getTimeInMillis());
					statement.execute();
					statement.close();
					if (currency == 57)
						activeChar.reduceAdena( (int) bidAmount,  true);
					else if (currency == 5575)
						activeChar.getInventory().reduceAncientAdena( (int) bidAmount);
					else
						activeChar.getInventory().destroyItem(currency, (int) bidAmount, true);
					activeChar.sendMessage("Your bid has been placed.");
				/* TODO
					if (activeChar.isGM() && Config.GMAUDIT)
						GMAudit.auditGMAction(activeChar.getName(), "auction", "_bbsauction_bid", lotId + " - " + currency + " - " + (int) bidAmount);
				*/
				}
				catch (Exception e)
				{
					_log.warning(e.getMessage());
				}
				finally
				{
					DatabaseUtils.closeDatabaseCSR(con, statement, rs);
				}
			}
			else
			{
				activeChar.sendMessage("Your bid is too low! Try again!");
				return false;
			}
		}
		else
		{
			activeChar.sendMessage("You do not have enough " + ItemTemplates.getInstance().getTemplate(currency).getName() + " to bid!");
			return false;
		}
		return true;
	}

	private void addItemToInventory(ThreadConnection con, int obj_Id, int objectId, int currency, long count, int enchantLevel) throws SQLException
	{
		FiltredPreparedStatement statement = con.prepareStatement("INSERT INTO items (owner_id, object_id, item_id, count, enchant_level, loc, loc_data) VALUES (?,?,?,?,?,?,?)");
		statement.setInt(1, obj_Id);
		statement.setInt(2, objectId);
		statement.setInt(3, currency);
		statement.setLong(4, count);
		statement.setInt(5, enchantLevel);
		statement.setString(6, "INVENTORY");
		statement.setInt(7, 0);
		statement.execute();
		statement.close();
	}

	private void sendMail(ThreadConnection con, int obj_Id, String recipient, String subject, String message) throws SQLException
	{
		FiltredPreparedStatement statement = con.prepareStatement("INSERT INTO character_mail (obj_Id, senderId, location, recipientNames, subject, message, sentDate, deleteDate, unread) VALUES (?,?,?,?,?,?,?,?,?)");
		statement.setInt(1, obj_Id);
		statement.setInt(2, 100100);
		statement.setString(3, "inbox");
		statement.setString(4, recipient);
		statement.setString(5, subject);
		statement.setString(6, message);
		statement.setLong(7, Calendar.getInstance().getTimeInMillis());
		statement.setLong(8, Calendar.getInstance().getTimeInMillis() + Long.valueOf("7948804000"));
		statement.setString(9, "true");
		statement.execute();
		statement.close();
	}

	private void endAuction(int lotId)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE auction_lots SET endDate = ? WHERE lotId = ?");
			statement.setLong(1, Calendar.getInstance().getTimeInMillis());
			statement.setInt(2, lotId);
			statement.execute();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rs);
		}
	}

	private int countBids(int lotId)
	{
		int count = 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result  = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT COUNT(*) FROM auction_bids WHERE lotId = ?");
			statement.setInt(1, lotId);
			result = statement.executeQuery();
			result.next();
			count = result.getInt(1);
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
		return count;
	}

	private int getHighestBid(int lotId)
	{
		int bidAmount = 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result  = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT MAX(bidAmount) FROM auction_bids WHERE lotId = ?");
			statement.setInt(1, lotId);
			result = statement.executeQuery();
			result.next();
			bidAmount = result.getInt(1);
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
		return bidAmount;
	}

	private BidList getHighestBidder(int lotId)
	{
		BidList bid = new BidList();
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result  = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT * FROM auction_bids WHERE lotId = ? ORDER BY bidId DESC");
			statement.setInt(1, lotId);
			result = statement.executeQuery();
			result.next();
			bid.bidderId = result.getInt("bidderId");
			bid.bidAmount = result.getLong("bidAmount");
			bid.bidDateFormated = "";
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
		return bid;
	}

	private void getLotsBiddedOn(L2Player activeChar)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result  = null;
		try
		{
			_lotsBidded = new ArrayList<Integer>();
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT lotId FROM auction_bids WHERE bidderId = ? GROUP BY 1");
			statement.setInt(1, activeChar.getObjectId());
			result = statement.executeQuery();
			while (result.next())
				_lotsBidded.add(result.getInt("lotId"));
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning(e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
	}

	private String getCharName(int obj_Id)
	{
		String charName = "No Name";
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet result  = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT char_name FROM characters WHERE obj_Id = ?");
			statement.setInt(1, obj_Id);
			result = statement.executeQuery();
			result.next();
			charName = result.getString("char_name");
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.warning("couldnt get char name for " + obj_Id + " " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, result);
		}
		return charName;
	}

	/*
	 * (non-Javadoc)
	 * @see com.l2jfree.gameserver.communitybbs.Manager.BaseBBSManager#parsewrite (java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, com.l2jfree.gameserver.model.actor.instance.L2Player)
	 */
	@Override
	public void parsewrite(String ar1, String ar2, String ar3, String ar4, String ar5, L2Player activeChar)
	{
		if (ar1.equals("Sort"))
		{
			String viewOnly;
			if (ar4.endsWith(" Armor"))
			{
				if (ar4.equals("Other Armor"))
					viewOnly = "None";
				else if (ar4.equals("Robe Armor"))
					viewOnly = "Magic";
				else
					viewOnly = ar4.replace(" Armor", "");
			}
			else
				viewOnly = ar4;
			showAuctionPage(activeChar, 1, viewOnly, false);
		}
	}

	@SuppressWarnings("synthetic-access")
	private static class SingletonHolder
	{
		protected static final AuctionBBSManager _instance = new AuctionBBSManager();
	}

	public static void notImplementedYet(L2Player activeChar, String command)
	{
		if (activeChar == null || command == null)
			return;
		ShowBoard.separateAndSend("<html><body><br><br><center>The command: [" + command + "] isn't implemented yet!</center><br><br></body></html>", activeChar);
	}
	
}
