package l2ft.gameserver.communitybbs.Manager;

import l2ft.extensions.scripts.Functions;
import l2ft.gameserver.Announcements;
import l2ft.gameserver.communitybbs.containers.Order;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.templates.L2Item;
import l2ft.gameserver.xml.ItemTemplates;
import l2ft.util.Files;
import l2ft.util.GArray;

import java.util.HashMap;
import java.util.Map;

public class MercenariesManager extends BaseBBSManager {
    // Список "Заказов", все сделано для максимально быстрого доступа и с наименьшим кол-вом запросов,
    // после ряда тестов было решено хранить всю информацию исключительно в самом ордере.
    public HashMap<Integer, Order> _orders = new HashMap<Integer, Order>();
    public GArray<Integer> _participants = new GArray<Integer>();
    public HashMap<Integer, Session> _sessions = new HashMap<Integer, Session>();

    private static MercenariesManager _Instance;

    public static MercenariesManager getInstance() {
        if (_Instance == null)
            _Instance = new MercenariesManager();
        return _Instance;
    }

    /**
     * Команды:
     * _bbsMercenaries; - открыть страницу с заказами.
     * _bbsMercenaries;showorder(ID цели) - посмотреть заказ.
     * _bbsMercenaries;addorder - меню создания заказа.
     * _bbsMercenaries;addreward;(ID-предмета Кол-во-предмета) - добавить награды
     * _bbsMercenaries;createorder; (Ник_перса Хайд/Не хайд) ;(ID-предмета Кол-во-предмета) - создать ордер
     * _bbsMercenaries;registration - регистрация
     * _bbsMercenaries;updatereward;(ID цели) - обновить награду
     * _bbsMercenaries;updateorder;(ID цели) - обновить заказ
     * _bbsMercenaries;addupdateorder;
     */

    @Override
    public void parsecmd(String command, L2Player activeChar) {
        //System.out.println(command);
        if (command.equals("_bbsMercenaries;") || command.equals("_bbsMercenaries")) {
            showHeadPage(activeChar, 0);
            //System.out.println(command);
        } else if (command.startsWith("_bbsMercenaries;page;")) {
            int page;
            try {
                page = Integer.parseInt(command.split(";")[2]);
            } catch (Exception e) {
                page = 0;
            }

            //System.out.println(page);

            showHeadPage(activeChar, page);
        } else if (command.startsWith("_bbsMercenaries;addorder")) {
            _sessions.remove(activeChar.getObjectId());
            showAddOrderPage(activeChar);
        } else if (command.startsWith("_bbsMercenaries;showorder;")) {
            if (command.length() > 26)
                command = command.substring(26, command.length());
            else {
                activeChar.sendMessage("Цель не выбрана.");
                return;
            }
            separateAndSend(getOrderInformation(Integer.parseInt(command)), activeChar);
        } else if (command.startsWith("_bbsMercenaries;addreward;")) {
            Session sessiong = _sessions.get(activeChar.getObjectId());

            if (sessiong == null)
                _sessions.put(activeChar.getObjectId(), sessiong = new Session());

            if (command.length() > 26) {
                String[] temp = command.substring(26, command.length()).split(" ");
                if (temp.length >= 2) {
                    long count;
                    try {
                        count = Long.parseLong(temp[1]);
                    } catch (Exception e) {
                        activeChar.sendMessage("Недопустимое кол-во предметов.");
                        return;
                    }
                    int id = Integer.parseInt(temp[0]);
                    L2ItemInstance item = activeChar.getInventory().getItemByItemId(id);
                    if (item != null && item.getCount() >= count) {
                        sessiong._items.put(item.getItemId(), count);
                        if (temp.length == 3) {
                            L2Player target = L2ObjectsStorage.getPlayer(Integer.parseInt(temp[2]));
                            if (target != null) {
                                showUpdateRewardPage(activeChar, target);
                                return;
                            }
                        }
                        showAddOrderPage(activeChar);
                    } else {
                        activeChar.sendMessage("Недопустимое кол-во предметов.");
                    }
                } else {
                    activeChar.sendMessage("Введите кол-во предметов отличное от нуля.");
                }
            }
        } else if (command.startsWith("_bbsMercenaries;updatereward;")) {
            if (command.length() > 29)
                command = command.substring(29, command.length());
            else return;

            L2Player target;
            try {
                target = L2ObjectsStorage.getPlayer(Integer.parseInt(command));
            } catch (Exception ingored) {
                activeChar.sendMessage("Цели не существует.");
                return;
            }

            Order order = _orders.get(target.getObjectId());

            if (order == null) {
                activeChar.sendMessage("Обновляемого ордера не существует.");
                return;
            }

            if (_sessions.get(activeChar.getObjectId()) != null && _sessions.get(activeChar.getObjectId())._items.size() > 0) {
                L2ItemInstance item;
                HashMap<Integer, Long> tempItems = _sessions.get(activeChar.getObjectId())._items;
                for (Map.Entry<Integer, Long> ent : tempItems.entrySet()) {
                    if ((item = activeChar.getInventory().getItemByItemId(ent.getKey())) != null && item.getCount() >= ent.getValue()) {
                        Functions.removeItem(activeChar, ent.getKey(), ent.getValue());
                        if (!order.reward.containsKey(ent.getKey()))
                            order.addReward(ent.getKey(), ent.getValue());
                        else order.addReward(ent.getKey(), order.reward.get(ent.getKey()) + ent.getValue());
                    }
                }
            } else {
                activeChar.sendMessage("Вы не добавили награду.");
                return;
            }
            _orders.put(target.getObjectId(), order);

            long price = (order.clientName.startsWith("Скрыт") ? 900000 : 300000);

            if (activeChar.getAdena() < price + (_sessions.get(activeChar.getObjectId())._items.containsKey(57) ? _sessions.get(activeChar.getObjectId())._items.get(57) : 0)) {
                activeChar.sendMessage("У вас недостаточно средств для создания ордера.");
                return;
            }
            Functions.removeItem(activeChar, 57, price);

            Announcements.getInstance().announceToAll("Заказ на убийство " + target.getName() + " был обновлен.");
            showHeadPage(activeChar, 0);
        } else if (command.startsWith("_bbsMercenaries;updateorder;")) {
            if (command.length() > 28)
                command = command.substring(28, command.length());
            else return;

            L2Player target;
            try {
                target = L2ObjectsStorage.getPlayer(Integer.parseInt(command));
            } catch (Exception ingored) {
                activeChar.sendMessage("Цели не существует.");
                return;
            }
            _sessions.remove(activeChar.getObjectId());
            showUpdateRewardPage(activeChar, target);
        } else if (command.startsWith("_bbsMercenaries;createorder")) {
            String[] temp = command.substring(29, command.length()).split(" ");

            if (temp.length < 3) {
                activeChar.sendMessage("Вы не ввели: кол-во убийств или ник персонажа.");
                return;
            }

            L2Player target = L2ObjectsStorage.getPlayer(temp[0]);
            if (target == null) {
                activeChar.sendMessage("Вы не ввели ник цели или цель не существует.");
                return;
            }

            Order order = _orders.get(target.getObjectId());
            if (order != null) {
                activeChar.sendMessage("Заказ на этого игрока уже существует.");
                return;
            }

            int needToKill = 0;
            try {
                needToKill = Integer.parseInt(temp[2]);
            } catch (Exception ignored) {
            }

            if (needToKill <= 0) {
                activeChar.sendMessage("Необходимо указать кол-во убийств больше нуля.");
                return;
            }

            if (_sessions.get(activeChar.getObjectId()) == null || _sessions.get(activeChar.getObjectId())._items.size() < 1) {
                activeChar.sendMessage("Вы должны назначить награду.");
                return;
            }

            long price = (temp[2].startsWith("Скрыт") ? 900000 : 300000);
            if (activeChar.getAdena() < price + (_sessions.get(activeChar.getObjectId())._items.containsKey(57) ? _sessions.get(activeChar.getObjectId())._items.get(57) : 0)) {
                activeChar.sendMessage("У вас недостаточно средств для создания ордера.");
                return;
            }

            Functions.removeItem(activeChar, 57, price);

            order = new Order(activeChar.getObjectId(), temp[1].equals("Hide") ? "Скрыто" : activeChar.getName(), target.getObjectId(), target.getName(), needToKill);


            L2ItemInstance item;
            HashMap<Integer, Long> tempItems = _sessions.get(activeChar.getObjectId())._items;
            for (Map.Entry<Integer, Long> ent : tempItems.entrySet()) {
                if ((item = activeChar.getInventory().getItemByItemId(ent.getKey())) != null && item.getCount() >= ent.getValue()) {
                    Functions.removeItem(activeChar, ent.getKey(), ent.getValue());
                    if (!order.reward.containsKey(ent.getKey()))
                        order.addReward(ent.getKey(), ent.getValue());
                    else order.addReward(ent.getKey(), order.reward.get(ent.getKey()) + ent.getValue());
                }
            }

            _orders.put(target.getObjectId(), order);
            Announcements.getInstance().announceToAll("Был размещен заказ на убийство " + target.getName() + ".");
            showHeadPage(activeChar, 0);
        } else if (command.startsWith("_bbsMercenaries;registration")) {
            if (_participants.contains(activeChar.getObjectId()))
                _participants.remove((Object) activeChar.getObjectId());
            else
                _participants.add(activeChar.getObjectId());
            showHeadPage(activeChar, 0);
        } else
            separateAndSend("<html><body><br><br><center>the command: " + command + " is not implemented yet</center><br><br></body></html>", activeChar);
    }

    public void showHeadPage(L2Player player, int page) {
        String content = Files.read("data/html/CommunityBoard/Mercenaries.htm");
        content = replace("%button_reg%", getButton(player), content);
        content = replace("%content%", getOrders(page), content);
        content = replace("%pages%", getPages(), content);
        separateAndSend(content, player);
    }

    public void showAddOrderPage(L2Player player) {
        String content = Files.read("data/html/CommunityBoard/MercenariesOrder.htm");
        Session sessiong = _sessions.get(player.getObjectId());
        String items = "<table width=600>";
        L2ItemInstance[] itemsTemp = player.getInventory().getItems();
        String button;
        int count = 0;
        for (L2ItemInstance item : itemsTemp) {
            if (count == 0)
                items += "<tr>";
            if (ItemTemplates.getInstance().getTemplate(item.getItemId()) == null || (sessiong != null && sessiong._items.containsKey(item.getItemId()) && sessiong._items.get(item.getItemId()) >= item.getCount()) || !item.getItem().isTradeable())
                continue;
            button = Functions.htmlButton("+", "bypass -h _bbsMercenaries;addreward;" + item.getItemId() + " $count", 27, 23);
            items += "<td align=center><img src=\"Icon." + ItemTemplates.getInstance().getTemplate(item.getItemId()).getIcon() + "\" width=32 height=32><br1>" + button + "</td>";
            if (count < 17)
                count++;
            else {
                count = 0;
                items += "</tr><tr><td></td></tr>";
            }
        }
        if (count != 0)
            items += "</tr>";
        items += "</table>";
        button = Functions.htmlButton("Создать ордер", "bypass -h _bbsMercenaries;createorder; $name $hide $countToKill", 150, 25);
        content = replace("%items%", items, content);
        content = replace("%button_addOrder%", button, content);
        separateAndSend(content, player);
    }

    public void showUpdateRewardPage(L2Player player, L2Player target) {
        if (target == null) {
            player.sendMessage("Цели не существует.");
            return;
        }
        String content = Files.read("data/html/CommunityBoard/MercenariesUpdateOrder.htm");
        Session sessiong = _sessions.get(player.getObjectId());
        String items = "<table width=600>";
        L2ItemInstance[] itemsTemp = player.getInventory().getItems();
        String button;
        int count = 0;
        for (L2ItemInstance item : itemsTemp) {
            if (count == 0)
                items += "<tr>";
            if (ItemTemplates.getInstance().getTemplate(item.getItemId()) == null || (sessiong != null && sessiong._items.containsKey(item.getItemId()) && sessiong._items.get(item.getItemId()) >= item.getCount()) || !item.getItem().isTradeable())
                continue;
            button = Functions.htmlButton("+", "bypass -h _bbsMercenaries;addreward;" + item.getItemId() + " $count " + target.getObjectId(), 27, 23);
            items += "<td align=center><img src=\"Icon." + ItemTemplates.getInstance().getTemplate(item.getItemId()).getIcon() + "\" width=32 height=32><br1>" + button + "</td>";
            if (count < 17)
                count++;
            else {
                count = 0;
                items += "</tr><tr><td></td></tr>";
            }
        }
        if (count != 0)
            items += "</tr>";
        items += "</table>";
        button = Functions.htmlButton("Создать ордер", "bypass -h _bbsMercenaries;updatereward;" + target.getObjectId(), 150, 25);
        content = replace("%items%", items, content);
        content = replace("%button_addOrder%", button, content);
        content = replace("%target_name%", target.getName(), content);
        separateAndSend(content, player);
    }

    @Override
    public void parsewrite(String ar1, String ar2, String ar3, String ar4, String ar5, L2Player activeChar) {

    }

    public String replace(String replaced, String replacing, String content) {
        StringBuilder content2 = new StringBuilder(content);
        int startIndex = content.indexOf(replaced);
        if (startIndex < 0)
            return content;
        content = content2.replace(startIndex, startIndex + replaced.length(), replacing).toString();
        return content;
    }

    public String getOrders(int page) {
        if (_orders.size() == 0)
            return "Заказы отсутствуют.";

        String content = "";
        boolean color = true;
        GArray<Order> temp = new GArray<Order>(_orders.size());
        temp.addAll(_orders.values());
        if (page < 1)
            page = 1;
        Order order;
        for (int i = page * 10 - 10; i <= page * 10 - 1; i++) {
            //System.out.println(i);
            order = temp.get(i);
            if (order == null)
                continue;
            //System.out.println("T:" + i);
            content += "<table width=600 bgcolor=" + (color ? "c4c4c4" : "797a75") + ">" +
                    "<tr>" +
                    "<td width=80 align=center><font color=LEVEL>" + order.clientName + "</font></td>" +
                    "<td width=80 align=center><font color=LEVEL>" + order.targetName + "</font></td>" +
                    "<td width=40 align=center><font color=LEVEL>" + order.countToKill + " раз</font></td>" +
                    "<td width=270 align=center><font color=b7dc2e>" + getReward(order) + "</font></td>" +
                    "<td width=100 align=center><button value=\"Подробнее\" action=\"bypass -h _bbsMercenaries;showorder;" + order.target + "\" width=100 height=25 back=\"L2UI_CT1.Button_DF_Down\" fore=\"L2UI_CT1.Button_DF\"></td>" +
                    "</tr>" +
                    "</table>";
            color = !color;
        }

        return content;
    }

    public String getPages() {
        String content = "";
        int pages = (int) Math.ceil((double) _orders.size() / 10);
        pages = pages > 0 ? pages : 1;

        content = "<table width=600>" +
                "<tr>" +
                "<td align=center>" +
                "<table>" +
                "<tr>";
        for (int i = 1; i <= pages; i++)
            content += "<td width=26 height=25 align=center>" + Functions.htmlButton(Integer.toString(i), "bypass -h _bbsMercenaries;page;" + i, 25, 25) + "</td>";

        content += "</tr>" +
                "</table>" +
                "</td>" +
                "</tr>" +
                "</table>";

        return content;
    }

    public String getOrderInformation(int objId) {
        Order order = _orders.get(objId);
        if (order == null)
            return "Такого ордера не существует.";

        String content = Files.read("data/html/CommunityBoard/MercenariesOrderInformation.htm");
        content = replace("%Killers%", getOrderInformationKillers(order), content);
        content = replace("%Rewards%", getOrderInformationReward(order), content);
        content = replace("%Owners%", getOrderInformationOwner(order), content);
        content = replace("%target_obj%", Integer.toString(objId), content);

        return content;
    }

    public String getOrderInformationKillers(Order order) {
        String content = "";
        boolean color = true;
        L2Player player;
        for (Map.Entry<Integer, Integer> kill : order.kills.entrySet()) {
            player = L2ObjectsStorage.getPlayer(kill.getKey());
            if (player == null)
                continue;
            content += "<tr>" +
                    "<td align=center>" +
                    "<table bgcolor=" + (color ? "a69a69" : "5c7024") + ">" +
                    "<tr><td fixwidth=150 align=center>" +
                    player.getName() +
                    "</td><td fixwidth=50 align=center>" +
                    kill.getValue() + " раз" +
                    "</td></tr></table></td></tr><tr><td></td></tr>";
            color = !color;
        }
        if (content.isEmpty())
            content = "<tr><td>Пока еще никто не убивал цель.</td></tr>";
        return content;
    }

    public String getOrderInformationOwner(Order order) {
        return "<tr>" +
                "<td align=center>" +
                "<table bgcolor=232f70>" +
                "<tr><td fixwidth=150 align=center>" +
                "Заказчик: <font color=LEVEL>" + order.clientName + "</font>" +
                "</td></tr></table></td></tr><tr><td></td></tr><tr><td align=center><table bgcolor=612370><tr><td fixwidth=150 align=center>" +
                "Цель: <font color=LEVEL>" + order.targetName + "</font>" +
                "</td></tr></table></td></tr><tr><td></td></tr><tr><td align=center><table bgcolor=232f70><tr><td fixwidth=150 align=center>" +
                "Необходимо убить <font color=LEVEL>" + order.countToKill + " раз</font>." +
                "</td></tr></table></td></tr>";
    }

    public String getOrderInformationReward(Order order) {
        String content = "";
        boolean color = true;
        L2Item item;
        for (Map.Entry<Integer, Long> it : order.reward.entrySet()) {
            item = ItemTemplates.getInstance().getTemplate(it.getKey());
            if (item == null)
                continue;
            content += "<tr>" +
                    "<td align=center>" +
                    "<table bgcolor=" + (color ? "a69a69" : "5c7024") + ">" +
                    "<tr><td fixwidth=150 align=center>" +
                    it.getValue() + " " + item.getName() +
                    "</td></tr></table></td></tr><tr><td></td></tr>";
            color = !color;
        }
        if (content.isEmpty())
            content = "<tr><td>Награда отсутствует.</td></tr>";
        return content;
    }

    public String getButton(L2Player player) {
        return Functions.htmlButton((_participants.contains(player.getObjectId()) ? "Отменить регистрацию" : "Зарегистрироваться"), "bypass -h _bbsMercenaries;registration", 130, 25);
    }

    public String getReward(Order order) {
        String content = "";

        if (order.reward.size() > 0)
            for (Map.Entry<Integer, Long> ent : order.reward.entrySet()) {
                if (ItemTemplates.getInstance().getTemplate(ent.getKey()) != null) {
                    content += ent.getValue() + " " + ItemTemplates.getInstance().getTemplate(ent.getKey()).getName();
                    content += ", ";
                    if (content.length() >= 40)
                        break;
                }
            }
        else
            content = "Награды нет.";

        return content.substring(0, content.length() - 2);
    }
}

class Session {
    HashMap<Integer, Long> _items = new HashMap<Integer, Long>();
}