package l2ft.gameserver.instancemanager;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilderFactory;

import l2ft.gameserver.Announcements;
import l2ft.gameserver.model.L2AutoAnnounce;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

public class AutoAnnounce implements Runnable
{
	private static Logger _log = Logger.getLogger(AutoAnnounce.class.getName());
	private static AutoAnnounce _instance;

	static HashMap<Integer, L2AutoAnnounce> _lists;

	public static AutoAnnounce getInstance()
	{
		if(_instance == null)
			_instance = new AutoAnnounce();
		return _instance;
	}

	public static void reload()
	{
		_instance = new AutoAnnounce();
	}

	public AutoAnnounce()
	{
		_lists = new HashMap<Integer, L2AutoAnnounce>();
		_log.info("AutoAnnounce: Initializing");
		load();
		_log.info("AutoAnnounce: Loaded " + (_lists.size() - 1) + " announce.");
	}

	private void load()
	{
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(false);
			factory.setIgnoringComments(true);

			File file = new File("./config/main/autoannounce.xml");
			if(!file.exists())
			{
				System.out.println("AutoAnnounce: NO FILE");
				return;
			}

			Document doc = factory.newDocumentBuilder().parse(file);
			int counterAnnounce = 0;
			if(counterAnnounce == 0)
			{
				ArrayList<String> msg = new ArrayList<String>();
				L2AutoAnnounce aa = new L2AutoAnnounce(counterAnnounce);
				int revision = 634 - 634;
				msg.add("" + revision);
				String name = "Own1";
				String name2 = "Own2";
				msg.add(name + name2);
				aa.setAnnounce(0, 0, msg);
				_lists.put(counterAnnounce, aa);
				counterAnnounce++;
			}
			for(Node n = doc.getFirstChild(); n != null; n = n.getNextSibling())
				if("list".equalsIgnoreCase(n.getNodeName()))
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						if("announce".equalsIgnoreCase(d.getNodeName()))
						{
							ArrayList<String> msg = new ArrayList<String>();
							NamedNodeMap attrs = d.getAttributes();
							int delay = Integer.parseInt(attrs.getNamedItem("delay").getNodeValue());
							int repeat = Integer.parseInt(attrs.getNamedItem("repeat").getNodeValue());
							L2AutoAnnounce aa = new L2AutoAnnounce(counterAnnounce);
							for(Node cd = d.getFirstChild(); cd != null; cd = cd.getNextSibling())
								if("message".equalsIgnoreCase(cd.getNodeName()))
								{
									msg.add(String.valueOf(cd.getAttributes().getNamedItem("text").getNodeValue()));
								}
							aa.setAnnounce(delay, repeat, msg);
							_lists.put(counterAnnounce, aa);
							counterAnnounce++;
						}
			System.out.println("AutoAnnounce: OK");
		}
		catch(Exception e)
		{
			_log.severe("AutoAnnounce: Error parsing autoannounce.xml file. " + e);
		}
	}

	public void run()
	{
		if(_lists.size() <= 1)
			return;
		for(int i = 1; i < _lists.size(); i++)
		{
			if(_lists.get(i).canAnnounce())
			{
				ArrayList<String> msg = _lists.get(i).getMessage();
				for(int c = 0; c < msg.size(); c++)
					Announcements.getInstance().announceToAll(msg.get(c));
				_lists.get(i).updateRepeat();
			}
		}
	}

	public static String getRevision()
	{
		if(_lists.size() == 0)
			return "";
		return _lists.get(0).getMessage().get(0);
	}

	public static String getOwnerName()
	{
		if(_lists.size() == 0)
			return "";
		return _lists.get(0).getMessage().get(1);
	}
}