package l2ft.gameserver.instancemanager;

import java.util.Arrays;
import java.util.HashMap;

import javolution.util.FastList;
import l2ft.Config;
import l2ft.common.ThreadPoolManager;
import l2ft.extensions.scripts.Functions;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Effect;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.L2Spawn;
import l2ft.gameserver.model.L2Summon;
import l2ft.gameserver.model.instances.L2BlockInstance;
import l2ft.gameserver.model.instances.L2MonsterInstance;
import l2ft.gameserver.serverpackets.ExCubeGameAddPlayer;
import l2ft.gameserver.serverpackets.ExCubeGameChangePoints;
import l2ft.gameserver.serverpackets.ExCubeGameChangeTimeToStart;
import l2ft.gameserver.serverpackets.ExCubeGameEnd;
import l2ft.gameserver.serverpackets.ExCubeGameExtendedChangePoints;
import l2ft.gameserver.serverpackets.ExCubeGameRemovePlayer;
import l2ft.gameserver.serverpackets.ExCubeGameTeamList;
import l2ft.gameserver.serverpackets.NpcInfo;
import l2ft.gameserver.serverpackets.SkillList;
import l2ft.gameserver.tables.NpcTable;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.util.GArray;
import l2ft.util.Rnd;

public class GameHandyManager
{
	private static GameHandyManager _instance;

	static int[][][] _spawnLocs = { { 
		
		// SpawnCoords for 1 arena
			{ -57814, -62257, -2370 }, 
			{ -57817, -62734, -2370 }, 
			{ -58365, -62634, -2370 }, 
			{ -58359, -62181, -2370 },
			{ -58053, -62180, -2370 } }, {
		// SpawnCoords for 2 arena
			{ -57817, -63397, -2408 }, 
			{ -57817, -63711, -2408 }, 
			{ -58290, -62634, -2408 }, 
			{ -58276, -63406, -2408 },
			{ -58050, -63325, -2408 } }, { 
		// SpawnCoords for 3 arena
			{ -56654, -63388, -2411 }, 
			{ -56649, -63709, -2411 }, 
			{ -57112, -63702, -2411 }, 
			{ -57112, -63404, -2411 },
			{ -56885, -63332, -2411 } }, {
		// SpawnCoords for 4 arena
			{ -56654, -62261, -2400 }, 
			{ -56655, -62566, -2400 }, 
			{ -57104, -62574, -2400 }, 
			{ -57110, -62268, -2400 },
			{ -56883, -62200, -2400 } } 
		};

	static int[][] _TpLocs = { 
								{ -58051, -62424, -2408 }, // TP arena 1
								{ -58051, -63562, -2408 }, // TP arena 2
								{ -56888, -63562, -2408 }, // TP arena 3
								{ -56888, -62424, -2408 }, // TP arena 4
	};

	private static final int BLUE_TEAM = 1;
	private static final int RED_TEAM = 2;
	private static final int[] BUFFS_WHITELIST = { 6035, 6036 };
	private static HashMap<Integer, HandyArena> _arenas = new HashMap<Integer, HandyArena>();


	static
	{
		_arenas.put(1, new HandyArena(1));
		_arenas.put(2, new HandyArena(2));
		_arenas.put(3, new HandyArena(3));
		_arenas.put(4, new HandyArena(4));
	}

	public static GameHandyManager getInstance()
	{
		if(_instance == null)
			_instance = new GameHandyManager();
		return _instance;
	}

	protected static final class HandyArena
	{
		public int _id = 0;
		public long _startTime = 0;
		public FastList<L2Player> _RedTeamPlayers = new FastList<L2Player>();
		public FastList<L2Player> _BlueTeamPlayers = new FastList<L2Player>();
		public FastList<L2Player> _tempPlayers = new FastList<L2Player>();
		public FastList<L2MonsterInstance> _mobs = new FastList<L2MonsterInstance>();
		public boolean _canRegister = true;
		public int _Red_playerTotalKill = 0;
		public int _Blue_playerTotalKill = 0;
		private GArray<L2MonsterInstance> _spawns1 = new GArray<L2MonsterInstance>();

		public HandyArena(int id)
		{
			_id = id;
		}
	}

	public static boolean registerPlayer(L2Player player, int arenaId)
	{
		int objectId = player.getObjectId();
		int TeamId = 1;

		HandyArena arena = _arenas.get(arenaId);

		if(getPlayerArena(player) != 0)
			return false;

		if(arena._tempPlayers.contains(objectId) || !arena._canRegister)
			return false;

		if(arena._RedTeamPlayers.size() > Config.MAX_PLAYERS_IN_TEAM && arena._BlueTeamPlayers.size() > Config.MAX_PLAYERS_IN_TEAM)
			return false;

		if(arena._RedTeamPlayers.size() == arena._BlueTeamPlayers.size())
			TeamId = Rnd.get(1, 2);
		else
			TeamId = arena._RedTeamPlayers.size() > arena._BlueTeamPlayers.size() ? BLUE_TEAM : RED_TEAM;
		arena._tempPlayers.add(player);

		if(TeamId == RED_TEAM)
			arena._RedTeamPlayers.add(player);
		else
			arena._BlueTeamPlayers.add(player);

		if(arena._RedTeamPlayers.size() >= Config.MIN_PLAYERS_IN_TEAM && arena._BlueTeamPlayers.size() >= Config.MIN_PLAYERS_IN_TEAM)
		{
		/*	for(L2Player plr : arena._BlueTeamPlayers)
			{
				if(plr != null)
				{
					allPlayers.add(plr);
					bluePlayers.add(plr);
					plr.sendPacket(new ExCubeGameChangeTimeToStart(120));
				}
			}
			for(L2Player plr : arena._RedTeamPlayers)
			{
				if(plr != null)
				{
					allPlayers.add(plr);
					redPlayers.add(plr);
					plr.sendPacket(new ExCubeGameChangeTimeToStart(120));
				}
			}
*/
			for(L2Player plr : arena._tempPlayers)
			{
				plr.sendPacket(new ExCubeGameTeamList(arena._RedTeamPlayers, arena._BlueTeamPlayers, arena._id));
				plr.sendPacket(new ExCubeGameChangeTimeToStart(Config.ARENA_HENDY_WAIT_TIME));
			}

			ThreadPoolManager.getInstance().scheduleGeneral(new startGamesHandy(arena._id), Config.ARENA_HENDY_WAIT_TIME * 1000);
		}

		return true;
	}

	public static void playerList(L2Player player, int arenaId)
	{
		HandyArena arena = _arenas.get(arenaId);
		player.sendPacket(new ExCubeGameTeamList(arena._RedTeamPlayers, arena._BlueTeamPlayers, arena._id));
	}

	public static void StartHandyGame(int arenaId)
	{
		HandyArena arena = _arenas.get(arenaId);
		arena._canRegister = false;
		arena._startTime = System.currentTimeMillis();

		for(L2Player player : arena._BlueTeamPlayers)
		{
			if(player != null)
			{
				//player._HandyGamePoints = 0;
				player.setTeam(BLUE_TEAM, false);
				player.altUseSkill(SkillTable.getInstance().getInfo(6036, 1), player);
				player.sendPacket(new SkillList(player));
			}
		}
		for(L2Player player : arena._RedTeamPlayers)
		{
			if(player != null)
			{
				//player._HandyGamePoints = 0;
				player.setTeam(RED_TEAM, false);
				player.altUseSkill(SkillTable.getInstance().getInfo(6035, 1), player);
				player.sendPacket(new SkillList(player));
			}
		}

		for(L2Player player : arena._tempPlayers)
		{
			doTeleport(player, _TpLocs[arena._id - 1][0], _TpLocs[arena._id - 1][1], _TpLocs[arena._id - 1][2]);
			removeBuffs(player);
			player.setTransformation(122);
			player.sendPacket(new ExCubeGameAddPlayer(player, (player.getTeam() == RED_TEAM ? true : false)));
			player.sendPacket(new ExCubeGameChangePoints(Config.ARENA_HENDY_GAME_TIME, arena._Blue_playerTotalKill, arena._Red_playerTotalKill));
		}
		
		ThreadPoolManager.getInstance().scheduleGeneral(new spawnMobs(arena._id), 10000L);
		ThreadPoolManager.getInstance().scheduleGeneral(new finishGameHandy(arena._id), Config.ARENA_HENDY_GAME_TIME * 1000); //test end game
	}

	private static void globalDespawn(HandyArena arena)
	{

		for(L2MonsterInstance mob : arena._spawns1)
			//DayNightSpawnManager.deleteMobs(_spawns1);
			//SpawnTable.getInstance().deleteSpawn(mob);
			//mob.doDie(killer)
			//mob = null;
			//SpawnTable.getInstance().deleteSpawn(mob, false);
			//mob.deleteMe();
			Functions.npcSay(mob, "You're a hard worker, Rayla!");

		arena._mobs.clear();
	}

	public static void doTeleport(L2Player player, int x, int y, int z)
	{
		if(!player.isOnline())
			return;
		player.teleToLocation(x, y, z, 0);

		L2Summon pet = player.getPet();
		if(pet != null)
			pet.teleToLocation(x, y, z, 0);
	}

	private static final void removeBuffs(L2Character ch)
	{
		L2Skill skill;
		for(L2Effect e : ch.getEffectList().getAllEffects())
		{
			if(e == null)
				continue;
			skill = e.getSkill();
			//if(skill.getSkillsByType(SkillType.DEBUFF) != null)
			//continue;
			if(Arrays.binarySearch(BUFFS_WHITELIST, skill.getId()) >= 0)
				continue;
			e.exit();
		}
		if(ch.getPet() == null)
			return;
		for(L2Effect e : ch.getPet().getEffectList().getAllEffects())
		{
			if(e == null)
				continue;
			skill = e.getSkill();
			//if(skill.getSkillsByType(SkillType.DEBUFF) != null)
			//continue;
			if(Arrays.binarySearch(BUFFS_WHITELIST, skill.getId()) >= 0)
				continue;
			e.exit();
		}
	}

	private static L2Spawn spawnNpc(int npcId, int x, int y, int z, int heading, int respawnTime, int instanceId)
	{
		NpcTable.getInstance();
		L2NpcTemplate npcTemplate = NpcTable.getTemplate(npcId);
		L2Spawn spawnDat = null;
		try
		{
			spawnDat = new L2Spawn(npcTemplate);
			spawnDat.setAmount(1);
			spawnDat.setLocx(x);
			spawnDat.setLocy(y);
			spawnDat.setLocz(z);
			spawnDat.setHeading(heading);
			spawnDat.setRespawnDelay(respawnTime);
			spawnDat.setReflection(instanceId);
			//RaidBossSpawnManager.getInstance().addNewSpawn(spawnDat);
			DayNightSpawnManager.getInstance().addDayMob(spawnDat);
			spawnDat.init();
			spawnDat.startRespawn();
			if(respawnTime == 0)
				spawnDat.stopRespawn();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return spawnDat;
	}

	public static void increaseKill(int Team, int arenaId, L2Player player)
	{
		HandyArena arena = _arenas.get(arenaId);
		if(Team == RED_TEAM)
			arena._Blue_playerTotalKill += 1;
		else if(Team == BLUE_TEAM)
			arena._Red_playerTotalKill += 1;

		int timeLeft = (int) ((arena._startTime + 300000 - System.currentTimeMillis()) / 1000);
		for(L2Player plr : arena._tempPlayers)
		{
			if(player != null)
			{
				plr.sendPacket(new ExCubeGameChangePoints(timeLeft, arena._Blue_playerTotalKill, arena._Red_playerTotalKill));
				plr.sendPacket(new ExCubeGameExtendedChangePoints(timeLeft, arena._Blue_playerTotalKill, arena._Red_playerTotalKill, player.getTeam() == RED_TEAM, player, 0));
			}
		}
	}

	public static void onLogin(L2Player player)
	{
		for(HandyArena arena : _arenas.values())
		{
			int playerId = player.getObjectId();
			Integer playerKey = new Integer(playerId);
			if(arena._BlueTeamPlayers.contains(playerId))
			{
				player.teleToLocation(-57470, -60637, -2364);
				arena._tempPlayers.remove(playerKey);
				arena._BlueTeamPlayers.remove(playerKey);
				for(L2Player plr : arena._tempPlayers)
				{
					if(plr != null)
						plr.sendPacket(new ExCubeGameRemovePlayer(player, false));
				}
			}
			else if(arena._RedTeamPlayers.contains(playerId))
			{
				player.teleToLocation(-57470, -60637, -2364);
				arena._tempPlayers.remove(playerKey);
				arena._RedTeamPlayers.remove(playerKey);
				for(L2Player plr : arena._tempPlayers)
				{
					if(plr != null)
						plr.sendPacket(new ExCubeGameRemovePlayer(player, true));
				}
			}
		}
	}

	public static int getPlayerTeam(L2Player player)
	{
		int teamId = 0;
		int playerId = player.getObjectId();
		for(HandyArena arena : _arenas.values())
		{
			if(arena._BlueTeamPlayers.contains(playerId))
				teamId = 1;
			if(arena._BlueTeamPlayers.contains(playerId))
				teamId = 2;
		}
		
		return teamId;
	}
	
	public static int getPlayerArena(L2Player player)
	{
		int arenaId = 0;
		for(HandyArena arena : _arenas.values())
			if(arena._tempPlayers.contains(player.getObjectId()))
				arenaId = arena._id;
		return arenaId;
	}

	private static class spawnMobs implements Runnable
	{
		private int _arenaId;

		public spawnMobs(int arenaId)
		{
			_arenaId = arenaId;
		}

		@Override
		public void run()
		{
			HandyArena arena = _arenas.get(_arenaId);
			int _instanceId = 0;
			int baseX = _TpLocs[_arenaId - 1][0];
			int baseY = _TpLocs[_arenaId - 1][1];
			int z = _TpLocs[_arenaId - 1][2];
			// Blue cubs
			for(int j = 0; j < 8; ++j)
			{
				int x = baseX + Rnd.get(-400, 400);
				int y = baseY + Rnd.get(-400, 400);
				L2Spawn spawnDat = spawnNpc(18671, x, y, z, 0, 60, _instanceId);
				L2MonsterInstance mob = (L2MonsterInstance) spawnDat.doSpawn(false);
				L2BlockInstance block = (L2BlockInstance) spawnDat.getLastSpawn();
				block.setColorHandyCubik(0, mob);
				arena._mobs.add(mob);
				mob.broadcastPacket(new NpcInfo(block, null));
			}
			// Red cubs
			for(int j = 0; j < 8; ++j)
			{
				int x = baseX + Rnd.get(-400, 400);
				int y = baseY + Rnd.get(-400, 400);
				L2Spawn spawnDat = spawnNpc(18671, x, y, z, 0, 60, _instanceId);
				L2MonsterInstance mob = (L2MonsterInstance) spawnDat.doSpawn(false);
				L2BlockInstance block = (L2BlockInstance) spawnDat.getLastSpawn();
				block.setColorHandyCubik(1, mob);
				arena._mobs.add(mob);
				mob.broadcastPacket(new NpcInfo(mob, null));
			}
		}
	}

	private static class finishGameHandy implements Runnable
	{
		private int _arenaId;

		public finishGameHandy(int arenaId)
		{
			_arenaId = arenaId;
		}

		@Override
		public void run()
		{
			HandyArena arena = _arenas.get(_arenaId);
			boolean isRedTeamWin = arena._Red_playerTotalKill > arena._Blue_playerTotalKill;
			int reward = 0;

			for(L2Player player : arena._RedTeamPlayers)
			{
				if(player != null)
				{
					player.setTransformation(0);
					player.setTeam(0, false);
					//player.removeSkill(SkillTable.getInstance().getInfo(5853, 1), false);
					player.teleToLocation(-57470, -60637, -2364);
					//player.sendPacket(new SkillList(null));
					player.sendPacket(new ExCubeGameEnd(isRedTeamWin));
					//player.sendPacket(new ExCubeGameCloseUI());
					//player.sendPacket(new ExCubeGameAddPlayer(player, false));
					if(arena._Red_playerTotalKill == arena._Blue_playerTotalKill)
						reward = 2;
					else if(arena._Red_playerTotalKill > arena._Blue_playerTotalKill)
						if(arena._Red_playerTotalKill / arena._RedTeamPlayers.size() > 25)
							reward = 25;
						else
							reward = 2 + arena._Red_playerTotalKill / arena._RedTeamPlayers.size();
					player.getInventory().addItem(13067, reward);
				}
			}
			for(L2Player player : arena._BlueTeamPlayers)
			{
				if(player != null)
				{
					player.setTransformation(0);
					player.setTeam(0, false);
					//player.removeSkill(SkillTable.getInstance().getInfo(5853, 1), false);
					player.teleToLocation(-57470, -60637, -2364);
					//player.sendPacket(new SkillList(null));
					player.sendPacket(new ExCubeGameEnd(isRedTeamWin));
					//player.sendPacket(new ExCubeGameCloseUI());
					//player.sendPacket(new ExCubeGameAddPlayer(player, false));
					if(arena._Red_playerTotalKill == arena._Blue_playerTotalKill)
						reward = 2;
					else if(arena._Red_playerTotalKill > arena._Blue_playerTotalKill)
						if(arena._Red_playerTotalKill / arena._RedTeamPlayers.size() > 25)
							reward = 25;
						else
							reward = 2 + arena._Red_playerTotalKill / arena._RedTeamPlayers.size();
					player.getInventory().addItem(13067, reward);
				}
			}
			arena._canRegister = true;
			arena._BlueTeamPlayers.clear();
			arena._RedTeamPlayers.clear();
			arena._tempPlayers.clear();
			arena._Red_playerTotalKill = 0;
			arena._Blue_playerTotalKill = 0;
			globalDespawn(arena);
			//arena._mobs.clear();			
		}
	}

	private static class startGamesHandy implements Runnable
	{
		private int _arenaId;

		public startGamesHandy(int arenaId)
		{
			_arenaId = arenaId;
		}

		@Override
		public void run()
		{
			StartHandyGame(_arenaId);
		}
	}
}