package l2ft.gameserver.instancemanager;

import java.util.logging.Logger;

import l2ft.Config;
import l2ft.common.ThreadPoolManager;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.model.items.MailParcelController;
import l2ft.gameserver.model.items.MailParcelController.Letter;
import l2ft.gameserver.xml.ItemTemplates;

public class ItemsForOnline 
{
	private static Logger _log = Logger.getLogger(ItemsForOnline.class.getName());
	
	private static ItemsForOnline _instance;
	
	public static ItemsForOnline getInstance()
	{
		if(_instance == null && Config.ENABLE_ONLINE_BONUS)
			_instance = new ItemsForOnline();
		return _instance;
	}
	
	public ItemsForOnline()
	{
		ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new GiveReward1(), 60000, 60000);
		ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new GiveReward2(), 60000, 60000);
		_log.info("ItemsForOnline: loaded sucesfully");
	}
	
	
	
	private synchronized void giveReward1()
	{
		long coinTime = System.currentTimeMillis();
		if (coinTime - Config.TIME_FOR_ONLINE_REWARD1 >= L2Player._TimeInWordAdena)
		{
			for(L2Player player : L2ObjectsStorage.getAllPlayers())
			{ 
				player.getInventory().addItem(Config.ID_ITEM_FOR_ONLINE_REWARD1, Config.COUNT_ITEM_FOR_ONLINE_REWARD1);
				L2Player._TimeInWordAdena = coinTime;
				player.sendMessage("Вы получили награду, за игру на нашем сервере");
			}
		}
	}
	
	private synchronized void giveReward2()
	{
		long coinTime = System.currentTimeMillis();
		if (coinTime - Config.TIME_FOR_ONLINE_REWARD2 >= L2Player._TimeInWordCoin)
		{
			for (L2Player player : L2ObjectsStorage.getAllPlayersForIterate())
			{
				if (player != null && player.isOnline() || player.isInOfflineMode() && !player.isDeleting())
				{
					Letter letter = new Letter();
					letter.receiverId = player.getObjectId();
					letter.receiverName = "";
					letter.senderId = 1;
					letter.senderName = "";
					letter.topic = "From Administration";
					letter.body = "Congratulations!";
					letter.price = 0;
					letter.unread = 1;
					letter.system = 1;
					letter.validtime = 1296000 + (int) (System.currentTimeMillis() / 1000); // 14 days
					L2ItemInstance reward1 = ItemTemplates.getInstance().createItem(Config.ID_ITEM_FOR_ONLINE_REWARD2);
					reward1.setCount(Config.COUNT_ITEM_FOR_ONLINE_REWARD2);
					MailParcelController.getInstance().attach(letter, reward1);
					MailParcelController.getInstance().sendLetter(letter);
				}
				L2Player._TimeInWordCoin = coinTime;
			}
		}
	}
	
	
	private class GiveReward1 implements Runnable
	{
		@Override
		public void run()
		{
			giveReward1();
		}
	}
	
	
	private class GiveReward2 implements Runnable
	{
		@Override
		public void run()
		{
			giveReward2();
		}
	}
}