package l2ft.gameserver.loginservercon;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import l2ft.Config;
import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.gameserver.loginservercon.gspackets.ReplyCharacters;
import l2ft.gameserver.loginservercon.lspackets.AuthResponse;
import l2ft.gameserver.loginservercon.lspackets.ChangePasswordResponse;
import l2ft.gameserver.loginservercon.lspackets.IpAction;
import l2ft.gameserver.loginservercon.lspackets.KickPlayer;
import l2ft.gameserver.loginservercon.lspackets.LoginServerBasePacket;
import l2ft.gameserver.loginservercon.lspackets.LoginServerFail;
import l2ft.gameserver.loginservercon.lspackets.MoveCharToAccResponse;
import l2ft.gameserver.loginservercon.lspackets.PlayerAuthResponse;
import l2ft.gameserver.loginservercon.lspackets.PointConnection;
import l2ft.gameserver.loginservercon.lspackets.RSAKey;
import l2ft.gameserver.loginservercon.lspackets.RequestCharacters;
import l2ft.gameserver.loginservercon.lspackets.TestConnection;


/**
 * @Author: Death
 * @Date: 12/11/2007
 * @Time: 22:41:04
 */
public class PacketHandler
{
	private static final Logger _log = Logger.getLogger(PacketHandler.class.getName());

	public static LoginServerBasePacket handlePacket(byte[] data, AttLS loginserver)
	{
		if(Config.DEBUG_GS_LS)
			_log.info("GS Debug: Processing packet from LS");

		LoginServerBasePacket packet = null;

		try
		{
			data = loginserver.decrypt(data);
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}

		int id = data[0] & 0xFF;

		switch(id)
		{
			case 0x00:
				packet = new RSAKey(data, loginserver);
				break;
			case 0x01:
				packet = new LoginServerFail(data, loginserver);
				break;
			case 0x02:
				packet = new AuthResponse(data, loginserver);
				break;
			case 0x03:
				packet = new PlayerAuthResponse(data, loginserver);
				break;
			case 0x04:
				packet = new KickPlayer(data, loginserver);
				break;
			case 0x05:
				RequestCharacters rc = new RequestCharacters(data, loginserver); 
				getCharsOnServer(rc.getAccount());
				break;
			case 0x06:
				packet = new ChangePasswordResponse(data, loginserver);
				break;
			case 0x07:
				packet = new IpAction(data, loginserver);
				break;
			case 0x08:
				packet = new MoveCharToAccResponse(data, loginserver);
				break;
			case 0x09:
				packet = new TestConnection(data, loginserver);
				break;
			case 0x0a:
				packet = new RequestCharacters(data, loginserver);
				break;
			case 0x1D:
				packet = new PointConnection(data, loginserver);
				break;
			default:
				_log.warning("LSConnection: Recieved unknown packet: " + id + ". Restarting connection.");
				//LSConnection.getInstance().shutdown();
				LSConnection.getInstance().restart();
		}

		return packet;
	}
	
	public static void getCharsOnServer(String account)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		int chars = 0;
		List<Long> charToDel = new ArrayList<Long>();
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT deletetime FROM characters WHERE account_name=?");
			statement.setString(1, account);
			ResultSet rset = statement.executeQuery();
			while(rset.next())
			{
				chars++;
				long delTime = rset.getLong("deletetime");
				if(delTime != 0)
					charToDel.add(delTime);
			}
			rset.close();
			statement.close();
		}
		catch(SQLException e)
		{
			_log.warning("Exception: getCharsOnServer: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		ReplyCharacters rc = new ReplyCharacters(account, chars, charToDel);
		try
		{
			LSConnection.getInstance().sendPacket(rc);
		}
		catch(Exception e)
		{
			_log.warning("Exception: ReplyCharacters: " + e);
		}
	}
	
}