package l2ft.gameserver.model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.serverpackets.ExShowContactList;
import l2ft.gameserver.serverpackets.L2GameServerPacket;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.util.Util;

public class L2ContactList
{
	private static final int LIMIT = 100;
	private final Logger _log = Logger.getLogger(getClass().getName());
	private final long ownerId;
	private FastMap<Integer, String> _contacts;
	private final String QUERY_LOAD = "SELECT contact_id, char_name FROM character_contacts LEFT JOIN characters ON ( character_contacts.contact_id = characters.obj_Id ) WHERE char_id=?";

	public L2ContactList(L2Player player)
	{
		ownerId = player.getStoredId().longValue();
	}

	public void restore()
	{
		L2Player owner = getOwner();
		int ownerObjId = getOwner().getObjectId();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT contact_id, char_name FROM character_contacts LEFT JOIN characters ON ( character_contacts.contact_id = characters.obj_Id ) WHERE char_id=?");
			statement.setInt(1, ownerObjId);
			rset = statement.executeQuery();

			while (rset.next())
			{
				int contactId = rset.getInt("contact_id");
				String contactName = rset.getString("char_name");
				if (contactName.equals(owner.getName()) || contactId == ownerObjId)
				{
					continue;
				}
				if (_contacts == null)
					_contacts = new FastMap<Integer, String>();
				_contacts.put(Integer.valueOf(contactId), contactName);
			}
		}
		catch (Exception e)
		{
			_log.log(Level.WARNING, "Error found in " + owner.getName() + "'s ContactsList: " + e.getMessage(), e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public boolean add(String name)
	{
		L2Player owner = getOwner();

		if (_contacts != null)
		{
			for (String contact : this._contacts.values())
				if (contact.equalsIgnoreCase(name))
				{
					owner.sendPacket(Msg.NAME_ALREADY_EXIST_ON_THE_ADDED_LIST);
					return false;
				}
		}
		if (owner.getName().equalsIgnoreCase(name))
		{
			owner.sendPacket(Msg.YOU_CANNOT_ADD_YOU_OWN_NAME);
			return false;
		}

		if (_contacts != null && _contacts.size() >= 100)
		{
			owner.sendPacket(new L2GameServerPacket[] { Msg.CONTACT_LIST_LIMIT_REACHED });
			return false;
		}

		L2Player contact = L2ObjectsStorage.getPlayer(name);
		int contactId = 0;
		if (contact != null)
			contactId = contact.getObjectId();
		else
		{
			contactId = Util.GetCharIDbyName(name);
		}
		if (contactId < 1)
		{
			owner.sendPacket(new SystemMessage(3215).addString(name));
			return false;
		}

		if (_contacts == null)
			_contacts = new FastMap<Integer, String>();
		_contacts.put(Integer.valueOf(contactId), name);
    
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO character_contacts(char_id, contact_id) VALUES (?, ?)");
			statement.setInt(1, getOwner().getObjectId());
			statement.setLong(2, contactId);
			statement.execute();
		}
		catch (SQLException e)
		{
			_log.warning("Lottery: Could not store new lottery data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
    
		owner.sendPacket(new SystemMessage(3214).addString(name), new ExShowContactList(owner));

		return true;
	}

	public void remove(String name)
	{
		L2Player owner = getOwner();
		int contactId = 0;

		if (_contacts != null)
		{
			for (Map.Entry entry : _contacts.entrySet())
				if (((String)entry.getValue()).equalsIgnoreCase(name))
				{
					contactId = ((Integer)entry.getKey()).intValue();
					break;
				}
		}
		if (contactId < 1)
		{
			owner.sendPacket(Msg.THE_NAME_IS_NOT_CURRENTLY_REGISTERED);
			return;
		}

		_contacts.remove(Integer.valueOf(contactId));
    
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM character_contacts WHERE char_id=? AND contact_id=?;");
			statement.setInt(1, Integer.valueOf(getOwner().getObjectId()));
			statement.setLong(2, Integer.valueOf(contactId));
			statement.execute();
		}
		catch (SQLException e)
		{
			_log.warning("Lottery: Could not store new lottery data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
    
		owner.sendPacket(new SystemMessage(3219).addString(name), new ExShowContactList(owner));
	}

	private L2Player getOwner()
	{
		return L2ObjectsStorage.getAsPlayer(ownerId);
	}

	public FastMap<Integer, String> getContacts()
	{
		return _contacts;
	}
}