package l2ft.gameserver.model.entity;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2ft.Config;
import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.database.mysql;
import l2ft.gameserver.instancemanager.CastleManager;
import l2ft.gameserver.model.L2Alliance;
import l2ft.gameserver.model.L2Clan;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2World;
import l2ft.gameserver.model.entity.olympiad.Olympiad;
import l2ft.gameserver.model.entity.residence.Castle;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.model.items.L2ItemInstance.ItemClass;
import l2ft.gameserver.serverpackets.NpcHtmlMessage;
import l2ft.gameserver.serverpackets.SocialAction;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.ClanTable;
import l2ft.gameserver.tables.NpcTable;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.templates.L2Item;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.Files;
import l2ft.util.GArray;

public class Hero
{
	private static Logger _log = Logger.getLogger(Hero.class.getName());
	private static Hero _instance;
	private static Map<Integer, StatsSet> _heroes;
	private static Map<Integer, StatsSet> _completeHeroes;
	private static Map<Integer, ArrayList<StatsSet>> _heroDiary;
	private static final int ACTION_RAID_KILLED = 1;
	private static final int ACTION_HERO_GAINED = 2;
	private static final int ACTION_CASTLE_TAKEN = 3;

	public static Hero getInstance()
	{
		if (_instance == null)
			_instance = new Hero();
		return _instance;
	}

	public Hero()
	{
		init();
	}

	private static void HeroSetClanAndAlly(int charId, StatsSet hero)
	{
		Entry<L2Clan, L2Alliance> e = ClanTable.getInstance().getClanAndAllianceByCharId(charId);
		hero.set("clan_crest", e.getKey() == null ? 0 : e.getKey().getCrestId());
		hero.set("clan_name", e.getKey() == null ? "" : e.getKey().getName());
		hero.set("ally_crest", e.getValue() == null ? 0 : e.getValue().getAllyCrestId());
		hero.set("ally_name", e.getValue() == null ? "" : e.getValue().getAllyName());
		e = null;
	}


	private void init()
	{
		_heroes = new FastMap<Integer, StatsSet>().setShared(true);
		_completeHeroes = new FastMap<Integer, StatsSet>().setShared(true);
		_heroDiary = new FastMap<Integer, ArrayList<StatsSet>>();

		int diaryEntries = 0;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT * FROM heroes WHERE played = 1");
			rset = statement.executeQuery();
			while (rset.next())
			{
				StatsSet hero = new StatsSet();
				int charId = rset.getInt("char_id");
				hero.set("char_name", Olympiad.getNobleInfo(charId).char_name);
				hero.set("class_id", Olympiad.getNobleInfo(charId).class_id);
				hero.set("count", rset.getInt("count"));
				hero.set("played", rset.getInt("played"));
				hero.set("active", rset.getInt("active"));
				hero.set("message", rset.getString("message"));
				HeroSetClanAndAlly(charId, hero);
				_heroes.put(Integer.valueOf(charId), hero);
				diaryEntries += loadDiary(charId);
			}
			DatabaseUtils.closeDatabaseSR(statement, rset);

			statement = con.prepareStatement("SELECT * FROM heroes");
			rset = statement.executeQuery();
			while (rset.next())
			{
				StatsSet hero = new StatsSet();
				int charId = rset.getInt("char_id");
				hero.set("char_name", Olympiad.getNobleInfo(charId).char_name);
				hero.set("class_id", Olympiad.getNobleInfo(charId).class_id);
				hero.set("count", rset.getInt("count"));
				hero.set("played", rset.getInt("played"));
				hero.set("active", rset.getInt("active"));
				hero.set("message", rset.getString("message"));
				HeroSetClanAndAlly(charId, hero);
				_completeHeroes.put(charId, hero);
			}
		}
		catch (SQLException e)
		{
			_log.warning("Hero System: Couldnt load Heroes");
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		_log.info("Hero System: Loaded " + _heroes.size() + " Heroes.");
		_log.info("Hero System: Loaded " + diaryEntries + " entries for Hero Diarys.");
		_log.info("Hero System: Loaded " + _completeHeroes.size() + " all time Heroes.");
	}

	private int loadDiary(int charId)
	{
		ArrayList<StatsSet> diary = new ArrayList<StatsSet>();
		int diaryEntries = 0;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT * FROM heroes_diary WHERE charId=? ORDER BY time ASC");
			statement.setInt(1, charId);
			rset = statement.executeQuery();
			while (rset.next())
			{
				StatsSet diaryEntry = new StatsSet();
				long time = rset.getLong("time");
				int action = rset.getInt("action");
				int param = rset.getInt("param");

				String date = new SimpleDateFormat("yyyy-MM-dd HH").format(new Date(time * 1000L));
				diaryEntry.set("date", date);

				switch (action)
				{
					case 1:
						L2NpcTemplate template = NpcTable.getTemplate(param);
						if (template == null) 
							break;
						diaryEntry.set("action", new StringBuilder().append(template.name).append(" was defeated.").toString());
						break;
					case 2:
						diaryEntry.set("action", "Gained Hero status.");
						break;
					case 3:
						Castle castle = CastleManager.getInstance().getCastleByIndex(param);
						if (castle == null)
							break;
						diaryEntry.set("action", new StringBuilder().append(castle.getName()).append(" Castle was successfuly taken.").toString()); break;
					default:
						_log.warning(new StringBuilder().append("Hero System: Unhandled action in heros diary for charId: ").append(charId).toString());
				}

				diary.add(diaryEntry);
				diaryEntries++;
			}
			_heroDiary.put(charId, diary);
			DatabaseUtils.closeDatabaseSR(statement, rset);
		}
		catch (SQLException e)
		{
			_log.warning("Hero System: Couldnt load Heroes Diarys");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		return diaryEntries;
	}

	public Map<Integer, StatsSet> getHeroes()
	{
		return _heroes;
	}

  public synchronized void clearHeroes()
  {
    mysql.set("DELETE FROM heroes_diary WHERE `time` < UNIX_TIMESTAMP() - 31*24*60*60");
    mysql.set("UPDATE heroes SET played = 0, active = 0");

		if(!_heroes.isEmpty())
			for(StatsSet hero : _heroes.values())
			{
				if(hero.getInteger("active") == 0)
					continue;

				String name = hero.getString("char_name");

				L2Player player = L2World.getPlayer(name);

				if(player != null)
				{
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_L_HAND, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_R_HAND, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_LR_HAND, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_HAIR, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_HAIRALL, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_DHAIR, null);

					for(L2ItemInstance item : player.getInventory().getItems())
					{
						if(item == null)
							continue;
						if(item.isHeroWeapon())
							player.getInventory().destroyItem(item, 1, true);
					}

					for(L2ItemInstance item : player.getWarehouse().listItems(ItemClass.EQUIPMENT))
					{
						if(item == null)
							continue;
						if(item.isHeroWeapon())
							player.getWarehouse().destroyItem(item.getItemId(), 1);
					}

					player.setHero(false);
					player.updatePledgeClass();
					player.broadcastUserInfo(true);
				}
			}

		_heroes.clear();
	}

  public synchronized boolean computeNewHeroes(GArray<StatsSet> newHeroes)
  {
		if(newHeroes.isEmpty())
			return true;

		Map<Integer, StatsSet> heroes = new FastMap<Integer, StatsSet>().setShared(true);
		boolean error = false;

		for(StatsSet hero : newHeroes)
		{
			int charId = hero.getInteger("char_id");

			if(_completeHeroes != null && _completeHeroes.containsKey(charId))
			{
				StatsSet oldHero = _completeHeroes.get(charId);
				int count = oldHero.getInteger("count");
				oldHero.set("count", count + 1);
				oldHero.set("played", 1);
				oldHero.set("active", 0);

				heroes.put(charId, oldHero);
			}
			else
			{
				StatsSet newHero = new StatsSet();
				newHero.set("char_name", hero.getString("char_name"));
				newHero.set("class_id", hero.getInteger("class_id"));
				newHero.set("count", 1);
				newHero.set("played", 1);
				newHero.set("active", 0);
				newHero.set("message", "");

				heroes.put(charId, newHero);
			}

		}

		_heroes.putAll(heroes);
		heroes.clear();

		updateHeroes(0);

		return error;
	}

	public void updateHeroes(int id)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("REPLACE INTO heroes VALUES (?,?,?,?,?)");

			for(Integer heroId : _heroes.keySet())
			{
				if(id > 0 && heroId != id)
					continue;
				StatsSet hero = _heroes.get(heroId);
				try
				{
					statement.setInt(1, heroId);
					statement.setInt(2, hero.getInteger("count"));
					statement.setInt(3, hero.getInteger("played"));
					statement.setInt(4, hero.getInteger("active"));
					statement.setString(5, hero.getString("message"));
					statement.execute();
					if(_completeHeroes != null && !_completeHeroes.containsKey(heroId))
					{
						HeroSetClanAndAlly(heroId, hero);
						_completeHeroes.put(heroId, hero);
					}
				}
				catch(SQLException e)
				{
					_log.warning("Hero System: Couldnt update Hero: " + heroId);
					e.printStackTrace();
				}
			}

		}
		catch(SQLException e)
		{
			_log.warning("Hero System: Couldnt update Heroes");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public boolean isHero(int id)
	{
		if(_heroes == null || _heroes.isEmpty())
			return false;
		if(_heroes.containsKey(id) && _heroes.get(id).getInteger("active") == 1)
			return true;
		return false;
	}

	public boolean isInactiveHero(int id)
	{
		if(_heroes == null || _heroes.isEmpty())
			return false;
		if(_heroes.containsKey(id) && _heroes.get(id).getInteger("active") == 0)
			return true;
		return false;
	}

	public void activateHero(L2Player player)
	{
		StatsSet hero = _heroes.get(player.getObjectId());
		hero.set("active", 1);

		if(player.getBaseClassId() == player.getActiveClassId())
			addSkills(player);

		player.setHero(true);
		player.updatePledgeClass();
		player.broadcastPacket(new SocialAction(player.getObjectId(), SocialAction.VICTORY));
		if(player.getClan() != null && player.getClan().getLevel() >= 5)
		{
			player.getClan().incReputation(1000, true, "Hero:activateHero:" + player);
			player.getClan().broadcastToOtherOnlineMembers(new SystemMessage(SystemMessage.CLAN_MEMBER_S1_WAS_NAMED_A_HERO_2S_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addString(player.getName()).addNumber(Math.round(1000 * Config.RATE_CLAN_REP_SCORE)), player);
		}
		else
			player.broadcastUserInfo(true);
		setHeroGained(player.getObjectId());
		updateHeroes(player.getObjectId());
	}

	public static int getHeroByClass(int classId)
	{
		if (!_heroes.isEmpty())
			for (Integer heroId : _heroes.keySet())
			{
				StatsSet hero = _heroes.get(heroId);
				if (hero.getInteger("class_id") == classId)
					return heroId.intValue();
			}
		return 0;
	}

	public static void showHeroDiary(L2Player activeChar, int heroClass, int charId, int page)
	{
		StatsSet hero = _heroes.get(Integer.valueOf(charId));
		if (hero == null) 
			return;

		String htmlContent = Files.read("data/html/olympiad/herodiary.htm", activeChar);
		if (htmlContent == null)
			return;

		int perpage = 10;
		htmlContent = htmlContent.replaceFirst("%heroname%", hero.getString("char_name"));
		htmlContent = htmlContent.replace("%message%", hero.getString("message"));
		ArrayList<StatsSet> list = _heroDiary.get(charId);
		if ((list == null) || (list.isEmpty()))
		{
			htmlContent = htmlContent.replaceFirst("%list%", "");
			htmlContent = htmlContent.replaceFirst("%buttprev%", "");
			htmlContent = htmlContent.replaceFirst("%buttnext%", "");
		}
		else
		{
			Collections.reverse(list);

			boolean color = true;
			StringBuilder fList = new StringBuilder();

			int counter = 0;
			int breakat = 0;
			for (int i = (page - 1) * 10; i < list.size(); i++)
			{
				breakat = i;
				StatsSet diaryEntry = list.get(i);
				fList.append("<tr><td>");
				if (color)
					fList.append("<table width=270 bgcolor=\"131210\">");
				else
					fList.append("<table width=270>");
					fList.append("<tr><td width=270><font color=\"LEVEL\">").append(diaryEntry.getString("date")).append(":xx</font></td></tr>");
					fList.append("<tr><td width=270>").append(diaryEntry.getString("action")).append("</td></tr>");
					fList.append("<tr><td>&nbsp;</td></tr></table>");
					fList.append("</td></tr>");
					color = !color;
					counter++;
					if (counter >= 10) 
					{
						break;
					}
			}
		if (breakat < list.size() - 1)
			htmlContent = htmlContent.replaceFirst("%buttprev%", new StringBuilder().append("<button value=\"Prev\" action=\"bypass -h _diary?class=").append(heroClass).append("&page=").append(page + 1).append("\" width=60 height=25 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\">").toString());
		else 
		{
			htmlContent = htmlContent.replaceFirst("%buttprev%", "");
		}
		if (page > 1)
			htmlContent = htmlContent.replaceFirst("%buttnext%", new StringBuilder().append("<button value=\"Next\" action=\"bypass -h _diary?class=").append(heroClass).append("&page=").append(page - 1).append("\" width=60 height=25 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\">").toString());
		else 	
		{
			htmlContent = htmlContent.replaceFirst("%buttnext%", "");
		}
		htmlContent = htmlContent.replaceFirst("%list%", fList.toString());
		}

		NpcHtmlMessage diaryReply = new NpcHtmlMessage(activeChar, 5);
		diaryReply.setHtml(htmlContent);
		activeChar.sendPacket(diaryReply);
	}

	private void setHeroGained(int charId)
	{
		addDiaryData(charId, "Gained Hero status.");
		saveDiaryData(charId, 2, 0);
	}

	public void setRaidKilled(int charId, int npcId)
	{
		addDiaryData(charId, new StringBuilder().append(NpcTable.getTemplate(npcId).name).append(" was defeated.").toString());
		saveDiaryData(charId, 1, npcId);
	}

	public void setCastleTaken(int charId, int castleId)
	{
		Castle castle = CastleManager.getInstance().getCastleByIndex(castleId);
		String action = new StringBuilder().append(castle.getName()).append(" Castle was successfuly taken.").toString();
		addDiaryData(charId, action);
		saveDiaryData(charId, 3, castleId);
	}

	private void addDiaryData(int charId, String action)
	{
		StatsSet diaryEntry = new StatsSet();
		String date = new SimpleDateFormat("yyyy-MM-dd HH").format(new Date(System.currentTimeMillis()));
		diaryEntry.set("date", date);
		diaryEntry.set("action", action);

		ArrayList<StatsSet> diary = _heroDiary.get(charId);
		if (diary == null)
		{
			diary = new ArrayList<StatsSet>();
		}
		diary.add(diaryEntry);
		_heroDiary.put(Integer.valueOf(charId), diary);
	}

	private void saveDiaryData(int charId, int action, int param)
	{
		mysql.set("INSERT INTO heroes_diary (charId, time, action, param) values(?,?,?,?)", charId, (int)(System.currentTimeMillis() / 1000), action, param);
	}

	public void setHeroMessage(L2Player player, String message)
	{
		if ((player == null) || (message == null))
			return;
		StatsSet hero = _heroes.get(Integer.valueOf(player.getObjectId()));
		if (hero == null)
			return;
		hero.set("message", message);
		updateHeroes(player.getObjectId());
	}

	public static void addSkills(L2Player player)
	{
		player.addSkill(SkillTable.getInstance().getInfo(395, 1));
		player.addSkill(SkillTable.getInstance().getInfo(396, 1));
		player.addSkill(SkillTable.getInstance().getInfo(1374, 1));
		player.addSkill(SkillTable.getInstance().getInfo(1375, 1));
		player.addSkill(SkillTable.getInstance().getInfo(1376, 1));
	}

	public static void removeSkills(L2Player player)
	{
		player.removeSkillById(395);
		player.removeSkillById(396);
		player.removeSkillById(1374);
		player.removeSkillById(1375);
		player.removeSkillById(1376);
	}
}