package l2ft.gameserver.model.entity;

import l2ft.gameserver.model.L2Effect;
import l2ft.gameserver.model.L2Playable;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Spawn;
import l2ft.gameserver.model.instances.L2DoorInstance;
import l2ft.gameserver.skills.EffectType;
import l2ft.util.Crontab;
import l2ft.util.GArray;
import l2ft.util.Location;

public class InstancedZone
{
	public static class SpawnInfo
	{
		private final int _locationId;
		private final L2Spawn _spawn;
		private final int _type;

		public SpawnInfo(int locationId, L2Spawn spawn, int type)
		{
			_locationId = locationId;
			_spawn = spawn;
			_type = type;
		}
		public int getLocationId()
		{
			return _locationId;
		}

		public L2Spawn getSpawn()
		{
			return _spawn;
		}

		public int getType()
		{
			return _type;
		}
	}

	private int _id;
	private final String _name;
	private final Crontab _resetReuse;
	private boolean _isShowTimer;
	private boolean _isIncreaseTimer;
	private boolean _setReuseOnEnter;
	private String _timerText;
	private final int _timelimit;
	private boolean _dispellBuffs;
	private final int _minLevel;
	private final int _maxLevel;
	private final int _minParty;
	private final int _maxParty;
	private final Location _teleportCoords;
	private final Location _returnCoords;
	private final GArray<L2DoorInstance> _doors;
	private final GArray<SpawnInfo> _spawnsInfo;

	public InstancedZone(String name, Crontab resetReuse, int timelimit, int minLevel, int maxLevel, int minParty, int maxParty, Location tele, Location ret)
	{
		_id = 0;
		_name = name;
		_resetReuse = resetReuse;
		_timelimit = timelimit;
		_dispellBuffs = true;
		_minLevel = minLevel;
		_maxLevel = maxLevel;
		_teleportCoords = tele;
		_returnCoords = ret;
		_doors = new GArray<L2DoorInstance>();
		_minParty = minParty;
		_maxParty = maxParty;
		_isShowTimer = false;
		_isIncreaseTimer = true;
		_setReuseOnEnter = true;
		_timerText = "";
		_spawnsInfo = new GArray<SpawnInfo>();
	}

	public void setInstancedZoneId(int id)
	{
		_id = id;
	}

	public int getInstancedZoneId()
	{
		return _id;
	}

	public String getName()
	{
		return _name;
	}

	public Crontab getResetReuse()
	{
		return _resetReuse;
	}

	public boolean isDispellBuffs()
	{
		return _dispellBuffs;
	}

	public void setDispellBuffs(boolean val)
	{
		_dispellBuffs = val;
	}

	public int getTimelimit()
	{
		return _timelimit;
	}

	public int getMinLevel()
	{
		return _minLevel;
	}

	public int getMaxLevel()
	{
		return _maxLevel;
	}

	public int getMinParty()
	{
		return _minParty;
	}

	public int getMaxParty()
	{
		return _maxParty;
	}

	public Location getTeleportCoords()
	{
		return _teleportCoords;
	}

	public Location getReturnCoords()
	{
		return _returnCoords;
	}

	public GArray<L2DoorInstance> getDoors()
	{
		return _doors;
	}

	public GArray<SpawnInfo> getSpawnsInfo()
	{
		return _spawnsInfo;
	}

	public boolean isShowTimer()
	{
		return _isShowTimer;
	}

	public boolean isIncreaseTimer()
	{
		return _isIncreaseTimer;
	}

	public String getTimerText()
	{
		return _timerText;
	}

	public boolean isSetReuseOnEnter()
	{
		return _setReuseOnEnter;
	}

	public void setReuseOnEnter(boolean val)
	{
		_setReuseOnEnter = val;
	}


	public static void dispellBuffs(L2Player player)
	{
		doDispellBuffs(player);
		doDispellBuffs(player.getPet());
	}

	private static void doDispellBuffs(L2Playable playable)
	{
		if (playable != null)
			for (L2Effect e : playable.getEffectList().getAllEffects())
			{
				if (e.getEffectType() == EffectType.Vitality)
					continue;
				if (!e.getSkill().isOffensive() && !e.getSkill().getName().startsWith("Adventurer's "))
				e.exit();
			}
	}
}
