package l2ft.gameserver.model.entity;

import java.lang.ref.WeakReference;

import l2ft.common.ThreadPoolManager;
import l2ft.gameserver.instancemanager.NavitManager;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.serverpackets.ExNevitAdventEffect;
import l2ft.gameserver.serverpackets.ExNevitAdventPointInfoPacket;
import l2ft.gameserver.serverpackets.ExNevitAdventTimeChange;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.skills.AbnormalEffect;

public class NevitBlessing
{
	private final WeakReference<L2Player> _owner;
	private int _pointsNevit = 0;

	private boolean _bonusNevitActive = false;
	
	private int _timeBonusNevit = 0;

	private boolean _nevitBuff = false;

	public NevitBlessing(L2Player player)
	{
		_owner = new WeakReference<L2Player>(player);
	}

	private L2Player getPlayer()
	{
		return _owner.get();
	}

	public synchronized void addPoints(int count)
	{
		L2Player player = getPlayer();

		if (count != 0)
		{
			int before = _pointsNevit;
			_pointsNevit += count;
			_pointsNevit = Math.max(Math.min(_pointsNevit, 7200), 0);

			if (before < 5250 && _pointsNevit >= 5250)
				player.sendPacket(new SystemMessage(3269));
			else if (before < 3500 && _pointsNevit >= 3500)
				player.sendPacket(new SystemMessage(3268));
			else if (before < 1750 && _pointsNevit >= 1750)
				player.sendPacket(new SystemMessage(3267));
		}
		if (_pointsNevit >= 7200 && !isBuffActive() && !player.isInZonePeace())
		{
			_pointsNevit = 0;
			ThreadPoolManager.getInstance().executeGeneral(new NevitBuffTask(180));
			stopBonus();
		}

		if (!isBuffActive())
			player.sendPacket(new ExNevitAdventPointInfoPacket(_pointsNevit));
	}

	public int getPoints()
	{
		return _pointsNevit;
	}

	public boolean onNevitBonusTimeTick()
	{
		if (!_bonusNevitActive)
			return false;

		setBonusTime(_timeBonusNevit + 1);
		addPoints(180);
		return _timeBonusNevit < 240;
	}

	public void setBonusTime(int val)
	{
		_timeBonusNevit = val;
		getPlayer().sendPacket(new ExNevitAdventTimeChange(_timeBonusNevit * 60, _bonusNevitActive));
	}

	public int getBonusTime()
	{
		return _timeBonusNevit;
	}

	public boolean isBonusActive()
	{
		return _bonusNevitActive;
	}

	public void startBonus()
	{
		L2Player player;
		if (!_bonusNevitActive && _timeBonusNevit < 240 && !(player = getPlayer()).isInPeaceZone() && !isBuffActive())
		{
			_bonusNevitActive = true;
			getPlayer().sendPacket(new ExNevitAdventTimeChange(_timeBonusNevit * 60, true));
			if (!NavitManager.getInstance().containPlayer(player.getStoredId().longValue()))
				NavitManager.getInstance().addPlayer(player.getStoredId().longValue());
		}
  }

	public void stopBonus()
	{
		if (_bonusNevitActive)
		{
			_bonusNevitActive = false;
			getPlayer().sendPacket(new ExNevitAdventTimeChange(_timeBonusNevit * 60, false));
		}
	}

	public boolean isBuffActive()
	{
		return _nevitBuff;
	}

	public void stopBuff()
	{
		if (!isBuffActive())
			return;

		_nevitBuff = false;

		L2Player player = getPlayer();
		player.stopAbnormalEffect(AbnormalEffect.S_UNKNOWN20);
		player.sendPacket(new ExNevitAdventEffect(-1), new ExNevitAdventPointInfoPacket(_pointsNevit));
	}

	public class NevitBuffTask implements Runnable
	{
		int _time = 0;

		NevitBuffTask(int time)
		{
			_time = time;
		}

		public void run()
		{
			L2Player player = getPlayer();
			player.sendPacket(new ExNevitAdventEffect(_time));
			if (_time > -1)
			{
				_nevitBuff = true;
				player.setOldVitality(player.getVitality());
				player.setVitality(20000);
				player.startAbnormalEffect(AbnormalEffect.S_UNKNOWN20);
				player.sendPacket(new SystemMessage(3274));
				ThreadPoolManager.getInstance().scheduleGeneral(new NevitBuffTask(-1), _time * 1000);
			}
			else
			{
				_nevitBuff = false;
				player.setVitality(player.getOldVitality() * 1.8);
				player.stopAbnormalEffect(AbnormalEffect.S_UNKNOWN20);
			}
			
			player.sendPacket(new SystemMessage(_time > -1 ? 3266 : 3275));
			addPoints(0);
		}
	}
}