package l2ft.gameserver.model.entity.olympiad;

import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Map;

import javolution.util.FastMap;
import l2ft.Config;
import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.gameserver.Announcements;
import l2ft.gameserver.instancemanager.ServerVariables;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.base.ClassId;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.GArray;

public class OlympiadDatabase
{
	public static final String OLYMPIAD_LOAD_NOBLES = "SELECT * FROM `olympiad_nobles`";
	public static final String OLYMPIAD_SAVE_NOBLES = "REPLACE INTO `olympiad_nobles` (`char_id`, `class_id`, `char_name`, `olympiad_points`, `olympiad_points_past`, `olympiad_points_past_static`, `competitions_done`, `competitions_win`, `competitions_loose`, `competitions_week_0`, `competitions_week_1`, `competitions_week_2`, `competitions_week_3`) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)";
	public static final String OLYMPIAD_GET_HEROS = "SELECT `char_id`, `char_name` FROM `olympiad_nobles` WHERE `class_id` = ? AND `competitions_done` >= 15 AND `competitions_win` > 0 ORDER BY `olympiad_points` DESC, `competitions_win` DESC, `competitions_done` DESC";
	public static final String OLYMPIAD_GET_HEROS_SOULHOUND = "SELECT `char_id`, `char_name` FROM `olympiad_nobles` WHERE `class_id` IN (?, 133) AND `competitions_done` >= 15 AND `competitions_win` > 0 ORDER BY `olympiad_points` DESC, `competitions_win` DESC, `competitions_done` DESC";
	public static final String GET_ALL_CLASSIFIED_NOBLESS = "SELECT `char_id` FROM `olympiad_nobles` ORDER BY olympiad_points_past_static DESC";
	public static final String GET_EACH_CLASS_LEADER = "SELECT `char_name` FROM `olympiad_nobles` WHERE `class_id` = ? AND `olympiad_points_past_static` != 0 ORDER BY `olympiad_points_past_static` DESC LIMIT 10";
	public static final String GET_EACH_CLASS_LEADER_SOULHOUND = "SELECT `char_name` FROM `olympiad_nobles` WHERE `class_id` IN (?, 133) AND `olympiad_points_past_static` != 0 ORDER BY `olympiad_points_past_static` DESC LIMIT 10";
	public static final String OLYMPIAD_CALCULATE_LAST_PERIOD = "UPDATE `olympiad_nobles` SET `olympiad_points_past` = `olympiad_points`, `olympiad_points_past_static` = `olympiad_points` WHERE `competitions_done` >= 15";
	public static final String OLYMPIAD_CLEANUP_NOBLES = "UPDATE `olympiad_nobles` SET `olympiad_points` = 10, `competitions_done` = 0, `competitions_win` = 0, `competitions_loose` = 0";


	public static synchronized void loadNobles()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT * FROM `olympiad_nobles`");
			rset = statement.executeQuery();

			while(rset.next())
			{
				NobleInfo nobleInfo = new NobleInfo(rset);
				Olympiad._nobles.put(nobleInfo.char_id, nobleInfo);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public static synchronized void loadNoblesRank()
	{
		Olympiad._noblesRank = new FastMap<Integer, Integer>().setShared(true);
		Map<Integer, Integer> tmpPlace = new FastMap<Integer, Integer>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT `char_id` FROM `olympiad_nobles` ORDER BY olympiad_points_past_static DESC");
			rset = statement.executeQuery();

			int place = 1;
			while(rset.next())
				tmpPlace.put(rset.getInt("char_id"), place++);

			rset.close();
			statement.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		int rank1 = (int) Math.round(tmpPlace.size() * 0.01);
		int rank2 = (int) Math.round(tmpPlace.size() * 0.10);
		int rank3 = (int) Math.round(tmpPlace.size() * 0.25);
		int rank4 = (int) Math.round(tmpPlace.size() * 0.50);

		if (rank1 == 0)
		{
			rank1 = 1;
			rank2++;
			rank3++;
			rank4++;
		}

		for(int charId : tmpPlace.keySet())
			if(tmpPlace.get(charId) <= rank1)
				Olympiad._noblesRank.put(charId, 1);
			else if(tmpPlace.get(charId) <= rank2)
				Olympiad._noblesRank.put(charId, 2);
			else if(tmpPlace.get(charId) <= rank3)
				Olympiad._noblesRank.put(charId, 3);
			else if(tmpPlace.get(charId) <= rank4)
				Olympiad._noblesRank.put(charId, 4);
			else
				Olympiad._noblesRank.put(charId, 5);
	}

	public static synchronized void cleanupNobles()
	{
		Olympiad._log.info("Olympiad: Calculating last period...");
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE `olympiad_nobles` SET `olympiad_points_past` = `olympiad_points`, `olympiad_points_past_static` = `olympiad_points` WHERE `competitions_done` >= 15");
			statement.execute();
		}
		catch(Exception e)
		{
			Olympiad._log.warning("Olympiad System: Couldn't calculate last period!");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		Olympiad._log.info("Olympiad: Clearing nobles table...");
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE `olympiad_nobles` SET `olympiad_points` = 10, `competitions_done` = 0, `competitions_win` = 0, `competitions_loose` = 0");
			statement.execute();
		}
		catch(Exception e)
		{
			Olympiad._log.warning("Olympiad System: Couldn't cleanup nobles table!");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		for(Integer nobleId : Olympiad._nobles.keySet())
		{
			NobleInfo nobleInfo = Olympiad._nobles.get(nobleId);
			int points = nobleInfo.olympiad_points;
			int compDone = nobleInfo.competitions_done;
			nobleInfo.olympiad_points = 10;
			if (compDone >= 15)
			{
				nobleInfo.olympiad_points_past = points;
				nobleInfo.olympiad_points_past_static = points;
			}
			else
			{
				nobleInfo.olympiad_points_past = 0;
				nobleInfo.olympiad_points_past_static = 0;
			}
			nobleInfo.competitions_done = 0;
			nobleInfo.competitions_win = 0;
			nobleInfo.competitions_loose = 0;
			nobleInfo.competitions_week_0 = 0;
			nobleInfo.competitions_week_1 = 0;
			nobleInfo.competitions_week_2 = 0;
			nobleInfo.competitions_week_3 = 0;
		}
	}

  public static GArray<String> getClassLeaderBoard(int classId)
  {
		GArray<String> names = new GArray<String>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement(classId == 132 ? "SELECT `char_name` FROM `olympiad_nobles` WHERE `class_id` IN (?, 133) AND `olympiad_points_past_static` != 0 ORDER BY `olympiad_points_past_static` DESC LIMIT 10" : "SELECT `char_name` FROM `olympiad_nobles` WHERE `class_id` = ? AND `olympiad_points_past_static` != 0 ORDER BY `olympiad_points_past_static` DESC LIMIT 10");
			statement.setInt(1, classId);
			rset = statement.executeQuery();

			while(rset.next())
				names.add(rset.getString("char_name"));

			return names;
		}
		catch(Exception e)
		{
			Olympiad._log.warning("Olympiad System: Couldnt get heros from db: ");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		return names;
	}

	public static synchronized void sortHerosToBe()
	{
		if(Olympiad._period != 1)
			return;

		Olympiad._heroesToBe = new GArray<StatsSet>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			StatsSet hero;

			for (ClassId id : ClassId.values())
			{
				if (id.getId() == 133)
					continue;
				if(id.level() == 3)
				{
					statement = con.prepareStatement(id.getId() == 132 ? "SELECT `char_id`, `char_name` FROM `olympiad_nobles` WHERE `class_id` IN (?, 133) AND `competitions_done` >= 15 AND `competitions_win` > 0 ORDER BY `olympiad_points` DESC, `competitions_win` DESC, `competitions_done` DESC" : "SELECT `char_id`, `char_name` FROM `olympiad_nobles` WHERE `class_id` = ? AND `competitions_done` >= 15 AND `competitions_win` > 0 ORDER BY `olympiad_points` DESC, `competitions_win` DESC, `competitions_done` DESC");
					statement.setInt(1, id.getId());
					rset = statement.executeQuery();

					if (rset.next())
					{
						hero = new StatsSet();
						hero.set("class_id", id.getId());
						hero.set("char_id", rset.getInt("char_id"));
						hero.set("char_name", rset.getString("char_name"));

						Olympiad._heroesToBe.add(hero);
					}
					DatabaseUtils.closeDatabaseSR(statement, rset);
				}
			}
		}
		catch(Exception e)
		{
			Olympiad._log.warning("Olympiad System: Couldnt heros from db");
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public static synchronized void saveNobleData(int nobleId)
	{
		L2Player player = L2ObjectsStorage.getPlayer(nobleId);

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			NobleInfo nobleInfo = Olympiad._nobles.get(nobleId);

			statement = con.prepareStatement("REPLACE INTO `olympiad_nobles` (`char_id`, `class_id`, `char_name`, `olympiad_points`, `olympiad_points_past`, `olympiad_points_past_static`, `competitions_done`, `competitions_win`, `competitions_loose`, `competitions_week_0`, `competitions_week_1`, `competitions_week_2`, `competitions_week_3`) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)");
			statement.setInt(1, nobleId);
			statement.setInt(2, nobleInfo.class_id);
			statement.setString(3, player != null ? player.getName() : nobleInfo.char_name);
			statement.setInt(4, nobleInfo.olympiad_points);
			statement.setInt(5, nobleInfo.olympiad_points_past);
			statement.setInt(6, nobleInfo.olympiad_points_past_static);
			statement.setInt(7, nobleInfo.competitions_done);
			statement.setInt(8, nobleInfo.competitions_win);
			statement.setInt(9, nobleInfo.competitions_loose);
			statement.setInt(10, nobleInfo.competitions_week_0);
			statement.setInt(11, nobleInfo.competitions_week_1);
			statement.setInt(12, nobleInfo.competitions_week_2);
			statement.setInt(13, nobleInfo.competitions_week_3);
			statement.execute();
		}
		catch (Exception e)
		{
			Olympiad._log.warning(new StringBuilder().append("Olympiad System: Couldnt save noble info in db for player ").append(player != null ? player.getName() : "null").toString());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public static synchronized void saveNobleData()
	{
		if(Olympiad._nobles == null)
			return;
		for(Integer nobleId : Olympiad._nobles.keySet())
			saveNobleData(nobleId);
	}

	public static synchronized void setNewOlympiadEnd()
	{
		Calendar currentTime = Calendar.getInstance();
		if (!nextEndDay(currentTime))
			return;
		currentTime.set(11, 0);
		currentTime.set(12, 0);
		Olympiad._olympiadEnd = currentTime.getTimeInMillis();
		Olympiad._currentRound += 1;
		Calendar nextChange = Calendar.getInstance();
		Olympiad._nextWeeklyChange = nextChange.getTimeInMillis() + Config.ALT_OLY_WPERIOD;

		Olympiad._isOlympiadEnd = false;
		Announcements.getInstance().announceToAll(new SystemMessage(SystemMessage.OLYMPIAD_PERIOD_S1_HAS_STARTED).addNumber(Olympiad._currentCycle));
	}

	public static boolean nextEndDay(Calendar currentTime)
	{
		int nextDay = 0;
		for (int i : Config.ALT_OLY_DATE_END)
		{
			if ((i > currentTime.get(5)) && (i < currentTime.getMaximum(5)))
			nextDay = i;
		}
		if (nextDay == 0)
			nextDay = Config.ALT_OLY_DATE_END[0];
		if ((nextDay > 0) && (nextDay <= currentTime.getMaximum(5)))
		{
			do
				currentTime.add(5, 1);
			while (currentTime.get(5) != nextDay);

			return true;
		}

		Olympiad._log.warning("Olympiad not is not initialized. Bad config 'AltOlyDateEnd'.");
		return false;
	}

	public static void save()
	{
		saveNobleData();
		ServerVariables.set("Olympiad_CurrentCycle", Olympiad._currentCycle);
		ServerVariables.set("Olympiad_Period", Olympiad._period);
		ServerVariables.set("Olympiad_End", Olympiad._olympiadEnd);
		ServerVariables.set("Olympiad_ValdationEnd", Olympiad._validationEnd);
		ServerVariables.set("Olympiad_NextWeeklyChange", Olympiad._nextWeeklyChange);
		ServerVariables.set("Olympiad_Round", Olympiad._currentRound);
	}
}