package l2ft.gameserver.model.entity.olympiad;

import java.util.concurrent.ScheduledFuture;

import l2ft.extensions.listeners.L2TerritoryEnterLeaveListener;
import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.instancemanager.InstancedZoneManager;
import l2ft.gameserver.instancemanager.ZoneManager;
import l2ft.gameserver.model.L2Object;
import l2ft.gameserver.model.L2Playable;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Territory;
import l2ft.gameserver.model.L2World;
import l2ft.gameserver.model.L2Zone;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.entity.InstancedZone;
import l2ft.gameserver.model.instances.L2DoorInstance;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.instances.L2OlympiadManagerInstance;
import l2ft.gameserver.serverpackets.ExOlympiadMatchEnd;
import l2ft.gameserver.serverpackets.ExOlympiadUserInfo;
import l2ft.gameserver.serverpackets.L2GameServerPacket;
import l2ft.gameserver.serverpackets.Say2;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.DoorTable;
import l2ft.util.GArray;
import l2ft.util.GCSArray;
import l2ft.util.Log;
import l2ft.util.Rnd;

public class OlympiadGame extends Reflection
{
	public static final int MAX_POINTS_LOOSE = 10;

	public TerritoryListener _territoryListener = new TerritoryListener();

	public boolean validated = false;

	private int _winner = 0;
	private int _state = 0;
	private int _gameId;
	private CompType _type;
	private OlympiadTeam _team1;
	private OlympiadTeam _team2;
	private GCSArray<L2Player> _spectators = new GCSArray<L2Player>();

	private GArray<L2NpcInstance> _buffers = new GArray<L2NpcInstance>();
	public L2Zone _zone;
	public L2Territory _door1Territory;
	public L2Territory _door2Territory;
	OlympiadGameTask _task;
	ScheduledFuture<?> _shedule;

	public L2Zone getZone()
	{
		return _zone;
	}

	public OlympiadGame(int id, CompType type, GCSArray<Integer> opponents)
	{
		super("Olympiad" + id);

		int arenaType = Rnd.get(4);

		InstancedZone il = InstancedZoneManager.getInstancedZoneById(500 + arenaType);

		FillSpawns(il.getSpawnsInfo());
		FillDoors(il.getDoors());

		_door1Territory = (getDoors().get(0)).getGeoPos();
		_door2Territory = (getDoors().get(1)).getGeoPos();

		_door1Territory.getListenerEngine().addMethodInvokedListener(_territoryListener);
		_door2Territory.getListenerEngine().addMethodInvokedListener(_territoryListener);

		_zone = ZoneManager.getInstance().getZoneByIndex(L2Zone.ZoneType.OlympiadStadia, arenaType + 1, false);

		_type = type;
		_gameId = id;

		_team1 = new OlympiadTeam(this, 1);
		_team2 = new OlympiadTeam(this, 2);

		for(int i = 0; i < opponents.size() / 2; i++)
			_team1.addMember(opponents.get(i));

		for(int i = opponents.size() / 2; i < opponents.size(); i++)
			_team2.addMember(opponents.get(i));
			
		closeDoors();

		for (L2NpcInstance buffer : findNPC(36402))
		{
			_buffers.add(buffer);
		}
		Log.add("Olympiad System: Game - " + id + ": " + _team1.getName() + " Vs " + _team2.getName(), "olympiad");
	}

	public OlympiadTeam getTeam1()
	{
		return _team1;
	}

	public OlympiadTeam getTeam2()
	{
		return _team2;
	}

	public void managerShout()
	{
		try
		{
			for (L2OlympiadManagerInstance npc : Olympiad.getNpcs())
			{
				if (_type == CompType.CLASSED)
					say(npc, "l2ft.gameserver.model.entity.OlympiadGame.OlympiadClassed", _gameId + 1);
				else if (_type == CompType.TEAM || _type == CompType.TEAM_RANDOM)
					say(npc, "l2ft.gameserver.model.entity.OlympiadGame.OlympiadTeam", _gameId + 1);
				else
					say(npc, "l2ft.gameserver.model.entity.OlympiadGame.OlympiadNonClassed", _gameId + 1);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void say(L2OlympiadManagerInstance npc, String msg, int arenaId)
	{
		for (L2Player player : L2World.getAroundPlayers(npc, 4000, 1000))
			if ((player != null) && (!player.isBlockAll()))
			{
				String[] m = new CustomMessage(msg, player, new Object[0]).addNumber(arenaId).toString().split(": ");
				player.sendPacket(new Say2(0, 1, m[0], m[1]));
			}
	}

	public void portPlayersToArena()
	{
		_team1.portPlayersToArena();
		_team2.portPlayersToArena();
	}

	public void preparePlayers()
	{
		_team1.preparePlayers();
		_team2.preparePlayers();
	}

	public void restoreHpMpCp()
	{
		_team1.restoreHpMpCp();
		_team2.restoreHpMpCp();
	}

	public void portPlayersBack()
	{
		_team1.portPlayersBack();
		_team2.portPlayersBack();
	}

	public void validateWinner(boolean aborted) throws Exception
	{
		int state = _state;
		_state = 0;

		broadcastPacket(new ExOlympiadMatchEnd(), true, false);

		if(validated)
		{
			Log.add("Olympiad Result: " + _team1.getName() + " vs " + _team2.getName() + " ... double validate check!!!", "olympiad");
			return;
		}
		validated = true;

		if ((state < 1) && (aborted))
		{
			_team1.takePointsForCrash();
			_team2.takePointsForCrash();
			broadcastPacket(Msg.THE_GAME_HAS_BEEN_CANCELLED_BECAUSE_THE_OTHER_PARTY_ENDS_THE_GAME, true, false);
			return;
		}

		boolean teamOneCheck = _team1.checkPlayers();
		boolean teamTwoCheck = _team2.checkPlayers();

		if(_winner <= 0)
			if(!teamOneCheck && !teamTwoCheck)
				_winner = 0;
			else if(!teamTwoCheck)
				_winner = 1; // Выиграла первая команда
			else if(!teamOneCheck)
				_winner = 2; // Выиграла вторая команда
			else if(_team1.getDamage() < _team2.getDamage()) // Вторая команда нанесла вреда меньше, чем первая
				_winner = 1; // Выиграла первая команда
			else if(_team1.getDamage() > _team2.getDamage()) // Вторая команда нанесла вреда больше, чем первая
				_winner = 2; // Выиграла вторая команда

		try
		{
			if(_winner == 1) // Выиграла первая команда
				_team1.winGame(_team2);
			else if(_winner == 2) // Выиграла вторая команда
				_team2.winGame(_team1);
			else
				_team1.tie(_team2);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		_team1.saveNobleData();
		_team2.saveNobleData();

    broadcastPacket(new SystemMessage(SystemMessage.YOU_WILL_GO_BACK_TO_THE_VILLAGE_IN_S1_SECOND_S).addNumber(20), true, true);
  }

	public int getGameId()
	{
		return _gameId;
	}

	public boolean isRegistered(int objId)
	{
		return _team1.contains(objId) || _team2.contains(objId);
	}

	public GCSArray<L2Player> getSpectators()
	{
		return _spectators;
	}

	public void addSpectator(L2Player spec)
	{
		_spectators.add(spec);
	}

	public void removeSpectator(L2Player spec)
	{
		_spectators.remove(spec);
	}

	public void clearSpectators()
	{
		for(L2Player pc : _spectators)
			if(pc != null && pc.inObserverMode())
				pc.leaveOlympiadObserverMode();
		_spectators.clear();
	}

	public void broadcastInfo(L2Player sender, L2Player receiver, boolean onlyToSpectators)
	{
		if(sender != null)
			if(receiver != null)
				receiver.sendPacket(new ExOlympiadUserInfo(sender, sender.getOlympiadSide()));
			else
				broadcastPacket(new ExOlympiadUserInfo(sender, sender.getOlympiadSide()), !onlyToSpectators, true);
		else
		{
			// Рассылаем информацию о первой команде
			for(L2Player player : _team1.getPlayers())
				if(receiver != null)
					receiver.sendPacket(new ExOlympiadUserInfo(player, player.getOlympiadSide()));
				else
					broadcastPacket(new ExOlympiadUserInfo(player, player.getOlympiadSide()), !onlyToSpectators, true);

			// Рассылаем информацию о второй команде
			for(L2Player player : _team2.getPlayers())
				if(receiver != null)
					receiver.sendPacket(new ExOlympiadUserInfo(player, player.getOlympiadSide()));
				else
					broadcastPacket(new ExOlympiadUserInfo(player, player.getOlympiadSide()), !onlyToSpectators, true);
		}
	}

	public void broadcastPacket(L2GameServerPacket packet, boolean toTeams, boolean toSpectators)
	{
		if(toTeams)
		{
			_team1.broadcast(packet);
			_team2.broadcast(packet);
		}

		if(toSpectators && _spectators != null)
			for(L2Player spec : _spectators)
				if(spec != null)
					spec.sendPacket(packet);
	}

	public GArray<L2Player> getAllPlayers()
	{
		GArray<L2Player> result = new GArray<L2Player>();
		for (L2Player player : _team1.getPlayers())
			result.add(player);
		for (L2Player player : _team2.getPlayers())
			result.add(player);
		if (_spectators != null)
			for (L2Player spec : _spectators)
				if (spec != null)
					result.add(spec);
		return result;
  }

	public void setWinner(int val)
	{
		_winner = val;
	}

	public void setState(int val)
	{
		_state = val;
	}

	public int getState()
	{
		return _state;
	}

	public GArray<L2Player> getTeamMembers(L2Player player)
	{
		return player.getOlympiadSide() == 1 ? _team1.getPlayers() : _team2.getPlayers();
	}

	public void addDamage(L2Player player, double damage)
	{
		if(player.getOlympiadSide() == 1)
			_team1.addDamage(damage);
		else
			_team2.addDamage(damage);
	}

	public boolean doDie(L2Player player)
	{
		return player.getOlympiadSide() == 1 ? _team1.doDie(player) : _team2.doDie(player);
	}

	public boolean checkPlayersOnline()
	{
		return _team1.checkPlayers() && _team2.checkPlayers();
	}

	public boolean logoutPlayer(L2Player player)
	{
		if(player == null)
			return false;
		return player.getOlympiadSide() == 1 ? _team1.logout(player) : _team2.logout(player);
	}

	public synchronized void sheduleTask(OlympiadGameTask task)
	{
		if(_shedule != null)
			_shedule.cancel(false);
		_task = task;
		_shedule = task.shedule();
	}

	public OlympiadGameTask getTask()
	{
		return _task;
	}

	public BattleStatus getStatus()
	{
		if(_task != null)
			return _task.getStatus();
		return BattleStatus.Begining;
	}

	public void endGame(long time, boolean aborted)
	{
		try
		{
			validateWinner(aborted);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		sheduleTask(new OlympiadGameTask(this, BattleStatus.Ending, 0, time));
	}

	public CompType getType()
	{
		return _type;
	}

	public void showBuffers()
	{
		for(L2NpcInstance buffer : _buffers)
			buffer.spawnMe();
	}

	public void hideBuffers()
	{
		for(L2NpcInstance buffer : _buffers)
			buffer.decayMe();
	}

	public void openDoors()
	{
		for (L2DoorInstance door : getDoors())
			door.openMe();
	}

	public void closeDoors()
	{
		for (L2DoorInstance door : getDoors())
			door.closeMe();
	}

	public void collapse()
	{
		super.collapse();
		_door1Territory.getListenerEngine().removeMethodInvokedListener(_territoryListener);
		_door2Territory.getListenerEngine().removeMethodInvokedListener(_territoryListener);
	}

	static
	{
		DoorTable.getInstance().getDoor(17100001).openMe();
		DoorTable.getInstance().getDoor(17100002).openMe();
		DoorTable.getInstance().getDoor(17100101).openMe();
		DoorTable.getInstance().getDoor(17100102).openMe();
		DoorTable.getInstance().getDoor(17100201).openMe();
		DoorTable.getInstance().getDoor(17100202).openMe();
		DoorTable.getInstance().getDoor(17100301).openMe();
		DoorTable.getInstance().getDoor(17100302).openMe();
	}

	public class TerritoryListener extends L2TerritoryEnterLeaveListener
	{
		public void objectEntered(L2Territory zone, L2Object object)
		{
		}

		@Override
		public void objectLeaved(L2Territory zone, L2Object object)
		{
			if (object.isPlayable())
			{
				Reflection reflection = object.getReflection();
				OlympiadGame game = null;
				if (reflection instanceof OlympiadGame)
					game = (OlympiadGame)reflection;
				if (game == null || game.getDoors().get(0).isOpen())
					return;
				int[] tele;
				if (zone == _door1Territory)
					tele = getZone().getSpawns().get(0);
				else
				{
					if (zone == _door2Territory)
						tele = getZone().getSpawns().get(1);
					else
						return;
				}
				((L2Playable)object).teleToLocation(tele[0], tele[1], tele[2]);
			}
		}
	}
}