package l2ft.gameserver.model.entity.olympiad;

import java.util.Collection;
import java.util.Map;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2ft.Config;
import l2ft.util.GArray;
import l2ft.util.GCSArray;
import l2ft.util.Rnd;

public class OlympiadManager extends Thread
{
	private static Logger _log = Logger.getLogger(OlympiadManager.class.getName());
	private FastMap<Integer, OlympiadGame> _olympiadGames = new FastMap<Integer, OlympiadGame>().setShared(true);
	private int _currentGameId = 0;

	public void wait2(long time)
	{
		try
		{
			Thread.sleep(time);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public synchronized void run()
	{
		if(Olympiad.isOlympiadEnd())
			return;

		while(Olympiad.inCompPeriod())
		{
			if(Olympiad._nobles.isEmpty())
			{
				wait2(60000);
				continue;
			}

			while(Olympiad.inCompPeriod())
			{
				// Подготовка и запуск внеклассовых боев
				if(Olympiad._nonClassBasedRegisters.size() >= Config.NONCLASS_GAME_MIN)
					prepareBattles(CompType.NON_CLASSED, Olympiad._nonClassBasedRegisters);

				// Подготовка и запуск классовых боев
				for(Map.Entry<Integer, GCSArray<Integer>> entry : Olympiad._classBasedRegisters.entrySet())
					if(entry.getValue().size() >= Config.CLASS_GAME_MIN)
						prepareBattles(CompType.CLASSED, entry.getValue());

				// Подготовка и запуск командных боев случайного типа
				if(Olympiad._teamRandomBasedRegisters.size() >= Config.RANDOM_TEAM_GAME_MIN)
					prepareBattles(CompType.TEAM_RANDOM, Olympiad._teamRandomBasedRegisters);

				// Подготовка и запуск командных боев
				if(Olympiad._teamBasedRegisters.size() >= Config.TEAM_GAME_MIN)
					prepareTeamBattles(CompType.TEAM, Olympiad._teamBasedRegisters.values());

				wait2(10000);
			}

			wait2(30000);
		}

		Olympiad._classBasedRegisters.clear();
		Olympiad._nonClassBasedRegisters.clear();
		Olympiad._teamRandomBasedRegisters.clear();
		Olympiad._teamBasedRegisters.clear();

		// when comp time finish wait for all games terminated before execute the cleanup code
		boolean allGamesTerminated = false;

		// wait for all games terminated
		while(!allGamesTerminated)
		{
			wait2(30000);

			if(_olympiadGames.isEmpty())
				break;

			allGamesTerminated = true;
			for(OlympiadGame game : _olympiadGames.values())
				if(game.getTask() != null && !game.getTask().isTerminated())
					allGamesTerminated = false;
		}

		_olympiadGames.clear();
	}

	private void prepareBattles(CompType type, GCSArray<Integer> list)
	{
		try
		{
			for (int i = 1; i <= 100; i++)
			{
				if (list.size() < type.getMinSize())
				{
					return;
				}
				OlympiadGame game = new OlympiadGame(_currentGameId, type, nextOpponents(list, type));
				game.sheduleTask(new OlympiadGameTask(game, BattleStatus.Begining, 0, 1));

				_olympiadGames.put(_currentGameId, game);
				_currentGameId += 1;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void prepareTeamBattles(CompType type, Collection<GCSArray<Integer>> list)
	{
		try
		{
			for (int i = 1; i <= 100; i++)
			{
				if (list.size() < type.getMinSize())
				{
					return;
				}
				GCSArray<Integer> nextOpponents = nextTeamOpponents(list, type);
				if (nextOpponents == null)
				{
					return;
				}
				OlympiadGame game = new OlympiadGame(_currentGameId, type, nextOpponents);
				game.sheduleTask(new OlympiadGameTask(game, BattleStatus.Begining, 0, 1));

				_olympiadGames.put(_currentGameId, game);
				_currentGameId += 1;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void freeOlympiadInstance(OlympiadGame game)
	{
		game.startCollapseTimer(0);
		_olympiadGames.remove(game.getGameId());
	}

	public OlympiadGame getOlympiadGame(int index)
	{
		return _olympiadGames.get(index);
	}

	public FastMap<Integer, OlympiadGame> getOlympiadGames()
	{
		return _olympiadGames;
	}

	private GCSArray<Integer> nextOpponents(GCSArray<Integer> list, CompType type)
	{
		GCSArray<Integer> opponents = new GCSArray<Integer>();

		int first = Rnd.get(list.size());
		Integer noble = list.remove(first);
		opponents.add(noble);
		removeOpponent(noble);

		for (int i = 0; i < type.getMinSize() - 1; i++)
		{
			int min = Math.max(first - 2, 0);
			int max = Math.min(first + 2, list.size() - 1);
			noble = list.remove(Rnd.get(min, max));
			opponents.add(noble);
			removeOpponent(noble);
		}

		return opponents;
	}

	private GCSArray<Integer> nextTeamOpponents(Collection<GCSArray<Integer>> list, CompType type)
	{
		GCSArray<Integer> opponents = new GCSArray<Integer>();
		GArray<GCSArray<Integer>> a = new GArray<GCSArray<Integer>>();
		for (int i = 0; i < type.getMinSize(); i++)
		{
			if (a.isEmpty())
			{
				_log.warning("OlympiadManager[184] error!");
				Thread.dumpStack();
				return null;
			}

			GCSArray<Integer> team = a.remove(Rnd.get(a.size()));
			if (team.size() == 3)
			{
				for (Integer noble : team)
				{
					opponents.add(noble);
					removeOpponent(noble);
				}
			}
			else
			{
				for (Integer noble : team)
					removeOpponent(noble);
				i--;
			}

		list.remove(team);
		}

		return opponents;
	}

	private void removeOpponent(Integer noble)
	{
		Olympiad._classBasedRegisters.removeValue(noble);
		Olympiad._nonClassBasedRegisters.remove(noble);
		Olympiad._teamRandomBasedRegisters.remove(noble);
		Olympiad._teamBasedRegisters.removeValue(noble);
	}
}