package l2ft.gameserver.model.events;

import java.util.logging.Logger;
import l2ft.extensions.scripts.Functions;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Object;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.model.items.L2ItemInstance;

public abstract class GameEvent
{
	private static Logger _log = Logger.getLogger(GameEvent.class.getName());
	public Long self;
	public Long npc;
	public static final int STATE_INACTIVE = 0;
	public static final int STATE_ACTIVE = 1;
	public static final int STATE_RUNNING = 2;

	public int getState()
	{
		return 0;
	}

	public abstract String getName();

	public long getNextTime()
	{
		return 0;
	}

	public boolean isRunning()
	{
		return getState() == 2;
	}

	public boolean canRegister(L2Player player, boolean first)
	{
		return (getState() == 1) && (!isParticipant(player)) && (player._event == null);
	}

	public abstract boolean isParticipant(L2Player paramL2Player);

	public abstract boolean register(L2Player paramL2Player);

	public abstract void unreg(L2Player paramL2Player);

	public abstract void remove(L2Player paramL2Player);

	public abstract void start();

	public abstract void finish();

	public abstract void abort();

	public boolean canAttack(L2Character attacker, L2Character target)
	{
		return true;
	}

	public boolean checkPvP(L2Character attacker, L2Character target)
	{
		return getState() != 2;
	}

	public boolean canUseItem(L2Player actor, L2ItemInstance item)
	{
		return true;
	}

	public boolean canUseSkill(L2Character caster, L2Character target, L2Skill skill)
	{
		return true;
	}

	public abstract void onLogout(L2Player paramL2Player);

	public abstract void doDie(L2Character paramL2Character1, L2Character paramL2Character2);

	public boolean canTeleportOnDie(L2Player player)
	{
		return getState() != 2;
	}

	public boolean canLostExpOnDie()
	{
		return getState() != 2;
	}

	public int getCountPlayers()
	{
		return 0;
	}

	public StringBuffer getInformation(L2Player player)
	{
		return null;
	}

	public boolean talkWithNpc(L2Player player, L2NpcInstance npc)
	{
		return false;
	}

	public static void unRide(L2Player player)
	{
		Functions.unRide(player);
	}

	public static void unSummonPet(L2Player player, boolean onlyPets)
	{
		Functions.unSummonPet(player, onlyPets);
	}
	
	public L2Object getSelf()
	{
		return L2ObjectsStorage.get(self);
	}
	
	public L2Player getSelfPlayer()
	{
		return L2ObjectsStorage.getAsPlayer(self);
	}

	public L2NpcInstance getNpc()
	{
		return L2ObjectsStorage.getAsNpc(npc);
	}
}