package l2ft.gameserver.model.events;

import java.util.Collection;
import java.util.HashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.logging.Logger;
import l2ft.common.ThreadPoolManager;
import l2ft.gameserver.model.L2Player;

public class GameEventManager
{
	private static Logger _log = Logger.getLogger(GameEventManager.class.getName());
	private static GameEventManager _instance;
	private HashMap<String, GameEvent> _events;
	private ScheduledFuture<?> event_sched;
	private GameEvent event;

	public GameEventManager()
	{
		_events = new HashMap<String, GameEvent>();
		event_sched = null;
		event = null;
	}

	public static GameEventManager getInstance()
	{
		if (_instance == null)
			_instance = new GameEventManager();
		return _instance;
	}

	public void registerEvent(GameEvent evt)
	{
		_events.put(evt.getName(), evt);
		nextEvent();
	}

	public boolean nextEvent()
	{
		long min = 0;
		long time = 0;
		GameEvent nextEv = null;
		for (GameEvent ev : _events.values())
		{
			time = ev.getNextTime();
			if (min > time || min == 0)
				min = time;
		}
		for (GameEvent ev : _events.values())
		{
			time = ev.getNextTime();
			if (min == time)
			{
				nextEv = ev;
				break;
			}
		}

		if (event_sched != null)
			event_sched.cancel(true);

		event = nextEv;

		if (event == null)
		{
			_log.info("Event load: error");
			return false;
		}

		_log.info("Event " + event.getName() + " started in " + Long.toString((time - System.currentTimeMillis() / 1000) / 60) + " mins.");
		event_sched = ThreadPoolManager.getInstance().scheduleGeneral(new EventStart(), time * 1000 - System.currentTimeMillis());
		return true;
	}

	public GameEvent findEvent(String name)
	{
		return _events.get(name);
	}

	public Collection<GameEvent> getAllEvents()
	{
		return _events.values();
	}

	public GameEvent participantOf(L2Player player)
	{
		for (GameEvent evt : getAllEvents())
		{
			if (evt.isParticipant(player))
				return evt;
		}
		return null;
	}

	private class EventStart implements Runnable
	{
		private EventStart()
		{
		}

		public void run()
		{
			if (event != null) 
				event.start();
			_log.info("Event " + event.getName() + " has been started.");
		}
	}
}