package l2ft.gameserver.model.instances;

import l2ft.gameserver.model.L2Clan;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2SkillLearn;
import l2ft.gameserver.model.entity.siege.territory.TerritorySiege;
import l2ft.gameserver.serverpackets.AcquireSkillList;
import l2ft.gameserver.serverpackets.NpcHtmlMessage;
import l2ft.gameserver.skills.EffectType;
import l2ft.gameserver.tables.ClanTable;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.tables.SkillTreeTable;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.util.Location;
import l2ft.util.Rnd;

public class L2CastleCourtInstance extends L2NpcInstance
{
	//private static Logger _log = Logger.getLogger(L2CastleCourtInstance.class.getName());

	protected static final int COND_ALL_FALSE = 0;
	protected static final int COND_BUSY_BECAUSE_OF_SIEGE = 1;
	protected static final int COND_OWNER = 2;

	/**
	 * @param template
	 */
	public L2CastleCourtInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		if(!canBypassCheck(player, this))
			return;

		int condition = validateCondition(player);
		if(condition <= COND_ALL_FALSE || condition == COND_BUSY_BECAUSE_OF_SIEGE)
			return;

		if((player.getClanPrivileges() & L2Clan.CP_CS_USE_FUNCTIONS) != L2Clan.CP_CS_USE_FUNCTIONS)
		{
			player.sendMessage("You don't have rights to do that.");
			return;
		}

		if (command.equals("subskills"))
		{
			if (player.getClan() != null)
			{
				if (player.isClanLeader())
				{
					L2SkillLearn[] skills = SkillTreeTable.getInstance().getAvailableClanSkills(player.getClan(), true);
					if(skills == null)
					{
						String filename = "data/html/castle/CourtMagician/CourtMagician-2.htm";
						showChatWindow(player, filename);
						return;
					}
					AcquireSkillList asl = new AcquireSkillList(AcquireSkillList.CLAN_ADDITIONAL);
					for(L2SkillLearn skill : skills)
						if(SkillTable.getInstance().getInfo(skill.getId(), skill.getLevel()) != null)
							asl.addSkill(skill.getId(), skill.getLevel(), skill.getLevel(), skill.getRepCost(), skill.getItemId());
					player.sendPacket(asl);
				}
			}
		}
		
		if(command.equalsIgnoreCase("ClanGate"))
		{
			final L2Clan clan = ClanTable.getInstance().getClan(getCastle().getOwnerId());
			if(clan != null)
			{
				final L2Player cl = clan.getLeader().getPlayer();
				if(cl != null && cl.getEffectList().getEffectByType(EffectType.ClanGate) != null)
				{
					Location dst = Rnd.coordsRandomize(cl, 50);
					player.teleToLocation(dst);
				}

				else
				{
					final NpcHtmlMessage html = new NpcHtmlMessage(player, this);
					html.setFile("data/html/castle/CourtMagician/CourtMagician-nocg.htm");

					html.replace("%objectId%", String.valueOf(getObjectId()));
					html.replace("%npcname%", String.valueOf(getName() + " " + getTitle()));
					player.sendPacket(html);
				}
			}
		}
		else
			super.onBypassFeedback(player, command);
		
		if(condition == COND_OWNER)
			super.onBypassFeedback(player, command);
	}

	@Override
	public void showChatWindow(L2Player player, int val)
	{
		player.sendActionFailed();
		String filename = "data/html/castle/CourtMagician/CourtMagician-no.htm";

		int condition = validateCondition(player);
		if(condition > COND_ALL_FALSE)
			if(condition == COND_BUSY_BECAUSE_OF_SIEGE)
				filename = "data/html/castle/CourtMagician/CourtMagician-busy.htm"; // Busy because of siege
			else if(condition == COND_OWNER)
				if(val == 0)
					filename = "data/html/castle/CourtMagician/CourtMagician.htm";
				else
					filename = "data/html/castle/CourtMagician/CourtMagician-" + val + ".htm";

		NpcHtmlMessage html = new NpcHtmlMessage(player, this);
		html.setFile(filename);
		html.replace("%objectId%", String.valueOf(getObjectId()));
		html.replace("%npcname%", getName());
		player.sendPacket(html);
	}

	protected int validateCondition(L2Player player)
	{
		if(player.isGM())
			return COND_OWNER;
		if(getCastle() != null && getCastle().getId() > 0)
			if(player.getClan() != null)
				if(getCastle().getSiege().isInProgress() || TerritorySiege.isInProgress())
					return COND_BUSY_BECAUSE_OF_SIEGE; // Busy because of siege
				else if(getCastle().getOwnerId() == player.getClanId()) // Clan owns castle
					return COND_OWNER;
		return COND_ALL_FALSE;
	}
}