package l2ft.gameserver.model.instances;

import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2World;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.serverpackets.NpcHtmlMessage;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.tables.NpcTable;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.util.Location;
import l2ft.util.Rnd;

public class L2DragonVortexInstance extends L2NpcInstance
{
	private final int ItemForSpawn = 17248;
	private static final int[] MOBS_ID = { 25718, 25719, 25720, 25721, 25722, 25723, 25724 };

	public L2DragonVortexInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		if(!canBypassCheck(player, this))
			return;

		if(command.startsWith("spawn_rb"))
		{
			L2ItemInstance item = player.getInventory().getItemByItemId(ItemForSpawn);
			if (item != null && item.getCount() >= 1)
			{
				int id = Rnd.get(MOBS_ID.length);
				int BossId = MOBS_ID[id];
				if(this.getNpcId() == 32871 && checkSpawn())
				{
					player.getInventory().destroyItemByItemId(ItemForSpawn, 1, true);
					player.sendPacket(SystemMessage.removeItems(ItemForSpawn, 1));
					addSpawn(BossId, 91592, 113512, -3098, 0);
				}
				else if(this.getNpcId() == 50728 && checkSpawn())
				{
					player.getInventory().destroyItemByItemId(ItemForSpawn, 1, true);
					player.sendPacket(SystemMessage.removeItems(ItemForSpawn, 1));
					addSpawn(BossId, 109592, 125080, -3712, 0);
				}
				else if(this.getNpcId() == 50729 && checkSpawn())
				{
					player.getInventory().destroyItemByItemId(ItemForSpawn, 1, true);
					player.sendPacket(SystemMessage.removeItems(ItemForSpawn, 1));
					addSpawn(BossId, 120472, 113448, -3818, 0);
				}
				else if(this.getNpcId() == 50730 && checkSpawn())
				{
					player.getInventory().destroyItemByItemId(ItemForSpawn, 1, true);
					player.sendPacket(SystemMessage.removeItems(ItemForSpawn, 1));
					addSpawn(BossId, 109528, 111496, -3072, 0);
				}
				else
				{
					showMessage(player, "Я уже вызвал для Вас Босса!");
					return;
				}
			}
			else
			{
				showMessage(player, "Нет итема для вызова!");
				return;
			}
		}
		else
			super.onBypassFeedback(player, command);
	}

	private static void addSpawn(int npcId, int x, int y, int z, int heading)
	{
		spawn(new Location(x, y, z, heading), npcId);
	}

	private static L2NpcInstance spawn(Location loc, int npcId)
	{
		try
		{
			L2NpcInstance npc = NpcTable.getTemplate(npcId).getNewInstance();
			npc.setSpawnedLoc(loc.correctGeoZ());
			npc.onSpawn();
			npc.spawnMe(npc.getSpawnedLoc());
			return npc;
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public String getHtmlPath(int npcId, int val)
	{
		return "data/html/dragonvortex/" + npcId + ".htm";
	}

	private void showMessage(L2Player player, String message)
	{
		NpcHtmlMessage html = new NpcHtmlMessage(player, this);
		html.setHtml(message);
		player.sendPacket(html);
	}

	private final boolean checkSpawn()
	{
		for(L2NpcInstance npc : L2World.getAroundNpc(this, 3000, 100))
		{
			if(npc.getNpcId() == 25718 || npc.getNpcId() == 25719 || npc.getNpcId() == 25720 
			|| npc.getNpcId() == 25721 || npc.getNpcId() == 25722 || npc.getNpcId() == 25723 || npc.getNpcId() == 25724)
			{
				return false;
			}
		}
		return true;
	}
}