package l2ft.gameserver.model.instances;

import java.util.logging.Logger;

import l2ft.Config;
import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.model.L2Multisell;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.entity.Hero;
import l2ft.gameserver.model.entity.olympiad.CompType;
import l2ft.gameserver.model.entity.olympiad.NobleInfo;
import l2ft.gameserver.model.entity.olympiad.Olympiad;
import l2ft.gameserver.model.entity.olympiad.OlympiadDatabase;
import l2ft.gameserver.serverpackets.ExHeroList;
import l2ft.gameserver.serverpackets.ExOlympiadMatchList;
import l2ft.gameserver.serverpackets.L2GameServerPacket;
import l2ft.gameserver.serverpackets.NpcHtmlMessage;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.util.Files;
import l2ft.util.GArray;

public class L2OlympiadManagerInstance extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2OlympiadManagerInstance.class.getName());

	public L2OlympiadManagerInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
		if (Config.ENABLE_OLYMPIAD && template.npcId == 31688)
			Olympiad.addOlympiadNpc(this);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		if (!canBypassCheck(player, this))
			return;
		if (!Config.ENABLE_OLYMPIAD)
			return;
		if (command.startsWith("OlympiadDesc"))
		{
			int val = Integer.parseInt(command.substring(13, 14));
			String suffix = command.substring(14);
			showChatWindow(player, val, suffix);
		}
		else if (command.startsWith("OlympiadNoble"))
		{
			int m;
			if (!Olympiad.isNoble(player.getObjectId()) || !player.isNoble() || player.getBaseClassId() != player.getClassId().getId()) 
			{
				return;
			}
			int val = Integer.parseInt(command.substring(14));
			NpcHtmlMessage html = new NpcHtmlMessage(player, this);

			NobleInfo noble = Olympiad.getNobleInfo(player.getObjectId());

			switch (val)
			{
				case 1:
					Olympiad.unRegisterNoble(player);
					break;
				case 2:
					int j = 0;
					int k = 0;
					m = 0;
					int n = 0;
					int[] arrayOfInt = Olympiad.getWaitingList();
					if (arrayOfInt != null)
					{
						j = arrayOfInt[0];
			            k = arrayOfInt[1];
			            m = arrayOfInt[2];
			            n = arrayOfInt[3];
					}
					html.setFile("data/html/olympiad/noble_registered.htm");
					if (Config.ALT_OLY_REG_DISPLAY > 0)
					{
			            String str1 = new CustomMessage("l2ft.gameserver.model.instances.L2OlympiadManagerInstance.Fewer", player, new Object[] { Integer.valueOf(Config.ALT_OLY_REG_DISPLAY) }).toString();
			            String str2 = new CustomMessage("l2ft.gameserver.model.instances.L2OlympiadManagerInstance.More", player, new Object[] { Integer.valueOf(Config.ALT_OLY_REG_DISPLAY) }).toString();
			            html.replace("%listClassed%", j < Config.ALT_OLY_REG_DISPLAY ? str1 : str2);
			            html.replace("%listNonClassedRandomTeam%", m < Config.ALT_OLY_REG_DISPLAY ? str1 : str2);
			            html.replace("%listNonClassedTeam%", n < Config.ALT_OLY_REG_DISPLAY ? str1 : str2);
			            html.replace("%listNonClassed%", k < Config.ALT_OLY_REG_DISPLAY ? str1 : str2);
					}
					else
					{
			        	  html.replace("%listClassed%", String.valueOf(j));
			        	  html.replace("%listNonClassedRandomTeam%", String.valueOf(m));
			        	  html.replace("%listNonClassedTeam%", String.valueOf(n));
			        	  html.replace("%listNonClassed%", String.valueOf(k));
					}
					player.sendPacket(new L2GameServerPacket[] { html });
					break;
				case 3:
					html.setFile("data/html/olympiad/noble_points1.htm");
					html.replace("%points%", String.valueOf(noble != null ? noble.olympiad_points : 0));
					player.sendPacket(new L2GameServerPacket[] { html });
					break;
				case 4:
					Olympiad.registerNoble(player, CompType.NON_CLASSED);
					break;
				case 5:
					Olympiad.registerNoble(player, CompType.CLASSED);
					break;
				case 6:
					int passes = Olympiad.getNoblessePasses(player);
					if (passes > 0)
					{
						player.getInventory().addItem(Config.ALT_OLY_COMP_RITEM, passes);
						player.sendPacket(SystemMessage.obtainItems(Config.ALT_OLY_COMP_RITEM, passes, 0));
					}
					else 
					{
						player.sendPacket(html.setFile("data/html/olympiad/noble_nopoints.htm"));
					}
					break;
				case 7:
					L2Multisell.getInstance().SeparateAndSend(102, player, 0.0D);
					break;
				case 8:
					html.setFile("data/html/olympiad/noble_points2.htm");
					html.replace("%points%", String.valueOf(noble != null ? noble.olympiad_points_past : 0));
					player.sendPacket(new L2GameServerPacket[] { html });
					break;
				case 9:
					L2Multisell.getInstance().SeparateAndSend(103, player, 0.0D);
					break;
				case 10:
					Olympiad.registerNoble(player, CompType.TEAM_RANDOM);
					break;
				case 11:
					Olympiad.registerNoble(player, CompType.TEAM);
					break;
				default:
				_log.warning(new StringBuilder().append("Olympiad System: Couldnt send packet for request ").append(val).toString());
			}

		}
		else if (command.startsWith("Olympiad"))
		{
			int val = Integer.parseInt(command.substring(9, 10));

			NpcHtmlMessage reply = new NpcHtmlMessage(player, this);

			switch (val)
			{
				case 1:
					if (!Olympiad.isProgress(player)) break;
					player.sendPacket(new ExOlympiadMatchList()); 
					break;
				case 2:
					int classId = Integer.parseInt(command.substring(11));
					if (classId < 88)
						break;
					reply.setFile("data/html/olympiad/olympiad_ranking.htm");

					GArray<String> names = OlympiadDatabase.getClassLeaderBoard(classId);

					int index = 1;
					for (String name : names)
					{
						reply.replace(new StringBuilder().append("%place").append(index).append("%").toString(), String.valueOf(index));
						reply.replace(new StringBuilder().append("%rank").append(index).append("%").toString(), name);
						index++;
						if (index > 10)
							break;
					}
					for (; index <= 10; index++)
					{
						reply.replace(new StringBuilder().append("%place").append(index).append("%").toString(), "");
						reply.replace(new StringBuilder().append("%rank").append(index).append("%").toString(), "");
					}

					player.sendPacket(reply);
					break;
				case 3:
					if (!Config.ENABLE_OLYMPIAD_SPECTATING)
						break;
					Olympiad.addSpectator(Integer.parseInt(command.substring(11)), player);
					break;
				case 4:
					player.sendPacket(new ExHeroList());
					break;
				case 5:
					StringBuilder replyMSG = new StringBuilder("<html><body>");
					if (Hero.getInstance().isInactiveHero(player.getObjectId()))
					{
						Hero.getInstance().activateHero(player);
						replyMSG.append("Congratulations! You are a Hero now.");
					}
					else
					{
						replyMSG.append("You cannot be a Hero.");
					}
					replyMSG.append("</body></html>");
					reply.setHtml(replyMSG.toString());
					player.sendPacket(reply);
					break;
				default:
				_log.warning(new StringBuilder().append("Olympiad System: Couldnt send packet for request ").append(val).toString());
			}
		}
		else
		{
			super.onBypassFeedback(player, command);
		}
	}

	private void showChatWindow(L2Player player, int val, String suffix)
	{
		String filename = "data/html/olympiad/";
		if (val == 2 && suffix.equals("a") && Olympiad.isRegistered(player))
		{
			filename = "data/html/olympiad/noble_cancel.htm";
		}
		else
		{
			filename = new StringBuilder().append(filename).append("noble_desc").append(val).toString();
			filename = new StringBuilder().append(filename).append(suffix != null ? new StringBuilder().append(suffix).append(".htm").toString() : ".htm").toString();
			if (filename.equals("data/html/olympiad/noble_desc0.htm"))
				filename = "data/html/olympiad/noble_main.htm";
		}
		NpcHtmlMessage html = new NpcHtmlMessage(player, this);

		String file = Files.read(filename, player);
		if (file != null)
		{
			file = file.replaceAll("%raund%", Integer.toString(Olympiad._currentRound));
			file = file.replaceAll("%circle%", Integer.toString(Olympiad._currentCycle));
			file = file.replaceAll("%participants%", Integer.toString(Olympiad.getCountParticipants()));
		} 
		else
		{
			file = "System error!";
		}
		player.sendPacket(html.setHtml(file));
	}
}