package l2ft.gameserver.serverpackets;

import java.util.TreeSet;
import java.util.concurrent.ConcurrentLinkedQueue;

import l2ft.gameserver.TradeController.NpcTradeList;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.TradeItem;
import l2ft.gameserver.model.items.Inventory;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.util.GArray;

public class ExBuySellList extends L2GameServerPacket
{
	private int _listId;
	private int _done;
	private final GArray<TradeItem> _Buylist;
	private final TreeSet<L2ItemInstance> _SellList;
	private final ConcurrentLinkedQueue<L2ItemInstance> _RefundList;
	private long _money;
	private double _TaxRate = 0;
	private double _markup = 0;

	public ExBuySellList(NpcTradeList Buylist, L2Player activeChar, double taxRate)
	{
		this(Buylist, activeChar, taxRate, false);
	}
	
	public ExBuySellList(NpcTradeList Buylist, L2Player activeChar, double taxRate, boolean done)
	{
		_done = (done ? 1 : 0);
		if(Buylist != null)
		{
			_listId = Buylist.getListId();
			_Buylist = cloneAndFilter(Buylist.getItems());
			activeChar.setBuyListId(_listId);
		}
        else
            _Buylist = null;

		_money = activeChar.getAdena();
		_TaxRate = taxRate;
		_RefundList = activeChar.getInventory().getRefundItemsList();

		_SellList = new TreeSet<L2ItemInstance>(Inventory.OrderComparator);
		for(L2ItemInstance item : activeChar.getInventory().getItemsList())
			if(item.getItemId() != 57 && item.getItem().isSellable() && item.canBeTraded(activeChar) && item.getReferencePrice() > 0)
				_SellList.add(item);
		activeChar.sendPacket(new L2GameServerPacket[] { new ExBuySellList_Buy(_Buylist, _listId, _money, _TaxRate, _markup) });

	}

	protected static GArray<TradeItem> cloneAndFilter(GArray<TradeItem> list)
	{
		if(list == null)
			return null;

		GArray<TradeItem> ret = new GArray<TradeItem>(list.size());

		for(TradeItem item : list)
		{
			// А не пора ли обновить количество лимитированных предметов в трейд листе?
			if(item.getCurrentValue() < item.getCount() && item.getLastRechargeTime() + item.getRechargeTime() <= System.currentTimeMillis() / 60000)
			{
				item.setLastRechargeTime(item.getLastRechargeTime() + item.getRechargeTime());
				item.setCurrentValue(item.getCount());
			}

			if(item.getCurrentValue() == 0 && item.getCount() != 0)
				continue;

			ret.add(item);
		}

		return ret;
	}

	@Override
	protected final void writeImpl()
	{
		writeC(EXTENDED_PACKET);
		writeH(0xB7);
		writeD(1);

	    if (_SellList == null) {
	    	writeH(0);
	    }
	    else {
	    	writeH(_SellList.size());
	    	for (L2ItemInstance item : _SellList)
	    	{
	    		writeItem(item);
	    		writeQ(item.getItem().getReferencePrice() / 2L);
	    	}
	    }

	    if (_RefundList == null) {
	    	writeH(0);
	    }
	    else {
	    	writeH(_RefundList.size());
	    	for (L2ItemInstance item : _RefundList)
	    	{
	    		writeItem(item);
	    		writeD(item.getObjectId());
	    		writeQ(item.getItem().getReferencePrice() / 2L);
	    	}
	    }

	    writeC(_done);
	}
}