package l2ft.gameserver.serverpackets;

public class ExShowScreenMessage extends L2GameServerPacket
{
	public static enum ScreenMessageAlign
	{
		TOP_LEFT,
		TOP_CENTER,
		TOP_RIGHT,
		MIDDLE_LEFT,
		MIDDLE_CENTER,
		MIDDLE_RIGHT,
		BOTTOM_CENTER,
		BOTTOM_RIGHT,
	}

	private int _type, _sysMessageId;
	private int _npcMessageId = -1;
	private boolean _big_font, _effect;
	private ScreenMessageAlign _text_align;
	private int _time;
	private String _text;
	private String[] _npcMessageArgs = { "", "", "", "" };

	private int _unk1 = 0;
	private int _unk2 = 0;
	private int _unk3 = 0;
	private int _unk4 = 1;

	public ExShowScreenMessage(String text, int time, ScreenMessageAlign text_align, boolean big_font, int npcMessageId, String[] npcMessageArgs)
	{
		_type = 1;
		_sysMessageId = -1;
		_text = text;
		_time = time;
		_text_align = text_align;
		_big_font = big_font;
		_effect = false;
		_npcMessageId = npcMessageId;
	    for (int i = 0; (i < npcMessageArgs.length) && (i < _npcMessageArgs.length); i++)
	    	_npcMessageArgs[i] = npcMessageArgs[i];
	}

	public ExShowScreenMessage(String text, int time, ScreenMessageAlign text_align, boolean big_font)
	{
		this(text, time, text_align, big_font, -1, new String[0]);
	}

	public ExShowScreenMessage(String text, int time, ScreenMessageAlign text_align)
	{
		this(text, time, text_align, true);
	}

	public ExShowScreenMessage(String text, int time)
	{
		this(text, time, ScreenMessageAlign.MIDDLE_CENTER);
	}

	public ExShowScreenMessage(String text, int time, ScreenMessageAlign text_align, boolean big_font, int type, int messageId, boolean showEffect)
	{
		_type = type;
		_sysMessageId = messageId;
		_text = text;
		_time = time;
		_text_align = text_align;
		_big_font = big_font;
		_effect = showEffect;
	}

	public ExShowScreenMessage(int npcMessageId, int time, ScreenMessageAlign text_align, boolean big_font, int type, int messageId, boolean showEffect)
	{
		_type = type;
		_sysMessageId = messageId;
		_text = "";
		_time = time;
		_text_align = text_align;
		_big_font = big_font;
		_effect = showEffect;
		_npcMessageId = npcMessageId;
	}
	
	public ExShowScreenMessage(String text, int time, ScreenMessageAlign text_align, boolean big_font, int type, int messageId, boolean showEffect, int unk1, int unk2, int unk3, int unk4)
	{
		this(text, time, text_align, big_font, type, messageId, showEffect);
		_unk1 = unk1;
	    _unk2 = unk2;
	    _unk3 = unk3;
	    _unk4 = unk4;
	}

	@Override
	protected final void writeImpl()
	{
		writeC(EXTENDED_PACKET);
		writeH(0x39);

		writeD(_type); // 0 - system messages, 1 - your defined text
		writeD(_sysMessageId); // system message id (_type must be 0 otherwise no effect)
		writeD(_text_align.ordinal() + 1); // размещение текста
		writeD(_unk1); // ?
		writeD(_big_font ? 0 : 1); // размер текста 0 - нормальный, 1 - маленький
		writeD(_unk2); // ?
		writeD(_unk3); // ?
		writeD(_effect == true ? 1 : 0); // upper effect (0 - disabled, 1 enabled) - _position must be 2 (center) otherwise no effect
		writeD(_time); // время отображения сообщения в милисекундах
		writeD(_unk4); // ?
		writeD(_npcMessageId);
	    writeS(_text != null ? _text.replace("\n", "#") : "");
	    for (String s : _npcMessageArgs)
	    	writeS(s);
	}
}