package l2ft.gameserver.serverpackets;

import java.sql.ResultSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.L2Player;
import l2ft.util.GArray;

public class FriendListExtended extends L2GameServerPacket
{

	public static class FriendInfo
	{
    	
    	public FriendInfo(int objectId, String string, boolean online2,
				int int1, int int2) 
    	{
    		objId = objectId;
    		name = string;
    		online = online2;
    		classId = int1;
    		level = int2;
		}
		int objId;
		String name;
		boolean online;
		int classId;
		int level;
	}
    
   
	private static Logger _log = Logger.getLogger(FriendListExtended.class.getName());
	private GArray<FriendInfo> _friends = new GArray<FriendInfo>();

	public FriendListExtended(L2Player player)
	{
	  
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT `friend_id`, `char_name`, `class_id`, `level` FROM `character_friends` LEFT JOIN `characters` ON ( `character_friends`.`friend_id` = `characters`.`obj_Id` ) LEFT JOIN `character_subclasses` ON ( `character_friends`.`friend_id` = `character_subclasses`.`char_obj_id` AND `character_subclasses`.`active` = 1 ) WHERE char_id=? LIMIT 128");
			statement.setInt(1, player.getObjectId());
			rset = statement.executeQuery();

			while (rset.next())
			{
				int objectId = rset.getInt("friend_id");
				boolean online = L2ObjectsStorage.getPlayer(rset.getInt("friend_id")) != null;
				FriendInfo friend = new FriendInfo(objectId, rset.getString("char_name"), online, rset.getInt("class_id"), rset.getInt("level"));
				_friends.add(friend);
			}
		}
		catch (Exception e)
		{
			_log.log(Level.WARNING, "Error in friendlist ", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	protected final void writeImpl()
	{
		writeC(EXTENDED_PACKET);
		writeH(0x88);
		writeD(this._friends.size());
		for (FriendInfo friend : _friends)
		{
			writeD(friend.objId);
			writeS(friend.name);
			writeD(friend.online ? 1 : 0);
			writeD(friend.online ? friend.objId : 0);
			writeD(friend.classId);
			writeD(friend.level);
		}
	}
}