package l2ft.gameserver.serverpackets;

import java.util.logging.Logger;

import l2ft.extensions.network.SendablePacket;
import l2ft.gameserver.model.TradeItem;
import l2ft.gameserver.model.base.MultiSellIngredient;
import l2ft.gameserver.model.items.L2ItemInstance;
import l2ft.gameserver.network.L2GameClient;
import l2ft.gameserver.templates.L2Item;
import l2ft.gameserver.xml.ItemTemplates;

public abstract class L2GameServerPacket extends SendablePacket<L2GameClient>
{
	//Пускай все наследники используют этот логер
	//P.S. Можно удалить "Logger _log = Logger.getLogger" со всех ServerPacket, они будут наследовать вот этот.
	protected static final Logger _log = Logger.getLogger(L2GameServerPacket.class.getName());
	@Override
	protected void write()
	{
		try
		{
			writeImpl();
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
	}

	protected abstract void writeImpl();

	protected final static int EXTENDED_PACKET = 0xFE;

	@Override
	protected int getHeaderSize()
	{
		return 2;
	}

	@Override
	protected void writeHeader(int dataSize)
	{
		writeH(dataSize + getHeaderSize());
	}

	protected void writeEnchantEffect(L2ItemInstance item)
	{
        writeH(0);
        writeH(0);
        writeH(0);
	}

	protected void writeItem(TradeItem item, boolean trade)
	{
		writeD(item.getObjectId());
		writeD(item.getItemId());
		writeQ(item.getCurrentValue()); // max amount of items that a player can buy at a time (with this itemid)
		writeD(0x00);
	    writeH(item.getItem().getType2ForPackets()); // item type2
		writeH(0x00);
		writeH(item.getCustomType1()); // getCustomType1?
		writeD(item.getItem().getBodyPart()); // rev 415  slot    0006-lr.ear  0008-neck  0030-lr.finger  0040-head  0080-??  0100-l.hand  0200-gloves  0400-chest  0800-pants  1000-feet  2000-??  4000-r.hand  8000-r.hand
		writeH(item.getEnchantLevel()); // enchant level
		writeH(item.getCustomType2());

	    writeD(0x00);// Augmentation Id
	    writeD(-1);// Mana
	    writeD(0x00);// Shadow Life Time
	    writeItemElements(item);
		writeEnchantEffect(item);
	}
	
	protected void writeItem(L2ItemInstance item)
	{
		writeD(item.getObjectId());
		writeD(item.getItemId());
		writeD(item.getEquipSlot());
		writeQ(item.getCount());
		writeH(item.getItem().getType2());
		writeH(item.getCustomType1());
		writeH(item.isEquipped() ? 1 : 0);
		writeD(item.getBodyPart());
		writeH(item.getEnchantLevel());
		writeH(item.getCustomType2());
		writeD(item.getAugmentationId());
		writeD(item.isShadowItem() ? item.getLifeTimeRemaining() : -1);
		writeD(item.isTemporalItem() ? item.getLifeTimeRemaining() : 0x00);

		writeItemElements(item);
		writeEnchantEffect(item);
	}
	protected void writeEnchantEffect(TradeItem item)
	{
        writeH(0);
        writeH(0);
        writeH(0);
	}

	protected void writeEnchantEffect()
	{
        writeH(0);
        writeH(0);
        writeH(0);
	}

	protected void writeItemElements(L2ItemInstance item)
	{
		writeH(item.getAttackElement());
		writeH(item.getAttackElementValue());
		for (int i = 0; i < 6; i++) 
			writeH(item.getDeffAttr()[i]);
	}

	protected void writeItemElements(TradeItem item)
	{
		writeH(item.getAttackElement()[0]); // attack element (-2 - none)
		writeH(item.getAttackElement()[1]); // attack element value
		writeH(item.getDefenceFire()); // водная стихия (fire pdef)
		writeH(item.getDefenceWater()); // огненная стихия (water pdef)
		writeH(item.getDefenceWind()); // земляная стихия (wind pdef)
		writeH(item.getDefenceEarth()); // воздушная стихия (earth pdef)
		writeH(item.getDefenceHoly()); // темная стихия (holy pdef)
		writeH(item.getDefenceUnholy()); // светлая стихия (dark pdef)
	}

	protected void writeItemElements(MultiSellIngredient item)
	{
		if(item.getItemId() <= 0)
		{
			writeItemElements();
			return;
		}
		L2Item i = ItemTemplates.getInstance().getTemplate(item.getItemId());
		if(i.isWeapon())
		{
			writeH(item.getElement()); // attack element (-2 - none)
			writeH(item.getElementValue()); // attack element value
			writeH(0); // водная стихия (fire pdef)
			writeH(0); // огненная стихия (water pdef)
			writeH(0); // земляная стихия (wind pdef)
			writeH(0); // воздушная стихия (earth pdef)
			writeH(0); // темная стихия (holy pdef)
			writeH(0); // светлая стихия (dark pdef)
		}
		else if(i.isArmor())
		{
			writeH(-2); // attack element (-2 - none)
			writeH(0); // attack element value
			writeH(item.getElement() == L2Item.ATTRIBUTE_FIRE ? item.getElementValue() : 0); // водная стихия (fire pdef)
			writeH(item.getElement() == L2Item.ATTRIBUTE_WATER ? item.getElementValue() : 0); // огненная стихия (water pdef)
			writeH(item.getElement() == L2Item.ATTRIBUTE_WIND ? item.getElementValue() : 0); // земляная стихия (wind pdef)
			writeH(item.getElement() == L2Item.ATTRIBUTE_EARTH ? item.getElementValue() : 0); // воздушная стихия (earth pdef)
			writeH(item.getElement() == L2Item.ATTRIBUTE_HOLY ? item.getElementValue() : 0); // темная стихия (holy pdef)
			writeH(item.getElement() == L2Item.ATTRIBUTE_DARK ? item.getElementValue() : 0); // светлая стихия (dark pdef)
		}
		else
			writeItemElements();
	}

	protected void writeItemElements()
	{
		writeH(-2); // attack element (-2 - none)
		writeH(0x00); // attack element value
		writeH(0x00); // водная стихия (fire pdef)
		writeH(0x00); // огненная стихия (water pdef)
		writeH(0x00); // земляная стихия (wind pdef)
		writeH(0x00); // воздушная стихия (earth pdef)
		writeH(0x00); // темная стихия (holy pdef)
		writeH(0x00); // светлая стихия (dark pdef)
	}

	public String getType()
	{
		return "[S] " + getClass().getSimpleName();
	}

	@Override
	public String toString()
	{
		return getType() + "; buffer: " + getByteBuffer();
	}
}