package l2ft.gameserver.serverpackets;

import l2ft.gameserver.model.L2Clan;
import l2ft.gameserver.model.L2Clan.SubPledge;
import l2ft.gameserver.model.L2Skill;
import l2ft.util.GArray;

/**
 * Пример (828 протокол):
 * 0000: fe 3a 00 05 00 00 00 00 00 00 00 72 01 00 00 01    .:.........r....
 * 0010: 00 00 00 79 01 00 00 01 00 00 00 7b 01 00 00 02    ...y.......{....
 * 0020: 00 00 00 86 01 00 00 01 00 00 00 87 01 00 00 01    ................
 * 0030: 00 00 00                                           ...
 */
public class PledgeSkillList extends L2GameServerPacket
{
	private GArray<SkillInfo> clanSkills = new GArray<SkillInfo>();
	private GArray<SkillInfo> additionalsClanSkills = new GArray<SkillInfo>();

	public PledgeSkillList(L2Clan clan)
	{
		for(L2Skill sk : clan.getAllSkills())
			clanSkills.add(new SkillInfo(sk.getId(), sk.getLevel()));
		for (SubPledge sp : clan.getAllSubPledges())
	    	for (L2Skill sk : sp.getSkills())
	    		additionalsClanSkills.add(new SkillInfo(sp.getType(), sk.getId(), sk.getLevel()));
	}

	@Override
	protected final void writeImpl()
	{
		writeC(EXTENDED_PACKET);
		writeH(0x3a);
		writeD(clanSkills.size());
	    writeD(additionalsClanSkills.size());
		for(SkillInfo info : clanSkills)
		{
			writeD(info._id);
			writeD(info._level);
		}
		for (SkillInfo info : additionalsClanSkills)
	    {
			writeD(info._unit);
	     	writeD(info._id);
	     	writeD(info._level);
	    }
		clanSkills.clear();
	    additionalsClanSkills.clear();
	}

	static class SkillInfo
	{
		public int _unit;
	    public int _id;
	    public int _level;

		public SkillInfo(int id, int level) 
		{ 
			_unit = 0;
			_id = id;
			_level = level;
		}
		
		public SkillInfo(int unit, int id, int level)
		{
			_unit = unit;
		    _id = id;
		    _level = level;
		}
	}
}