package l2ft.gameserver.skills.conditions;

import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Clan;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Zone.ZoneType;
import l2ft.gameserver.model.entity.Duel;
import l2ft.gameserver.skills.Env;

public class ConditionTargetRelation extends Condition
{
	private final Relation _relation;

	
	public static enum Relation
	{
		Friend,
		Enemy,
		Neutral
	}

	public ConditionTargetRelation(Relation relation)
	{
		_relation = relation;
	}

	protected boolean testImpl(Env env)
	{
		return getRelation(env.character, env.target) == _relation;
	}

	public static Relation getRelation(L2Character activeChar, L2Character target)
	{
		if (activeChar.isPlayable() && activeChar.getPlayer() != null)
		{
			if (target.isMonster())
			{
				return Relation.Enemy;
			}
			L2Player player = activeChar.getPlayer();
			if (target.isPlayable() && target.getPlayer() != null)
			{
				L2Player pcTarget = target.getPlayer();

				if (player == pcTarget)
				{
					return Relation.Friend;
				}

				Duel duel1 = player.getDuel();
				Duel duel2 = pcTarget.getDuel();
				if (player != pcTarget && duel1 != null && duel1 == duel2)
				{
					if (duel1.getTeamForPlayer(pcTarget) == duel1.getTeamForPlayer(player))
						return Relation.Friend;
					return Relation.Enemy;
				}

				if (player.getParty() != null && player.getParty() == pcTarget.getParty())
				{
					return Relation.Friend;
				}
				if (player.isInOlympiadMode() && player.isOlympiadCompStart() && player.getOlympiadSide() == pcTarget.getOlympiadSide())
				{
					return Relation.Friend;
				}
				if (player.getTeam() > 0 && player.isChecksForTeam() && pcTarget.getTeam() > 0 && pcTarget.isChecksForTeam() && player.getTeam() == pcTarget.getTeam())
				{
					return Relation.Friend;
				}
				if (activeChar.isInZoneBattle()) 
				{
					return Relation.Enemy;
				}
				if (player.getClanId() != 0 && player.getClanId() == pcTarget.getClanId())
				{
					return Relation.Friend;
				}
				if (activeChar.isInZone(ZoneType.Siege) && target.isInZone(ZoneType.Siege))
				{
					if (player.getTerritorySiege() > -1 && player.getTerritorySiege() == pcTarget.getTerritorySiege())
						return Relation.Friend;
					L2Clan clan1 = player.getClan();
					L2Clan clan2 = pcTarget.getClan();
					if (clan1 != null && clan2 != null && clan1.getSiege() == null && clan1.getSiege() == clan2.getSiege())
					{
						if (clan1.isDefender() && clan2.isDefender())
							return Relation.Friend;
						if (!clan1.getSiege().isMidVictory() && clan1.isAttacker() && clan2.isAttacker())
							return Relation.Friend;
						return Relation.Enemy;
					}
				}

				if (activeChar.isInZonePeace())
				{
					return Relation.Neutral;
				}
				if (player.atMutualWarWith(pcTarget))
				{
					return Relation.Enemy;
				}
				if (pcTarget.getKarma() > 0)
				{
					return Relation.Enemy;
				}
				return Relation.Neutral;
			}

			return Relation.Neutral;
		}
		if (activeChar.isMonster())
		{
			if (!target.isNpc())
				return Relation.Enemy;
			return Relation.Neutral;
		}

		return Relation.Neutral;
	}
}
