package l2ft.gameserver.skills.effects;

import l2ft.gameserver.ai.CtrlIntention;
import l2ft.gameserver.model.L2Effect;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2World;
import l2ft.gameserver.model.L2WorldRegion;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.skills.Env;

public final class EffectInvisible extends L2Effect
{
	public EffectInvisible(Env env, EffectTemplate template)
	{
		super(env, template);
	}

	@Override
	public boolean checkCondition()
	{
		if(!_effected.isPlayer())
			return false;
		L2Player player = (L2Player) _effected;
		if(player.isInvisible())
			return false;
		if(player.isCombatFlagEquipped() || player.isTerritoryFlagEquipped())
			return false;
		return super.checkCondition();
	}

	@Override
	public void onStart()
	{
		super.onStart();
		L2Player player = (L2Player) _effected;
		player.setInvisible(true);
		player.sendUserInfo(true);
		if(player.getCurrentRegion() != null)
			for(L2WorldRegion neighbor : player.getCurrentRegion().getNeighbors())
				neighbor.removeObjectsFromPlayer(player);
		for (L2Character npc : L2World.getAroundNpc(player, 900, 200))
			if (npc != null && npc.getAI().getAttackTarget() == player)
			{
				npc.setTarget(null);
				npc.abortAttack(true, false);
				npc.abortCast(true);
				npc.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE);
			}
	}

	@Override
	public void onExit()
	{
		super.onExit();
		L2Player player = (L2Player) _effected;
		if(!player.isInvisible())
			return;
		player.setInvisible(false);
		player.broadcastUserInfo(true);
		if(player.getPet() != null)
			player.getPet().broadcastPetInfo();
	}

	@Override
	public boolean onActionTime()
	{
		return false;
	}
}