package l2ft.gameserver.skills.skillclasses;

import l2ft.gameserver.ai.CtrlEvent;
import l2ft.gameserver.ai.CtrlIntention;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Playable;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.instances.L2DecoyInstance;
import l2ft.gameserver.model.instances.L2NpcInstance;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.GArray;

public class Aggression extends L2Skill
{
	private final boolean _unaggring;
	private final boolean _silent;

	public Aggression(StatsSet set)
	{
		super(set);
		_unaggring = set.getBool("unaggroing", false);
		_silent = set.getBool("silent", false);
	}

	@Override
	public void useSkill(L2Character activeChar, GArray<L2Character> targets)
	{
		int effect = _effectPoint;

		if(isSSPossible() && (activeChar.getChargedSoulShot() || activeChar.getChargedSpiritShot() > 0))
			effect *= 2;

		for(L2Character target : targets)
			if(target != null)
			{
				if(!target.isNpc() && !target.isPlayable())
					continue;
				if(_unaggring)
				{
					if(target.isNpc() && activeChar.isPlayable())
						((L2Playable) activeChar).addDamageHate((L2NpcInstance) target, 0, -effect);
				}
				else if(activeChar instanceof L2DecoyInstance)
				{
					L2Player player = ((L2DecoyInstance) activeChar).getPlayer();
					if(player == target || player.getClanId() != 0 && player.getClanId() == target.getClanId())
						return;
					target.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, activeChar);
				}
				else
				{
					target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, activeChar, effect);
					if(!_silent && target.isNpc())
						((L2NpcInstance) target).callFriends(activeChar);
				}

				getEffects(activeChar, target, getActivateRate() > 0, false);
			}

		if(isSSPossible())
			activeChar.unChargeShots(isMagic());
	}
}