package l2ft.gameserver.skills.skillclasses;

import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Effect;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.skills.Formulas;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.GArray;

public class Cancel extends L2Skill
{
	private final String _dispelType;
	private final int _cancelRate;
	private final int _negateCount;

	public Cancel(StatsSet set)
	{
		super(set);
		_dispelType = set.getString("dispelType", "");
		_cancelRate = set.getInteger("cancelRate", 0);
		_negateCount = set.getInteger("negateCount", 5);
	}

	@Override
	public void useSkill(L2Character activeChar, GArray<L2Character> targets)
	{
		for (L2Character target : targets) 
		{
			if (target != null)
			{
				if (target.checkReflectSkill(activeChar, this)) 
					target = activeChar;
				
				byte counter = 0;
				L2Effect[] effect_list = target.getEffectList().getAllEffects().toArray(new L2Effect[0]);

				if (_dispelType.isEmpty())
				{
					for (int i = effect_list.length - 1; i >= 0; i--)
					{
						L2Effect e = effect_list[i];
						if (counter >= _negateCount || !e.getSkill().isCancelable() || !Formulas.calcCancelSuccess(this, e, activeChar, target, _cancelRate))
							continue;
						e.exit();
						counter = (byte)(counter + 1);
					}
				}
				else
				{
					counter = 0;
					if (_dispelType.contains("negative")) {
						for (int i = effect_list.length - 1; i >= 0; i--)
						{
							L2Effect e = effect_list[i];
							if (counter >= _negateCount || !e.getSkill().isOffensive() || !e.getSkill().isCancelable()) 
								continue;
	            	
							e.exit();
							counter = (byte)(counter + 1);
						}
					}

					counter = 0;
					if (_dispelType.contains("positive")) {
						for (int i = effect_list.length - 1; i >= 0; i--)
						{
							L2Effect e = effect_list[i];
							if (counter >= _negateCount || e.getSkill().isOffensive() || !e.getSkill().isCancelable() || !Formulas.calcCancelSuccess(this, e, activeChar, target, _cancelRate))
								continue;
							e.exit();
							counter = (byte)(counter + 1);
						}
					}
				}

				getEffects(activeChar, target, getActivateRate() > 0, false);
			}
		}
	    if (isSSPossible())
	    	activeChar.unChargeShots(isMagic());
	}

	public boolean isOffensive()
	{
		return !_dispelType.contains("negative");
	}
}