package l2ft.gameserver.skills.skillclasses;

import java.util.Comparator;
import java.util.TreeSet;

import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.instances.L2SiegeHeadquarterInstance;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.skills.Stats;
import l2ft.gameserver.skills.conditions.ConditionTargetRelation;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.GArray;

public class ChainHeal extends L2Skill
{
	private final int _healsCount;

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(target.isDoor() || target instanceof L2SiegeHeadquarterInstance)
			return false;
		if(activeChar.isPlayable() && target.isMonster())
			return false;
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	public ChainHeal(StatsSet set)
	{
		super(set);
		_healsCount = set.getInteger("healsCount", 10);
	}

	public void useSkill(L2Character activeChar, GArray<L2Character> targets)
	{
		int counter = 0;
		for (L2Character target : targets)
		{
			if (counter == 10)
				return;

			double maxNewHp = Math.max(30 - counter * 3, 10) * target.getMaxHp() / 100;

			counter++;

			double limit = target.calcStat(Stats.HP_LIMIT, null, null) - 100;
			double addToHp = Math.max(0, Math.min(maxNewHp, (limit == 0 ? 100 : limit) * target.getMaxHp() / 100 - target.getCurrentHp()));

			if (addToHp > 0)
				target.setCurrentHp(addToHp + target.getCurrentHp(), false);

			if (getId() == 4051)
			{
				target.sendPacket(Msg.REJUVENATING_HP);
			} 
			else if (target.isPlayer())
			{
				if (activeChar == target)
				{
					activeChar.sendPacket(new SystemMessage(1066).addNumber(Math.round(addToHp)));
				}
				else
					target.sendPacket(new SystemMessage(1067).addName(activeChar).addNumber(Math.round(addToHp)));
			}
			else if ((target.isSummon()) || (target.isPet()))
			{
				L2Player owner = target.getPlayer();
				if (owner != null)
				{
					if (activeChar == target)
						owner.sendMessage(new CustomMessage("YOU_HAVE_RESTORED_S1_HP_OF_YOUR_PET", owner, new Object[0]).addNumber(Math.round(addToHp)));
					else if (owner == activeChar)
						owner.sendMessage(new CustomMessage("YOU_HAVE_RESTORED_S1_HP_OF_YOUR_PET", owner, new Object[0]).addNumber(Math.round(addToHp)));
					else
						owner.sendMessage(new CustomMessage("S1_HAS_BEEN_RESTORED_S2_HP_OF_YOUR_PET", owner, new Object[0]).addString(activeChar.getName()).addNumber(Math.round(addToHp)));
				}
			}
		}
		if (isSSPossible())
			activeChar.unChargeShots(isMagic());
	}

	public GArray<L2Character> getTargets(L2Character activeChar, L2Character aimingTarget, boolean forceUse)
	{
		
		TreeSet<L2Character> targets_recollected = new TreeSet<L2Character>(ChainHeal.HealthComparator.instance);
		for (L2Character possible : aimingTarget.getAroundCharacters(350, 200)) 
		{
			if ((!possible.isDead()) && (!possible.isHealBlocked(true)) && (!possible.isCursedWeaponEquipped()) && (!possible.isCurrentHpFull()) && (ConditionTargetRelation.getRelation(activeChar, possible) == ConditionTargetRelation.Relation.Friend))
				targets_recollected.add(possible);
	    }
	    if ((!aimingTarget.isDead()) && (!aimingTarget.isHealBlocked(true)) && (!aimingTarget.isCursedWeaponEquipped()) && (!aimingTarget.isCurrentHpFull()) && (ConditionTargetRelation.getRelation(activeChar, aimingTarget) == ConditionTargetRelation.Relation.Friend)) 
	    {
	    	targets_recollected.add(aimingTarget);
	    }
	    GArray<L2Character> targets = new GArray<L2Character>();
	    for (int i = 0; (i < _healsCount) && (!targets_recollected.isEmpty()); i++) {
	    	targets.add(targets_recollected.pollFirst());
	    }
	    return targets;	
		
	}	

	private static class HealthComparator    implements Comparator<L2Character>
	{
		private static HealthComparator instance = new HealthComparator();

		public int compare(L2Character o1, L2Character o2)
		{
			if(o1 == null || o2 == null)
				return 0;
			if (o1.getCurrentHp() > o2.getCurrentHp())
				return 1;
			if (o1.getCurrentHp() < o2.getCurrentHp())
				return -1;
			
			return 0;
		}
	}
}