package l2ft.gameserver.skills.skillclasses;

import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Player;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.model.instances.L2ChestInstance;
import l2ft.gameserver.serverpackets.SystemMessage;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.GArray;
import l2ft.util.Rnd;

public class DeluxeKey extends L2Skill
{
	final int _unlockPower;

	public DeluxeKey(StatsSet set)
	{
		super(set);
		_unlockPower = set.getInteger("unlockPower", 0) + 100;
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(target == null || target instanceof L2ChestInstance && target.isDead())
		{
			activeChar.sendPacket(Msg.INVALID_TARGET);
			return false;
		}

		if(target instanceof L2ChestInstance && activeChar.isPlayer())
			return super.checkCondition(activeChar, target, forceUse, dontMove, first);

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, GArray<L2Character> targets)
	{
		for(L2Character targ : targets)
		{
			if(targ != null)
			{
				L2ChestInstance target = (L2ChestInstance) targ;

				if (!(activeChar instanceof L2Player))
					return;

				if(target.isDead())
					return;

				if(target.isFake())
				{
					target.onOpen((L2Player) activeChar);
					return;
				}

				double chance = getActivateRate();
				double levelmod = (double) getMagicLevel() - target.getLevel();
				chance += levelmod * getLevelModifier();

				if(chance < 0)
					chance = 1;

				if(chance < 100)
					activeChar.sendMessage(new CustomMessage("l2ft.gameserver.skills.skillclasses.Unlock.Chance", activeChar).addString(getName()).addNumber((long) chance));

				if(Rnd.chance(chance))
				{
					activeChar.sendMessage(new CustomMessage("l2ft.gameserver.skills.skillclasses.Unlock.Success", activeChar));
					target.onOpen((L2Player) activeChar);
				}
				else
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_FAILED).addString(getName()));
					target.doDie(activeChar);
				}
			}
		}
	}
}