package l2ft.gameserver.skills.skillclasses;

import java.util.Iterator;

import l2ft.extensions.multilang.CustomMessage;
import l2ft.gameserver.cache.Msg;
import l2ft.gameserver.model.L2Character;
import l2ft.gameserver.model.L2Effect;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.skills.EffectType;
import l2ft.gameserver.skills.Formulas;
import l2ft.gameserver.templates.StatsSet;
import l2ft.util.GArray;

public class StealBuff extends L2Skill
{
	private final int _stealCount;
	private final int _stealRate;

	public StealBuff(StatsSet set)
	{
		super(set);
		_stealCount = set.getInteger("stealCount", 1);
		_stealRate = set.getInteger("stealRate", 100);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if ((target == null) || (!target.isPlayer()))
		{
			activeChar.sendPacket(Msg.THAT_IS_THE_INCORRECT_TARGET);
			return false;
	    }

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, GArray<L2Character> targets)
	{
		for (L2Character target : targets) 
		{
			if (target != null)
			{
				if (!target.isPlayer()) 
					continue;
				
				if (target.checkReflectSkill(activeChar, this)) 
					target = activeChar;
				
				L2Effect[] effects = target.getEffectList().getAllFirstEffects();
				if (effects.length == 0) 
					continue;
	        
				Result result = new Result();

				steal(activeChar, target, effects, true, result);

				steal(activeChar, target, effects, false, result);

				if (result.update)
				{
					activeChar.sendMessage(new CustomMessage("l2ft.gameserver.skills.skillclasses.StealBuff.Success", activeChar, new Object[0]).addNumber(result.counter));
					activeChar.sendChanges();
					activeChar.updateEffectIcons();
				}

				getEffects(activeChar, target, getActivateRate() > 0, false);
			}
		}
		activeChar.abortCast(true);
		if (isSSPossible())
			activeChar.unChargeShots(isMagic());
	}

	private void steal(L2Character activeChar, L2Character target, L2Effect[] effects, boolean music, Result result)
	{
		for (int i = effects.length - 1; (result.counter < _stealCount) && (i >= 0); i--)
		{
			L2Effect e = effects[i];
			if (e == null || e.getSkill().isMusic() != music || !checkEffect(e))
				continue;
			if (!Formulas.calcCancelSuccess(this, e, activeChar, target, _stealRate))
				continue;
			GArray<L2Effect> effectsBySkill = target.getEffectList().getEffectsBySkill(e.getSkill());
			if (effectsBySkill == null)
				continue;
			Iterator<L2Effect> i$ = effectsBySkill.iterator();
			for(i$ = effectsBySkill.iterator(); i$.hasNext();)
			{
				L2Effect el = i$.next();
				if(!checkEffect(el))
					continue ;
				break;
			}

			i$ = effectsBySkill.iterator();
			do
			{
				if(!i$.hasNext())
					break;
				L2Effect el = i$.next();
				L2Effect stealedEffect = el.clone(activeChar);
				el.exit();
				if(stealedEffect != null)
				{
					activeChar.getEffectList().addEffect(stealedEffect, false);
					result.update = true;
				}
			} 
			while(true);
			result.counter++;
		}
	}

	private boolean checkEffect(L2Effect e)
	{
		return e.getSkill().isCancelable() && !e.getSkill().isPassive() && !e.isOffensive() && e.getEffectType() != EffectType.Vitality;
	}

	private static class Result
	{
	    int counter;
	    boolean update;
	}
}