package l2ft.gameserver.tables;

import java.io.File;
import java.util.HashMap;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;

import l2ft.Config;
import l2ft.gameserver.idfactory.IdFactory;
import l2ft.gameserver.model.L2Territory;
import l2ft.gameserver.model.instances.L2DoorInstance;
import l2ft.gameserver.templates.L2CharTemplate;
import l2ft.gameserver.templates.StatsSet;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

public class DoorTable
{
	private static final Logger _log = Logger.getLogger(DoorTable.class.getName());

	private HashMap<Integer, L2DoorInstance> _staticItems;

	private static DoorTable _instance;

	public static DoorTable getInstance()
	{
		if(_instance == null)
			new DoorTable();
		return _instance;
	}

	public DoorTable()
	{
		_instance = this;
		_staticItems = new HashMap<Integer, L2DoorInstance>();
		FillDoors();
	}

	public void respawn()
	{
		for(L2DoorInstance door : _staticItems.values())
			if(door != null)
				door.decayMe();
		_instance = new DoorTable();
	}

	private void FillDoors()
	{
		DocumentBuilderFactory localDocumentBuilderFactory = DocumentBuilderFactory.newInstance();
		try
		{
			File localFile = new File(Config.DATAPACK_ROOT + "/data/xml/doors.xml");
			if(localFile == null)
				_log.config("DoorTable: NO Loadedes " + localFile + " doors.");
			Document localDocument = localDocumentBuilderFactory.newDocumentBuilder().parse(localFile);
			for (Node localNodeDoor = localDocument.getFirstChild(); localNodeDoor != null; localNodeDoor = localNodeDoor.getNextSibling())
			{
				if (!localNodeDoor.getNodeName().equalsIgnoreCase("doors"))
					continue;
				for (Node localNodeDoor1 = localNodeDoor.getFirstChild(); localNodeDoor1 != null; localNodeDoor1 = localNodeDoor1.getNextSibling())
				{
					if (!localNodeDoor1.getNodeName().equalsIgnoreCase("door"))
						continue;
					NamedNodeMap localNamedNodeMap = localNodeDoor1.getAttributes();
					int _id = Integer.parseInt(localNamedNodeMap.getNamedItem("id").getNodeValue());
					String _name = localNamedNodeMap.getNamedItem("name").getNodeValue();
					int _hp = Integer.parseInt(localNamedNodeMap.getNamedItem("hp").getNodeValue());
					int _pdef = Integer.parseInt(localNamedNodeMap.getNamedItem("pdef").getNodeValue());
					int _mdef = Integer.parseInt(localNamedNodeMap.getNamedItem("mdef").getNodeValue());
					boolean _unlockable = Boolean.parseBoolean(localNamedNodeMap.getNamedItem("unlockable").getNodeValue());
					int _key = Integer.parseInt(localNamedNodeMap.getNamedItem("key").getNodeValue());
					byte _level = Byte.parseByte(localNamedNodeMap.getNamedItem("level").getNodeValue());
					boolean _showHp = Boolean.parseBoolean(localNamedNodeMap.getNamedItem("showHp").getNodeValue());
					int _posx = Integer.parseInt(localNamedNodeMap.getNamedItem("posx").getNodeValue());
					int _posy = Integer.parseInt(localNamedNodeMap.getNamedItem("posy").getNodeValue());
					int _posz = Integer.parseInt(localNamedNodeMap.getNamedItem("posz").getNodeValue());
					int _ax = Integer.parseInt(localNamedNodeMap.getNamedItem("ax").getNodeValue());
					int _ay = Integer.parseInt(localNamedNodeMap.getNamedItem("ay").getNodeValue());
					int _bx = Integer.parseInt(localNamedNodeMap.getNamedItem("bx").getNodeValue());
					int _by = Integer.parseInt(localNamedNodeMap.getNamedItem("by").getNodeValue());
					int _cx = Integer.parseInt(localNamedNodeMap.getNamedItem("cx").getNodeValue());
					int _cy = Integer.parseInt(localNamedNodeMap.getNamedItem("cy").getNodeValue());
					int _dx = Integer.parseInt(localNamedNodeMap.getNamedItem("dx").getNodeValue());
					int _dy = Integer.parseInt(localNamedNodeMap.getNamedItem("dy").getNodeValue());
					int _minz = Integer.parseInt(localNamedNodeMap.getNamedItem("minz").getNodeValue());
					int _maxz = Integer.parseInt(localNamedNodeMap.getNamedItem("maxz").getNodeValue());
					boolean _siege_weapon = Boolean.parseBoolean(localNamedNodeMap.getNamedItem("siege_weapon").getNodeValue());
					boolean _geodata = Boolean.parseBoolean(localNamedNodeMap.getNamedItem("geodata").getNodeValue());

					StatsSet baseDat = new StatsSet();
					baseDat.set("level", 0);
					baseDat.set("jClass", "door");
					baseDat.set("baseSTR", 0);
					baseDat.set("baseCON", 0);
					baseDat.set("baseDEX", 0);
					baseDat.set("baseINT", 0);
					baseDat.set("baseWIT", 0);
					baseDat.set("baseMEN", 0);
					baseDat.set("baseShldDef", 0);
					baseDat.set("baseShldRate", 0);
					baseDat.set("baseAccCombat", 38);
					baseDat.set("baseEvasRate", 38);
					baseDat.set("baseCritRate", 38);
					baseDat.set("collision_radius", 5);
					baseDat.set("collision_height", 0);
					baseDat.set("sex", "male");
					baseDat.set("type", "");
					baseDat.set("baseAtkRange", 0);
					baseDat.set("baseMpMax", 0);
					baseDat.set("baseCpMax", 0);
					baseDat.set("revardExp", 0);
					baseDat.set("revardSp", 0);
					baseDat.set("basePAtk", 0);
					baseDat.set("baseMAtk", 0);
					baseDat.set("basePAtkSpd", 0);
					baseDat.set("aggroRange", 0);
					baseDat.set("baseMAtkSpd", 0);
					baseDat.set("rhand", 0);
					baseDat.set("lhand", 0);
					baseDat.set("armor", 0);
					baseDat.set("baseWalkSpd", 0);
					baseDat.set("baseRunSpd", 0);
					baseDat.set("baseHpReg", 0);
					baseDat.set("baseCpReg", 0);
					baseDat.set("baseMpReg", 0);
					baseDat.set("siege_weapon", false);
					baseDat.set("geodata", true);
					
					StatsSet npcDat = baseDat.clone();
					npcDat.set("npcId", _id);
					npcDat.set("name", _name);
					npcDat.set("baseHpMax", _hp);
					npcDat.set("basePDef", _pdef);
					npcDat.set("baseMDef", _mdef);
					
					L2CharTemplate template = new L2CharTemplate(npcDat);
					L2DoorInstance door = new L2DoorInstance(IdFactory.getInstance().getNextId(), template, _id, _name, _unlockable, _showHp);
					_staticItems.put(_id, door);
					L2Territory pos = new L2Territory(_id);
					door.setGeoPos(pos);
					pos.add(_ax, _ay, _minz, _maxz);
					pos.add(_bx, _by, _minz, _maxz);
					pos.add(_cx, _cy, _minz, _maxz);
					pos.add(_dx, _dy, _minz, _maxz);
					door.getTemplate().collisionHeight = _maxz - _minz & 0xfff0;
					door.getTemplate().collisionRadius = Math.max(50, Math.min(_posx - pos.getXmin(), _posy - pos.getYmin()));
					door.setXYZInvisible(_posx, _posy, _minz + 32);
					door.setCurrentHpMp(door.getMaxHp(), door.getMaxMp(), true);
					door.setOpen(false);
					door.level = _level;
					door.key = _key;
					// Дверь/стена может быть атакована только осадным оружием
					door.setSiegeWeaponOlnyAttackable(_siege_weapon);
					door.setGeodata(_geodata);
					door.spawnMe(door.getLoc());
				}
			}
		}
		catch (Exception localException)
		{
			localException.printStackTrace();
		}

		_log.config("DoorTable: Loaded " + _staticItems.size() + " doors.");
	}

	public boolean isInitialized()
	{
		return _initialized;
	}

	private boolean _initialized = true;

	public L2DoorInstance getDoor(Integer id)
	{
		return _staticItems.get(id);
	}

	public void putDoor(Integer id, L2DoorInstance door)
	{
		_staticItems.put(id, door);
	}

	public void removeDoor(Integer id)
	{
		_staticItems.remove(id);
	}

	public L2DoorInstance[] getDoors()
	{
		return _staticItems.values().toArray(new L2DoorInstance[_staticItems.size()]);
	}

	/**
	 * Performs a check and sets up a scheduled task for
	 * those doors that require auto opening/closing.
	 */
	public void checkAutoOpen()
	{
		for(L2DoorInstance doorInst : getDoors())

			// Garden of Eva (every 7 minutes)
			if(doorInst.getDoorName().startsWith("Eva_"))
				doorInst.setAutoActionDelay(420000);

			// Tower of Insolence (every 5 minutes)
			else if(doorInst.getDoorName().startsWith("hubris_"))
				doorInst.setAutoActionDelay(300000);

			// Giran Devil island (every 5 minutes)
			else if(doorInst.getDoorName().startsWith("Devil.opendoor"))
				doorInst.setAutoActionDelay(300000);

			// Coral Garden Gate (every 15 minutes)
			else if(doorInst.getDoorName().startsWith("Coral_garden"))
				doorInst.setAutoActionDelay(900000);

			// Normil's cave (every 5 minutes)
			else if(doorInst.getDoorName().startsWith("Normils_cave"))
				doorInst.setAutoActionDelay(300000);

			// Normil's Garden (every 15 minutes)
			else if(doorInst.getDoorName().startsWith("Normils_garden"))
				doorInst.setAutoActionDelay(900000);
	}
}