package l2ft.gameserver.tables;

import java.sql.ResultSet;
import java.util.logging.Logger;

import l2ft.database.DatabaseUtils;
import l2ft.database.FiltredPreparedStatement;
import l2ft.database.L2DatabaseFactory;
import l2ft.database.ThreadConnection;
import l2ft.gameserver.model.L2Skill;
import l2ft.gameserver.skills.Stats;
import l2ft.gameserver.skills.funcs.Func;
import l2ft.gameserver.skills.funcs.FuncAdd;
import l2ft.gameserver.templates.L2NpcTemplate;
import l2ft.util.GArray;

public final class PreloadedAttributeTable
{
	private static final Logger _log = Logger.getLogger(PreloadedAttributeTable.class.getName());

	public static void load()
	{
		int count = 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT * FROM `npc_elementals`");
			rs = statement.executeQuery();

			while (rs.next())
			{
				L2NpcTemplate templ = NpcTable.getTemplate(rs.getInt("npc_id"));

				if (templ == null)
				{
					_log.warning("Not found template for " + rs.getInt("npc_id") + ", but preloaded attribute is defined.");
					continue;
				}

				if (templ.funcs == null) {
					templ.funcs = new GArray<Func>(7);
				}
				if (rs.getInt("elemAtkValue") > 0)
					templ.funcs.add(new FuncAdd(L2Skill.Element.getElementById(rs.getInt("elemAtkType")).getAttack(), 0, templ, rs.getInt("elemAtkValue")));
				if (rs.getInt("fireDefValue") != 0)
					templ.funcs.add(new FuncAdd(Stats.FIRE_RECEPTIVE, 0, templ, -rs.getInt("fireDefValue")));
				if (rs.getInt("waterDefValue") != 0)
					templ.funcs.add(new FuncAdd(Stats.WATER_RECEPTIVE, 0, templ, -rs.getInt("waterDefValue")));
				if (rs.getInt("earthDefValue") != 0)
					templ.funcs.add(new FuncAdd(Stats.EARTH_RECEPTIVE, 0, templ, -rs.getInt("earthDefValue")));
				if (rs.getInt("windDefValue") != 0)
					templ.funcs.add(new FuncAdd(Stats.WIND_RECEPTIVE, 0, templ, -rs.getInt("windDefValue")));
				if (rs.getInt("holyDefValue") != 0)
					templ.funcs.add(new FuncAdd(Stats.SACRED_RECEPTIVE, 0, templ, -rs.getInt("holyDefValue")));
				if (rs.getInt("darkDefValue") != 0) {
					templ.funcs.add(new FuncAdd(Stats.UNHOLY_RECEPTIVE, 0, templ, -rs.getInt("darkDefValue")));
				}
				count++;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rs);
		}
		_log.info("Preloaded element attributes for " + count + " monsters.");
	}
}