package l2ft.gameserver.tables;

import java.lang.ref.WeakReference;
import java.util.Map.Entry;
import javolution.util.FastMap;
import l2ft.gameserver.model.L2ObjectsStorage;
import l2ft.gameserver.model.Reflection;
import l2ft.gameserver.model.entity.KamalokaNightmare;
import l2ft.util.GArray;

public class ReflectionTable
{
	private FastMap<Integer, WeakReference<Reflection>> _playersMatchList = new FastMap<Integer, WeakReference<Reflection>>();
	public static long SOD_REFLECTION_ID = 0;
	public static final long DEFAULT = 0;
	public static final long PARNASSUS = -1;
	public static final long GH = -2;
	public static final long JAIL = -3;
	public static final long MULTILAYER = -4;
	public static final int SOI_HEART_OF_INFINITY_DEFENCE = 122;
	public static final int SOI_HALL_OF_EROSION_DEFENCE = 120;
	public static final int SOI_HALL_OF_EROSION_ATTACK = 119;
	public static final int SOI_HALL_OF_SUFFERING_SECTOR2 = 116;
	public static final int SOI_HALL_OF_SUFFERING_SECTOR1 = 115;
	public static final int DISCIPLES_NECROPOLIS = 112;

	private static ReflectionTable _instance;
	private static Reflection _default = new Reflection(0);

	public static ReflectionTable getInstance()
	{
		if(_instance == null)
			_instance = new ReflectionTable();
		return _instance;
	}

	private FastMap<Integer, Long> _soloKamalokaList = new FastMap<Integer, Long>();

	private Reflection[] staticData = new Reflection[10];

	public synchronized void addReflection(Reflection r)
	{
		if(r.getId() <= 0) // отрицательные номера у статичных отражений
		{
			staticData[(int) Math.abs(r.getId())] = r;
			return;
		}

		r.setId(L2ObjectsStorage.put(r));
	}

	public Reflection get(long index, boolean CreateIfNonExist)
	{
		Reflection ret = null;
		if(index <= 0)
		{
			if(staticData.length > Math.abs(index))
				ret = staticData[(int) Math.abs(index)];
		}
		else
			ret = (Reflection) L2ObjectsStorage.get(index);
		if(CreateIfNonExist && ret == null)
			ret = new Reflection(index);
		return ret;
	}

	public Reflection get(long index)
	{
		return get(index, false);
	}

	public Reflection getDefault()
	{
		return _default;
	}

	public void addSoloKamaloka(Integer player, KamalokaNightmare r)
	{
		_soloKamalokaList.put(player, r.getId());
	}

	public void removeSoloKamaloka(Integer player)
	{
		_soloKamalokaList.remove(player);
	}

	public KamalokaNightmare findSoloKamaloka(Integer player)
	{
		Long index = _soloKamalokaList.get(player);
		if(index == null)
			return null;
		Reflection found = get(index);
		if(found == null || !(found instanceof KamalokaNightmare) || ((KamalokaNightmare) found).getPlayerId() != player)
		{
			_soloKamalokaList.remove(player);
			return null;
		}
		return (KamalokaNightmare) get(index);
	}

	public Reflection playerFindReflection(Integer playerObjId)
	{
		WeakReference<Reflection> ref = _playersMatchList.get(playerObjId);
		if (ref == null)
			return null;
		Reflection found = ref.get();
		if ((found == null) || (found.isCollapseStarted()))
		{
			_playersMatchList.remove(playerObjId);
			return null;
		}
		return found;
	}

	public void playerAddReflection(Integer playerObjId, Reflection r)
	{
		_playersMatchList.put(playerObjId, new WeakReference<Reflection>(r));
	}

	public void playerRemoveReflection(Integer playerObjId)
	{
		_playersMatchList.remove(playerObjId);
	}

	@SuppressWarnings("unchecked")
	public GArray<Integer> getPlayersEntered(Reflection r)
	{
		GArray list = new GArray(9);
		for (Entry entry : _playersMatchList.entrySet())
			if ((entry.getValue() != null) && (((Reflection)((WeakReference)entry.getValue()).get()).equals(r)))
				list.add(entry.getKey());
		return list.isEmpty() ? null : list;
	}
}