package l2ft.gameserver.templates;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import l2ft.gameserver.model.base.ClassType;
import l2ft.gameserver.skills.Stats;
import l2ft.gameserver.skills.funcs.FuncTemplate;
import l2ft.gameserver.tables.SkillTable;
import l2ft.gameserver.templates.L2Weapon.WeaponType;

public final class L2Armor extends L2Item
{
	public static final double EMPTY_RING = 5;
	public static final double EMPTY_EARRING = 9;
	public static final double EMPTY_NECKLACE = 13;
	public static final double EMPTY_HELMET = 12;
	public static final double EMPTY_BODY_FIGHTER = 31;
	public static final double EMPTY_LEGS_FIGHTER = 18;
	public static final double EMPTY_BODY_MYSTIC = 15;
	public static final double EMPTY_LEGS_MYSTIC = 8;
	public static final double EMPTY_GLOVES = 8;
	public static final double EMPTY_BOOTS = 7;

	private final int _pDef;
	private final int _mDef;
	private final int _mpBonus;
	private final double _evsmod;
	private final ClassType _classtype;
    private ConcurrentHashMap<String, List<Integer>> exInfo;

    public enum ArmorType
	{
		NONE(1, "None"),
		LIGHT(2, "Light"),
		HEAVY(3, "Heavy"),
		MAGIC(4, "Magic"),
		PET(5, "Pet"),
		SIGIL(6, "Sigil");

		final int _id;
		final String _name;

		ArmorType(int id, String name)
		{
			_id = id;
			_name = name;
		}

		public long mask()
		{
			return 1L << (_id + WeaponType.values().length);
		}

		@Override
		public String toString()
		{
			return _name;
		}
	}

	/**
	 * Constructor<?> for Armor.<BR><BR>
	 * <U><I>Variables filled :</I></U><BR>
	 * <LI>_avoidModifier</LI>
	 * <LI>_pDef & _mDef</LI>
	 * <LI>_mpBonus & _hpBonus</LI>
	 * @param type : L2ArmorType designating the type of armor
	 * @param set : StatsSet designating the set of couples (key,value) caracterizing the armor
	 * @see L2Item constructor
	 */
	public L2Armor(ArmorType type, StatsSet set)
	{
		super(type, set);

		_pDef = set.getInteger("p_def", 0);
		_mDef = set.getInteger("m_def", 0);
		_mpBonus = set.getInteger("mp_bonus", 0);
		_evsmod = set.getDouble("avoid_modify", 0.0);
		_classtype = set.getEnum("player_class", ClassType.class, null);

		int sId = set.getInteger("enchant4_skill_id", 0);
		int sLv = set.getInteger("enchant4_skill_lvl", 0);
		if(sId > 0 && sLv > 0)
			_enchant4Skill = SkillTable.getInstance().getInfo(sId, sLv);

		int sId4 = set.getInteger("enchantdo4_skill_id", 0);
		int sLv4 = set.getInteger("enchantdo4_skill_lvl", 0);
		if(sId4 > 0 && sLv4 > 0)
			_enchantdo4Skill = SkillTable.getInstance().getInfo(sId4, sLv4);

		int sId5 = set.getInteger("enchantdo5_skill_id", 0);
		int sLv5 = set.getInteger("enchantdo5_skill_lvl", 0);
		if(sId5 > 0 && sLv5 > 0)
			_enchantdo5Skill = SkillTable.getInstance().getInfo(sId5, sLv5);

		int sId6 = set.getInteger("enchantdo6_skill_id", 0);
		int sLv6 = set.getInteger("enchantdo6_skill_lvl", 0);
		if(sId6 > 0 && sLv6 > 0)
			_enchantdo6Skill = SkillTable.getInstance().getInfo(sId6, sLv6);

		int sId7 = set.getInteger("enchantdo7_skill_id", 0);
		int sLv7 = set.getInteger("enchantdo7_skill_lvl", 0);
		if(sId7 > 0 && sLv7 > 0)
			_enchantdo7Skill = SkillTable.getInstance().getInfo(sId7, sLv7);

		int sId71 = set.getInteger("enchantdo71_skill_id", 0);
		int sLv71 = set.getInteger("enchantdo71_skill_lvl", 0);
		if(sId71 > 0 && sLv71 > 0)
			_enchantdo71Skill = SkillTable.getInstance().getInfo(sId71, sLv71);
		
		int sId8 = set.getInteger("enchantdo8_skill_id", 0);
		int sLv8 = set.getInteger("enchantdo8_skill_lvl", 0);
		if(sId8 > 0 && sLv8 > 0)
			_enchantdo8Skill = SkillTable.getInstance().getInfo(sId8, sLv8);

		int sId81 = set.getInteger("enchantdo81_skill_id", 0);
		int sLv81 = set.getInteger("enchantdo81_skill_lvl", 0);
		if(sId81 > 0 && sLv81 > 0)
			_enchantdo81Skill = SkillTable.getInstance().getInfo(sId81, sLv81);
		
		int sId9 = set.getInteger("enchantdo9_skill_id", 0);
		int sLv9 = set.getInteger("enchantdo9_skill_lvl", 0);
		if(sId9 > 0 && sLv9 > 0)
			_enchantdo9Skill = SkillTable.getInstance().getInfo(sId9, sLv9);

		int sId91 = set.getInteger("enchantdo91_skill_id", 0);
		int sLv91 = set.getInteger("enchantdo91_skill_lvl", 0);
		if(sId91 > 0 && sLv91 > 0)
			_enchantdo91Skill = SkillTable.getInstance().getInfo(sId91, sLv91);

		int sId10 = set.getInteger("enchantdo10_skill_id", 0);
		int sLv10 = set.getInteger("enchantdo10_skill_lvl", 0);
		if(sId10 > 0 && sLv10 > 0)
			_enchantdo10Skill = SkillTable.getInstance().getInfo(sId10, sLv10);

		int sId101 = set.getInteger("enchantdo101_skill_id", 0);
		int sLv101 = set.getInteger("enchantdo101_skill_lvl", 0);
		if(sId101 > 0 && sLv101 > 0)
			_enchantdo101Skill = SkillTable.getInstance().getInfo(sId101, sLv101);

		if(_pDef > 0)
			attachFunction(new FuncTemplate(null, "Add", Stats.POWER_DEFENCE, 0x10, _pDef));
		if(_mDef > 0)
			attachFunction(new FuncTemplate(null, "Add", Stats.MAGIC_DEFENCE, 0x10, _mDef));
		if(_mpBonus > 0)
			attachFunction(new FuncTemplate(null, "Add", Stats.MAX_MP, 0x10, _mpBonus));
		if(_evsmod != 0)
			attachFunction(new FuncTemplate(null, "Add", Stats.EVASION_RATE, 0x10, _evsmod));

		if(_crystalType != Grade.NONE)
		{
			if(_pDef > 0)
			{
				attachFunction(new FuncTemplate(null, "Enchant", Stats.POWER_DEFENCE, 0x0C, 0));
				if(set.getInteger("type2") == L2Item.TYPE2_SHIELD_ARMOR)
					attachFunction(new FuncTemplate(null, "Enchant", Stats.MAX_HP, 0x80, 0));
			}
			if(_mDef > 0)
				attachFunction(new FuncTemplate(null, "Enchant", Stats.MAGIC_DEFENCE, 0x0C, 0));
		}
	}

	/**
	 * Returns the type of the armor.
	 * @return L2ArmorType
	 */
	@Override
	public ArmorType getItemType()
	{
		return (ArmorType) super.type;
	}

	/**
	 * Returns the ID of the item after applying the mask.
	 * @return int : ID of the item
	 */
	@Override
	public final long getItemMask()
	{
		return getItemType().mask();
	}

	public ClassType getClassType()
	{
		return _classtype;
	}
}